@@ -8,6 +8,7 @@ import java.net.URL;
 
 public class ArrayClassLoader extends ClassLoader
 {
+  /** Map String to union(byte[], Class, ClassType). */
   Hashtable map = new Hashtable(100);
 
   /** If non-null, context to use for finding resources. */
@@ -53,12 +54,8 @@ public class ArrayClassLoader extends ClassLoader
   }
 
   public void addClass (ClassType ctype)
-    throws java.io.IOException
   {
-    if ((ctype.flags & ClassType.EXISTING_CLASS) != 0)
-      addClass(ctype. getReflectClass());
-    else
-      addClass(ctype.getName(), ctype.writeToArray());
+    map.put(ctype.getName(), ctype);
   }
 
   protected URL findResource(String name)
@@ -79,24 +76,34 @@ public class ArrayClassLoader extends ClassLoader
     return super.findResource(name);
   }
 
-  /* #ifdef JAVA2 */
-  public Class findClass (String name)
-  /* #else */
-  // public Class loadClass (String name, boolean resolve)
-  /* #endif */
+  public Class loadClass (String name, boolean resolve)
+    throws ClassNotFoundException
+  {
+    Class clas = loadClass(name);
+    if (resolve)
+      resolveClass(clas);
+    return clas;
+  }
+
+  /** Load named class.
+   * Note we deliberately don't follow the Java2 delegation model,
+   * in order to allow classes to be overridden and replaced.
+   * Specifically, we depend on this for the "session class-loader".
+   */
+  public Class loadClass (String name)
     throws ClassNotFoundException
   {
     Object r = map.get(name);
-    Class clas;
-    if (r == null)
+    if (r instanceof ClassType)
       {
-        /* #ifdef JAVA2 */
-        throw new ClassNotFoundException(name);
-        /* #else */
-        // clas = Class.forName(name);
-        /* #endif */
+        ClassType ctype = (ClassType) r;
+        if (ctype.isExisting())
+          r = ctype.reflectClass;
+        else
+          r = ctype.writeToArray();
       }
-    else if (r instanceof byte[])
+    Class clas;
+    if (r instanceof byte[])
       {
         // double-locking? is this safe?  or needed? FIXME.
 	synchronized (this)
@@ -112,12 +119,27 @@ public class ArrayClassLoader extends ClassLoader
 	      clas = (Class) r;
 	  }
       }
+    else if (r == null)
+      clas = getParent().loadClass(name);
     else
       clas = (Class) r;
-    /* #ifndef JAVA2 */
-    // if (resolve && clas != null)
-    //   resolveClass (clas);
-    /* #endif */
     return clas;
   }
+
+  /* #ifdef JAVA2 */
+  public static Package getContextPackage (String cname)
+  {
+    ClassLoader loader;
+    try
+      {
+        loader = Thread.currentThread().getContextClassLoader();
+        if (loader instanceof ArrayClassLoader)
+          return ((ArrayClassLoader) loader).getPackage(cname);
+      }
+    catch (java.lang.SecurityException ex)
+      {
+      }
+    return Package.getPackage(cname);
+  }
+  /* #endif */
 }
