@@ -1,15 +1,23 @@
-// Copyright (c) 1997, 1998, 1999, 2001, 2002, 2004, 2005  Per M.A. Bothner.
+// Copyright (c) 1997, 1998, 1999, 2001, 2002, 2004, 2005, 2008, 2009  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
 import java.io.*;
-import java.util.Vector;
+import java.util.*;
 
 public class ClassType extends ObjectType 
-  implements AttrContainer, Externalizable
+  implements AttrContainer, Externalizable, Member
 {
+  public static final int JDK_1_1_VERSION = 45 * 0x10000 + 3;
+  public static final int JDK_1_2_VERSION = 46 * 0x10000 + 0;
+  public static final int JDK_1_3_VERSION = 47 * 0x10000 + 0;
+  public static final int JDK_1_4_VERSION = 48 * 0x10000 + 0;
+  public static final int JDK_1_5_VERSION = 49 * 0x10000 + 0;
+  public static final int JDK_1_6_VERSION = 50 * 0x10000 + 0;
+  public static final int JDK_1_7_VERSION = 51 * 0x10000 + 0;
+
   // An old but generally valid default value.
-  int classfileFormatVersion = 45 * 0x10000 + 3;
+  int classfileFormatVersion = JDK_1_1_VERSION;
 
   public short getClassfileMajorVersion ()
   {
@@ -23,9 +31,17 @@ public class ClassType extends ObjectType
   {
     classfileFormatVersion = (major & 0xFFFF) * 0x10000 + (minor * 0xFFFF);
   }
+  public void setClassfileVersion (int code)
+  {
+    classfileFormatVersion = code;
+  }
+  public int getClassfileVersion ()
+  {
+    return classfileFormatVersion;
+  }
   public void setClassfileVersionJava5 ()
   {
-    setClassfileVersion(49, 0);
+    setClassfileVersion(JDK_1_5_VERSION);
   }
 
   /** Find a ClassType with the given name, or create a new one.
@@ -57,7 +73,7 @@ public class ClassType extends ObjectType
 
   ClassType[] interfaces;
   int[] interfaceIndexes;
-  public int access_flags;
+  int access_flags;
 
   Attribute attributes;
   public final Attribute getAttributes () { return attributes; }
@@ -89,8 +105,154 @@ public class ClassType extends ObjectType
     return access_flags;
   }
 
+  public final boolean getStaticFlag () {
+    return (getModifiers() & Access.STATIC) != 0;
+  }
+
   /** Set the modifiers (access flags) for this class. */
   public final void setModifiers(int flags) { access_flags = flags; }
+  public final void addModifiers(int flags) { access_flags |= flags; }
+
+  public String getSimpleName ()
+  {
+    /* #ifdef JAVA5 */
+    if ((flags & EXISTING_CLASS) != 0 && getReflectClass() != null)
+      {
+        try
+          {
+            return reflectClass.getSimpleName();
+          }
+        catch (Throwable ex)
+          { /* ... fall thorugh ... */ }
+      }
+    /* #endif */
+    String name = getName();
+    int dot = name.lastIndexOf('.');
+    if (dot > 0)
+      name = name.substring(dot+1);
+    int dollar = name.lastIndexOf('$');
+    if (dollar >= 0)
+      {
+        int len = name.length();
+        int start = dollar + 1;
+        while (start < len)
+          {
+            char ch = name.charAt(start);
+            if (ch >= '0' && ch <= '9')
+              start++;
+            else
+              break;
+          }
+        name = name.substring(start);
+      }
+    return name;
+  }
+
+  public void addMemberClass (ClassType member)
+  {
+    ClassType prev = null;
+    ClassType entry = firstInnerClass;
+    while (entry != null)
+      {
+        if (entry == member)
+          return;
+        prev = entry;
+        entry = entry.nextInnerClass;
+      }
+    if (prev == null)
+      firstInnerClass = member;
+    else
+      prev.nextInnerClass = member;
+  }
+
+  public ClassType getDeclaredClass (String simpleName)
+  {
+    addMemberClasses();
+    for (ClassType member = firstInnerClass;  member != null;
+         member = member.nextInnerClass)
+      {
+        if (simpleName.equals(member.getSimpleName()))
+          return member;
+      }
+    return null;
+  }
+
+  ClassType firstInnerClass;
+  ClassType nextInnerClass;
+
+  Member enclosingMember;
+  public ClassType getDeclaringClass()
+  {
+    addEnclosingMember();
+    if (enclosingMember instanceof ClassType)
+      return (ClassType) enclosingMember;
+    else
+      return null;
+  }
+
+  public Member getEnclosingMember ()
+  {
+    addEnclosingMember();
+    return enclosingMember;
+  }
+
+  public void setEnclosingMember (Member member)
+  {
+    enclosingMember = member;
+  }
+
+  void addEnclosingMember ()
+  {
+    if ((flags & (ADD_ENCLOSING_DONE|EXISTING_CLASS)) != EXISTING_CLASS)
+      return;
+    Class clas = getReflectClass();
+    flags |= ADD_ENCLOSING_DONE;
+
+    Class dclas;
+    /* #ifdef JAVA5 */
+    dclas = clas.getEnclosingClass();
+    if (dclas == null)
+      return;
+    if (! clas.isMemberClass())
+      {
+        java.lang.reflect.Method rmeth = clas.getEnclosingMethod();
+        if (rmeth != null)
+          {
+            enclosingMember = addMethod(rmeth);
+            return;
+          }
+        java.lang.reflect.Constructor rcons = clas.getEnclosingConstructor();
+        if (rcons != null)
+          {
+            enclosingMember = addMethod(rcons);
+            return;
+          }
+      }
+    enclosingMember = (ClassType) Type.make(dclas);
+    /* #else */
+    // dclas = clas.getDeclaringClass();
+    // if (dclas != null)
+    //   enclosingMember = (ClassType) Type.make(dclas);
+    /* #endif */
+  }
+
+  public void addMemberClasses ()
+  {
+    if ((flags & (ADD_MEMBERCLASSES_DONE|EXISTING_CLASS)) != EXISTING_CLASS)
+      return;
+    Class clas = getReflectClass();
+    flags |= ADD_MEMBERCLASSES_DONE;
+    Class[] memberClasses = clas.getClasses();
+    int numMembers = memberClasses.length;
+    if (numMembers > 0)
+      {
+        for (int i = 0;  i < numMembers;  i++)
+          {
+            ClassType member = (ClassType) Type.make(memberClasses[i]);
+            addMemberClass(member);
+          }
+      }
+  }
 
   public final boolean hasOuterLink ()
   {
@@ -139,15 +301,31 @@ public class ClassType extends ObjectType
     return field;
   }
 
+  /** Check if a member is accessible from this class.
+   * @param member the member (Field, Method) we're trting to access.
+   * @param receiver the type of the receiver object, if applicable.
+   * @return true if the specified component can be accessed from this class.
+   */
+  public boolean isAccessible (Member member, ObjectType receiver)
+  {
+    if (member.getStaticFlag())
+      receiver = null;
+    return isAccessible(member.getDeclaringClass(), receiver,
+                        member.getModifiers());
+  }
+
   /** Check if a component is accessible from this class.
    * @param declaring the class containing the component (a field, method,
    *   or inner class)
+   * @param receiver the type of the receiver object, if applicable.
    * @param modifiers the access flags of the component
    * @return true if the specified component can be accessed from this class.
    */
-  public boolean isAccessible (ClassType declaring, int modifiers)
+  public boolean isAccessible (ClassType declaring,
+                               ObjectType receiver, int modifiers)
   {
     int cmods = declaring.getModifiers();
+    // Fast, hopefully-common case.
     if ((modifiers & Access.PUBLIC) != 0 && (cmods & Access.PUBLIC) != 0)
       return true;
     String callerName = getName();
@@ -162,8 +340,12 @@ public class ClassType extends ObjectType
     String classPackage = dot >= 0 ? className.substring(0, dot) : "";
     if (callerPackage.equals(classPackage))
       return true;
+    if ((cmods & Access.PUBLIC) == 0)
+      return false;
     if ((modifiers & Access.PROTECTED) != 0
-        && this.isSubclass(declaring))
+        && this.isSubclass(declaring)
+        && (!(receiver instanceof ClassType)
+            || ((ClassType) receiver).isSubclass(this)))
       return true;
     return false;
   }
@@ -492,6 +674,29 @@ public class ClassType extends ObjectType
     return method;
   }
 
+  public Method addMethod (java.lang.reflect.Method method)
+  {
+    int modifiers = method.getModifiers();
+    Class[] paramTypes = method.getParameterTypes();
+    int j = paramTypes.length;
+    Type[] args = new Type[j];
+    while (--j >= 0)
+      args[j] = Type.make(paramTypes[j]);
+    Type rtype = Type.make(method.getReturnType());
+    return addMethod(method.getName(), modifiers, args, rtype);
+  }
+
+  public Method addMethod (java.lang.reflect.Constructor method)
+  {
+    Class[] paramTypes = method.getParameterTypes();
+    int modifiers = method.getModifiers();
+    int j = paramTypes.length;
+    Type[] args = new Type[j];
+    while (--j >= 0)
+      args[j] = Type.make(paramTypes[j]);
+    return addMethod("<init>", modifiers, args, Type.voidType);
+  }
+
   public Method addMethod (String name,  String signature, int flags)
   {
     Method meth = addMethod(name, flags);
@@ -524,12 +729,14 @@ public class ClassType extends ObjectType
    * @param filter to select methods to return
    * @param searchSupers 0 if only current class should be searched,
    *   1 if superclasses should also be searched,
-   *   2 if super-interfaces should also be search
+   *   2 if super-interfaces should also be searched
    * @return number of methods that match
    */
   public final int countMethods (Filter filter, int searchSupers)
   {
-    return getMethods(filter, searchSupers, null, 0);
+    Vector vec = new Vector();
+    getMethods(filter, searchSupers, vec);
+    return vec.size();
   }
 
   public Method[] getMethods (Filter filter, boolean searchSupers)
@@ -546,9 +753,12 @@ public class ClassType extends ObjectType
    */
   public Method[] getMethods (Filter filter, int searchSupers)
   {
-    int count = getMethods(filter, searchSupers, null, 0);
+    Vector<Method> vec = new Vector();
+    getMethods(filter, searchSupers, vec);
+    int count = vec.size();
     Method[] result = new Method[count];
-    getMethods(filter, searchSupers, result, 0);
+    for (int i = 0;  i < count;  i++)
+      result[i] = vec.elementAt(i);
     return result;
   }
 
@@ -565,32 +775,11 @@ public class ClassType extends ObjectType
   public int getMethods (Filter filter, int searchSupers,
 			 Method[] result, int offset)
   {
-    int count = 0;
-    for (ClassType ctype = this;  ctype != null;
-	 ctype = ctype.getSuperclass())
-    {
-      for (Method meth = ctype.getDeclaredMethods();
-	   meth != null;  meth = meth.getNext())
-	if (filter.select(meth))
-	  {
-	    if (result != null)
-	      result[offset + count] = meth;
-	    count++;
-	  }
-      if (searchSupers == 0)
-	break;
-
-      if (searchSupers > 1)
-	{
-	  ClassType[] interfaces = ctype.getInterfaces();
-	  if (interfaces != null)
-	    {
-	      for (int i = 0;  i < interfaces.length;  i++)
-		count += interfaces[i].getMethods(filter, searchSupers,
-						  result, offset+count);
-	    }
-	}
-    }
+    Vector<Method> vec = new Vector<Method>();
+    getMethods(filter, searchSupers, vec);
+    int count = vec.size();
+    for (int i = 0;  i < count;  i++)
+      result[offset+i] = vec.elementAt(i);
     return count;
   }
 
@@ -599,30 +788,51 @@ public class ClassType extends ObjectType
    * @param searchSupers 0 if only current class should be searched,
    *   1 if superclasses should also be searched,
    *   2 if super-interfaces should also be searched
-   * @param result Vector to add selected methods in
-   * @param context If non-null, skip if class not visible in named package.
-   * @return number of methods placed in result array
+   * @param result List to add selected methods in
+   * @return number of methods placed in result list
    */
-  public int getMethods (Filter filter, int searchSupers, Vector result,
-			 String context)
+  public int getMethods (Filter filter, int searchSupers,
+                         /* #ifdef JAVA5 */
+                         List<Method>
+                         /* #else */
+                         // Vector
+                         /* #endif */
+                         result)
   {
     int count = 0;
+    String inheritingPackage = null;
     for (ClassType ctype = this;  ctype != null;
 	 ctype = ctype.getSuperclass())
     {
-      if (context == null
-	  || (ctype.getModifiers() & Access.PUBLIC) != 0
-	  || context.equals(ctype.getPackageName()))
-	{
-	  for (Method meth = ctype.getDeclaredMethods();
-	       meth != null;  meth = meth.getNext())
-	    if (filter.select(meth))
-	      {
-		if (result != null)
-		  result.addElement(meth);
-		count++;
-	      }
-	}
+      String curPackage = ctype.getPackageName();
+      for (Method meth = ctype.getDeclaredMethods();
+           meth != null;  meth = meth.getNext())
+        {
+          if (ctype != this)
+            {
+              int mmods = meth.getModifiers();
+              if ((mmods & Access.PRIVATE) != 0)
+                continue;
+              if ((mmods & (Access.PUBLIC|Access.PROTECTED)) == 0
+                  && ! curPackage.equals(inheritingPackage))
+                continue;
+            }
+          if (filter.select(meth))
+            {
+              if (result != null)
+                {
+                  /* #ifdef JAVA2 */
+                  result.add(meth);
+                  /* #else */
+                  // result.addElement(meth);
+                  /* #endif */
+                }
+              count++;
+            }
+        }
+
+      inheritingPackage = curPackage;
+
       if (searchSupers == 0)
 	break;
 
@@ -633,7 +843,7 @@ public class ClassType extends ObjectType
 	    {
 	      for (int i = 0;  i < interfaces.length;  i++)
 		count += interfaces[i].getMethods(filter, searchSupers,
-						  result, context);
+						  result);
 	    }
 	}
     }
@@ -760,15 +970,7 @@ public class ClassType extends ObjectType
         java.lang.reflect.Method method = methods[i];
         if (! method.getDeclaringClass().equals(clas))
           continue;
-        int modifiers = method.getModifiers();
-        Class[] paramTypes = method.getParameterTypes();
-        int j = paramTypes.length;
-        Type[] args = new Type[j];
-        while (--j >= 0)
-          args[j] = Type.make(paramTypes[j]);
-        Method meth = addMethod(method.getName(), modifiers);
-        meth.arg_types = args;
-        meth.return_type = Type.make(method.getReturnType());
+        addMethod(method);
       }
 
     java.lang.reflect.Constructor[] cmethods;
@@ -786,17 +988,7 @@ public class ClassType extends ObjectType
         java.lang.reflect.Constructor method = cmethods[i];
         if (! method.getDeclaringClass().equals(clas))
           continue;
-        int modifiers = method.getModifiers();
-        if ((modifiers & (Access.PUBLIC|Access.PROTECTED)) == 0)
-          continue;
-        Class[] paramTypes = method.getParameterTypes();
-        int j = paramTypes.length;
-        Type[] args = new Type[j];
-        while (--j >= 0)
-          args[j] = Type.make(paramTypes[j]);
-        Method meth = addMethod("<init>", modifiers);
-        meth.arg_types = args;
-        meth.return_type = Type.void_type;
+        addMethod(method);
       }
   }
 
@@ -849,7 +1041,57 @@ public class ClassType extends ObjectType
     }
     for (Method method = methods; method != null; method = method.next)
       method.assignConstants();
+    if (enclosingMember instanceof Method)
+      {
+        EnclosingMethodAttr attr =
+          EnclosingMethodAttr.getFirstEnclosingMethod(getAttributes());
+        if (attr == null)
+          attr = new EnclosingMethodAttr(this);
+        attr.method = (Method) enclosingMember;
+      }
+    else if (enclosingMember instanceof ClassType)
+      constants.addClass((ClassType) enclosingMember);
+    for (ClassType member = firstInnerClass;  member != null;
+         member = member.nextInnerClass)
+      {
+        constants.addClass(member);
+      }
+
+    InnerClassesAttr innerAttr
+      = InnerClassesAttr.getFirstInnerClasses(getAttributes());
+    if (innerAttr != null)
+      {
+        // Should never happen ...
+        innerAttr.setSkipped(true);
+      }
+
     Attribute.assignConstants(this, this);
+
+    // Add any needed entries to the InnerClasses attribute.  We do this at
+    // the end, because the JVM spec requires the InnerClasses attribute to
+    // have an entry for all non-package-level classes in the constant-pool.
+    // Note that count is not constant in this loop.
+    for (int i = 1; i <= constants.count; i++)
+      {
+	CpoolEntry entry = constants.pool[i];
+        if (! (entry instanceof CpoolClass))
+          continue;
+        CpoolClass centry = (CpoolClass) entry;
+        if (! (centry.clas instanceof ClassType))
+          continue;
+        ClassType ctype = (ClassType) centry.clas;
+        if (ctype.getEnclosingMember() != null)
+          {
+            if (innerAttr == null)
+              innerAttr = new InnerClassesAttr(this);
+            innerAttr.addClass(centry, this);
+          }
+      }
+    if (innerAttr != null)
+      {
+        innerAttr.setSkipped(false);                                       
+        innerAttr.assignConstants(this);
+      }
   }
 
   public void writeToStream (OutputStream stream)
@@ -1004,8 +1246,8 @@ public class ClassType extends ObjectType
   {
     if (other.isInterface())
       return implementsInterface(other);
-    if ((this == tostring_type && other == string_type)
-	|| (this == string_type && other == tostring_type))
+    if ((this == toStringType && other == javalangStringType)
+	|| (this == javalangStringType && other == toStringType))
       return true;
     ClassType baseClass = this;
     while (baseClass != null)
@@ -1031,9 +1273,9 @@ public class ClassType extends ObjectType
       return -1;
     if (cother.isSubclass(this))
       return 1;
-    if (this == tostring_type)
+    if (this == toStringType)
       return cother == Type.pointer_type ? -1 : 1;
-    if (cother == tostring_type)
+    if (cother == toStringType)
       return this == Type.pointer_type ? 1 : -1;
     if (this.isInterface() || cother.isInterface())
       return -2;
@@ -1064,9 +1306,9 @@ public class ClassType extends ObjectType
   {
     String name = getName();
     /* #ifdef JAVA5 */
-    // java.util.HashMap<String,Type> map = mapNameToType;
+    java.util.HashMap<String,Type> map = mapNameToType;
     /* #else */
-    java.util.Hashtable map = mapNameToType;
+    // java.util.Hashtable map = mapNameToType;
     /* #endif */
     synchronized (map)
       {
