@@ -2,7 +2,7 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
-import java.util.Vector;
+import java.util.*;
 
 /**
   * Semi-abstract class object reference types.
@@ -25,16 +25,24 @@ public class ObjectType extends Type
   // Miscellaneous bits:
   final static int ADD_FIELDS_DONE  = 1;
   final static int ADD_METHODS_DONE = 2;
+  final static int ADD_MEMBERCLASSES_DONE = 4;
+  final static int ADD_ENCLOSING_DONE = 8;
   // A ClassType that we can expect to have a corresponding reflectClass.
-  final static int EXISTING_CLASS = 4;
+  final static int EXISTING_CLASS = 16;
 
-  final static int HAS_OUTER_LINK = 8;
+  final static int HAS_OUTER_LINK = 32;
 
   /* */ public int flags;
 
   public final boolean isExisting()
   {
-    return (flags & EXISTING_CLASS) != 0;
+    Type t = getImplementationType();
+    if (t instanceof ArrayType)
+      t = ((ArrayType) t).getComponentType();
+    if (t == this)
+      return (flags & EXISTING_CLASS) != 0;
+    else
+      return ! (t instanceof ObjectType) || ((ObjectType) t).isExisting();
   }
 
   public final void setExisting(boolean existing)
@@ -56,18 +64,18 @@ public class ObjectType extends Type
 
   /* #ifdef JAVA2 */
   /* #ifndef JAVA5 */
-  static ClassLoader thisClassLoader;
-  static
-  {
-    try
-      {
-        thisClassLoader
-          = Class.forName("gnu.mapping.ObjectType").getClassLoader();
-      }
-    catch (Throwable ex)
-      {
-      }
-  }
+  // static ClassLoader thisClassLoader;
+  // static
+  // {
+  //   try
+  //     {
+  //       thisClassLoader
+  //         = Class.forName("gnu.mapping.ObjectType").getClassLoader();
+  //     }
+  //   catch (Throwable ex)
+  //     {
+  //     }
+  // }
   /* #endif */
   /* #endif */
 
@@ -78,28 +86,36 @@ public class ObjectType extends Type
     throws java.lang.ClassNotFoundException
   {
     /* #ifdef JAVA2 */
-    ClassLoader loader;
-    try
-      {
-        loader = Thread.currentThread().getContextClassLoader();
-      }
-    catch (java.lang.SecurityException ex)
-      {
-        /* The .class syntax below also works for JDK 1.4, but it's just
-           syntactic sugar, so no benefit it using it. */
-        /* #ifdef JAVA5 */
-        // loader = ObjectType.class.getClassLoader();
-        /* #else */
-        loader = thisClassLoader;
-        /* #endif */
-      }
     /* Specifies optional 'initialize' argument. */
-    return Class.forName(cname, false, loader);
+    return Class.forName(cname, false, getContextClassLoader());
     /* #else */
     // return Class.forName(cname);
     /* #endif */
   }
 
+  /* #ifdef JAVA2 */
+  public static ClassLoader getContextClassLoader ()
+  {
+    try
+      {
+        /* #ifdef Android */
+        // return ClassLoader.getSystemClassLoader();
+        /* #else */
+        return Thread.currentThread().getContextClassLoader();
+        /* #endif */
+      }
+    catch (java.lang.SecurityException ex)
+      {
+        /* The .class syntax below also works for JDK 1.4, but it's just
+           syntactic sugar, so there is no benefit in using it. */
+        /* #ifdef JAVA5 */
+        return ObjectType.class.getClassLoader();
+        /* #else */
+        // return thisClassLoader;
+        /* #endif */
+      }
+  }
+  /* #endif */
 
   /** Get the java.lang.Class object for the representation type. */
   public Class getReflectClass()
@@ -127,13 +143,13 @@ public class ObjectType extends Type
 
   public Type getImplementationType()
   {
-    return this == nullType ? pointer_type
-      : this == tostring_type ? string_type : this;
+    return this == nullType ? objectType
+      : this == toStringType ? javalangStringType : this;
   }
 
   public Type promote ()
   {
-    return this == nullType ? pointer_type : this;
+    return this == nullType ? objectType : this;
   }
 
   public boolean isInstance (Object obj)
@@ -143,10 +159,32 @@ public class ObjectType extends Type
     return super.isInstance(obj);
   }
 
-  public int getMethods (Filter filter, int searchSupers, Vector result,
-			 String context)
+  public Field getField(String name, int mask)
   {
-    return 0;
+    return null;
+  }
+
+  public Method getMethod(String name, Type[] arg_types)
+  {
+    return Type.objectType.getMethod(name, arg_types);
+  }
+
+  /** @deprecated */
+  public final int getMethods (Filter filter, int searchSupers,
+                               Vector result, String context)
+  {
+    return getMethods(filter, searchSupers, result);
+  }
+
+  public int getMethods (Filter filter, int searchSupers,
+                         /* #ifdef JAVA5 */
+                         List<Method>
+                         /* #else */
+                         // Vector
+                         /* #endif */
+			 result)
+  {
+    return Type.objectType.getMethods(filter, searchSupers, result);
   }
 
   public int compare(Type other)
@@ -156,7 +194,7 @@ public class ObjectType extends Type
   }
 
   /* #ifdef JAVA5 */
-  // @SuppressWarnings("unchecked")
+  @SuppressWarnings("unchecked")
   /* #endif */
   /** Convert an object to a value of this Type.
    * Throw a ClassCastException when this is not possible. */
@@ -164,7 +202,7 @@ public class ObjectType extends Type
   {
     if (obj != null)
       {
-	if (this == Type.tostring_type)
+	if (this == Type.toStringType)
 	  return obj.toString();
         Class clas = getReflectClass();
         Class objClass = obj.getClass();
@@ -179,7 +217,7 @@ public class ObjectType extends Type
   /** Compile (in given method) cast from Object to this Type. */
   public void emitCoerceFromObject (CodeAttr code)
   {
-    if (this == Type.tostring_type)
+    if (this == Type.toStringType)
       {
 	// This would be nice but it doesn't verify, alas!
 	// code.reserve(4);
@@ -195,7 +233,7 @@ public class ObjectType extends Type
 	code.emitInvokeVirtual(Type.toString_method);
 	code.emitFi();
       }
-    else if (this != Type.pointer_type)
+    else if (this != Type.objectType)
       code.emitCheckcast(this);
   }
 }
