@@ -1,10 +1,15 @@
-// Copyright (c) 1997, 2000, 2003, 2004, 2006  Per M.A. Bothner.
+// Copyright (c) 1997, 2000, 2003, 2004, 2006, 2009  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
 import java.util.*;
+import gnu.kawa.util.AbstractWeakHashTable;
 
-public abstract class Type {
+public abstract class Type
+/* #ifdef JAVA5 */
+ implements java.lang.reflect.Type
+/* #endif */
+{
   String signature;
   // Fully-qualified name (in external format, i.e. using '.' to separate).
   String this_name;
@@ -29,31 +34,22 @@ public abstract class Type {
     return this;
   }
 
-  // Maps java.lang.Class to corresponding Type.
-  /* #ifdef JAVA2 */
-  /* #ifdef JAVA5 */
-  // static WeakHashMap<Class,Type> mapClassToType;
-  /* #else */
-  static WeakHashMap mapClassToType;
-  /* #endif */
-  /* #else */
-  // static Hashtable mapClassToType;
-  /* #endif */
+  static ClassToTypeMap mapClassToType;
 
   /** Maps Java type name (e.g. "java.lang.String[]") to corresponding Type. */
   /* #ifdef JAVA5 */
-  // static java.util.HashMap<String,Type> mapNameToType;
+  static java.util.HashMap<String,Type> mapNameToType;
   /* #else */
-  static java.util.Hashtable mapNameToType;
+  // static java.util.Hashtable mapNameToType;
   /* #endif */
 
   public static Type lookupType (String name)
   {
     /* #ifdef JAVA5 */
-    // java.util.HashMap<String,Type> map = mapNameToType;
-    // synchronized (map) { return map.get(name); }
+    java.util.HashMap<String,Type> map = mapNameToType;
+    synchronized (map) { return map.get(name); }
     /* #else */
-    return (Type) mapNameToType.get(name);
+    // return (Type) mapNameToType.get(name);
     /* #endif */
   }
 
@@ -65,9 +61,9 @@ public abstract class Type {
   public static Type getType (String name)
   {
     /* #ifdef JAVA5 */
-    // java.util.HashMap<String,Type> map = mapNameToType;
+    java.util.HashMap<String,Type> map = mapNameToType;
     /* #else */
-    java.util.Hashtable map = mapNameToType;
+    // java.util.Hashtable map = mapNameToType;
     /* #endif */
     synchronized (map)
       {
@@ -91,23 +87,11 @@ public abstract class Type {
   /** Register that the Type for class is type. */
   public synchronized static void registerTypeForClass(Class clas, Type type)
   {
-    /* #ifdef JAVA2 */
-    /* #ifdef JAVA5 */
-    // WeakHashMap<Class,Type> map = mapClassToType;
-    // if (map == null)
-    //   mapClassToType = map = new WeakHashMap<Class,Type>(100);
-    /* #else */
-    WeakHashMap map = mapClassToType;
+    ClassToTypeMap map = mapClassToType;
     if (map == null)
-      mapClassToType = map = new WeakHashMap(100);
-    /* #endif */
-    /* #else */
-    // Hashtable map = mapClassToType;
-    // if (map == null)
-    //   mapClassToType = map = new Hashtable(100);
-    /* #endif */
-    map.put(clas, type);
+      mapClassToType = map = new ClassToTypeMap(100);
     type.reflectClass = clas;
+    map.put(clas, type);
   }
 
   public synchronized static Type make(Class reflectClass)
@@ -116,9 +100,9 @@ public abstract class Type {
     
     if (mapClassToType != null)
       {
-	Object t = mapClassToType.get(reflectClass);
+	Type t = mapClassToType.get(reflectClass);
 	if (t != null)
-	  return (Type) t;
+	  return t;
       }
     if (reflectClass.isArray())
       type = ArrayType.make(Type.make(reflectClass.getComponentType()));
@@ -128,9 +112,9 @@ public abstract class Type {
       {
 	String name = reflectClass.getName();
         /* #ifdef JAVA5 */
-        // java.util.HashMap<String,Type> map = mapNameToType;
+        java.util.HashMap<String,Type> map = mapNameToType;
         /* #else */
-        java.util.Hashtable map = mapNameToType;
+        // java.util.Hashtable map = mapNameToType;
         /* #endif */
         synchronized (map)
           {
@@ -167,7 +151,7 @@ public abstract class Type {
   }
 
   public Type promote () {
-    return size < 4 ? int_type : this;
+    return size < 4 ? intType : this;
   }
 
   public final int getSize() { return size; }
@@ -181,15 +165,15 @@ public abstract class Type {
   {
     switch(sig)
       {
-      case 'B':  return Type.byte_type;
-      case 'C':  return Type.char_type;
-      case 'D':  return Type.double_type;
-      case 'F':  return Type.float_type;
-      case 'S':  return Type.short_type;
-      case 'I':  return Type.int_type;
-      case 'J':  return Type.long_type;
-      case 'Z':  return Type.boolean_type;
-      case 'V':  return Type.void_type;
+      case 'B':  return Type.byteType;
+      case 'C':  return Type.charType;
+      case 'D':  return Type.doubleType;
+      case 'F':  return Type.floatType;
+      case 'S':  return Type.shortType;
+      case 'I':  return Type.intType;
+      case 'J':  return Type.longType;
+      case 'Z':  return Type.booleanType;
+      case 'V':  return Type.voidType;
       }
     return null;
   }
@@ -223,6 +207,30 @@ public abstract class Type {
     return signatureToType(sig, 0, sig.length());
   }
 
+  public static void printSignature (String sig, int off, int len,
+                                     java.io.PrintWriter out)
+  {
+    if (len == 0)
+      return;
+    char c = sig.charAt(off);
+    Type type;
+    if (len == 1)
+      {
+	type = signatureToPrimitive(c);
+	if (type != null)
+	  out.print(type.getName());
+        return;
+      }
+    if (c == '[')
+      {
+        printSignature(sig, off+1, len-1, out);
+        out.print("[]");
+        return;
+      }
+    if (c == 'L' && len > 2 && sig.indexOf(';', off) == len-1+off)
+      out.print(sig.substring(off+1,len-1+off).replace('/', '.'));
+  }
+
   /** Return the length of the signature starting at a given string position.
    * Returns -1 for an invalid signature. */
   public static int signatureLength (String sig, int pos)
@@ -356,7 +364,14 @@ public abstract class Type {
       return t1;
     if (t1 == null || t2 == null)
      return null;
-
+    if (t1 instanceof PrimType && t2 instanceof PrimType)
+      {
+        if (t1 == t2)
+          return t1;
+        t1 = ((PrimType) t1).promotedType();
+        t2 = ((PrimType) t2).promotedType();
+        return t1 == t2 ? t1 : null;
+      }
     if (t1.isSubtype(t2))
       return t2;
     else if (t2.isSubtype(t1))
@@ -365,13 +380,11 @@ public abstract class Type {
       {
        // the only chance left is that t1 and t2 are ClassTypes.
        if (!(t1 instanceof ClassType && t2 instanceof ClassType))
-         return null;
+         return Type.objectType;
        ClassType c1 = (ClassType) t1;
        ClassType c2 = (ClassType) t2;
-       if (c1.isInterface())
-         return Type.pointer_type;
-       if (c2.isInterface())
-         return Type.pointer_type;
+       if (c1.isInterface() || c2.isInterface())
+         return Type.objectType;
 
        return lowestCommonSuperType(c1.getSuperclass(), c2.getSuperclass());
       }
@@ -444,86 +457,113 @@ public abstract class Type {
     throw new Error ("unimplemented emitCoerceFromObject for "+this);
   }
 
-  public static final PrimType byte_type
+  public static final PrimType byteType
     = new PrimType ("byte", "B", 1, java.lang.Byte.TYPE);
-  public static final PrimType short_type
+  public static final PrimType shortType
     = new PrimType ("short", "S", 2, java.lang.Short.TYPE);
-  public static final PrimType int_type
+  public static final PrimType intType
     = new PrimType ("int", "I", 4, java.lang.Integer.TYPE);
-  public static final PrimType long_type
+  public static final PrimType longType
     = new PrimType ("long", "J", 8, java.lang.Long.TYPE);
-  public static final PrimType float_type
+  public static final PrimType floatType
     = new PrimType ("float", "F", 4, java.lang.Float.TYPE);
-  public static final PrimType double_type
+  public static final PrimType doubleType
     = new PrimType ("double", "D", 8, java.lang.Double.TYPE);
-  public static final PrimType boolean_type
+  public static final PrimType booleanType
     = new PrimType ("boolean", "Z", 1, java.lang.Boolean.TYPE);
-  public static final PrimType char_type
+  public static final PrimType charType
     = new PrimType ("char", "C", 2, java.lang.Character.TYPE);
-  public static final PrimType void_type
+  public static final PrimType voidType
     = new PrimType ("void", "V", 0, java.lang.Void.TYPE);
 
+  /* @deprecated */  public static final PrimType byte_type = byteType;
+  /* @deprecated */  public static final PrimType short_type = shortType;
+  /* @deprecated */  public static final PrimType int_type = intType;
+  /* @deprecated */  public static final PrimType long_type = longType;
+  /* @deprecated */  public static final PrimType float_type = floatType;
+  /* @deprecated */  public static final PrimType double_type = doubleType;
+  /* @deprecated */  public static final PrimType boolean_type = booleanType;
+  /* @deprecated */  public static final PrimType char_type = charType;
+  /* @deprecated */  public static final PrimType void_type = voidType;
+
   static
   {
     /* #ifdef JAVA5 */
-    // mapNameToType = new java.util.HashMap<String,Type>();
+    mapNameToType = new java.util.HashMap<String,Type>();
     /* #else */
-    mapNameToType = new java.util.Hashtable();
+    // mapNameToType = new java.util.Hashtable();
     /* #endif */
-	mapNameToType.put("byte",    byte_type);
-	mapNameToType.put("short",   short_type);
-	mapNameToType.put("int",     int_type);
-	mapNameToType.put("long",    long_type);
-	mapNameToType.put("float",   float_type);
-	mapNameToType.put("double",  double_type);
-	mapNameToType.put("boolean", boolean_type);
-	mapNameToType.put("char",    char_type);
-	mapNameToType.put("void",    void_type);
+	mapNameToType.put("byte",    byteType);
+	mapNameToType.put("short",   shortType);
+	mapNameToType.put("int",     intType);
+	mapNameToType.put("long",    longType);
+	mapNameToType.put("float",   floatType);
+	mapNameToType.put("double",  doubleType);
+	mapNameToType.put("boolean", booleanType);
+	mapNameToType.put("char",    charType);
+	mapNameToType.put("void",    voidType);
   }
 
   /** The "return type" of an expression that never returns, such as a throw. */
-  public static final PrimType neverReturnsType = new PrimType (void_type);
+  public static final PrimType neverReturnsType = new PrimType (voidType);
   static { neverReturnsType.this_name = "(never-returns)"; }
 
   /** The magic type of null. */
   public static final ObjectType nullType = new ObjectType("(type of null)");
 
-  static public ClassType string_type = ClassType.make("java.lang.String");
+  public static final ObjectType errorType = new ClassType("(error type)");
+
+  static public ClassType javalangStringType = ClassType.make("java.lang.String");
   /* The String type. but coercion is handled by toString. */
-  public static final ClassType tostring_type
+  public static final ClassType toStringType
     = new ClassType("java.lang.String");
 
-  public static final ClassType pointer_type
+  public static final ClassType javalangObjectType
     = ClassType.make("java.lang.Object");
-  public static final ClassType boolean_ctype
+  public static final ClassType objectType = javalangObjectType;
+  public static final ClassType javalangBooleanType
     = ClassType.make("java.lang.Boolean");
-  public static final ClassType throwable_type
+  public static final ClassType javalangThrowableType
     = ClassType.make("java.lang.Throwable");
   public static final Type[] typeArray0 = new Type[0];
   public static final Method toString_method
-    = pointer_type.getDeclaredMethod("toString", 0);
-  public static final ClassType number_type
+    = objectType.getDeclaredMethod("toString", 0);
+  public static final ClassType javalangNumberType
     = ClassType.make("java.lang.Number");
   public static final Method clone_method
-    = Method.makeCloneMethod(pointer_type);
+    = Method.makeCloneMethod(objectType);
   public static final Method intValue_method
-    = number_type.addMethod("intValue", typeArray0,
-			    int_type, Access.PUBLIC);
+    = javalangNumberType.addMethod("intValue", typeArray0,
+			    intType, Access.PUBLIC);
   public static final Method longValue_method
-    = number_type.addMethod("longValue", typeArray0,
-			    long_type, Access.PUBLIC);
+    = javalangNumberType.addMethod("longValue", typeArray0,
+			    longType, Access.PUBLIC);
   public static final Method floatValue_method
-    = number_type.addMethod("floatValue", typeArray0,
-			    float_type, Access.PUBLIC);
+    = javalangNumberType.addMethod("floatValue", typeArray0,
+			    floatType, Access.PUBLIC);
   public static final Method doubleValue_method
-    = number_type.addMethod("doubleValue", typeArray0,
-			    double_type, Access.PUBLIC);
+    = javalangNumberType.addMethod("doubleValue", typeArray0,
+			    doubleType, Access.PUBLIC);
   public static final Method booleanValue_method
-    = boolean_ctype.addMethod("booleanValue", typeArray0,
-			      boolean_type, Access.PUBLIC);
-  public static final ClassType java_lang_Class_type
+    = javalangBooleanType.addMethod("booleanValue", typeArray0,
+			      booleanType, Access.PUBLIC);
+  public static final ClassType javalangClassType
     = ClassType.make("java.lang.Class");
-
+  /** @deprecated */
+  public static final ClassType pointer_type = javalangObjectType;
+  /** @deprecated */
+  public static final ClassType string_type = javalangStringType;
+  /** @deprecated */
+  public static final ClassType tostring_type = toStringType;
+  /** @deprecated */
+  public static final ClassType java_lang_Class_type = javalangClassType;
+  /** @deprecated */
+  public static final ClassType boolean_ctype = javalangBooleanType;
+  /** @deprecated */
+  public static final ClassType throwable_type = javalangThrowableType;
+  /** @deprecated */
+  public static final ClassType number_type = javalangNumberType;
+  
   protected Class reflectClass;
 
   /** Get the java.lang.Class object for the representation type. */
@@ -548,3 +588,26 @@ public abstract class Type {
     return name == null ? 0 : name.hashCode ();
   }
 }
+
+class ClassToTypeMap extends AbstractWeakHashTable<Class,Type>
+{
+  public ClassToTypeMap ()
+  {
+    super(64);
+  }
+
+  public ClassToTypeMap (int capacity)
+  {
+    super(capacity);
+  }
+
+  protected Class getKeyFromValue (Type type)
+  {
+    return type.reflectClass;
+  }
+
+  protected boolean matches (Type oldValue, Type newValue)
+  {
+    return oldValue.reflectClass == newValue.reflectClass;
+  }
+}
