@@ -104,6 +104,13 @@ public class Variable extends Location implements java.util.Enumeration
       code.setMaxLocals(varIndex + size);
     offset = varIndex;
     flags |= LIVE_FLAG;
+    if (offset == 0)
+      {
+        // Note that the this parameter in <init> is uninitialized,
+        // until we call super(...) or this(...) on it.
+        if ("<init>".equals(code.getMethod().getName()))
+          setType(code.local_types[0]);
+      }
     return true;
   }
 
