@@ -11,7 +11,6 @@ import gnu.bytecode.Type;
 import gnu.kawa.lispexpr.LangPrimType;
 import gnu.kawa.functions.DisplayFormat;
 import gnu.kawa.functions.NumberCompare;
-import gnu.kawa.lispexpr.ReadTable;
 
 public class CommonLisp extends Lisp2
 {
@@ -192,7 +191,7 @@ public class CommonLisp extends Lisp2
 	if (name.equals("boolean"))
 	  {
 	    if (booleanType == null)
-	      booleanType = new LangPrimType(Type.boolean_type, this);
+	      booleanType = new LangPrimType(Type.booleanType, this);
 	    return booleanType;
 	  }
 	return Scheme.getNamedType(name);
