@@ -88,7 +88,7 @@ public class ApplyExp extends Expression
 	    // is that we have to do some extra stack operations.
 	    // However, these are cheap (and get compiled away when
 	    // compiling to native code).
-	    arg.compile (comp, Target.pushObject);
+	    arg.compileWithPosition(comp, Target.pushObject);
 	    code.emitSwap();
 	    code.emitDup(1, 1);
 	    code.emitSwap();
@@ -99,7 +99,7 @@ public class ApplyExp extends Expression
 	  {
 	    code.emitDup(Compilation.objArrayType);
 	    code.emitPushInt(i);
-	    arg.compile (comp, Target.pushObject);
+	    arg.compileWithPosition(comp, Target.pushObject);
 	  }
 	code.emitArrayStore(Type.pointer_type);
       }
@@ -116,7 +116,7 @@ public class ApplyExp extends Expression
   }
 
   static void compile (ApplyExp exp, Compilation comp, Target target,
-                              boolean checkInlineable)
+                       boolean checkInlineable)
   {
     int args_length = exp.args.length;
     Expression exp_func = exp.func;
@@ -185,6 +185,8 @@ public class ApplyExp extends Expression
 	  throw new Error ("internal error - wrong number of parameters for "
 			   + func_lambda);
 	int conv = func_lambda.getCallConvention();
+        // Mostly duplicates logic with LambdaExp.inline.
+        // See comment there.
 	if (comp.inlineOk(func_lambda)
 	    && (conv <= Compilation.CALL_WITH_CONSUMER
 		|| (conv == Compilation.CALL_WITH_TAILCALLS
@@ -209,19 +211,20 @@ public class ApplyExp extends Expression
                   func_lambda.getOwningLambda().loadHeapFrame(comp);
 	      }
 
-	    pproc.compile(extraArg ? Type.void_type : null,
+	    pproc.compile(extraArg ? Type.voidType : null,
 			  exp, comp, target);
 	    return;
 	  }
       }
 
+    /*
     if (comp.usingCPStyle())
       {
 	  {
 	    Label l = new Label(code);
 	    gnu.bytecode.SwitchState fswitch = comp.fswitch;
 	    int pc = fswitch.getMaxValue() + 1;
-	    fswitch.addCase(pc, l, code);
+	    fswitch.addCase(pc, code, l);
             exp_func.compile(comp, new StackTarget(Compilation.typeProcedure));
 	    comp.loadCallContext();
 
@@ -261,15 +264,15 @@ public class ApplyExp extends Expression
 		  }
 	      }
 
-	    /* FIXME
+	    // FIXME
 	    // Load result from stack.value to target.
 	    comp.loadCallContext();
 	    code.emitGetField(comp.valueCallContextField);
 	    target.compileFromStack(comp, Type.pointer_type);
-	    */
 	  }
 	return;
       }
+    */
 
     // Check for tail-recursion.
     boolean tail_recurse
@@ -279,17 +282,24 @@ public class ApplyExp extends Expression
     if (func_lambda != null && func_lambda.getInlineOnly() && !tail_recurse
 	&& func_lambda.min_args == args_length)
       {
-        pushArgs(func_lambda, exp.args, comp);
+        pushArgs(func_lambda, exp.args, null, comp);
+        if (func_lambda.getFlag(LambdaExp.METHODS_COMPILED))
+          {
+            popParams(code, func_lambda, null, false);
+            code.emitTailCall(false, func_lambda.getVarScope());
+            return;
+          }
+        func_lambda.flags |= LambdaExp.METHODS_COMPILED;
 	LambdaExp saveLambda = comp.curLambda;
 	comp.curLambda = func_lambda;
 	func_lambda.allocChildClasses(comp);
 	func_lambda.allocParameters(comp);
-	popParams (code, func_lambda, false);
+	popParams (code, func_lambda, null, false);
 	func_lambda.enterFunction(comp);
 	func_lambda.body.compileWithPosition(comp, target);
 	func_lambda.compileEnd(comp);
-	func_lambda.compileChildMethods(comp);
-	func_lambda.popScope(code);
+	func_lambda.generateApplyMethods(comp);
+        code.popScope();
 	comp.curLambda = saveLambda;
 	return;
       }
@@ -304,7 +314,7 @@ public class ApplyExp extends Expression
 	if (args_length <= 4)
 	  {
 	    for (int i = 0; i < args_length; ++i)
-	      exp.args[i].compile(comp, Target.pushObject);
+	      exp.args[i].compileWithPosition(comp, Target.pushObject);
 	    comp.loadCallContext();
 	    code.emitInvoke(Compilation.typeProcedure
 			    .getDeclaredMethod("check"+args_length,
@@ -341,6 +351,7 @@ public class ApplyExp extends Expression
     boolean toArray
       = (tail_recurse ? func_lambda.min_args != func_lambda.max_args
          : args_length > 4);
+    int[] incValues = null; // Increments if we can use iinc.
     if (toArray)
       {
 	compileToArray(exp.args, comp);
@@ -348,14 +359,15 @@ public class ApplyExp extends Expression
       }
     else if (tail_recurse)
       {
-        pushArgs(func_lambda, exp.args, comp);
+        incValues = new int[exp.args.length];
+        pushArgs(func_lambda, exp.args, incValues, comp);
         method = null;
       }
     else
       {
 	for (int i = 0; i < args_length; ++i)
           {
-            exp.args[i].compile (comp, Target.pushObject);
+            exp.args[i].compileWithPosition(comp, Target.pushObject);
             if (! code.reachableHere())
               break;
           }
@@ -368,7 +380,7 @@ public class ApplyExp extends Expression
       }
     if (tail_recurse)
       {
-	popParams(code, func_lambda, toArray);
+	popParams(code, func_lambda, incValues, toArray);
 	code.emitTailCall(false, func_lambda.getVarScope());
 	return;
       }
@@ -376,11 +388,33 @@ public class ApplyExp extends Expression
     target.compileFromStack(comp, Type.pointer_type);
   }
 
+  public Expression deepCopy (gnu.kawa.util.IdentityHashTable mapper)
+  {
+    Expression f = deepCopy(func, mapper);
+    Expression[] a = deepCopy(args, mapper);
+    if ((f == null && func != null) || (a == null && args != null))
+      return null;
+    ApplyExp copy = new ApplyExp(f, a);
+    copy.flags = getFlags();
+    return copy;
+  }
+
   protected Expression walk (ExpWalker walker)
   {
     return walker.walkApplyExp(this);
   }
 
+  public void walkArgs (ExpWalker walker)
+  {
+    args = walker.walkExps(args, args.length);
+  }
+
+  public void walkArgs (ExpWalker walker, boolean argsInlined)
+  {
+    if (! argsInlined)
+      args = walker.walkExps(args, args.length);
+  }
+
   protected void walkChildren(ExpWalker walker)
   {
     func = walker.walk(func);
@@ -410,7 +444,8 @@ public class ApplyExp extends Expression
   }
 
   /** Only used for inline- and tail-calls. */
-  private static void pushArgs (LambdaExp lexp, Expression[] args, Compilation comp)
+  private static void pushArgs (LambdaExp lexp, Expression[] args,
+                                int[] incValues, Compilation comp)
   {
     Declaration param = lexp.firstDecl();
     int args_length = args.length;
@@ -419,14 +454,18 @@ public class ApplyExp extends Expression
         Expression arg = args[i];
         if (param.ignorable())
           arg.compile(comp, Target.Ignore);
+        else if (incValues != null
+            && (incValues[i] = SetExp.canUseInc(arg, param)) != SetExp.BAD_SHORT)
+          ;
         else
-          arg.compile(comp, param.getType());
+          arg.compileWithPosition(comp,
+                                  StackTarget.getInstance(param.getType()));
         param = param.nextDecl();
       }
   }
 
   private static void popParams (CodeAttr code, LambdaExp lexp,
-                                 boolean toArray)
+                                 int[] incValues, boolean toArray)
   {
     Variable vars = lexp.getVarScope().firstVar();
     Declaration decls = lexp.firstDecl();
@@ -438,24 +477,31 @@ public class ApplyExp extends Expression
       {
 	if (toArray)
 	  {
-	    popParams (code, 1, decls, vars);
+	    popParams (code, 0, 1, null, decls, vars);
 	    return;
 	  }
         vars = vars.nextVar();
       }
-    popParams (code, lexp.min_args, decls, vars);
+    popParams (code, 0, lexp.min_args, incValues, decls, vars);
   }
 
   // Recursive helper function.
-  private static void popParams (CodeAttr code, int count,
+  private static void popParams (CodeAttr code, int paramNo, int count,
+                                 int[] incValues,
                                  Declaration decl, Variable vars)
   {
     if (count > 0)
       {
-	popParams (code, count - 1, decl.nextDecl(),
+        count--;
+	popParams (code, paramNo+1, count, incValues, decl.nextDecl(),
                    decl.getVariable() == null ? vars : vars.nextVar());
         if (! decl.ignorable())
-          code.emitStore(vars);
+          {
+            if (incValues != null && incValues[paramNo] != SetExp.BAD_SHORT)
+              code.emitInc(vars, (short) incValues[paramNo]);
+            else
+              code.emitStore(vars);
+          }
       }
   }
 
@@ -546,6 +592,8 @@ public class ApplyExp extends Expression
 
   public String toString ()
   {
-    return "ApplyExp/"+args.length+'['+func+']';
+    if (this==LambdaExp.unknownContinuation)
+      return "ApplyExp[unknownContinuation]";
+    return "ApplyExp/"+(args == null?0:args.length)+'['+func+']';
   }
 }
