@@ -1,3 +1,6 @@
+// Copyright (c) 2008  Per M.A. Bothner.
+// This is free software;  for terms and warranty disclaimer see ./COPYING.
+
 package gnu.expr;
 import gnu.bytecode.*;
 import gnu.mapping.*;
@@ -9,6 +12,11 @@ public class ClassExp extends LambdaExp
   public boolean isSimple() { return simple; }
   public void setSimple(boolean value) { simple = value; }
 
+  public final boolean isAbstract () { return getFlag(IS_ABSTRACT); }
+  public static final int IS_ABSTRACT = LambdaExp.NEXT_AVAIL_FLAG;
+  public static final int INTERFACE_SPECIFIED = 2 * LambdaExp.NEXT_AVAIL_FLAG;
+  public static final int CLASS_SPECIFIED = 4 * LambdaExp.NEXT_AVAIL_FLAG;
+
   /** True if there is at least one explicit "<init>" ("*init*"} method. */
   boolean explicitInit;
 
@@ -18,14 +26,18 @@ public class ClassExp extends LambdaExp
    * Using an interface plus a class gives us true multiple inheritance. */
   ClassType instanceType;
 
+  public String classNameSpecifier;
+
   /** True if we should make a pair of an interface and a class. */
   public boolean isMakingClassPair()
   {
-    return ! simple;
+    return type != instanceType;
   }
 
   /** List of base classes and implemented interfaces. */
   public Expression[] supers;
+  /** Index in supers array of class we extend, or -1. */
+  public int superClassIndex = -1;
 
   /** An artificial method named {@code "$finit$"} for evaluating
    * non-static initializations.
@@ -49,13 +61,7 @@ public class ClassExp extends LambdaExp
     if (simple)
       instanceType = type = new ClassType();
     else
-      {
-        PairClassType ptype = new PairClassType();
-        type = ptype;
-        instanceType = new ClassType();
-        ptype.setInterface(true);
-        ptype.instanceType = instanceType;
-      }
+      instanceType = type = new PairClassType(); // For now.
     setCanRead(true);
   }
 
@@ -65,7 +71,7 @@ public class ClassExp extends LambdaExp
   {
     if (target instanceof IgnoreTarget)
       return;
-    compile (comp);
+    compileMembers(comp);
     compilePushClass(comp, target);
   }
 
@@ -79,6 +85,8 @@ public class ClassExp extends LambdaExp
     ClassType typeType;
     int nargs;
     boolean needsLink = getNeedsClosureEnv();
+    if (isSimple() && ! needsLink)
+      return;
     if (isMakingClassPair() || needsLink)
       {
         if (new_class == instanceType)
@@ -109,29 +117,91 @@ public class ClassExp extends LambdaExp
     target.compileFromStack(comp, typeType);
   }
 
-  public String getJavaName ()
-  {
-    String name = getName();
-    return name == null ? "object" : Compilation.mangleNameIfNeeded (name);
-  }
-
   protected ClassType getCompiledClassType(Compilation comp)
   {
     return type;
   }
 
-  public void setClassName (Compilation comp)
+  public void setTypes(Compilation comp)
   {
+    int nsupers = supers == null ? 0 : supers.length;
+    ClassType[] superTypes = new ClassType[nsupers];
+    ClassType superType = null;
+    int j = 0;
+    for (int i = 0;  i < nsupers;  i++)
+      {
+	Type st = Language.getDefaultLanguage().getTypeFor(supers[i]);
+	if (! (st instanceof ClassType))
+          {
+            comp.setLine(supers[i]);
+            comp.error('e', "invalid super type");
+            continue;
+          }
+	ClassType t = (ClassType) st;
+	int modifiers;
+	try
+	  {
+	    modifiers = t.getModifiers();
+	  }
+	catch (RuntimeException ex)
+	  {
+	    modifiers = 0;
+	    if (comp != null)
+	      comp.error('e', "unknown super-type "+t.getName());
+	  }
+	if ((modifiers & Access.INTERFACE) == 0)
+	  {
+	    if (j < i)
+              comp.error('e', "duplicate superclass for "+this);
+	    superType = t;
+            superClassIndex = i;
+	  }
+	else
+	  superTypes[j++] = t;
+      }
+    if (superType != null && (flags & INTERFACE_SPECIFIED) != 0)
+      comp.error('e', "cannot be interface since has superclass");
+    if (! simple && superType == null && (flags & CLASS_SPECIFIED) == 0)
+      {
+        PairClassType ptype = (PairClassType) type;
+        instanceType = new ClassType();
+        ptype.setInterface(true);
+        ptype.instanceType = instanceType;
+        ClassType[] interfaces = { type };
+        // Can we better.  FIXME.
+        instanceType.setSuper(Type.pointer_type);
+        instanceType.setInterfaces(interfaces);
+      }
+    else if (getFlag(INTERFACE_SPECIFIED))
+      instanceType.setInterface(true);
+    type.setSuper(superType == null ? Type.pointer_type : superType);
+
+    ClassType[] interfaces;
+    if (j == nsupers)
+      interfaces = superTypes;
+    else
+      {
+	interfaces = new ClassType[j];
+	System.arraycopy(superTypes, 0, interfaces, 0, j);
+      }
+    type.setInterfaces(interfaces);
+
     if (type.getName() == null)
       {
-	String name = getName();
-	if (name != null)
-	  {
-	    int nlen = name.length();
-	    if (nlen > 2
-		&& name.charAt(0) == '<' && name.charAt(nlen-1) == '>')
-	      name = name.substring(1, nlen-1);
-	  }
+	String name;
+        if (classNameSpecifier != null)
+          name = classNameSpecifier;
+        else
+          {
+            name = getName();
+            if (name != null)
+              {
+                int nlen = name.length();
+                if (nlen > 2
+                    && name.charAt(0) == '<' && name.charAt(nlen-1) == '>')
+                  name = name.substring(1, nlen-1);
+              }
+          }
         if (name == null)
           {
 	    StringBuffer nbuf = new StringBuffer(100);
@@ -161,8 +231,9 @@ public class ClassExp extends LambdaExp
 		  break;
 		nbuf.append(Compilation
 			    .mangleNameIfNeeded(name.substring(start, dot)));
-		nbuf.append('.');
 		start = dot + 1;
+                if (start < name.length())
+                  nbuf.append('.');
 	      }
 	    if (start == 0)
 	      {
@@ -174,6 +245,12 @@ public class ClassExp extends LambdaExp
 		else if (comp.classPrefix != null)
 		  nbuf.append(comp.classPrefix);
 	      }
+            else if (start == 1 && start < name.length())
+              {
+                nbuf.setLength(0);
+                nbuf.append(comp.mainClass.getName());
+                nbuf.append('$');
+              }
 	    if (start < name.length())
 	      nbuf.append(Compilation
 			  .mangleNameIfNeeded(name.substring(start)));
@@ -189,70 +266,6 @@ public class ClassExp extends LambdaExp
       }
   }
 
-  public void setTypes(Compilation comp)
-  {
-    int len = supers == null ? 0 : supers.length;
-    ClassType[] superTypes = new ClassType[len];
-    ClassType superType = null;
-    int j = 0;
-    for (int i = 0;  i < len;  i++)
-      {
-        // setTypes may be called at name-resolution time (so we can can
-        // resolve against inherited field and method names).  Therefore do
-        // inlining now.  Needed (for example) for deprecated PREXIX:<> syntax.
-        supers[i] = new InlineCalls(comp).walk(supers[i]);
-
-	Type st = Language.getDefaultLanguage().getTypeFor(supers[i]);
-	if (! (st instanceof ClassType))
-          {
-            comp.setLine(supers[i]);
-            comp.error('e', "invalid super type");
-            continue;
-          }
-	ClassType t = (ClassType) st;
-	int modifiers;
-	try
-	  {
-	    modifiers = t.getModifiers();
-	  }
-	catch (RuntimeException ex)
-	  {
-	    modifiers = 0;
-	    if (comp != null)
-	      comp.error('e', "unknown super-type "+t.getName());
-	  }
-	if ((modifiers & Access.INTERFACE) == 0)
-	  {
-	    if (j < i)
-              comp.error('e', "duplicate superclass for "+this);
-	    superType = t;
-	  }
-	else
-	  superTypes[j++] = t;
-      }
-    if (! isSimple())
-      {
-        if (superType != null)
-          comp.error('e', "non-simple class inherts from non-interface "+superType.getName());
-          
-        ClassType[] interfaces = { type };
-        // Can we better.  FIXME.
-        instanceType.setSuper(Type.pointer_type);
-        instanceType.setInterfaces(interfaces);
-      }
-    type.setSuper(superType == null ? Type.pointer_type : superType);
-
-    ClassType[] interfaces;
-    if (j == len)
-      interfaces = superTypes;
-    else
-      {
-	interfaces = new ClassType[j];
-	System.arraycopy(superTypes, 0, interfaces, 0, j);
-      }
-    type.setInterfaces(interfaces);
-  }
-
   boolean partsDeclared;
 
   public void declareParts(Compilation comp)
@@ -277,7 +290,7 @@ public class ClassExp extends LambdaExp
 			       flags, Type.typeArray0, ftype);
 		Type[] stypes = { ftype };
 		type.addMethod(slotToMethodName("set",decl.getName()),
-			       flags, stypes, Type.void_type);
+			       flags, stypes, Type.voidType);
 	      }
 	    else
 	      {
@@ -293,17 +306,38 @@ public class ClassExp extends LambdaExp
     for (LambdaExp child = firstChild;  child != null;
 	 child = child.nextSibling)
       {
+        if (child.isAbstract())
+          setFlag(IS_ABSTRACT);
         if ("*init*".equals(child.getName()))
-          explicitInit = true;
-	if ((child != initMethod && child != clinitMethod)
+          {
+            explicitInit = true;
+            if (child.isAbstract())
+              comp.error('e', "*init* method cannot be abstract", child);
+          }
+        // Setting child.outer isn't normally needed.  The exception is
+        // if we're called from object.rewriteClassDef and there is some
+        // funny macro expansion going on, in which case outer
+        // might be a TemplateScope.
+        child.outer = this;
+	if ((child != initMethod && child != clinitMethod
+             && ! child.nameDecl.getFlag(Declaration.STATIC_SPECIFIED))
 	    || ! isMakingClassPair())
 	  child.addMethodFor(type, comp, null);
 	if (isMakingClassPair())
-          // FIXME this is wrong if the method is static
 	  child.addMethodFor(instanceType, comp, type);
       }
-    if (! explicitInit)
+    if (! explicitInit && ! instanceType.isInterface())
       Compilation.getConstructor(instanceType, this);
+    if (isAbstract())
+      instanceType.setModifiers(instanceType.getModifiers() | Access.ABSTRACT);
+    if (nameDecl != null)
+      {
+        short access = nameDecl.getAccessFlags((short) 0);
+        if (access != 0)
+          instanceType.setModifiers((instanceType.getModifiers()
+                                     & ~Access.PUBLIC)
+                                    | access);
+      }
   }
 
   /** Return implementation method matching name and param types.
@@ -328,8 +362,24 @@ public class ClassExp extends LambdaExp
       return;
     else
       {
-	String implTypeName = interfaceType.getName() + "$class";
-	implType = ClassType.make(implTypeName);
+        try
+          {
+            Class reflectClass = interfaceType.getReflectClass();
+            if (reflectClass == null)
+              return;
+            String implTypeName = interfaceType.getName() + "$class";
+            ClassLoader loader = reflectClass.getClassLoader();
+            /* #ifdef JAVA2 */
+            Class implClass = Class.forName(implTypeName, false, loader);
+            /* #else */
+            // Class implClass = Class.forName(implTypeName);
+            /* #endif */
+            implType = (ClassType) Type.make(implClass);
+          }
+        catch (Throwable ex)
+          {
+            return;
+          }
       }
     Type[] itypes = new Type[paramTypes.length + 1];
     itypes[0] = interfaceType;
@@ -361,7 +411,7 @@ public class ClassExp extends LambdaExp
       }
   }
 
-  public ClassType compile (Compilation comp)
+  public ClassType compileMembers (Compilation comp)
   {
     ClassType saveClass = comp.curClass;
     Method saveMethod = comp.method;
@@ -370,6 +420,26 @@ public class ClassExp extends LambdaExp
 	ClassType new_class = getCompiledClassType(comp);
 	comp.curClass = new_class;
 
+        LambdaExp outer = outerLambda();
+        Member enclosing = null;
+        if (outer instanceof ClassExp)
+          enclosing = outer.type;
+        else if (outer != null && ! (outer instanceof ModuleExp))
+          enclosing = saveMethod;
+        else if (outer instanceof ModuleExp && type.getName().indexOf('$') > 0)
+         enclosing = outer.type;
+        if (enclosing != null)
+          {
+            new_class.setEnclosingMember(enclosing);
+            if (enclosing instanceof ClassType)
+              ((ClassType) enclosing).addMemberClass(new_class);
+          }
+        if (instanceType != new_class)
+          {
+            instanceType.setEnclosingMember(type);
+            type.addMemberClass(instanceType);
+          }
+              
 	usedSuperClasses(type, comp);
 	if (type != instanceType)
 	  usedSuperClasses(instanceType, comp);
@@ -384,8 +454,11 @@ public class ClassExp extends LambdaExp
 	allocFrame(comp);
 	CodeAttr code;
 
-	for (LambdaExp child = firstChild;  child != null; )
+	for (LambdaExp child = firstChild;  child != null;
+             child = child.nextSibling)
 	  {
+            if (child.isAbstract())
+              continue;
 	    Method save_method = comp.method;
 	    LambdaExp save_lambda = comp.curLambda;
             String saveFilename = comp.getFileName();
@@ -403,11 +476,17 @@ public class ClassExp extends LambdaExp
 	    comp.method.initCode();
             child.allocChildClasses(comp);
 	    child.allocParameters(comp);
-	    child.enterFunction(comp);
             if ("*init*".equals(child.getName()))
               {
                 code = comp.getCode();
 
+                if (staticLinkField != null)
+                  {
+                    code.emitPushThis();
+                    code.emitLoad(code.getCurrentScope().getVariable(1));
+                    code.emitPutField(staticLinkField);
+                  }
+
                 // Extract "first" expression to see if it is special.
                 Expression bodyFirst = child.body;
                 while (bodyFirst instanceof BeginExp)
@@ -442,16 +521,9 @@ public class ClassExp extends LambdaExp
                   {
                     // Call default super constructor if there isn't an explicit
                     // call to a super constructor.
-                    Method superConstructor
-                      = superClass.getDeclaredMethod("<init>", 0);
-                    if (superConstructor == null)
-                      comp.error('e', "super class does not have a default constructor");
-                    else
-                      {
-                        code.emitPushThis();
-                        code.emitInvokeSpecial(superConstructor);
-                      }
+                    invokeDefaultSuperConstructor(superClass, comp, this);
                   }
+                child.enterFunction(comp);
                 if (calledInit != instanceType)
                   comp.callInitMethods(getCompiledClassType(comp),
                                        new Vector(10));
@@ -462,22 +534,35 @@ public class ClassExp extends LambdaExp
                   child.compileBody(comp);
               }
             else
-              child.compileBody(comp);
+              {
+                child.enterFunction(comp);
+                child.compileBody(comp);
+              }
 	    child.compileEnd(comp);
-	    child.compileChildMethods(comp);
+	    child.generateApplyMethods(comp);
 	    comp.method = save_method;
 	    comp.curClass = new_class;
 	    comp.curLambda = save_lambda;
             comp.setLine(saveFilename, saveLine, saveColumn);
-	    child = child.nextSibling;
 	  }
-        if (! explicitInit)
+        if (! explicitInit && ! instanceType.isInterface())
           comp.generateConstructor(instanceType, this);
         else if (initChain != null)
           initChain.reportError("unimplemented: explicit constructor cannot initialize ", comp);
 
-	Method[] methods = type.getMethods(AbstractMethodFilter.instance, 2);
-	for (int i = 0;  i < methods.length;  i++)
+	Method[] methods;
+        int nmethods;
+        if (isAbstract())
+          {
+            methods = null;
+            nmethods = 0;
+          }
+        else
+          {
+            methods = type.getMethods(AbstractMethodFilter.instance, 2);
+            nmethods = methods.length;
+          }
+	for (int i = 0;  i < nmethods;  i++)
 	  {
 	    Method meth = methods[i];
 	    String mname = meth.getName();
@@ -581,6 +666,7 @@ public class ClassExp extends LambdaExp
   {
     LambdaExp save = walker.currentLambda;
     walker.currentLambda = this;
+    supers = walker.walkExps(supers, supers.length);
     try
       {
 	for (LambdaExp child = firstChild;
@@ -602,6 +688,38 @@ public class ClassExp extends LambdaExp
       }
   }
 
+  static void loadSuperStaticLink (Expression superExp, ClassType superClass,
+                                   Compilation comp)
+  {
+    CodeAttr code = comp.getCode();
+    // This can be optimized in most cases. FIXME.
+    superExp.compile(comp, Target.pushValue(Compilation.typeClassType));
+    code.emitInvokeStatic(ClassType.make("gnu.expr.PairClassType").getDeclaredMethod("extractStaticLink", 1));
+    code.emitCheckcast(superClass.getOuterLinkType());
+  }
+
+  static void invokeDefaultSuperConstructor (ClassType superClass,
+                                             Compilation comp,
+                                             LambdaExp lexp)
+  {
+    CodeAttr code = comp.getCode();
+    Method superConstructor
+      = superClass.getDeclaredMethod("<init>", 0);
+    if (superConstructor == null)
+      comp.error('e', "super class does not have a default constructor");
+    else
+      {
+        code.emitPushThis();
+        if (superClass.hasOuterLink() && lexp instanceof ClassExp)
+          {
+            ClassExp clExp = (ClassExp) lexp;
+            Expression superExp = clExp.supers[clExp.superClassIndex];
+            loadSuperStaticLink(superExp, superClass, comp);
+          }
+        code.emitInvokeSpecial(superConstructor);
+      }
+  }
+
   public void print (OutPort out)
   {
     out.startLogicalBlock("("+getExpClassName()+"/", ")", 2);
@@ -613,7 +731,17 @@ public class ClassExp extends LambdaExp
       }
     out.print(id);
     out.print("/fl:");  out.print(Integer.toHexString(flags));
-    out.print(" (");
+    if (supers.length > 0)
+      {
+
+        out.print(" supers:");
+        for (int i = 0;  i < supers.length;  i++)
+          {
+            supers[i].print(out);
+            out.print(' ');
+          }
+      }
+    out.print('(');
     Special prevMode = null;
     int i = 0;
     int key_args = keywords == null ? 0 : keywords.length;
