@@ -133,6 +133,18 @@ public class Compilation implements SourceLocator
     return currentOptions.getBoolean(key);
   }
 
+  public static int defaultClassFileVersion =
+    /* #ifdef JAVA6 */
+    // ClassType.JDK_1_6_VERSION
+    /* #else */
+    /* #ifdef JAVA5 */
+    ClassType.JDK_1_5_VERSION
+    /* #else */
+    // ClassType.JDK_1_1_VERSION
+    /* #endif */
+    /* #endif */
+    ;
+
   /** The default calling convention.
    * One of the following CALL_WITHG_xxx values. */
   public static int defaultCallConvention;
@@ -182,23 +194,20 @@ public class Compilation implements SourceLocator
   Field fswitchIndex;
 
   // Various standard classes
-  static public ClassType typeObject = Type.pointer_type;
+  static public ClassType typeObject = Type.objectType;
   static public ClassType scmBooleanType = ClassType.make("java.lang.Boolean");
   static public ClassType typeString = ClassType.make("java.lang.String");
   static public ClassType javaStringType = typeString;
   static public ClassType scmKeywordType = ClassType.make("gnu.expr.Keyword");
   static public ClassType scmSequenceType = ClassType.make("gnu.lists.Sequence");
-  static public ClassType javaIntegerType = ClassType.make("java.lang.Integer");
   static public ClassType scmListType = ClassType.make("gnu.lists.LList");
   static public ClassType typePair = ClassType.make("gnu.lists.Pair");
-  static public ClassType scmPairType = typePair;
   public static final ArrayType objArrayType = ArrayType.make(typeObject);
-  static public ClassType scmNamedType = ClassType.make("gnu.mapping.Named");
   static public ClassType typeRunnable = ClassType.make("java.lang.Runnable");
   public static ClassType typeType = ClassType.make("gnu.bytecode.Type");
   public static ClassType typeObjectType
     = ClassType.make("gnu.bytecode.ObjectType", typeType);
-  public static ClassType typeClass = Type.java_lang_Class_type;
+  public static ClassType typeClass = Type.javalangClassType;
   static public ClassType typeClassType = ClassType.make("gnu.bytecode.ClassType", typeObjectType);
   static public ClassType typeProcedure
     = ClassType.make("gnu.mapping.Procedure");
@@ -216,7 +225,7 @@ public class Compilation implements SourceLocator
     = typeLanguage.getDeclaredMethod("getSymbolProcedure", 1);
   static public final Method getLocationMethod
     = typeLocation.addMethod("get", Type.typeArray0,
-			    Type.pointer_type, Access.PUBLIC);
+			    Type.objectType, Access.PUBLIC);
   static public final Method getProcedureBindingMethod
     = typeSymbol.addMethod("getProcedure", Type.typeArray0,
 			    typeProcedure, Access.PUBLIC);
@@ -229,7 +238,7 @@ public class Compilation implements SourceLocator
     = typeProcedure.getDeclaredMethod("setName", 1);
   static Method makeListMethod;
   
-  public static final Type[] int1Args = { Type.int_type };
+  public static final Type[] int1Args = { Type.intType  };
   public static final Type[] string1Arg = { javaStringType };
   public static final Type[] sym1Arg = string1Arg;
 
@@ -237,14 +246,14 @@ public class Compilation implements SourceLocator
   = typeEnvironment.getDeclaredMethod("getLocation", 1);
   static public final Method getLocation2EnvironmentMethod;
   static {
-    Type[] args = { typeSymbol, Type.pointer_type };
+    Type[] args = { typeSymbol, Type.objectType    };
     getLocation2EnvironmentMethod
       = typeEnvironment.addMethod("getLocation", args,
 				  typeLocation, Access.PUBLIC|Access.FINAL);
   }
 
   static {
-    Type[] makeListArgs = { objArrayType, Type.int_type };
+    Type[] makeListArgs = { objArrayType, Type.intType  };
     makeListMethod = scmListType.addMethod ("makeList",
 					     makeListArgs, scmListType,
 					     Access.PUBLIC|Access.STATIC);
@@ -286,9 +295,9 @@ public class Compilation implements SourceLocator
 						typeObject, Access.PUBLIC);
     Type[] args = new Type[2];
     args[0] = typeProcedure;
-    args[1] = Type.int_type;
+    args[1] = Type.intType;
     checkArgCountMethod
-      = typeProcedure.addMethod("checkArgCount", args, Type.void_type,
+      = typeProcedure.addMethod("checkArgCount", args, Type.voidType,
 				   Access.PUBLIC|Access.STATIC);
   }
 
@@ -339,7 +348,7 @@ public class Compilation implements SourceLocator
     = typeCallContext.getDeclaredField("proc");
   private static Type[] applyCpsArgs = { typeCallContext};
   public static Method applyCpsMethod
-    = typeProcedure.addMethod("apply", applyCpsArgs, Type.void_type,
+    = typeProcedure.addMethod("apply", applyCpsArgs, Type.voidType,
 				 Access.PUBLIC);
 
   public static ClassType[] typeProcedureArray = {
@@ -428,24 +437,6 @@ public class Compilation implements SourceLocator
               } 
             return;
           }
-        else if (target instanceof SeriesTarget)
-          {
-            SeriesTarget starget = (SeriesTarget) target;
-            Label saveDone = starget.done;
-            if (len > 0)
-              {
-                starget.done = null;
-                for (int i = 0;  i < len;  i++)
-                  {
-                    if (i+1 == len)
-                      starget.done = saveDone;
-                    compileConstant(values[i], target);
-                  }
-              }
-            else if (saveDone != null && getCode().reachableHere())
-              getCode().emitGoto(saveDone);
-            return;
-          }
       }
     if (target instanceof ConditionalTarget)
       {
@@ -507,6 +498,11 @@ public class Compilation implements SourceLocator
 		// should print an ERROR.
 	      }
 	  }
+        if (type == Compilation.typeClass && value instanceof ClassType)
+          {
+            loadClassRef((ClassType) value);
+            return;
+          }
         try
           {
             value = type.coerceFromObject(value);
@@ -519,7 +515,10 @@ public class Compilation implements SourceLocator
 	    else
 	      {
 		sbuf.append("cannot convert literal (of type ");
-                sbuf.append(value.getClass().getName());
+                if (value == null)
+                  sbuf.append("<null>");
+                else
+                  sbuf.append(value.getClass().getName());
 		sbuf.append(") to ");
 	      }
 	    sbuf.append(type.getName());
@@ -645,7 +644,7 @@ public class Compilation implements SourceLocator
 
   public static String mangleNameIfNeeded (String name)
   {
-    if (isValidJavaName(name))
+    if (name == null || isValidJavaName(name))
       return name;
     else
       return mangleName(name, 0);
@@ -907,23 +906,6 @@ public class Compilation implements SourceLocator
       }
   }
 
-  public Compilation (boolean immediate, SourceMessages messages)
-  {
-    this(messages);
-    this.immediate = immediate;
-  }
-
-  public Compilation (SourceMessages messages)
-  {
-    this.messages = messages;
-    lexical = new NameLookup(getLanguage());
-  }
-
-  public Compilation (Language language, SourceMessages messages)
-  {
-    this(language, messages, new NameLookup(language));
-  }
-
   public Compilation (Language language, SourceMessages messages,
 		      NameLookup lexical)
   {
@@ -963,14 +945,22 @@ public class Compilation implements SourceLocator
 	if (parent != null)
 	  new File(parent).mkdirs();
 	clas.writeToFile(out_name);
-
-        clas.cleanupAfterCompilation();
       }
+    cleanupAfterCompilation();
+    litTable = null;
+  }
 
+  public void cleanupAfterCompilation ()
+  {
+    for (int iClass = 0;  iClass < numClasses;  iClass++)
+      classes[iClass].cleanupAfterCompilation();
+    classes = null;
     minfo.comp = null;
+    // We don't clear minfo.exp itself, since it might be re-required.
+    if (minfo.exp != null)
+      minfo.exp.body = null;
     mainLambda.body = null;
     mainLambda = null;
-    litTable = null;
   }
 
   public void compileToArchive (ModuleExp mexp, String fname)
@@ -998,10 +988,7 @@ public class Compilation implements SourceLocator
       zout = new java.util.jar.JarOutputStream(new FileOutputStream(zar_file));
     else
     /* #endif */
-      {
-	zout = new ZipOutputStream (new FileOutputStream (zar_file));
-	zout.setMethod(ZipOutputStream.STORED); // no compression
-      }
+      zout = new ZipOutputStream(new FileOutputStream(zar_file));
 
     byte[][] classBytes = new byte[numClasses][];
     CRC32 zcrc = new CRC32();
@@ -1036,8 +1023,8 @@ public class Compilation implements SourceLocator
 	System.arraycopy (classes, 0, new_classes, 0, numClasses);
 	classes = new_classes;
       }
-    new_class.access_flags |= new_class.isInterface() ? Access.PUBLIC
-      : Access.PUBLIC|Access.SUPER;
+    new_class.addModifiers(new_class.isInterface() ? Access.PUBLIC
+                           : Access.PUBLIC|Access.SUPER);
     if (new_class == mainClass && numClasses > 0)
       {
         // Ensure mainClass is written first when writing an archive.
@@ -1056,9 +1043,13 @@ public class Compilation implements SourceLocator
 	new_class.setSourceFile(mainLambda.filename);
       }
     registerClass(new_class);
-    /* #ifdef JAVA5 */
-    // new_class.setClassfileVersionJava5();
-    /* #endif */
+    new_class.setClassfileVersion(defaultClassFileVersion);
+  }
+
+  public boolean makeRunnable ()
+  {
+    return ! generateServlet && ! generateApplet
+      && ! getModule().staticInitRun();
   }
 
   public void addMainClass (ModuleExp module)
@@ -1081,7 +1072,7 @@ public class Compilation implements SourceLocator
 	else
 	  sup = getModuleType();
       }
-    if (! generateServlet)
+    if (makeRunnable())
       type.addInterface(typeRunnable);
     type.setSuper(sup);
 
@@ -1108,7 +1099,7 @@ public class Compilation implements SourceLocator
       }
     else
       args = apply0args;
-    return clas.addMethod("<init>", Access.PUBLIC, args, Type.void_type);
+    return clas.addMethod("<init>", Access.PUBLIC, args, Type.voidType);
   }
 
   public final void generateConstructor (LambdaExp lexp)
@@ -1134,14 +1125,11 @@ public class Compilation implements SourceLocator
 	code.emitLoad(code.getCurrentScope().getVariable(1));
 	code.emitPutField(lexp.staticLinkField);
       }
-    Method superConstructor
-      = clas.getSuperclass().addMethod("<init>", Access.PUBLIC,
-				       apply0args, Type.void_type);
-    code.emitPushThis();
-    code.emitInvokeSpecial(superConstructor);
+    ClassType superClass = clas.getSuperclass();
+    ClassExp.invokeDefaultSuperConstructor(superClass, this, lexp);
 
     if (curClass == mainClass
-        // Optimization: No pointing in calling register if we aren't
+        // Optimization: No point in calling ModuleInfo.register if we aren't
         // compiling a named module.
         && minfo != null && minfo.sourcePath != null)
       {
@@ -1150,7 +1138,7 @@ public class Compilation implements SourceLocator
                               .getDeclaredMethod("register", 1));
       }
 
-    if (lexp.initChain != null)
+    if (lexp != null && lexp.initChain != null)
       {
 	// Create dummy lambda, for its closureEnv.  This may be needed
 	// if init.value contains a reference that uses our heap frame.
@@ -1210,19 +1198,22 @@ public class Compilation implements SourceLocator
       }
 
     int clEnvArgs = 1;
-    if (clas instanceof PairClassType)
-      clas = ((PairClassType) clas).instanceType;
-    else if (clas.isInterface())
-      {
-	try
-	  {
-	    clas = ((ClassType)
-		    Type.make(Class.forName(clas.getName() + "$class")));
-	  }
-	catch (Throwable ex)
-	  {
-	    return;
-	  }
+    if (clas.isInterface())
+      {	
+        if (clas instanceof PairClassType)
+          clas = ((PairClassType) clas).instanceType;
+        else
+          {
+            try
+              {
+                clas = ((ClassType)
+                        Type.make(Class.forName(clas.getName() + "$class")));
+              }
+            catch (Throwable ex)
+              {
+                return;
+              }
+          }
       }
     else
       clEnvArgs = 0;
@@ -1299,13 +1290,13 @@ public class Compilation implements SourceLocator
 		    matchArgs[2] = typeCallContext;
 		  }
 		matchArgs[0] = procType;
-		method = curClass.addMethod (mname, matchArgs, Type.int_type,
+		method = curClass.addMethod (mname, matchArgs, Type.intType,
 					     Access.PUBLIC);
 		code = method.startCode();
 
 		code.emitLoad(code.getArg(1)); // method
 		code.emitGetField(procType.getField("selector"));
-		aswitch = new SwitchState(code);
+		aswitch = code.startSwitch();
 
 		needThisMatch = true;
 	      }
@@ -1326,7 +1317,7 @@ public class Compilation implements SourceLocator
 		    code.emitLoad(ctxVar);
 		    code.emitLoad(code.getArg(k+1));
 		    Type ptype = var.getType();
-		    if (ptype != Type.pointer_type)
+		    if (ptype != Type.objectType)
 		      {
 			if (ptype instanceof TypeValue)
 			  {
@@ -1344,8 +1335,8 @@ public class Compilation implements SourceLocator
 			    trueLabel.define(code);
 			  }
 			else if (ptype instanceof ClassType
-			    && ptype != Type.pointer_type
-			    && ptype != Type.tostring_type)  // FIXME
+			    && ptype != Type.objectType
+			    && ptype != Type.toStringType)  // FIXME
 			  {
 			    code.emitDup();
 			    ptype.emitIsInstance(code);
@@ -1419,14 +1410,14 @@ public class Compilation implements SourceLocator
 
     // First LambdaExp we seen suitable for this i.
     method = curClass.addMethod ("apply", applyArgs,
-				 (Type) Type.void_type,
+				 (Type) Type.voidType,
 				 Access.PUBLIC);
     code = method.startCode();
     Variable ctxVar = code.getArg(1);
 
     code.emitLoad(ctxVar);
     code.emitGetField(pcCallContextField);
-    SwitchState aswitch = new SwitchState(code);
+    SwitchState aswitch = code.startSwitch();
 
     for (int j = 0;  j < numApplyMethods;  ++j)
       {
@@ -1446,6 +1437,7 @@ public class Compilation implements SourceLocator
 
 	    aswitch.addCase(source.getSelectorValue(this) + i, code);
 
+            SourceLocator saveLoc1 = messages.swapSourceLocator(source);
 	    int line = source.getLineNumber();
 	    if (line > 0)
 	      code.putLineNumber(source.getFileName(), line);
@@ -1458,13 +1450,13 @@ public class Compilation implements SourceLocator
 
 	    if (i > 4 && numMethods > 1) // FIXME
 	      {
-		counter = code.addLocal(Type.int_type);
+		counter = code.addLocal(Type.intType);
 		code.emitLoad(ctxVar);
 		code.emitGetField(typeCallContext.getDeclaredField("count"));
 		if (source.min_args != 0)
 		  {
 		    code.emitPushInt(source.min_args);
-		    code.emitSub(Type.int_type);
+		    code.emitSub(Type.intType);
 		  }
 		code.emitStore(counter);
 	      }
@@ -1480,6 +1472,8 @@ public class Compilation implements SourceLocator
 	      }
 
 	    Declaration var = source.firstDecl();
+            if (var != null && var.isThisParameter())
+              var = var.nextDecl();
 	    for (int k = 0; k < singleArgs;  k++)
 	      {
 		if (counter != null && k >= source.min_args)
@@ -1502,12 +1496,16 @@ public class Compilation implements SourceLocator
 		    code.emitGetField(typeCallContext
 				      .getDeclaredField("values"));
 		    code.emitPushInt(k);
-		    code.emitArrayLoad(Type.pointer_type);
+		    code.emitArrayLoad(Type.objectType);
 		  }
 		Type ptype = var.getType();
-		if (ptype != Type.pointer_type)
-		  CheckedTarget.emitCheckedCoerce(this, source,
-						  k+1, ptype);
+		if (ptype != Type.objectType)
+                  {
+                    SourceLocator saveLoc2 = messages.swapSourceLocator(var);
+                    CheckedTarget.emitCheckedCoerce(this, source,
+                                                    k+1, ptype);
+                    messages.swapSourceLocator(saveLoc2);
+                  }
 		var = var.nextDecl();
 	      }
 
@@ -1545,6 +1543,7 @@ public class Compilation implements SourceLocator
 	    if (defaultCallConvention < Compilation.CALL_WITH_CONSUMER)
 	      Target.pushObject.compileFromStack(this,
 						 source.getReturnType());
+            messages.swapSourceLocator(saveLoc1);
 	    code.emitReturn();
           }
       }
@@ -1585,7 +1584,7 @@ public class Compilation implements SourceLocator
 	String mname = null;
 	Type[] applyArgs = null;
 
-	for (int j = numApplyMethods;  --j >= 0; )
+	for (int j = 0;  j < numApplyMethods;  j++)
 	  {
 	    LambdaExp source = (LambdaExp) lexp.applyMethods.elementAt(j);
 	    // Select the subset of source.primMethods[*] that are suitable
@@ -1632,19 +1631,20 @@ public class Compilation implements SourceLocator
 		  }
 		applyArgs[0] = procType;
 		method = curClass.addMethod (mname, applyArgs,
-					     defaultCallConvention >= Compilation.CALL_WITH_CONSUMER ? (Type) Type.void_type : (Type) Type.pointer_type,
+					     defaultCallConvention >= Compilation.CALL_WITH_CONSUMER ? (Type) Type.voidType : (Type) Type.objectType,
 					     Access.PUBLIC);
 		code = method.startCode();
 
 		code.emitLoad(code.getArg(1)); // method
 		code.emitGetField(procType.getField("selector"));
-		aswitch = new SwitchState(code);
+		aswitch = code.startSwitch();
 
 		needThisApply = true;
 	      }
 
 	    aswitch.addCase(source.getSelectorValue(this), code);
 
+            SourceLocator saveLoc1 = messages.swapSourceLocator(source);
 	    int line = source.getLineNumber();
 	    if (line > 0)
 	      code.putLineNumber(source.getFileName(), line);
@@ -1657,13 +1657,13 @@ public class Compilation implements SourceLocator
 
 	    if (i > 4 && numMethods > 1)
 	      {
-		counter = code.addLocal(Type.int_type);
+		counter = code.addLocal(Type.intType);
 		code.emitLoad(code.getArg(2));
 		code.emitArrayLength();
 		if (source.min_args != 0)
 		  {
 		    code.emitPushInt(source.min_args);
-		    code.emitSub(Type.int_type);
+		    code.emitSub(Type.intType);
 		  }
 		code.emitStore(counter);
 	      }
@@ -1679,6 +1679,8 @@ public class Compilation implements SourceLocator
 	      }
 
 	    Declaration var = source.firstDecl();
+            if (var != null && var.isThisParameter())
+              var = var.nextDecl();
 	    for (int k = 0; k < singleArgs;  k++)
 	      {
 		if (counter != null && k >= source.min_args)
@@ -1702,12 +1704,16 @@ public class Compilation implements SourceLocator
 		    // Load Object[]args value:
 		    code.emitLoad(code.getArg(2));
 		    code.emitPushInt(k);
-		    code.emitArrayLoad(Type.pointer_type);
+		    code.emitArrayLoad(Type.objectType);
 		  }
 		Type ptype = var.getType();
-		if (ptype != Type.pointer_type)
-		  CheckedTarget.emitCheckedCoerce(this, source,
-						  k+1, ptype, pvar);
+		if (ptype != Type.objectType)
+                  {
+                    SourceLocator saveLoc2 = messages.swapSourceLocator(var);
+                    CheckedTarget.emitCheckedCoerce(this, source,
+                                                    k+1, ptype, pvar);
+                    messages.swapSourceLocator(saveLoc2);
+                  }
 		var = var.nextDecl();
 	      }
 
@@ -1727,21 +1733,22 @@ public class Compilation implements SourceLocator
 			code.pushScope();
 			if (counter == null)
 			  {
-			    counter = code.addLocal(Type.int_type);
+			    counter = code.addLocal(Type.intType);
 			    code.emitLoad(code.getArg(2));
 			    code.emitArrayLength();
 			    if (singleArgs != 0)
 			      {
 				code.emitPushInt(singleArgs);
-				code.emitSub(Type.int_type);
+				code.emitSub(Type.intType);
 			      }
 			    code.emitStore(counter);
 			  }
 			code.emitLoad(counter);
 			code.emitNewArray(elType);
 			Label testLabel = new Label(code);
-			code.emitGoto(testLabel);
 			Label loopTopLabel = new Label(code);
+                        loopTopLabel.setTypes(code);
+			code.emitGoto(testLabel);
 			loopTopLabel.define(code);
 
 			code.emitDup(1); // new array
@@ -1751,9 +1758,9 @@ public class Compilation implements SourceLocator
 			if (singleArgs != 0)
 			  {
 			    code.emitPushInt(singleArgs);
-			    code.emitAdd(Type.int_type);
+			    code.emitAdd(Type.intType);
 			  }
-			code.emitArrayLoad(Type.pointer_type);
+			code.emitArrayLoad(Type.objectType);
 			if (mustConvert)
 			  {
 			    CheckedTarget.emitCheckedCoerce
@@ -1786,6 +1793,7 @@ public class Compilation implements SourceLocator
 	    if (defaultCallConvention < Compilation.CALL_WITH_CONSUMER)
 	      Target.pushObject.compileFromStack(this,
 						 source.getReturnType());
+            messages.swapSourceLocator(saveLoc1);
 	    code.emitReturn();
           }
 	if (needThisApply)
@@ -1815,7 +1823,7 @@ public class Compilation implements SourceLocator
 
   private Method startClassInit ()
   {
-    method = curClass.addMethod ("<clinit>", apply0args, Type.void_type,
+    method = curClass.addMethod ("<clinit>", apply0args, Type.voidType,
 				 Access.PUBLIC|Access.STATIC);
 
     CodeAttr code = method.startCode();
@@ -1876,6 +1884,11 @@ public class Compilation implements SourceLocator
           }
         if (wantedState >= COMPILED && getState() < COMPILED)
           {
+            if (immediate)
+              {
+                ClassLoader parentLoader = ObjectType.getContextClassLoader();
+                loader = new ArrayClassLoader(parentLoader);
+              }
             generateBytecode();
             setState(messages.seenErrors() ? ERROR_SEEN : COMPILED);
           }
@@ -1929,7 +1942,7 @@ public class Compilation implements SourceLocator
 	moduleClass = new ClassType(generateClassName("frame"));
 	moduleClass.setSuper(neededSuper);
 	addClass(moduleClass);
-	generateConstructor(moduleClass, module);
+	generateConstructor(moduleClass, null);
       }
 
     curClass = module.type;
@@ -1963,7 +1976,7 @@ public class Compilation implements SourceLocator
     boolean staticModule = module.isStatic();
     Method apply_method;
     
-    apply_method = curClass.addMethod ("run", arg_types, Type.void_type,
+    apply_method = curClass.addMethod ("run", arg_types, Type.voidType,
 				       Access.PUBLIC+Access.FINAL);
     method = apply_method;
     // For each parameter, assign it to its proper slot.
@@ -1996,20 +2009,18 @@ public class Compilation implements SourceLocator
       {
 	loadCallContext();
         code.emitGetField(pcCallContextField);
-        fswitch = new SwitchState(code);
-	Label l = new Label(code);
-	l.define(code);
-	fswitch.addCase(0, l, code);
+        fswitch = code.startSwitch();
+	fswitch.addCase(0, code);
       }
 
     module.compileBody(this);
-    module.compileChildMethods(this);
+    module.compileEnd(this);
 
     Label startLiterals = null;
     Label afterLiterals = null;
     Method initMethod = null;
 
-    if (curClass == mainClass)
+    if (curClass == mainClass) // redundant - never false.
       {
 	Method save_method = method;
         Variable callContextSave = callContextVar;
@@ -2022,7 +2033,7 @@ public class Compilation implements SourceLocator
         startLiterals = new Label(code);
         afterLiterals = new Label(code);
         code.fixupChain(afterLiterals, startLiterals);
-	  
+
 	if (staticModule)
 	  {
 	    generateConstructor (module);
@@ -2031,7 +2042,7 @@ public class Compilation implements SourceLocator
 	    code.emitDup(moduleClass);
 	    code.emitInvokeSpecial(moduleClass.constructor);
 	    moduleInstanceMainField
-	      = moduleClass.addField("$instance", mainClass,
+	      = moduleClass.addField("$instance", moduleClass,
 				     Access.STATIC|Access.PUBLIC|Access.FINAL);
 	    code.emitPutStatic(moduleInstanceMainField);
 	  }
@@ -2045,7 +2056,7 @@ public class Compilation implements SourceLocator
 	if (! immediate && module.staticInitRun())
 	  {
 	    code.emitGetStatic(moduleInstanceMainField);
-	    code.emitInvokeInterface(typeRunnable.getDeclaredMethod("run", 0));
+	    code.emitInvoke(typeModuleBody.getDeclaredMethod("run", 0));
 	  }
 	code.emitReturn();
 
@@ -2054,11 +2065,11 @@ public class Compilation implements SourceLocator
 	  {
 	    // Compare the run methods in ModuleBody.
 	    method = curClass.addMethod("run", Access.PUBLIC,
-					Type.typeArray0, Type.void_type);
+					Type.typeArray0, Type.voidType);
 	    code = method.startCode();
 	    Variable ctxVar = code.addLocal(typeCallContext);
 	    Variable saveVar = code.addLocal(typeConsumer);
-	    Variable exceptionVar = code.addLocal(Type.throwable_type);
+	    Variable exceptionVar = code.addLocal(Type.javalangThrowableType);
 	    // ctx = CallContext.getInstance();
 	    code.emitInvokeStatic(getCallContextInstanceMethod);
 	    code.emitStore(ctxVar);
@@ -2073,7 +2084,7 @@ public class Compilation implements SourceLocator
 			       .getDeclaredField("instance"));
 	    code.emitPutField(consumerFld);
 	    // try {
-	    code.emitTryStart(false, Type.void_type);
+	    code.emitTryStart(false, Type.voidType);
 	    // this.apply(ctx):
 	    code.emitPushThis();
 	    code.emitLoad(ctxVar);
@@ -2099,7 +2110,7 @@ public class Compilation implements SourceLocator
         callContextVar = callContextSave;
       }
 
-    module.compileEnd(this);
+    module.generateApplyMethods(this);
 
     curLambda = saveLambda;
 
@@ -2149,7 +2160,7 @@ public class Compilation implements SourceLocator
       {
 	Type[] args = { new ArrayType(javaStringType) };
 	method = curClass.addMethod("main", Access.PUBLIC|Access.STATIC,
-				    args, Type.void_type);
+				    args, Type.voidType);
 				    
 	code = method.startCode();
 
@@ -2208,10 +2219,10 @@ public class Compilation implements SourceLocator
         registerClass(mapClass);
 
         method = mapClass.addMethod("<init>", Access.PUBLIC,
-                                apply0args, Type.void_type);
+                                apply0args, Type.voidType);
         Method superConstructor
           = typeModuleSet.addMethod("<init>", Access.PUBLIC,
-                                    apply0args, Type.void_type);
+                                    apply0args, Type.voidType);
         code = method.startCode();
         code.emitPushThis();
         code.emitInvokeSpecial(superConstructor);
@@ -2219,14 +2230,14 @@ public class Compilation implements SourceLocator
 
         ClassType typeModuleManager = ClassType.make("gnu.expr.ModuleManager");
         Type[] margs = { typeModuleManager };
-        method = mapClass.addMethod("register", margs, Type.void_type,
+        method = mapClass.addMethod("register", margs, Type.voidType,
                                     Access.PUBLIC);
         code = method.startCode();
         Method reg = typeModuleManager.getDeclaredMethod("register", 3);
 
-        for (ModuleInfo mi = manager.firstModule();
-             mi != null;  mi = mi.nextModule())
+        for (int i = manager.numModules;  --i >= 0; )
           {
+            ModuleInfo mi = manager.modules[i];
             String miClassName = mi.className;
             if (miClassName == null
                 || ! miClassName.startsWith(mainPrefix))
@@ -2447,20 +2458,15 @@ public class Compilation implements SourceLocator
   /** Called for classes referenced in bytecode.
    * Since this only does something when immediate, we only care about
    * classes referenced in the bytecode when immediate.
-   * It is used to ensure that we can inherit from classes defines when in
+   * It is used to ensure that we can inherit from classes defined when in
    * immediate mode (in Scheme using define-class or similar).
    */
   public void usedClass (Type type)
   {
     while (type instanceof ArrayType)
       type = ((ArrayType) type).getComponentType();
-    if (! immediate || ! (type instanceof ClassType))
-      return;
-    ClassType clas = (ClassType) type;
-    if (loader != null && clas.isExisting())
-      {
-	loader.addClass(clas.getReflectClass());
-      }
+    if (immediate && type instanceof ClassType)
+      loader.addClass((ClassType) type);
   }
 
   public SourceMessages getMessages() { return messages; }
@@ -2679,31 +2685,26 @@ public class Compilation implements SourceLocator
   /** If non-null, a helper method generated by getForNameHelper. */
   Method forNameHelper;
 
-  public void loadClassRef (ClassType clas)
+  /** Generate code to load a named Class without initializing it.
+   */
+  public void loadClassRef (ObjectType clas)
   {
+    CodeAttr code = getCode();
     // Try an optimization
-    if (clas == mainClass && mainLambda.isStatic()
+    if (curClass.getClassfileVersion() >= ClassType.JDK_1_5_VERSION)
+      code.emitPushClass(clas);
+    else if (clas == mainClass && mainLambda.isStatic()
         // moduleInstanceMainField may not have been set yet.
         && moduleInstanceMainField != null)
       {
-        CodeAttr code = getCode();
         code.emitGetStatic(moduleInstanceMainField);
-        code.emitInvokeVirtual(Type.pointer_type.getDeclaredMethod("getClass", 0));
+        code.emitInvokeVirtual(Type.objectType.getDeclaredMethod("getClass", 0));
       }
-    else
-      loadClassRef(clas.getName());
-  }
-
-  /** Generate code to load a named Class without initializing it.
-   */
-  public void loadClassRef (String className)
-  {
-    CodeAttr code = getCode();
-    if (curClass.getClassfileMajorVersion() >= 49) // Java5 feature
-      code.emitPushClass(className);
     else
       {
-        code.emitPushString(className);
+        String name = clas instanceof ClassType ? clas.getName()
+          : clas.getInternalName().replace('/', '.');
+        code.emitPushString(name);
         code.emitInvokeStatic(getForNameHelper());
       }
   }
@@ -2796,6 +2797,7 @@ public class Compilation implements SourceLocator
             clas.getDeclaredField(init.field.getName())
               .set(null, init.value);
 	  }
+        comp.litTable = null;
       }
     catch (Throwable ex)
       {
@@ -2830,6 +2832,7 @@ public class Compilation implements SourceLocator
               chainUninitialized = next;
             else
               prev.nextUninitialized = next;
+            comp.nextUninitialized = null;
             return comp;
           }
         prev = comp;
