@@ -70,13 +70,21 @@ public class Declaration
   public final void setTypeExp (Expression typeExp)
   {
     this.typeExp = typeExp;
-    Object typeValue;
-    Type t = (typeExp instanceof QuoteExp
-              && (typeValue = ((QuoteExp) typeExp).getValue()) instanceof Type
-              ? (Type) typeValue
-              : typeExp instanceof TypeValue
-              ? ((TypeValue) typeExp).getImplementationType()
-              : (Type) Type.pointer_type);
+    Type t;
+    if (typeExp instanceof TypeValue)
+      t = ((TypeValue) typeExp).getImplementationType();
+    else if (typeExp instanceof QuoteExp)
+      {
+        Object typeValue = ((QuoteExp) typeExp).getValue();
+        if (typeValue instanceof Type)
+          t = (Type) typeValue;
+        else if (typeValue instanceof Class)
+          t = Type.make((Class) typeValue);
+        else
+          t = Type.pointer_type;
+      }
+    else
+      t = Type.pointer_type;
     this.type = t;
     if (var != null) var.setType(t);
   }
@@ -200,6 +208,7 @@ public class Declaration
         if (field != null)
           {
             comp.usedClass(field.getDeclaringClass());
+            comp.usedClass(field.getType());
             if (! field.getStaticFlag())
               {
                 loadOwningObject(owner, comp);
@@ -269,15 +278,16 @@ public class Declaration
                 Label endTry = new Label(code);
                 endTry.define(code);
                 Label endLabel = new Label(code);
+                endLabel.setTypes(code);
                 if (isInTry)
                   code.emitGoto(endLabel);
+                else
+                  code.setUnreachable();
                 int fragment_cookie = 0;
                 if (! isInTry)
-                  fragment_cookie
-                    = code.beginFragment(new Label(code), endLabel);
+                  fragment_cookie = code.beginFragment(endLabel);
                 code.addHandler(startTry, endTry, typeUnboundLocationException);
 
-                code.pushType(typeUnboundLocationException);
                 code.emitDup(typeUnboundLocationException);
                 code.emitPushString(filename);
                 code.emitPushInt(line);
@@ -296,12 +306,7 @@ public class Declaration
             rtype = Type.pointer_type;
           }
       }
-    if (target instanceof SeriesTarget
-	&& getFlag(Declaration.IS_SINGLE_VALUE))
-      // A kludge until we get a better type system.
-      ((SeriesTarget) target).compileFromStackSimple(comp, rtype);
-    else
-      target.compileFromStack(comp, rtype);
+    target.compileFromStack(comp, rtype);
   }
 
   /* Compile code to store a value (which must already be on the
@@ -371,6 +376,12 @@ public class Declaration
     return ((QuoteExp) v).getValue();
   }
 
+  public final boolean hasConstantValue ()
+  {
+    Object v = getValue();
+    return (v instanceof QuoteExp) && v != QuoteExp.undefined_exp;
+  }
+
   /** This prefix is prepended to field names for unknown names. */
   static final String UNKNOWN_PREFIX = "loc$";
 
@@ -780,6 +791,17 @@ public class Declaration
   public void printInfo(StringBuffer sbuf)
   {
     sbuf.append(symbol);
+    if (true || // DEBUGGING
+        symbol == null)
+      ;
+    else if (symbol instanceof SimpleSymbol)
+      sbuf.append("[simple-symbol]");
+    else if (symbol instanceof Symbol)
+      sbuf.append("[symbol]");
+    else if (symbol.toString().intern() == symbol)
+      sbuf.append("[interned-string]");
+    else if (symbol instanceof String)
+      sbuf.append("[noninterned-string]");
     sbuf.append('/');
     sbuf.append(id);
     /*
@@ -798,6 +820,8 @@ public class Declaration
     */
     sbuf.append("/fl:");
     sbuf.append(Integer.toHexString(flags));
+    if (ignorable())
+      sbuf.append("(ignorable)");
     Expression tx = typeExp;
     Type t = getType();
     if (tx != null && ! (tx instanceof QuoteExp))
@@ -855,7 +879,10 @@ public class Declaration
     boolean typeSpecified = getFlag(TYPE_SPECIFIED);
     if (isPublic() && ! isConstant && ! typeSpecified)
       setIndirectBinding(true);
-    if (isPublic() || external_access)
+    // In immediate mode we may need to access the field from a future
+    // command in a different "runtime package" (see JVM spec) because it
+    // gets loaded by a different class loader.  So make the field public.
+    if (isPublic() || external_access || comp.immediate)
       fflags |= Access.PUBLIC;
     if (isStatic()
 	|| (isConstant && value instanceof QuoteExp)
