@@ -5,6 +5,8 @@ import gnu.text.Printable;
 import gnu.text.SourceLocator;
 import gnu.lists.Consumer;
 import java.io.PrintWriter;
+import gnu.kawa.util.IdentityHashTable;
+import gnu.kawa.reflect.OccurrenceType;
 
 /**
  * Abstract class for syntactic forms that evaluate to a value.
@@ -198,6 +200,51 @@ public abstract class Expression extends Procedure0
       }
   }
 
+  /** Make a deep copy of this expression, if possible.
+   * @param mapper used to lookup parts (expressions, declarations)
+   *   that have been translated in the parent.
+   *   Needed for copied Declarations and BlockExps.
+   * @return a copy, or null if we can't copy.
+   */
+  public static Expression deepCopy (Expression exp, IdentityHashTable mapper)
+  {
+    if (exp == null)
+      return null;
+    Object tr = mapper.get(exp);
+    if (tr != null) return (Expression) tr;
+    Expression copy = exp.deepCopy(mapper);
+    mapper.put(exp, copy);
+    return copy;
+  }
+
+  public static Expression[] deepCopy (Expression[] exps,
+                                       IdentityHashTable mapper)
+  {
+    if (exps == null)
+      return null;
+    int nargs = exps.length;
+    Expression[] a = new Expression[nargs];
+    for (int i = 0; i < nargs;  i++)
+      {
+        Expression ei = exps[i];
+        Expression ai = deepCopy(ei, mapper);
+        if (ai == null && ei != null)
+          return null;
+        a[i] = ai;
+      }
+    return a;
+  }
+
+  protected static Expression deepCopy (Expression exp)
+  {
+    return deepCopy(exp, new IdentityHashTable());
+  }
+
+  protected Expression deepCopy (IdentityHashTable mapper)
+  {
+    return null;
+  }
+
   protected Expression walk (ExpWalker walker)
   {
     return walker.walkExpression(this);
@@ -206,16 +253,20 @@ public abstract class Expression extends Procedure0
   protected void walkChildren (ExpWalker walker) { }
 
   /** Apply inlining transformations on a given ApplyExp.
-   * Assumes the ApplyExp's function is the this expression,
+   * Assumes the ApplyExp's function is this expression,
    * or can be optimized to this expression.
    * @param exp an application whose function expression can be simplified
    *  to this expression.
    * @param walker the context for the current inlining pass
-   * @param decl if non-null, a Declaration bound to this expression
-   * @return an Expression equivalent to te passed-in exp.
+   * @param decl if non-null, a Declaration bound to this expression.
+   * @param argsInlined true iff {@code exp.getArgs()} have been walked.
+   * @return an Expression equivalent to the passed-in exp.
    */
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
   {
+    if (! argsInlined)
+      exp.args = walker.walkExps(exp.args, exp.args.length);
     return exp;
   }
 
@@ -325,6 +376,11 @@ public abstract class Expression extends Procedure0
     return Type.pointer_type;
   }
 
+  public boolean isSingleValue()
+  {
+    return OccurrenceType.itemCountIsOne(getType());
+  }
+
   /** Return value if it is constant, or null if non-constant or unknown. */
   public Object valueIfConstant ()
   {
@@ -345,7 +401,7 @@ public abstract class Expression extends Procedure0
     flags |= flag;
   }
 
-  public int  getFlags()
+  public int getFlags()
   {
     return flags;
   }
