@@ -1,4 +1,4 @@
-// Copyright (c) 2003  Per M.A. Bothner.
+// Copyright (c) 2003, 2009  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -42,10 +42,41 @@ public class FindCapturedVars extends ExpWalker
 	      {
 		LambdaExp lexp = (LambdaExp) value;
 		if (! lexp.getNeedsClosureEnv())
-		  skipFunc = true;
+                  skipFunc = true;
 	      }
 	  }
       }
+    // Similar hack for constructor calls, but here we want to
+    // avoid walking the type argument.
+    else if (exp.func instanceof QuoteExp && exp.getArgCount() > 0)
+      {
+        Object val = ((QuoteExp) exp.func).getValue();
+        Expression arg0 = exp.getArg(0);
+        if (val instanceof PrimProcedure && arg0 instanceof ReferenceExp)
+          {
+            PrimProcedure pproc = (PrimProcedure) val;
+            Declaration decl
+              = Declaration.followAliases(((ReferenceExp) arg0).binding);
+            if (decl != null && decl.context instanceof ModuleExp
+                && ! decl.getFlag(Declaration.NONSTATIC_SPECIFIED))
+              {
+                Expression value = decl.getValue();
+                if (value instanceof ClassExp)
+                  {
+                    Expression[] args = exp.getArgs();
+                    LambdaExp lexp = (LambdaExp) value;
+                    if (! lexp.getNeedsClosureEnv())
+                      {
+                        exp.nextCall = decl.firstCall;
+                        decl.firstCall = exp;
+                        for (int i = 1;  i < args.length;  i++)
+                          args[i].walk(this);
+                        return exp;
+                      }
+                  }
+              }
+          }
+      }
     if (! skipFunc)
       exp.func = (Expression) exp.func.walk(this);
     if (exitValue == null)
@@ -80,10 +111,25 @@ public class FindCapturedVars extends ExpWalker
   protected Expression walkClassExp (ClassExp exp)
   {
     Expression ret = super.walkClassExp(exp);
-    // Make sure <init> has been declared, in case we need to invoke it.
-    // However, this is only safe to do if ! getNeedsClosureEnv().  FIXME.
-    if (! exp.explicitInit && ! exp.getNeedsClosureEnv())
+    if (! exp.explicitInit && ! exp.instanceType.isInterface())
+      // Make sure <init> has been declared, in case we need to invoke it.
       Compilation.getConstructor(exp.instanceType, exp);
+    else if (exp.getNeedsClosureEnv())
+      {
+        for (LambdaExp child = exp.firstChild;  child != null;
+             child = child.nextSibling)
+          {
+            if ("*init*".equals(child.getName()))
+              child.setNeedsStaticLink(true);
+          }
+      }
+    if (exp.isSimple() && exp.getNeedsClosureEnv() && exp.nameDecl != null)
+      {
+        comp.error('w', "simple class requiring lexical link - use define-class instead");
+        //exp.setSimple(false);
+        if (exp.nameDecl.getType() == Compilation.typeClass)
+          exp.nameDecl.setType(Compilation.typeClassType);
+      }
     return ret;
   }
 
@@ -164,18 +210,84 @@ public class FindCapturedVars extends ExpWalker
     return super.walkLetExp(exp);
   }
 
+  static Expression checkInlineable (LambdaExp current,
+                                     java.util.Set<LambdaExp> seen)
+  {
+    if (current.returnContinuation == LambdaExp.unknownContinuation)
+      return current.returnContinuation;
+    if (seen.contains(current))
+      return current.returnContinuation;
+    if (current.getCanRead()
+        || current.isClassMethod()
+        || current.min_args != current.max_args)
+      {
+        current.returnContinuation = LambdaExp.unknownContinuation;
+        return LambdaExp.unknownContinuation;
+      }
+    seen.add(current);
+    Expression r = current.returnContinuation;
+    if (current.tailCallers != null)
+      {
+        LambdaExp outer = current.outerLambda();
+        for (LambdaExp p : current.tailCallers)
+          {
+            Expression t = checkInlineable(p, seen);
+            if (t == LambdaExp.unknownContinuation)
+              {
+                if (r == null || r == p.body)
+                  {
+                    r = p.body;
+                    current.inlineHome = p;
+                  }
+                else
+                  {
+                    current.returnContinuation = LambdaExp.unknownContinuation;
+                    return t;
+                  }
+              }
+            else if (r == null)
+              {
+                r = t;
+                if (current.inlineHome == null)
+                  current.inlineHome = current.nestedIn(p) ? p : p.inlineHome;
+              }
+            else if ((t != null && r != t)
+                     || current.getFlag(LambdaExp.CANNOT_INLINE))
+              {
+                current.returnContinuation = LambdaExp.unknownContinuation;
+                return LambdaExp.unknownContinuation;
+              }
+          }
+      }
+    return r;
+  }
+
+  protected Expression walkLambdaExp (LambdaExp exp)
+  {
+    java.util.Set<LambdaExp> seen = new java.util.LinkedHashSet<LambdaExp>();
+    // Finish the job that was started in FindTailCalls.
+    Expression caller = checkInlineable(exp, seen);
+    if (caller != LambdaExp.unknownContinuation)
+      exp.setInlineOnly(true);
+    return super.walkLambdaExp(exp);
+  }
+
   public void capture(Declaration decl)
   {
     if (! (decl.getCanRead() || decl.getCanCall()))
       return;
     if (decl.field != null && decl.field.getStaticFlag())
       return;
+    // This catches the "(module-instance)" dummy context variable
+    // created in Translator.rewrite.
+    if (comp.immediate && decl.hasConstantValue())
+      return;
 
     LambdaExp curLambda = getCurrentLambda ();
     LambdaExp declLambda = decl.getContext().currentLambda ();
 
     // If curLambda is inlined, the function that actually needs a closure
-    // is its caller.  We get its caller using returnContinuation.context.
+    // is its caller.  We get its caller using getCaller().
     // A complication is that we can have a chain of functions that
     // recursively call each other, and are hence inlined in each other.
     // Since a function is only inlined if it has a single call site,
@@ -199,14 +311,13 @@ public class FindCapturedVars extends ExpWalker
             chain = curParent.firstChild;
             oldParent = curParent;
           }
-        ApplyExp curReturn = curLambda.returnContinuation;
-        if (chain == null || curReturn == null)
+        if (chain == null || curLambda.inlineHome == null)
           {
             // Infinite loop of functions that are inlined in each other.
             curLambda.setCanCall(false);
             return;
-        }
-        curLambda = curReturn.context;
+          }
+        curLambda = curLambda.getCaller();
         chain = chain.nextSibling;
       }
     if (comp.usingCPStyle())
@@ -313,6 +424,7 @@ public class FindCapturedVars extends ExpWalker
 	if (currentModule.isStatic())
 	  decl.setFlag(Declaration.STATIC_SPECIFIED);
 	decl.setCanRead(true);
+	decl.setCanWrite(true);
 	decl.setFlag(Declaration.IS_UNKNOWN);
 	decl.setIndirectBinding(true);
 	unknownDecls.put(key, decl);
@@ -331,10 +443,6 @@ public class FindCapturedVars extends ExpWalker
       }
     if (decl.getFlag(Declaration.IS_UNKNOWN))
       {
-	Type type = getCompilation().getLanguage().getTypeFor(exp);
-	if (type instanceof Externalizable && ! exp.getDontDereference())
-	  return new QuoteExp(type);
-
 	if (comp.getBooleanOption("warn-undefined-variable", false))
 	  {
 	    Object resolved
