@@ -1,4 +1,4 @@
-// Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004  Per M.A. Bothner.
+// Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2007, 2008, 2009  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -6,6 +6,7 @@ import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.lists.LList;
 import java.util.Vector;
+import gnu.kawa.functions.Convert;
 
 /**
  * Class used to implement Scheme lambda expressions.
@@ -72,15 +73,24 @@ public class LambdaExp extends ScopeExp
   /** A magic value to indicate there is no unique return continuation. */
   final static ApplyExp unknownContinuation = new ApplyExp ((Expression) null, null);
 
-  /** The unique caller that calls this lambda.
-      The value is null, if no callers have been seen.
-      A value of unknownContinuation means there are multiple call sites.
-      Tail-recursive calls do not count as multiple call sites. (With a
-      little more analysis, we could also allow multiple non-self tail-calls
-      as long as they all are ultimately called from the same place.)
-      This is used to see if we can inline the function at its unique
-      call site. */
-  public ApplyExp returnContinuation;
+  /** The unique call site that calls this lambda.
+   * The value is null if no callers have been seen.
+   * A value of unknownContinuation means there are multiple call sites.
+   * Tail-recursive calls do not count as multiple call sites.
+   * This is used to see if we can inline the function at its unique call site.
+   * Usually this is an ApplyExp, but it can also be the "tail position"
+   * for some outer expression, such as an IfExp.  This allows inlining f
+   * in the call 'if (cond) f(x) else f(y)' since both calls have the same
+   * return point.
+   */
+  public Expression returnContinuation;
+
+  /** If non-null, set of functions that tail-call this function. */
+  java.util.Set<LambdaExp> tailCallers;
+
+  /** If this lambda gets inlined this is the containing lambda.
+      Otherwise this is null. */
+  public LambdaExp inlineHome;
 
   /** Expressions that name classes that may be thrown. */
   ReferenceExp[] throwsSpecification;
@@ -128,7 +138,8 @@ public class LambdaExp extends ScopeExp
   static final int DEFAULT_CAPTURES_ARG = 512;
   public static final int SEQUENCE_RESULT = 1024;
   public static final int OVERLOADABLE_FIELD = 2048;
-  protected static final int NEXT_AVAIL_FLAG = 4096;
+  public static final int ATTEMPT_INLINE = 4096;
+  protected static final int NEXT_AVAIL_FLAG = 8192;
 
   /** True iff this lambda is only "called" inline. */
   public final boolean getInlineOnly() { return (flags & INLINE_ONLY) != 0; }
@@ -225,6 +236,11 @@ public class LambdaExp extends ScopeExp
     return isModuleBody() || this instanceof ClassExp;
   }
 
+  public boolean isAbstract ()
+  {
+    return body == QuoteExp.abstractExp;
+  }
+
   /** Specify the calling convention used for this function.
    * @return One of the CALL_WITH_xxx values in Compilation. */
   public int getCallConvention ()
@@ -302,6 +318,9 @@ public class LambdaExp extends ScopeExp
    * primMethods[primMethods.length-1] is used otherwise.
    */
   Method[] primMethods;
+  /** If in a ClassExp which isMakingClassPair, the static body methods.
+   * Otherwise, same as primMethods. */
+  Method[] primBodyMethods;
 
   /** Select the method used given an argument count. */
   public final Method getMethod(int argCount)
@@ -319,7 +338,7 @@ public class LambdaExp extends ScopeExp
    * (The other methods are just stubs that call that method.) */
   public final Method getMainMethod()
   {
-    Method[] methods = primMethods;
+    Method[] methods = primBodyMethods;
     return methods == null ? null : methods[methods.length-1];
   }
 
@@ -368,19 +387,10 @@ public class LambdaExp extends ScopeExp
    * except in the case that outer.getInlineOnly(). */
   boolean inlinedIn (LambdaExp outer)
   {
-    if (! getInlineOnly())
-      return false;
-    for (ScopeExp exp = getCaller(); exp != null;  exp = exp.outer)
+    for (LambdaExp exp = this; exp.getInlineOnly(); exp = exp.getCaller())
       {
-	if (exp instanceof LambdaExp)
-	  {
-	    
-	    LambdaExp result = (LambdaExp) exp;
-	    if (result == outer)
-	      return true;
-	    if (! result.getInlineOnly())
-	      return false;
-	  }
+        if (exp == outer)
+          return true;
       }
     return false;
   }
@@ -388,7 +398,7 @@ public class LambdaExp extends ScopeExp
   /** For an INLINE_ONLY function, return the function it gets inlined in. */
   public LambdaExp getCaller ()
   {
-    return returnContinuation.context;
+    return inlineHome;
   }
 
   Variable thisVariable;
@@ -417,21 +427,28 @@ public class LambdaExp extends ScopeExp
 	  parent = parent.outerLambda();
 	Variable parentFrame = parent.heapFrame != null ?  parent.heapFrame
 	  : parent.closureEnv;
-	if (isClassMethod())
-	  closureEnv = declareThis(type);
+	if (isClassMethod() && ! "*init*".equals(getName()))
+          closureEnv = declareThis(type);
 	else if (parent.heapFrame == null && ! parent.getNeedsStaticLink()
 		 && ! (parent instanceof ModuleExp))
 	  closureEnv = null;
 	else if (! isClassGenerated() && ! getInlineOnly())
 	  {
 	    Method primMethod = getMainMethod();
-	    if (! primMethod.getStaticFlag())
+            boolean isInit = "*init*".equals(getName());
+	    if (! primMethod.getStaticFlag()
+                && ! isInit)
 	      closureEnv = declareThis(primMethod.getDeclaringClass());
 	    else
 	      {
 		Type envType = primMethod.getParameterTypes()[0];
 		closureEnv = new Variable("closureEnv", envType);
-		getVarScope().addVariableAfter(null, closureEnv);
+                Variable prev;
+                if (isInit)
+                  prev = declareThis(primMethod.getDeclaringClass());
+                else
+                  prev = null;
+                getVarScope().addVariableAfter(prev, closureEnv);
 		closureEnv.setParameter(true);
 	      }
 	  }
@@ -525,10 +542,17 @@ public class LambdaExp extends ScopeExp
 	  code.popScope(); // Undoes pushScope in method.initCode.
       }
 
+    for (LambdaExp child = firstChild;  child != null; )
+      {
+	if (! child.getCanRead() && ! child.getInlineOnly())
+	  {
+	    child.compileAsMethod(comp);
+	  }
+	child = child.nextSibling;
+      }
+
     if (heapFrame != null)
-      comp.generateConstructor((ClassType) heapFrame.getType(), this);
-    
-    generateApplyMethods(comp);
+      comp.generateConstructor(this);
   }
 
   public void generateApplyMethods(Compilation comp)
@@ -544,7 +568,9 @@ public class LambdaExp extends ScopeExp
   {
     if (nameDecl != null && nameDecl.field != null)
       return nameDecl.field;
-    ClassType frameType = getOwningLambda().getHeapFrameType();
+    boolean needsClosure = getNeedsClosureEnv();
+    ClassType frameType = needsClosure ? getOwningLambda().getHeapFrameType()
+      : comp.mainClass;
     String name = getName();
     String fname
       = name == null ? "lambda" : Compilation.mangleNameIfNeeded(name);
@@ -563,7 +589,10 @@ public class LambdaExp extends ScopeExp
             if (! ((ModuleExp) nameDecl.context).isStatic())
               fflags &= ~Access.FINAL;
           }
-	if (! nameDecl.isPrivate() || external_access)
+        // In immediate mode we may need to access the field from a future
+        // command in a different "runtime package" (see JVM spec) because it
+        // gets loaded by a different class loader.  So make the field public.
+	if (! nameDecl.isPrivate() || external_access || comp.immediate)
 	  fflags |= Access.PUBLIC;
         if ((flags & OVERLOADABLE_FIELD) != 0)
           {
@@ -576,8 +605,8 @@ public class LambdaExp extends ScopeExp
     else
       {
 	fname = fname + "$Fn" + ++comp.localFieldIndex;
-	if (! getNeedsClosureEnv())
-	  fflags = (fflags | Access.STATIC) & ~Access.FINAL;
+        if (! needsClosure)
+          fflags |= Access.STATIC;
       }
     Type rtype = Compilation.typeModuleMethod;
     Field field = frameType.addField (fname, rtype, fflags);
@@ -586,21 +615,26 @@ public class LambdaExp extends ScopeExp
     return field;
   }
 
-  final void addApplyMethod (Compilation comp)
+  final void addApplyMethod (Compilation comp, Field field)
   {
     LambdaExp owner = this;
-    // Similar to getOwningLambda(), but we can't add apply methods
-    // to a ClassExp - at least not unless it extends ModuleBody.
-    for (;;)
-      {
-        owner = owner.outerLambda();
-	if (owner instanceof ModuleExp
-	    || owner.heapFrame != null)
-          break;
-      }
-    ClassType frameType = owner.getHeapFrameType();
-    if (! (frameType.getSuperclass().isSubtype(Compilation.typeModuleBody)))
+    if (field != null && field.getStaticFlag())
       owner = comp.getModule();
+    else
+      {
+        // Similar to getOwningLambda(), but we can't add apply methods
+        // to a ClassExp - at least not unless it extends ModuleBody.
+        for (;;)
+          {
+            owner = owner.outerLambda();
+            if (owner instanceof ModuleExp
+                || owner.heapFrame != null)
+              break;
+          }
+        ClassType frameType = owner.getHeapFrameType();
+        if (! (frameType.getSuperclass().isSubtype(Compilation.typeModuleBody)))
+          owner = comp.getModule();
+      }
     if (owner.applyMethods == null)
       owner.applyMethods = new Vector();
     owner.applyMethods.addElement(this);
@@ -608,15 +642,16 @@ public class LambdaExp extends ScopeExp
 
   public Field compileSetField (Compilation comp)
   {
+    Field field = allocFieldFor(comp);
     if (comp.usingCPStyle())
-      compile(comp, Type.pointer_type);
+      compile(comp, Type.objectType);
     else
       {
 	compileAsMethod(comp);
-	addApplyMethod(comp);
+	addApplyMethod(comp, field);
       }
 
-    return (new ProcInitializer(this, comp)).field;
+    return (new ProcInitializer(this, comp, field)).field;
   }
 
   public void compile (Compilation comp, Target target)
@@ -627,6 +662,7 @@ public class LambdaExp extends ScopeExp
     Type rtype;
     CodeAttr code = comp.getCode();
 
+    /*
     if (comp.usingCPStyle())
       {
 	//	Label func_start = new Label(code);
@@ -635,71 +671,67 @@ public class LambdaExp extends ScopeExp
 	comp.curLambda = this;
 	type = saveLambda.type;
 	closureEnv = saveLambda.closureEnv;
-        /*
-	if (comp.usingCPStyle())
-	  {
-	    heapFrame = comp.thisDecl;
-	    for (Declaration var = firstDecl();
-		 var != null; var = var.nextDecl())
-	      var.assignField(comp);
-	  }
-        */
+	// if (comp.usingCPStyle())
+	//   {
+	//     heapFrame = comp.thisDecl;
+	//     for (Declaration var = firstDecl();
+	// 	 var != null; var = var.nextDecl())
+	//       var.assignField(comp);
+	//   }
 	gnu.bytecode.SwitchState fswitch = comp.fswitch;
 	int pc = comp.fswitch.getMaxValue() + 1;
 	code.emitGoto(func_end);
 	Type[] stackTypes = code.saveStackTypeState(true);
 
 	fswitch.addCase(pc, code);
-        /*
-	code.emitPushThis();
-	code.emitGetField(comp.argsCallContextField);
-	code.emitStore(comp.argsArray);
-        */
+	// code.emitPushThis();
+	// code.emitGetField(comp.argsCallContextField);
+	// code.emitStore(comp.argsArray);
 	allocParameters(comp);
 	enterFunction(comp);
 
 	compileBody(comp);
 	compileEnd(comp);
+	generateApplyMethods(comp);
 	comp.curLambda = saveLambda;
 	func_end.define(code);
 	code.restoreStackTypeState(stackTypes);
 	ClassType ctype = comp.curClass;
 	rtype = ctype;
-	/*
-	code.emitNew(ctype);
-	code.emitDup(ctype);
-	code.emitInvokeSpecial(ctype.constructor);
-	code.emitDup(ctype);
-	code.emitPushInt(pc);
-	code.emitPutField(comp.saved_pcCallFrameField);
-	if (isHandlingTailCalls())
-	  {
-	    // Set name field.
-	    if (name != null)
-	      {
-		code.emitDup(ctype);
-		code.emitPushString(name);
-		code.emitInvokeVirtual(comp.setNameMethod);
-	      }
-	    // Set numArgs field.
-	    code.emitDup(ctype);
-	    code.emitPushInt(min_args | (max_args << 12));
-	    code.emitPutField(comp.numArgsCallFrameField);
-	    // Set static link field to this CallFrame.
-	    code.emitDup(ctype);
-	    code.emitPushThis();
-	    code.emitPutField(comp.callerCallFrameField);
-	  }
-	*/
+	// code.emitNew(ctype);
+	// code.emitDup(ctype);
+	// code.emitInvokeSpecial(ctype.constructor);
+	// code.emitDup(ctype);
+	// code.emitPushInt(pc);
+	// code.emitPutField(comp.saved_pcCallFrameField);
+	// if (isHandlingTailCalls())
+	//   {
+	//     // Set name field.
+	//     if (name != null)
+	//       {
+	// 	code.emitDup(ctype);
+	// 	code.emitPushString(name);
+	// 	code.emitInvokeVirtual(comp.setNameMethod);
+	//       }
+	//     // Set numArgs field.
+	//     code.emitDup(ctype);
+	//     code.emitPushInt(min_args | (max_args << 12));
+	//     code.emitPutField(comp.numArgsCallFrameField);
+	//     // Set static link field to this CallFrame.
+	//     code.emitDup(ctype);
+	//     code.emitPushThis();
+	//     code.emitPutField(comp.callerCallFrameField);
+	//   }
       }
     else
+    */
       { LambdaExp outer = outerLambda();
 	rtype = Compilation.typeModuleMethod;
 	if ((flags & NO_FIELD) != 0
 	    || (comp.immediate && outer instanceof ModuleExp))
 	  {
 	    compileAsMethod(comp);
-	    addApplyMethod(comp);
+	    addApplyMethod(comp, null);
 	    ProcInitializer.emitLoadModuleMethod(this, comp);
 	  }
 	else
@@ -771,7 +803,14 @@ public class LambdaExp extends ScopeExp
     int numStubs =
       ((flags & DEFAULT_CAPTURES_ARG) != 0) ? 0 : opt_args;
     boolean varArgs = max_args < 0 || min_args + numStubs < max_args;
-    primMethods = new Method[numStubs + 1];
+
+    Method[] methods = new Method[numStubs + 1];
+    // We assume that for "pair" class methods that ClassExp.declareParts first
+    // calls this method to create the interface method, and then calls us
+    // to create the static implementation method.
+    primBodyMethods = methods;
+    if (primMethods == null)
+      primMethods = methods;
 
     boolean isStatic;
     // 'I' if initMethod ($finit$); 'C' if clinitMethod (<clinit>).
@@ -853,15 +892,81 @@ public class LambdaExp extends ScopeExp
 	    mflags = (mflags & ~Access.PUBLIC+Access.PROTECTED)+Access.PRIVATE;
 	  }
       }
-    if (ctype.isInterface())
+    if (ctype.isInterface() || isAbstract())
       mflags |= Access.ABSTRACT;
-    if (! isStatic)
-      declareThis(ctype);
+
+    // If a class method has unspecified parameter types, see if we
+    // can "inherit" the parameter types from an inherited method.
+    if (isClassMethod() && outer instanceof ClassExp
+        && min_args == max_args)
+      {
+        Method[] inherited = null;
+        int iarg = 0;
+      param_loop:
+        for (Declaration param = firstDecl(); ;
+             param = param.nextDecl(), iarg++)
+          {
+            if (param == null)
+              {
+                if (returnType != null)
+                  break;
+              }
+            else if (param.isThisParameter())
+              {
+                iarg--;
+                continue;
+              }
+            else if (param.getFlag(Declaration.TYPE_SPECIFIED))
+              continue;
+            if (inherited == null)
+              {
+                final String mangled = nameBuf.toString();
+                gnu.bytecode.Filter filter
+                  = new gnu.bytecode.Filter() {
+                      public boolean select(Object value)
+                      {
+                        gnu.bytecode.Method method = (gnu.bytecode.Method) value;
+                        if (! method.getName().equals(mangled))
+                          return false;
+                        Type[] ptypes = method.getParameterTypes();
+                        return ptypes.length == min_args;
+                      }
+                    };
+                inherited = ctype.getMethods(filter, 2);
+              }
+            Type type = null;
+            for (int i = inherited.length;  --i >= 0; )
+              {
+                Method method = inherited[i];
+                Type ptype = param == null ? method.getReturnType()
+                  : method.getParameterTypes()[iarg];
+                if (type == null)
+                  type = ptype;
+                else if (ptype != type)
+                  {
+                    // More than one method with inconsistent parameter type.
+                    if (param == null)
+                      break param_loop;
+                    else
+                      continue param_loop;
+                  }
+              }
+            if (type != null)
+              {
+                if (param != null)
+                  param.setType(type);
+                else
+                  setCoercedReturnType(type);
+              }
+            if (param == null)
+              break param_loop;
+          }
+      }
 
     Type rtype
       = (getFlag(SEQUENCE_RESULT)
 	 || getCallConvention () >= Compilation.CALL_WITH_CONSUMER)
-      ? Type.void_type
+      ? Type.voidType
       : getReturnType().getImplementationType();
     int extraArg = (closureEnvType != null && closureEnvType != ctype) ? 1 : 0;
 
@@ -890,11 +995,19 @@ public class LambdaExp extends ScopeExp
 	  atypes[atypes.length-1] = Compilation.typeCallContext;
 	if (plainArgs < numArgs)
 	  {
-	    nameBuf.append("$V");
-	    name = nameBuf.toString();
 	    Type lastType = var.getType();
 	    String lastTypeName = lastType.getName();
+            if (ctype.getClassfileVersion() >= ClassType.JDK_1_5_VERSION
+                && "java.lang.Object[]".equals(lastTypeName))
+              mflags |= Access.VARARGS;
+            else 
+              nameBuf.append("$V");
 	    if (key_args > 0 || numStubs < opt_args
+                // We'd like to support the the #!rest parameter an arbitrary
+                // array type or implementation of java.util.List.  However,
+                // we currently only support gnu.lists.LList and Object[].
+                // For Object[] we prefer to use the Java5 VARARGS feature;
+                // for gnu.lists.LList we use the old $V mechanism.
 		|| ! ("gnu.lists.LList".equals(lastTypeName)
 		      || "java.lang.Object[]".equals(lastTypeName)))
 	      {
@@ -944,7 +1057,7 @@ public class LambdaExp extends ScopeExp
 	    }
 	}
 	Method method = ctype.addMethod(name, atypes, rtype, mflags);
-	primMethods[i] = method;
+	methods[i] = method;
 
 	if (throwsSpecification != null && throwsSpecification.length > 0)
 	  {
@@ -987,6 +1100,9 @@ public class LambdaExp extends ScopeExp
   {
     Method main = getMainMethod();
 	
+    if (main != null && ! main.getStaticFlag())
+      declareThis(main.getDeclaringClass());
+
     Declaration decl = firstDecl();
     for (;;)
       {
@@ -1069,7 +1185,7 @@ public class LambdaExp extends ScopeExp
         if (child instanceof ClassExp)
           {
             ClassExp cl = (ClassExp) child;
-            if (cl.getNeedsStaticLink())
+            if (cl.getNeedsClosureEnv())
               {
                 ClassType parentFrameType;
                 if (this instanceof ModuleExp || this instanceof ClassExp)
@@ -1130,7 +1246,7 @@ public class LambdaExp extends ScopeExp
 	  {
 	    code.emitLoad(argsArray);
 	    code.emitPushInt(j);
-	    code.emitArrayLoad(Type.pointer_type);
+	    code.emitArrayLoad(Type.objectType);
 	    decl.getType().emitCoerceFromObject(code);
 	    code.emitStore(decl.getVariable());
 	  }
@@ -1153,7 +1269,7 @@ public class LambdaExp extends ScopeExp
     CodeAttr code = comp.getCode();
 
     // Tail-calls loop back to here!
-    getVarScope().setStartPC(code);
+    getVarScope().noteStartFunction(code);
 
     if (closureEnv != null && ! closureEnv.isParameter()
 	&& ! comp.usingCPStyle())
@@ -1194,6 +1310,7 @@ public class LambdaExp extends ScopeExp
 					       closureEnv.getType());
         if (! (this instanceof ModuleExp) && ! (this instanceof ClassExp))
           {
+            frameType.setEnclosingMember(comp.method);
 	    code.emitNew(frameType);
 	    code.emitDup(frameType);
 	    Method constructor = Compilation.getConstructor(frameType, this);
@@ -1201,7 +1318,7 @@ public class LambdaExp extends ScopeExp
 
             if (staticLinkField != null)
               {
-                code.emitDup(heapFrame.getType());
+                code.emitDup(frameType);
                 code.emitLoad(closureEnv);
                 code.emitPutField(staticLinkField);
               }
@@ -1255,7 +1372,7 @@ public class LambdaExp extends ScopeExp
 	  {
 	    Type paramType = param.getType();
 	    Type stackType
-	      = (mainMethod == null || plainArgs >= 0 ? Type.pointer_type
+	      = (mainMethod == null || plainArgs >= 0 ? Type.objectType
 		 : paramType);
 	    // If the parameter is captured by an inferior lambda,
 	    // then the incoming parameter needs to be copied into its
@@ -1274,7 +1391,7 @@ public class LambdaExp extends ScopeExp
 	      { // This is a required parameter, in argsArray[i].
 		code.emitLoad(argsArray);
 		code.emitPushInt(i);
-		code.emitArrayLoad(Type.pointer_type);
+		code.emitArrayLoad(Type.objectType);
 	      }
             else if (i < min_args + opt_args)
 	      { // An optional parameter
@@ -1284,7 +1401,7 @@ public class LambdaExp extends ScopeExp
 		code.emitIfIntLt();
                 code.emitLoad(argsArray);
 		code.emitPushInt(i - plainArgs);
-		code.emitArrayLoad(Type.pointer_type);
+		code.emitArrayLoad();
 		code.emitElse();
 		defaultArgs[defaultStart + opt_i++].compile(comp, paramType);
 		code.emitFi();
@@ -1313,13 +1430,13 @@ public class LambdaExp extends ScopeExp
 		      {
 			Type[] argts = new Type[4];
 			argts[0] = Compilation.objArrayType;
-			argts[1] = Type.int_type;
-			argts[2] = Type.pointer_type;
-			argts[3] = Type.pointer_type;
+			argts[1] = Type.intType;
+			argts[2] = Type.objectType;
+			argts[3] = Type.objectType;
 			searchForKeywordMethod4
 			  = Compilation.scmKeywordType.addMethod
 			  ("searchForKeyword",  argts,
-			   Type.pointer_type, Access.PUBLIC|Access.STATIC);
+			   Type.objectType, Access.PUBLIC|Access.STATIC);
 		      }
 		    defaultArg.compile(comp, paramType);
 		    code.emitInvokeStatic(searchForKeywordMethod4);
@@ -1330,12 +1447,12 @@ public class LambdaExp extends ScopeExp
 		      {
 			Type[] argts = new Type[3];
 			argts[0] = Compilation.objArrayType;
-			argts[1] = Type.int_type;
-			argts[2] = Type.pointer_type;
+			argts[1] = Type.intType;
+			argts[2] = Type.objectType;
 			searchForKeywordMethod3
 			  = Compilation.scmKeywordType.addMethod
 			  ("searchForKeyword",  argts,
-			   Type.pointer_type, Access.PUBLIC|Access.STATIC);
+			   Type.objectType, Access.PUBLIC|Access.STATIC);
 		      }
 		    code.emitInvokeStatic(searchForKeywordMethod3);
 		    code.emitDup(1);
@@ -1362,23 +1479,13 @@ public class LambdaExp extends ScopeExp
     comp.callContextVar = callContextSave;
   }
 
-  void compileChildMethods (Compilation comp)
-  {
-    for (LambdaExp child = firstChild;  child != null; )
-      {
-	if (! child.getCanRead() && ! child.getInlineOnly())
-	  {
-	    child.compileAsMethod(comp);
-	  }
-	child = child.nextSibling;
-      }
-  }
-
   void compileAsMethod (Compilation comp)
   {
-    if ((flags & METHODS_COMPILED) != 0)
+    if ((flags & METHODS_COMPILED) != 0 || isAbstract())
       return;
     flags |= METHODS_COMPILED;
+    if (primMethods == null)
+      return;
     Method save_method = comp.method;
     LambdaExp save_lambda = comp.curLambda;
     comp.curLambda = this;
@@ -1476,10 +1583,10 @@ public class LambdaExp extends ScopeExp
 
 	    compileBody(comp);
 	    compileEnd(comp);
+	    generateApplyMethods(comp);
 	  }
       }
 
-    compileChildMethods(comp);
     comp.method = save_method;
     comp.curLambda = save_lambda;
   }
@@ -1576,8 +1683,32 @@ public class LambdaExp extends ScopeExp
     ctx.writeValue(new Closure(this, ctx));
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  Object evalDefaultArg(int index, CallContext ctx)
   {
+    try
+      {
+        return defaultArgs[index].eval(ctx);
+      }
+    catch (Throwable ex)
+      {
+        throw new WrappedException("error evaluating default argument", ex);
+      }
+  }
+
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
+  {
+    Expression[] args = exp.getArgs();
+    if (! argsInlined)
+      {
+        if ((flags & ATTEMPT_INLINE) != 0)
+          {
+            Expression inlined = InlineCalls.inlineCall(this, args, true);
+            if (inlined != null)
+              return walker.walk(inlined);
+          }
+        exp.args = walker.walkExps(exp.args, exp.args.length);
+      }
     int args_length = exp.args.length;
     String msg = WrongArguments.checkArgCount(getName(),
                                               min_args, max_args, args_length);
@@ -1586,6 +1717,7 @@ public class LambdaExp extends ScopeExp
     int conv = getCallConvention();
     Compilation comp = walker.getCompilation();
     Method method;
+    // Mostly duplicates logic with ApplyExp.compile.
     if (comp.inlineOk(this) && isClassMethod()
         && (conv <= Compilation.CALL_WITH_CONSUMER
             || (conv == Compilation.CALL_WITH_TAILCALLS))
@@ -1595,10 +1727,23 @@ public class LambdaExp extends ScopeExp
         // same ClassExp.  The result is a call to a PrimProcedure instead.
         // This isn't just an optimization, since the re-write is
         // needed to ensure that if we're in an inner lambda that the
-        // $this$ declaration is captured in a closure.
+        // $this$ declaration is captured in a closure.  (See the
+        // 'new ThisExp(d)' below.)  Otherwise, we could could defer this
+        // optimization to ApplyExp.compile.  (Conversely, we can't do the
+        // latter optimization here instead, because we may not have called
+        // addMethodFor yet - since (except in the case of class methods)
+        // that happens later, after FindCapturedVars.  Yuck.)
+        boolean isStatic = nameDecl.isStatic();
+        if (! isStatic && outer instanceof ClassExp)
+          {
+            ClassExp cl = (ClassExp) outer;
+            if (cl.isMakingClassPair())
+              {
+              }
+          }
         PrimProcedure mproc = new PrimProcedure(method, this);
         Expression[] margs;
-        if (mproc.getStaticFlag())
+        if (isStatic)
           margs = exp.args;
         else
           {
@@ -1745,9 +1890,9 @@ public class LambdaExp extends ScopeExp
   {
     if (returnType == null)
       {
-	returnType = Type.pointer_type;  // To guards against cycles.
+	returnType = Type.objectType;  // To guards against cycles.
 	// body may not be set if define scan'd but not yet rewrit'ten.
-	if (body != null)
+	if (body != null && ! isAbstract())
 	  returnType = body.getType();
       }
     return returnType;
@@ -1758,6 +1903,20 @@ public class LambdaExp extends ScopeExp
   {
     this.returnType = returnType;
   }
+
+  public final void setCoercedReturnType (Type returnType)
+  {
+    this.returnType = returnType;
+    if (returnType != null
+        && returnType != Type.objectType
+        && returnType != Type.voidType
+        && body != QuoteExp.abstractExp)
+      {
+        Expression value = body;
+	body = Convert.makeCoercion(value, returnType);
+	body.setLine(value);
+      }
+  }
 }
 
 class Closure extends MethodProc
@@ -1827,6 +1986,7 @@ class Closure extends MethodProc
       : lambda.defaultArgs.length - key_args;
     int i = 0;
     int opt_i = 0;
+    int key_i = 0;
     int min_args = lambda.min_args;
     for (Declaration decl = lambda.firstDecl(); decl != null;
          decl = decl.nextDecl())
@@ -1839,7 +1999,8 @@ class Closure extends MethodProc
             if (i < nargs)
               value = args[i++];
             else
-              value = ((QuoteExp) lambda.defaultArgs[opt_i++]).getValue();
+              value = lambda.evalDefaultArg(opt_i, ctx);
+            opt_i++;
           }
 	else if (lambda.max_args < 0 && i == min_args + opt_args)
           {
@@ -1847,7 +2008,7 @@ class Closure extends MethodProc
               {
                 int rem = nargs - i;
                 Type elementType = ((ArrayType) decl.type).getComponentType();
-                if (elementType == Type.pointer_type)
+                if (elementType == Type.objectType)
                   {
                     Object[] rest = new Object[rem];
                     System.arraycopy(args, i, rest, 0, rem);
@@ -1876,8 +2037,15 @@ class Closure extends MethodProc
             else
               value = LList.makeList(args, i);
           }
-        else // Should never happen
-          return MethodProc.NO_MATCH_TOO_MANY_ARGS|max;
+        else
+          { // Keyword argument.
+            Keyword keyword = lambda.keywords[key_i++];
+            int key_offset = min_args + opt_args;
+            value = Keyword.searchForKeyword(args, key_offset, keyword);
+            if (value == Special.dfault)
+              value = lambda.evalDefaultArg(opt_i, ctx);
+            opt_i++;
+          }
         if (decl.type != null)
           {
             try
@@ -1922,6 +2090,24 @@ class Closure extends MethodProc
 
     try
       {
+        if (lambda.body == null)
+          {
+            // This can happen if a syntax-case macro calls a function
+            // in the same compilation unit.  FIXME.
+            StringBuffer sbuf = new StringBuffer("procedure ");
+            String name = lambda.getName();
+            if (name == null)
+              name = "<anonymous>";
+            sbuf.append(name);
+            int line = lambda.getLineNumber();
+            if (line > 0)
+              {
+                sbuf.append(" at line ");
+                sbuf.append(line);
+              }
+            sbuf.append(" was called before it was expanded");
+            throw new RuntimeException(sbuf.toString());
+          }
         lambda.body.apply(ctx);
       }
     finally
