@@ -66,7 +66,7 @@ public abstract class Language
 
   /** Add a language to the list.
    *
-   * @param langMapping is a language definition, the first index
+   * @param langMapping is a language definition, the first element
    *  is the language name, subsequent indexes are file types that
    *  might cause the language to be used and the final index is the
    *  name of the class that implements the language.
@@ -122,7 +122,7 @@ public abstract class Language
   public static Language detect (String line)
   {
     String str = line.trim();
-    // Does the line contain the string "kawa:LANGAUGE" for a valid LANGUAGE?
+    // Does the line contain the string "kawa:LANGUAGE" for a valid LANGUAGE?
     int k = str.indexOf("kawa:");
     if (k >= 0)
       {
@@ -357,66 +357,6 @@ public abstract class Language
     environ.define(getSymbol(name), property, proc);
   }
 
-  /** Import all the public fields of an object. */
-  private void defineAll(Object object)
-  {
-    Class clas = object.getClass();
-    java.lang.reflect.Field[] fields = clas.getFields();
-    for (int i = fields.length;  --i >= 0; )
-      {
-	java.lang.reflect.Field field = fields[i];
-	String name = field.getName();
-	if (name.startsWith(Declaration.PRIVATE_PREFIX)
-	    || name.endsWith("$instance"))
-	  continue;
-	if ((field.getModifiers() & java.lang.reflect.Modifier.FINAL) != 0)
-	  {
-	    try
-	      {
-		defineFromFieldValue(field, field.get(object));
-	      }
-	    catch (Throwable ex)
-	      {
-		throw new WrappedException("error accessing field "+field, ex);
-	      }
-	  }
-	else
-	  {
-	    System.err.println("INTERNAL ERROR in defineAll for "
-			       + name + " in " + clas);
-	  }
-      }
-  }
-
-  private void defineFromFieldValue(java.lang.reflect.Field fld, Object value)
-    throws Throwable
-  {
-    if (value instanceof Location)
-      {
-	Location loc = (Location) value;
-	Symbol sym = loc.getKeySymbol();
-	if (sym != null)
-	  {
-	    environ.addLocation(sym, loc.getKeyProperty(), loc);
-	    return;
-	  }
-      }
-    else
-      {
-	Object vname;
-	if (value instanceof Named)
-	  vname = ((Named) value).getSymbol();
-	else
-	  vname = null;
-	if (vname == null)
-	  vname = Compilation.demangleName(fld.getName(), true).intern();
-	Symbol symbol = vname instanceof Symbol ? (Symbol) vname
-	  : environ.getSymbol(vname.toString());
-	Object prop = getEnvPropertyFor(fld, value);
-	environ.define(symbol, prop, value);
-      }
-  }
-
   public Object getEnvPropertyFor (java.lang.reflect.Field fld, Object value)
   {
     if (! hasSeparateFunctionNamespace())
@@ -441,7 +381,7 @@ public abstract class Language
       {
 	Class clas = Class.forName(name);
 	Object inst = clas.newInstance ();
-	defineAll(inst);
+	ClassMemberLocation.defineAll(inst, this, Environment.getCurrent());
 	if (inst instanceof ModuleBody)
 	  ((ModuleBody)inst).run();
       }
@@ -489,21 +429,25 @@ public abstract class Language
 
   public abstract Lexer getLexer(InPort inp, SourceMessages messages);
 
-  public Compilation getCompilation (Lexer lexer, SourceMessages messages)
+  public Compilation getCompilation (Lexer lexer, SourceMessages messages, NameLookup lexical)
   {
-    return new Compilation(this, messages);
+    return new Compilation(this, messages, lexical);
   }
 
   /** Flag to tell parse that expression will be evaluated immediately.
    * I.e. we're not creating class files for future execution. */
   public static final int PARSE_IMMEDIATE = 1;
+  /** Flag to tell parse to use current NameLookup.
+   * As opposed to creating a new instance. */
+  public static final int PARSE_CURRENT_NAMES = 2;
   /** Flag to tell parse to only read a single line if possible.
    * Multiple lines may be read if syntactically required. */
-  public static final int PARSE_ONE_LINE = 2;
+  public static final int PARSE_ONE_LINE = 4;
   /** Flag to tell parser to continue until we have the module name.
    * The parser is allowed to continue further, but must stop before
    * any module import. */
-  public static final int PARSE_PROLOG = 4;
+  public static final int PARSE_PROLOG = 8;
+  public static final int PARSE_FOR_EVAL = PARSE_IMMEDIATE|PARSE_CURRENT_NAMES;
 
   public static boolean requirePedantic;
 
@@ -531,14 +475,26 @@ public abstract class Language
     return parse(getLexer(port, messages), Language.PARSE_PROLOG, info);
   }
 
+  public final Compilation parse(InPort port,
+                                 gnu.text.SourceMessages messages,
+                                 int options, ModuleInfo info)
+    throws java.io.IOException, gnu.text.SyntaxException
+  {
+    return parse(getLexer(port, messages), options, info);
+  }
+
   public final Compilation parse(Lexer lexer, int options, ModuleInfo info)
     throws java.io.IOException, gnu.text.SyntaxException
   {
     SourceMessages messages = lexer.getMessages();
-    Compilation tr = getCompilation(lexer, messages);
+    NameLookup lexical = ((options & PARSE_CURRENT_NAMES) != 0
+                          ? NameLookup.getInstance(getEnvironment(), this)
+                          : new NameLookup(this));
+    boolean immediate = (options & PARSE_IMMEDIATE) != 0;
+    Compilation tr = getCompilation(lexer, messages, lexical);
     if (requirePedantic)
       tr.pedantic = true;
-    tr.immediate = (options & PARSE_IMMEDIATE) != 0;
+    tr.immediate = immediate;;
     if ((options & PARSE_PROLOG) != 0)
       tr.setState(Compilation.PROLOG_PARSING);
     tr.pushNewModule(lexer);
@@ -641,7 +597,12 @@ public abstract class Language
   {
     if (exp instanceof QuoteExp)
       {
-        return getTypeFor(((QuoteExp) exp).getValue(), lenient);
+        Object value = ((QuoteExp) exp).getValue();
+        if (value instanceof Type)
+          return (Type) value;
+        if (value instanceof Class)
+          return Type.make((Class) value);
+        return getTypeFor(value, lenient);
       }
     else if (exp instanceof ReferenceExp)
       {
@@ -650,9 +611,15 @@ public abstract class Language
         String name = rexp.getName();
         if (decl != null)
 	  {
-	    name = decl.getName();
 	    exp = decl.getValue();
-	    if (decl.isAlias()
+            if (exp instanceof QuoteExp
+                && decl.getFlag(Declaration.IS_CONSTANT)
+                && ! decl.isIndirectBinding())
+              {
+		Object val = ((QuoteExp) exp).getValue();
+                return getTypeFor(val, lenient);
+              }
+	    else if (decl.isAlias()
 		&& exp instanceof QuoteExp)
 	      {
 		Object val = ((QuoteExp) exp).getValue();
@@ -660,7 +627,7 @@ public abstract class Language
 		  {
 		    Location loc = (Location) val;
 		    if (loc.isBound())
-		      return asType(loc.get());
+		      return getTypeFor(loc.get(), lenient);
 		    if (! (loc instanceof Named))
 		      return null;
 		    name = ((Named) loc).getName();
@@ -710,15 +677,30 @@ public abstract class Language
           }
 	fdname = Compilation.demangleName(fname, true).intern();
       }
-    Type dtype = isAlias ? Type.pointer_type
+    if (fdname instanceof String)
+      {
+        String uri = mod.getNamespaceUri();
+        String sname = (String) fdname;
+        Symbol sym;
+        if (uri == null)
+          fdname = SimpleSymbol.valueOf(sname);
+        else
+          fdname = Symbol.make(uri, sname);
+      }
+    Type dtype = isAlias ? Type.objectType
       : getTypeFor(ftype.getReflectClass());
     Declaration fdecl = mod.addDeclaration(fdname, dtype);
     boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
     boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
     if (isAlias)
       fdecl.setIndirectBinding(true);
-    else if (isFinal && ftype.isSubtype(Compilation.typeProcedure))
-      fdecl.setProcedureDecl(true);
+    else if (isFinal && ftype instanceof ClassType)
+      {
+        if (ftype.isSubtype(Compilation.typeProcedure))
+          fdecl.setProcedureDecl(true);
+        else if (((ClassType) ftype).isSubclass("gnu.mapping.Namespace"))
+          fdecl.setFlag(Declaration.IS_NAMESPACE_PREFIX);
+      }
     if (isStatic)
       fdecl.setFlag(Declaration.STATIC_SPECIFIED);
     fdecl.field = fld; 
@@ -794,7 +776,7 @@ public abstract class Language
   {
     Language.setDefaultLanguage(lang);
     current.setGlobal(lang);
-    // Assuming this is the initial (main) thread, make it's Environment
+    // Assuming this is the initial (main) thread, make its Environment
     // the default (global) one, so child threads can inherit from it.
     // Thus command-line definitions etc get inherited.
     if (Environment.getGlobal() == BuiltinEnvironment.getInstance())
@@ -895,7 +877,7 @@ public abstract class Language
     setDefaultLanguage(this);
     try
       {
-	Compilation comp = parse(port, messages, PARSE_IMMEDIATE);
+	Compilation comp = parse(port, messages, PARSE_FOR_EVAL);
 	ModuleExp.evalModule(getEnvironment(), ctx, comp, null, null);
       }
     finally
