@@ -3,6 +3,11 @@ import java.io.*;
 import gnu.bytecode.*;
 import java.lang.reflect.Array;
 import java.util.*;
+import gnu.mapping.Table2D;
+import gnu.mapping.Values;
+/* #ifdef use:java.util.regex */
+import java.util.regex.*;
+/* #endif */
 
 /** Manages the literals of a Compilation.
  * Implements ObjectOutput, because we use externalization to determine
@@ -19,7 +24,18 @@ public class LitTable implements ObjectOutput
   // Hashtable literalTable = new Hashtable(100);
   /* #endif */
 
-  static Hashtable staticTable =  new Hashtable (100);
+  /** A table mapping objects to public static final field literals.
+   * When we a need a literal for a value that is an instance of some
+   * class we automatically search the class for static fields.
+   * We use a {@code Table2D} primarily to make use of weak references,
+   * but we also use the 2nd argument:
+   * {@code staticTable(value, null, defaultValue)} yields a Literal
+   * if there is a public static final field for {@code value},
+   * and {@code defaultValue} otherwise.
+   * {@code staticTable(class, Boolean.TRUE, null) != null} if and only if
+   * we have scanned {@code class} (a {@code java.lang.Class} object).
+   */
+  static Table2D staticTable =  new Table2D (100);
 
   int literalsCount;
 
@@ -118,42 +134,42 @@ public class LitTable implements ObjectOutput
 
   public void writeBoolean(boolean v)
   {
-    push(new Boolean(v), Type.boolean_type);
+    push(new Boolean(v), Type.booleanType);
   }
 
   public void writeChar(int v)
   {
-    push(new Character((char) v), Type.char_type);
+    push(new Character((char) v), Type.charType);
   }
 
   public void writeByte(int v)
   {
-    push(new Byte((byte) v), Type.byte_type);
+    push(new Byte((byte) v), Type.byteType);
   }
 
   public void writeShort(int v)
   {
-    push(new Short((short) v), Type.short_type);
+    push(new Short((short) v), Type.shortType);
   }
 
   public void writeInt(int v)
   {
-    push(new Integer(v), Type.int_type);
+    push(new Integer(v), Type.intType);
   }
 
   public void writeLong(long v)
   {
-    push(new Long(v), Type.long_type);
+    push(new Long(v), Type.longType);
   }
 
   public void writeFloat(float v)
   {
-    push(new Float(v), Type.float_type);
+    push(new Float(v), Type.floatType);
   }
 
   public void writeDouble(double v)
   {
-    push(new Double(v), Type.double_type);
+    push(new Double(v), Type.doubleType);
   }
 
   public void writeUTF(String v) 
@@ -229,21 +245,29 @@ public class LitTable implements ObjectOutput
             /* #endif */
           }
 	else if (obj instanceof Integer)
-	  push(obj, Type.int_type);
+	  push(obj, Type.intType);
 	else if (obj instanceof Short)
-	  push(obj, Type.short_type);
+	  push(obj, Type.shortType);
 	else if (obj instanceof Byte)
-	  push(obj, Type.byte_type);
+	  push(obj, Type.byteType);
 	else if (obj instanceof Long)
-	  push(obj, Type.long_type);
+	  push(obj, Type.longType);
 	else if (obj instanceof Double)
-	  push(obj, Type.double_type);
+	  push(obj, Type.doubleType);
 	else if (obj instanceof Float)
-	  push(obj, Type.float_type);
+	  push(obj, Type.floatType);
 	else if (obj instanceof Character)
-	  push(obj, Type.char_type);
+	  push(obj, Type.charType);
         else if (obj instanceof Class)
           push(obj, Type.java_lang_Class_type);
+        /* #ifdef use:java.util.regex */
+        else if (obj instanceof Pattern)
+          {
+            Pattern pat = (Pattern) obj;
+            push(pat.pattern(), Type.string_type);
+            push(Integer.valueOf(pat.flags()), Type.intType);
+          }
+        /* #endif */
 	else
 	  error(obj.getClass().getName()+" does not implement Externalizable");
 	int nargs = stackPointer - oldStack;
@@ -279,7 +303,7 @@ public class LitTable implements ObjectOutput
 
     synchronized (staticTable)
       {
-	literal = (Literal) staticTable.get(value);
+	literal = (Literal) staticTable.get(value, null, null);
 	if ((literal == null || literal.value != value)
 	    && valueType instanceof ClassType)
 	  {
@@ -287,10 +311,10 @@ public class LitTable implements ObjectOutput
 	    int needed_mod = Access.STATIC | Access.FINAL | Access.PUBLIC;
 	    Class fldClass = valueClass;
 	    ClassType fldType = (ClassType) valueType;
-	    while (staticTable.get(fldClass) == null)
+	    while (staticTable.get(fldClass, Boolean.TRUE, null) == null)
 	      {
 		// This is a convention to note that we've scanned valueType.
-		staticTable.put(fldClass, fldClass);
+		staticTable.put(fldClass, Boolean.TRUE, fldClass);
 		for (Field fld = fldType.getFields();
 		     fld != null;  fld = fld.getNext())
 		  {
@@ -304,8 +328,7 @@ public class LitTable implements ObjectOutput
 				|| ! fldClass.isInstance(litValue))
 			      continue;
 			    Literal lit = new Literal (litValue, fld, this);
-			    staticTable.put(litValue, lit);
-			    staticTable.put(lit, litValue);
+			    staticTable.put(litValue, null, lit);
 			    if (value == litValue)
 			      literal = lit;
 			  }
@@ -402,7 +425,7 @@ public class LitTable implements ObjectOutput
 	    if (aType.isSubtype(pType))
 	      ; // OK so far
 	    else if (pType instanceof ArrayType && iparam < 64
-		     && (aType == Type.int_type || aType == Type.short_type))
+		     && (aType == Type.intType || aType == Type.shortType))
 	      {
 		int count = ((Number) literal.argValues[iarg]).intValue();
 		if (count < 0 && type.getName().equals("gnu.math.IntNum"))
@@ -554,7 +577,19 @@ public class LitTable implements ObjectOutput
       }
     else if (literal.value instanceof Class)
       {
-	comp.loadClassRef(((Class) literal.value).getName());
+        Class clas = (Class) literal.value;
+        if (clas.isPrimitive())
+          {
+            String cname = clas.getName();
+            if (cname.equals("int"))
+              cname = "integer";
+            cname = "java.lang."
+              +Character.toUpperCase(cname.charAt(0))
+              +cname.substring(1);
+            code.emitGetStatic(ClassType.make(cname).getDeclaredField("TYPE"));
+          }
+        else
+          comp.loadClassRef((ObjectType)Type.make(clas));
 	store(literal, ignore, code);
       }
     else if (literal.value instanceof ClassType
@@ -581,13 +616,23 @@ public class LitTable implements ObjectOutput
 	  {
 	    // look for matching "valueOf" or "make" method;
 	    method = getMethod(type, "valueOf", literal, true);
-            if (method == null)
-              method = getMethod(type, "make", literal, true);
+            if (method == null
+                // Values.make has return type Object, so use the constructor.
+                && ! (literal.value instanceof Values))
+              {
+                String mname = "make";
+                /* #ifdef use:java.util.regex */
+                if (literal.value instanceof Pattern)
+                  mname = "compile";
+                /* #endif */
+                method = getMethod(type, mname, literal, true);
+              }
 	    // otherwise look for matching constructor;
 	    if (method != null)
 	      makeStatic = true;
 	    else if (literal.argTypes.length > 0)
 	      method = getMethod(type, "<init>", literal, false);
+
 	    if (method == null)
 	      useDefaultInit = true;
 	  }
@@ -618,7 +663,8 @@ public class LitTable implements ObjectOutput
 	    code.emitInvokeSpecial(method);
 	  }
 	Method resolveMethod
-	  = makeStatic ? null : type.getDeclaredMethod("readResolve", 0);
+	  = makeStatic || literal.value instanceof Values ? null
+          : type.getDeclaredMethod("readResolve", 0);
 	if (resolveMethod != null)
 	  {
 	    code.emitInvokeVirtual(resolveMethod);
