@@ -23,12 +23,6 @@ public class ModuleExp extends LambdaExp
   public static final int LAZY_DECLARATIONS = STATIC_RUN_SPECIFIED << 1;
   public static final int IMMEDIATE = LAZY_DECLARATIONS << 1;
 
-  public String getJavaName ()
-  {
-    String name = getName();
-    return name == null ? "lambda" : Compilation.mangleName(name, 0);
-  }
-
   public ModuleExp ()
   {
   }
@@ -48,32 +42,23 @@ public class ModuleExp extends LambdaExp
   public static int interactiveCounter;
 
   public static Class evalToClass (Compilation comp, URL url)
+    throws SyntaxException
   {
     ModuleExp mexp = comp.getModule();
     SourceMessages messages = comp.getMessages();
     try
       {
-        ClassLoader parentLoader;
-        try
-          {
-            parentLoader = Thread.currentThread().getContextClassLoader();
-          }
-        catch (SecurityException ex)
-          {
-            parentLoader = comp.getClass().getClassLoader();
-          }
-
-	ArrayClassLoader loader = new ArrayClassLoader(parentLoader);
-        if (url == null)
-          url = Path.currentPath().toURL();
-        loader.setResourceContext(url);
-	comp.loader = loader;
 
         comp.minfo.loadByStages(Compilation.COMPILED);
 
 	if (messages.seenErrors())
 	  return null;
 
+	ArrayClassLoader loader = comp.loader;
+        if (url == null)
+          url = Path.currentPath().toURL();
+        loader.setResourceContext(url);
+
 	java.util.zip.ZipOutputStream zout = null;
 	if (dumpZipPrefix != null)
 	  {
@@ -95,8 +80,6 @@ public class ModuleExp extends LambdaExp
 	    String className = clas.getName ();
 	    byte[] classBytes = clas.writeToArray ();
 	    loader.addClass(className, classBytes);
-            // This reduces memory leaks if we do lots of evalToClass.
-            clas.cleanupAfterCompilation();
 
 	    if (zout != null)
 	      {
@@ -123,6 +106,12 @@ public class ModuleExp extends LambdaExp
 	*/
 
         Class clas = null;
+        // Use the "session" ClassLoader, for remembering classes
+        // created in one command (Compilation) through further command,
+        // while still allowing the classes to be replaced and collected.
+        ArrayClassLoader context = loader;
+        while (context.getParent() instanceof ArrayClassLoader)
+          context = (ArrayClassLoader) context.getParent();
 	for (int iClass = 0;  iClass < comp.numClasses;  iClass++)
 	  {
 	    ClassType ctype = comp.classes[iClass];
@@ -131,18 +120,26 @@ public class ModuleExp extends LambdaExp
             ctype.setExisting(true);
             if (iClass == 0)
               clas = cclass;
+            // Add all classes except the main module class to the "session"
+            // ClassLoader.  Don't add the main module class, as it's
+            // anonymous.  We might go further and skip other anonymous
+            // classes (theough defining which classes are anonymous is tricky).
+            else if (context != loader)
+              context.addClass(cclass);
           }
 
         ModuleInfo minfo = comp.minfo;
-        minfo.moduleClass = clas;
+        minfo.setModuleClass(clas);
+        comp.cleanupAfterCompilation();
         int ndeps = minfo.numDependencies;
 
         for (int idep = 0;  idep < ndeps;  idep++)
           {
             ModuleInfo dep = minfo.dependencies[idep];
-            if (dep.moduleClass == null)
-              dep.moduleClass = evalToClass(dep.comp, null);
-            comp.loader.addClass(dep.moduleClass);
+            Class dclass = dep.getModuleClassRaw();
+            if (dclass == null)
+              dclass = evalToClass(dep.comp, null);
+            comp.loader.addClass(dclass);
           }
 
         return clas;
@@ -157,18 +154,33 @@ public class ModuleExp extends LambdaExp
       }
     catch (Throwable ex)
       {
-	comp.error('f', "internal compile error - caught "+ex);
-	throw WrappedException.wrapIfNeeded(ex);
+	comp.getMessages()
+          .error('f', "internal compile error - caught "+ex, ex);
+        throw new SyntaxException(messages);
       }
   }
 
   /** Flag to force compilation, even when not required. */
-  public static boolean alwaysCompile = false;
+  public static boolean alwaysCompile = true;
 
   public final static boolean evalModule (Environment env, CallContext ctx,
                                        Compilation comp, URL url,
                                        OutPort msg)
     throws Throwable
+  {
+    ModuleExp mexp = comp.getModule();
+    Language language = comp.getLanguage();
+    Object inst = evalModule1(env, comp, url, msg);
+    if (inst == null)
+      return false;
+    evalModule2(env, ctx, language, mexp, inst, msg);
+    return true;
+  }
+
+  public final static Object evalModule1 (Environment env,
+                                          Compilation comp, URL url,
+                                          OutPort msg)
+    throws SyntaxException
   {
     comp.getLanguage().resolve(comp);
     ModuleExp mexp = comp.getModule();
@@ -184,36 +196,34 @@ public class ModuleExp extends LambdaExp
         if (comp != orig_comp)
           Compilation.setCurrent(comp);
 
-        if (alwaysCompile || comp.mustCompile)
+        if (alwaysCompile)
+          comp.mustCompile = true;
+
+        if (comp.mustCompile)
           comp.addMainClass(mexp);
 
         comp.walkModule(mexp);
         comp.setState(Compilation.WALKED);
 
         if (msg != null ? messages.checkErrors(msg, 20) : messages.seenErrors())
-          return false;
+          return null;
 
-	if (! alwaysCompile && ! comp.mustCompile)
-	  { // optimization - don't generate unneeded Class.
-	    if (Compilation.debugPrintFinalExpr)
+	if (! comp.mustCompile)
+	  {
+	    if (Compilation.debugPrintFinalExpr && msg != null)
 	      {
 		msg.println ("[Evaluating final module \""+mexp.getName()+"\":");
 		mexp.print(msg);
 		msg.println(']');
 		msg.flush();
 	      }
-	    mexp.body.apply(ctx);
-	  }
-	else
-	  {
-            if (comp.mainClass == null)
-              comp.addMainClass(mexp);
-
-	    try
-	      {
+            return Boolean.TRUE;
+          }
+        else
+          {
 		Class clas = evalToClass(comp, url);
 		if (clas == null)
-		  return false;
+		  return null;
                 try
                   {
                     thread = Thread.currentThread();
@@ -225,7 +235,47 @@ public class ModuleExp extends LambdaExp
                     thread = null;
                   }
 
-                Object inst;
+                mexp.body = null;
+                mexp.thisVariable = null;
+                if (msg != null ? messages.checkErrors(msg, 20)
+                    : messages.seenErrors())
+                  return null;
+                return clas;
+          }
+      }
+    finally
+      {
+	if (env != orig_env)
+	  Environment.setCurrent(orig_env);
+	if (comp != orig_comp)
+	  Compilation.setCurrent(orig_comp);
+        if (thread != null)
+          thread.setContextClassLoader(savedLoader);
+      }
+  }
+
+  public final static void evalModule2 (Environment env, CallContext ctx,
+                                        Language language, ModuleExp mexp,
+                                        Object inst, OutPort msg)
+    throws Throwable
+  {
+    Environment orig_env = Environment.getCurrent();
+    ClassLoader savedLoader = null;
+    Thread thread = null; // Non-null if we need to restore context ClassLoader.
+    try
+      {
+	if (env != orig_env)
+	  Environment.setCurrent(env);
+
+	if (inst == Boolean.TRUE)
+	  { // optimization - don't generate unneeded Class.
+	    mexp.body.apply(ctx);
+	  }
+	else
+	  {
+            if (inst instanceof Class)
+              {
+                Class clas = (Class) inst;
                 try
                   {
                     inst = clas.getDeclaredField("$instance").get(null);
@@ -240,6 +290,12 @@ public class ModuleExp extends LambdaExp
                     throw ex.getCause();
                   }
                 /* #endif */
+              }
+
+            //	    try
+	      {
+                if (inst instanceof ModuleBody)
+                  ((ModuleBody) inst).run(ctx);
 
 		// Import declarations defined in module into the Environment.
 		for (Declaration decl = mexp.firstDecl();
@@ -251,13 +307,13 @@ public class ModuleExp extends LambdaExp
 		    Field fld = decl.field;
 		    Symbol sym = dname instanceof Symbol ? (Symbol) dname
 		      : Symbol.make("", dname.toString().intern());
-		    Object property = comp.getLanguage()
-		      .getEnvPropertyFor(decl);
-		    // Would it be better to check if fld is FINAL?
-                    // If it is, gets its value; otherwise create
-                    // a FieldLocation to access it?  FIXME.
+		    Object property = language.getEnvPropertyFor(decl);
                     Expression dvalue = decl.getValue();
-		    if (decl.getFlag(Declaration.PROCEDURE|Declaration.IS_CONSTANT|Declaration.INDIRECT_BINDING)
+                    // It would be cleaner to not bind these values in
+                    // the environment, and just require lexical lookup.
+                    // However, various parts of the code makes use of
+                    // the environment.
+		    if ((decl.field.getModifiers() & Access.FINAL) != 0
                         && ! (dvalue instanceof ReferenceExp
                               && ((ReferenceExp) dvalue).getBinding().needsContext()))
 		      {
@@ -266,12 +322,17 @@ public class ModuleExp extends LambdaExp
 			    && dvalue != QuoteExp.undefined_exp)
 			  value = ((QuoteExp) dvalue).getValue();
 			else
-                          value = decl.field.getReflectField().get(null);
+			  {
+			    value = decl.field.getReflectField().get(null);
+                            if (! decl.isIndirectBinding())
+                              decl.setValue(QuoteExp.getInstance(value));
+                            else if (! decl.isAlias() || ! (dvalue instanceof ReferenceExp))
+                              decl.setValue(null);
+                          }
 			if (decl.isIndirectBinding())
-			  env.addLocation(sym, property, (Location) value);
-			else
-			  env.define(sym, property, value);
-			// if IS_CONSTANT ... make Location constant.  FIXME.
+                          env.addLocation(sym, property, (Location) value);
+                        else
+                          env.define(sym, property, value);
 		      }
 		    else
 		      {
@@ -280,18 +341,16 @@ public class ModuleExp extends LambdaExp
                                                     fld.getName());
 			loc.setDeclaration(decl);
 			env.addLocation(sym, property, loc);
+			decl.setValue(null);
 		      }
 		  }
-                if (msg != null ? messages.checkErrors(msg, 20)
-                    : messages.seenErrors())
-                  return false;
-                if (inst instanceof ModuleBody)
-                  ((ModuleBody) inst).run(ctx);
 	      }
+            /*
 	    catch (IllegalAccessException ex)
 	      {
 		throw new RuntimeException("class illegal access: in lambda eval");
 	      }
+            */
 	  }
 	ctx.runUntilDone();
       }
@@ -299,12 +358,9 @@ public class ModuleExp extends LambdaExp
       {
 	if (env != orig_env)
 	  Environment.setCurrent(orig_env);
-	if (comp != orig_comp)
-	  Compilation.setCurrent(orig_comp);
         if (thread != null)
           thread.setContextClassLoader(savedLoader);
       }
-    return true;
   }
 
   ClassType superType;
@@ -312,6 +368,8 @@ public class ModuleExp extends LambdaExp
 
   ModuleInfo info;
 
+  public String getNamespaceUri () { return info.uri; }
+
   public final ClassType getSuperType() { return superType; }
   public final void setSuperType(ClassType s) { superType = s; }
   public final ClassType[] getInterfaces() { return interfaces; }
@@ -378,9 +436,9 @@ public class ModuleExp extends LambdaExp
 	  continue;
 	if (decl.getFlag(Declaration.IS_UNKNOWN))
 	  continue;
-        if (value instanceof ModuleExp) // if decl set by a module-name command.
-          continue;
-	if (value instanceof LambdaExp && ! (value instanceof ClassExp))
+	if (value instanceof LambdaExp
+            && ! (value instanceof ModuleExp // from a module-name command.
+                  || value instanceof ClassExp))
 	  {
 	    ((LambdaExp) value).allocFieldFor(comp);
 	  }
