@@ -1,4 +1,4 @@
-// Copyright (c) 2006 Per M.A. Bothner.
+// Copyright (c) 2006, 2008 Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -6,20 +6,20 @@ import gnu.mapping.*;
 import gnu.bytecode.*;
 import gnu.kawa.reflect.FieldLocation;
 import gnu.text.*;
+import java.util.*;
+import gnu.kawa.util.AbstractWeakHashTable;
 
 public class ModuleInfo
 {
-  /** Next element in list head by {@link ModuleManager#modules}. */
-  public ModuleInfo nextModule () { return next; }
-  ModuleInfo next;
-
   /** Name of class that implements module.
    * Must be non-null unless we're currently compiling the module,
    * in which case sourcePath and comp must both be non-null.
    */
   public String className;
 
-  public Class moduleClass;
+  Class moduleClass;
+
+  static ClassToInfoMap mapClassToInfo = new ClassToInfoMap();
 
   /** The namespace URI associated with this module, or {@code null}.
    * This is null for Scheme modules, but non-null for XQuery modules.
@@ -31,7 +31,7 @@ public class ModuleInfo
   ModuleExp exp;
   Compilation comp;
 
-  public Compilation getCompilation() { return comp; }
+  public Compilation getCompilation () { return comp; }
 
   public void setCompilation (Compilation comp)
   {
@@ -66,6 +66,9 @@ public class ModuleInfo
   String sourceAbsPathname;
 
   public long lastCheckedTime;
+  /** Last time the source file was modified.
+   * At least the last time we checked ...
+   */
   public long lastModifiedTime;
 
   public Path getSourceAbsPath ()
@@ -191,43 +194,46 @@ public class ModuleInfo
     Class mclass = moduleClass;
     if (mclass != null)
       return mclass;
-    mclass = Class.forName(className);
+    mclass = ClassType.getContextClass(className);
     moduleClass = mclass;
     return mclass;
   }
 
+  public Class getModuleClassRaw ()
+  {
+    return moduleClass;
+  }
+
+  public void setModuleClass (Class clas)
+  {
+    moduleClass = clas;
+    className = clas.getName();
+    mapClassToInfo.put(clas, this);
+  }
+
   public static ModuleInfo findFromInstance (Object instance)
   {
     return ModuleContext.getContext().findFromInstance(instance);
   }
 
-  public static ModuleInfo find (String className)
+  public static ModuleInfo find (ClassType type)
   {
-    return ModuleManager.getInstance().findWithClassName(className);
-  }
-
-  public static ModuleInfo find (Type type)
-  {
-    ModuleInfo info = ModuleManager.getInstance().findWithClassName(type.getName());
-    if (type instanceof ObjectType && ((ObjectType) type).isExisting())
+    if (type.isExisting())
       {
         try
           {
-            info.moduleClass = type.getReflectClass();
+            return ModuleManager.findWithClass(type.getReflectClass());
           }
         catch (Exception ex)
           {
           }
       }
-    return info;
+    return ModuleManager.getInstance().findWithClassName(type.getName());
   }
 
   public static void register (Object instance)
   {
-    Class clas = instance.getClass();
-    ModuleInfo info = find(clas.getName());
-    info.moduleClass = clas;
-    ModuleContext.getContext().setInstance(info, instance);
+    ModuleContext.getContext().setInstance(instance);
   }
 
   public Object getInstance ()
@@ -347,10 +353,14 @@ public class ModuleInfo
    */
   public boolean checkCurrent (ModuleManager manager, long now)
   {
-    if (lastCheckedTime + manager.lastModifiedCacheTime >= now)
+    if (sourceAbsPath == null)
       return true;
+    if (lastCheckedTime + manager.lastModifiedCacheTime >= now)
+      return moduleClass != null || comp != null;
     lastCheckedTime = now;
     long lastModifiedTime = sourceAbsPath.getLastModified();
+    long oldModifiedTime = this.lastModifiedTime;
+    this.lastModifiedTime = lastModifiedTime;
     if (moduleClass == null && className != null)
       {
         try
@@ -359,11 +369,10 @@ public class ModuleInfo
           }
         catch (ClassNotFoundException ex)
           {
-            this.lastModifiedTime = lastModifiedTime;
             return false;
           }
       }
-    if (this.lastModifiedTime == 0 && moduleClass != null)
+    if (oldModifiedTime == 0 && moduleClass != null)
       {
         String classFilename = className;
         int dot = classFilename.lastIndexOf('.');
@@ -375,7 +384,7 @@ public class ModuleInfo
           {
             try
               {
-                this.lastModifiedTime = resource.openConnection().getLastModified();
+                oldModifiedTime = resource.openConnection().getLastModified();
               }
             catch (java.io.IOException ex)
               {
@@ -386,21 +395,22 @@ public class ModuleInfo
           {
             // Couldn't open timestand of the .class file.
             // Assume it is current.
-            this.lastModifiedTime = lastModifiedTime;
             return true;
           }
       }
-    if (className == null || lastModifiedTime > this.lastModifiedTime)
+    if (className == null || lastModifiedTime > oldModifiedTime)
       {
         moduleClass = null;
-        this.lastModifiedTime = lastModifiedTime;
         return false;
       }
     for (int i = numDependencies;  --i >= 0; )
       {
         ModuleInfo dep = dependencies[i];
         if (! dep.checkCurrent(manager, now))
-          return false;
+          {
+            moduleClass = null;
+            return false;
+          }
       }
     return true;
   }
@@ -423,3 +433,21 @@ public class ModuleInfo
     return sbuf.toString();
   }
 }
+
+class ClassToInfoMap extends AbstractWeakHashTable<Class,ModuleInfo>
+{
+  public ClassToInfoMap ()
+  {
+    super(64);
+  }
+
+  protected Class getKeyFromValue (ModuleInfo minfo)
+  {
+    return minfo.moduleClass;
+  }
+
+  protected boolean matches (ModuleInfo oldValue, ModuleInfo newValue)
+  {
+    return oldValue.moduleClass == newValue.moduleClass;
+  }
+}
