@@ -15,7 +15,7 @@ public class ObjectExp extends ClassExp
 
   public void compile (Compilation comp, Target target)
   {
-    compile (comp);
+    compileMembers(comp);
     CodeAttr code = comp.getCode();
     code.emitNew(type);
     code.emitDup(1);
