@@ -1,9 +1,10 @@
-// Copyright (c) 1999, 2000  Per M.A. Bothner.
+// Copyright (c) 1999, 2000, 2008  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
 import gnu.bytecode.*;
 import gnu.mapping.*;
+import gnu.kawa.lispexpr.LangObjType;
 
 /** A primitive Procedure implemented by a plain Java method. */
 
@@ -43,6 +44,8 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
   {
     if (method != null)
       {
+        if ((method.getModifiers() & Access.VARARGS) != 0)
+          return true;
 	String name = method.getName();
 	return name.endsWith("$V") || name.endsWith("$V$X");
       }
@@ -59,6 +62,24 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     return method.getName().endsWith("$X");
   }
 
+  public int isApplicable(Type[] argTypes)
+  {
+    int app = super.isApplicable(argTypes);
+    int nargs = argTypes.length;
+    if (app == -1 && method != null
+        && (method.getModifiers() & Access.VARARGS) != 0
+        && nargs > 0 && argTypes[nargs-1] instanceof ArrayType)
+      {
+        // For a Java5-style VARARGS method, you're also allowed to
+        // explicitly pass in an array as the last argument.
+        Type[] tmp = new Type[nargs];
+        System.arraycopy(argTypes, 0, tmp, 0, nargs-1);
+        tmp[nargs-1] = ((ArrayType) argTypes[nargs-1]).getComponentType();
+        return super.isApplicable(tmp);
+      }
+    return app;
+  }
+
   public final boolean isConstructor()
   {
     // invokespecial == primitive-constructor
@@ -144,11 +165,11 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     if (takesVarArgs)
       {
 	Type restType = argTypes[paramCount-1];
-	if (restType == Compilation.scmListType)
+        if (restType == Compilation.scmListType || restType == LangObjType.listType)
 	  { // FIXME
 	    rargs[paramCount-1] = gnu.lists.LList.makeList(args, fixArgs);
 	    nargs = fixArgs;
-            elementType = Type.pointer_type;
+            elementType = Type.objectType;
 	  }
 	else
 	  {
@@ -179,7 +200,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
       {
         Object arg = args[i];
         Type type = i < fixArgs ? argTypes[i-extraCount] : elementType;
-        if (type != Type.pointer_type)
+        if (type != Type.objectType)
           {
             try
               {
@@ -273,7 +294,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
   public PrimProcedure(Method method)
   {
     init(method);
-    this.retType = method.getName().endsWith("$X") ? Type.pointer_type
+    this.retType = method.getName().endsWith("$X") ? Type.objectType
       : method.getReturnType();
   }
 
@@ -315,16 +336,16 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     if (isConstructor())
       retType = method.getDeclaringClass();
     else if (method.getName().endsWith("$X"))
-      retType = Type.pointer_type;
+      retType = Type.objectType;
     else
       {
         retType = language.getLangTypeFor(method.getReturnType());
 
-        // Kludge - tostring_type doesn't have methods.
+        // Kludge - toStringType doesn't have methods.
         // It shouldn't be used as the "type" of anything -
         // it's just a type with a coercion.  FIXME.
-        if (retType == Type.tostring_type)
-          retType = Type.string_type;
+        if (retType == Type.toStringType)
+          retType = Type.javalangStringType;
       }
   }
   
@@ -415,9 +436,11 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
 
   /** Compile arguments and push unto stack.
    * @param args arguments to evaluate and push.
+   * @param normally 0, but 1 in the case of a constructor,
+   *   or the case of "static" method of a non-static class.
    * @param thisType If we are calling a non-static function,
    *   then args[0] is the receiver and thisType is its expected class.
-   *   If thisType==Type.void_type, ignore argTypes[0].  (It is used to to
+   *   If thisType==Type.voidType, ignore argTypes[0].  (It is used to to
    *   pass a link to a closure environment, which was pushed by our caller.)
    *   If thisType==null, no special handling of args[0] or argTypes[0].
    */
@@ -427,12 +450,35 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     String name = getName();
     Type arg_type = null;
     gnu.bytecode.CodeAttr code = comp.getCode();
-    int skipArg = thisType == Type.void_type ? 1 : 0;
+    int skipArg = thisType == Type.voidType ? 1 : 0;
     int arg_count = argTypes.length - skipArg;
     if (takesContext())
       arg_count--;
+    int nargs = args.length - startArg;
     boolean is_static = thisType == null || skipArg != 0;
-    int fix_arg_count = variable ? arg_count - 1 : args.length - startArg;
+
+    // If Java5-style VARARS we allow both a variable-length argument list,
+    // or if the last argument already is an array we can use it as is.
+    // The tricky part is we sometimes have to distinguish these cases
+    // at run-time, in which case we set createVarargsArrayIfNeeded.
+    boolean createVarargsArrayIfNeeded = false;
+    if (variable && (method.getModifiers() & Access.VARARGS) != 0
+        && nargs > 0 && argTypes.length > 0
+        && nargs == arg_count + (is_static ? 0 : 1))
+      {
+        Type lastType = args[args.length-1].getType();
+        Type lastParam = argTypes[argTypes.length-1];
+        if (lastType instanceof ObjectType
+            && lastParam instanceof ArrayType // should always be true
+            && ! (((ArrayType) lastParam).getComponentType()
+                  instanceof ArrayType))
+          {
+            if (! (lastType instanceof ArrayType))
+              createVarargsArrayIfNeeded = true;
+            variable = false;
+          }
+      }
+    int fix_arg_count = variable ? arg_count - (is_static ? 1 : 0) : args.length - startArg;
     Declaration argDecl = source == null ? null : source.firstDecl();
     if (argDecl != null && argDecl.isThisParameter())
       argDecl = argDecl.nextDecl();
@@ -441,7 +487,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
         if (variable && i == fix_arg_count)
           {
             arg_type = argTypes[arg_count-1+skipArg];
-	    if (arg_type == Compilation.scmListType)
+	    if (arg_type == Compilation.scmListType || arg_type == LangObjType.listType)
 	      {
 		gnu.kawa.functions.MakeList.compile(args, startArg+i, comp);
 		break;
@@ -450,8 +496,9 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
             arg_type = ((ArrayType) arg_type).getComponentType();
             code.emitNewArray(arg_type);
           }
-        if (i + startArg >= args.length)
+        if (i >= nargs)
           break;
+        boolean createVarargsNow = createVarargsArrayIfNeeded && i + 1 == nargs;
         if (i >= fix_arg_count)
           {
             code.emitDup(1); // dup array.
@@ -463,10 +510,31 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
             : i==0 ? thisType
             : argTypes[i-1];
 	comp.usedClass(arg_type);
+        Type argTypeForTarget = createVarargsNow ? Type.objectType : arg_type;
 	Target target =
-	  source == null ? CheckedTarget.getInstance(arg_type, name, i+1)
-	  : CheckedTarget.getInstance(arg_type, source, i);
+	  source == null ? CheckedTarget.getInstance(argTypeForTarget, name, i+1)
+	  : CheckedTarget.getInstance(argTypeForTarget, source, i);
 	args[startArg+i].compileNotePosition(comp, target, args[startArg+i]);
+        if (createVarargsNow)
+          {
+            // Wrap final argument in array if not already an array:
+            // Emit: (arg instanceof T[] ? (T[]) arg : new T[]{arg})
+            Type eltype = ((ArrayType) arg_type).getComponentType();
+            code.emitDup();
+            code.emitInstanceof(arg_type);
+            code.emitIfIntNotZero();
+            code.emitCheckcast(arg_type);
+            code.emitElse();
+            code.emitPushInt(1);
+            code.emitNewArray(eltype); // Stack: value array
+            code.emitDupX(); // Stack: array value array
+            code.emitSwap(); // Stack: array array value
+            code.emitPushInt(0); // Stack array array value 0
+            code.emitSwap(); // Stack: array array 0 value
+            eltype.emitCoerceFromObject(code);
+            code.emitArrayStore(arg_type); // Stack: array
+            code.emitFi();
+          }
         if (i >= fix_arg_count)
           code.emitArrayStore(arg_type);
 	if (argDecl != null && (is_static || i > 0))
@@ -502,14 +570,9 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
         ClassType mclass = method == null ? null :  method.getDeclaringClass();
         if (mclass.hasOuterLink())
           {
-            // This can be optimized in most cases. FIXME.
-            args[0].compile(comp, Target.pushValue(Compilation.typeClassType));
-            code.emitInvokeStatic(ClassType.make("gnu.expr.PairClassType").getDeclaredMethod("extractStaticLink", 1));
-            code.emitCheckcast(mclass.getOuterLinkType());
-            thisType = Type.void_type;
+            ClassExp.loadSuperStaticLink(args[0], mclass, comp);
           }
-        else
-          thisType = null;
+        thisType = null;
         startArg = 1;
       }
     else if (takesTarget() && method.getStaticFlag())
@@ -588,9 +651,9 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     else
       {
         comp.loadCallContext();
-        stackType = Type.pointer_type;
+        stackType = Type.objectType;
         code.pushScope();
-        Variable saveIndex = code.addLocal(Type.int_type);
+        Variable saveIndex = code.addLocal(Type.intType);
         comp.loadCallContext();
         code.emitInvokeVirtual(Compilation.typeCallContext.
                                getDeclaredMethod("startFromContext", 0));
@@ -617,7 +680,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     if (takesTarget())
       {
         if (index == 0)
-          return isConstructor() ? Type.pointer_type
+          return isConstructor() ? Type.objectType
             : method.getDeclaringClass();
         index--;
       }
@@ -632,7 +695,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     if (restType instanceof ArrayType)
       return ((ArrayType) restType).getComponentType();
     else // Should be LList or some other Sequence class.
-      return Type.pointer_type;
+      return Type.objectType;
   }
 
   // This is null in JDK 1.1 and something else in JDK 1.2.
