@@ -1,15 +1,17 @@
 package gnu.expr;
 import gnu.bytecode.*;
+import gnu.mapping.PropertySet;
 
 public class ProcInitializer extends Initializer
 {
   LambdaExp proc;
 
-  public ProcInitializer(LambdaExp lexp, Compilation comp)
+  public ProcInitializer(LambdaExp lexp, Compilation comp, Field field)
   {
-    field = lexp.allocFieldFor(comp);
+    this.field = field;
     proc = lexp;
-    LambdaExp heapLambda = lexp.getOwningLambda();
+    LambdaExp heapLambda = field.getStaticFlag() ? comp.getModule()
+      : lexp.getOwningLambda() ;
     if (heapLambda instanceof ModuleExp && comp.isStatic())
       {
 	next = comp.clinitChain;
@@ -29,7 +31,8 @@ public class ProcInitializer extends Initializer
     ClassType procClass = Compilation.typeModuleMethod;
     code.emitNew(procClass);
     code.emitDup(1);
-    LambdaExp owning = proc.getOwningLambda();
+    LambdaExp owning = proc.getNeedsClosureEnv() ? proc.getOwningLambda()
+      : comp.getModule();
     if (owning instanceof ClassExp && owning.staticLinkField != null)
       code.emitLoad(code.getCurrentScope().getVariable(1));
     else if (! (owning instanceof ModuleExp)
@@ -80,7 +83,7 @@ public class ProcInitializer extends Initializer
 	  {
 	    Object key = proc.properties[i];
 	    // Skip "name" property since we've taken care of that specially.
-	    if (key != null && key != "name")
+	    if (key != null && key != PropertySet.nameKey)
 	      {
 		Object val = proc.properties[i+1];
 		code.emitDup(1);
