@@ -21,7 +21,7 @@ public class QuoteExp extends Expression
     if (type != null)
       return type;
     if (value == Values.empty)
-      return Type.void_type;
+      return Type.voidType;
     else if (value == null)
       return Type.nullType;
     else if (this == undefined_exp)
@@ -32,21 +32,28 @@ public class QuoteExp extends Expression
 
   static public QuoteExp undefined_exp =
     new QuoteExp (Special.undefined);
+  static public QuoteExp abstractExp =
+    new QuoteExp(Special.abstractSpecial);
   static public QuoteExp voidExp =
-    new QuoteExp (Values.empty, Type.void_type);
+    new QuoteExp (Values.empty, Type.voidType);
   static public QuoteExp trueExp = new QuoteExp(Boolean.TRUE);
   static public QuoteExp falseExp = new QuoteExp(Boolean.FALSE);
   static public QuoteExp nullExp =
     new QuoteExp(null, Type.nullType);
+  public static final QuoteExp classObjectExp =
+    new QuoteExp(Type.pointer_type);
 
   public static QuoteExp getInstance (Object value)
   {
     if (value == null)
       return nullExp;
+    if (value == Type.pointer_type)
+      return classObjectExp;
     if (value == Special.undefined)
       return undefined_exp;
     if (value == Values.empty)
       return voidExp;
+    // Note we deliberately don't map abstractSpecial to abstractExp.
     if (value instanceof Boolean)
       return ((Boolean) value).booleanValue() ? trueExp : falseExp;
     return new QuoteExp(value);
@@ -77,12 +84,18 @@ public class QuoteExp extends Expression
       }
   }
  
+  public Expression deepCopy (gnu.kawa.util.IdentityHashTable mapper)
+  {
+    return this;
+  }
+
   protected Expression walk (ExpWalker walker)
   {
     return walker.walkQuoteExp(this);
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
   {
     if (this == QuoteExp.undefined_exp)
       return exp;
@@ -97,7 +110,9 @@ public class QuoteExp extends Expression
     if (msg != null)
       return walker.noteError(msg);
     if (proc instanceof CanInline)
-      return ((CanInline) proc).inline(exp, walker);
+      return ((CanInline) proc).inline(exp, walker, argsInlined);
+    if (! argsInlined)
+      exp.args = walker.walkExps(exp.args, exp.args.length);
     if (exp.getFlag(ApplyExp.INLINE_IF_CONSTANT))
       {
 	Expression e = exp.inlineIfConstant(proc, walker);
@@ -147,13 +162,14 @@ public class QuoteExp extends Expression
   {
     out.startLogicalBlock("(Quote", ")", 2);
     out.writeSpaceLinear();
-    if (value instanceof Expression)
-      value = value.toString(); // To avoid cycles.
+    Object val = this.value;
+    if (val instanceof Expression)
+      val = val.toString(); // To avoid cycles.
     gnu.lists.AbstractFormat saveFormat = out.objectFormat;
     try
       {
 	out.objectFormat = Language.getDefaultLanguage().getFormat(true);
-	out.print(value);
+	out.print(val);
         /*
         if (value != null)
           {
