@@ -15,11 +15,28 @@
 
 (define (purecopy obj) obj)
 
-(define (call-interactively command)
-  (invoke-static <gnu.jemacs.buffer.Command> 'perform command))
-
 (define (minibuffer-depth) 0)  ;; FIXME
 
+;; The 'if' primitive in version takes an arbitary number of 'else'
+;; expressions, in contrast to the Scheme and CommonLisp definitions.
+
+(%define-syntax if
+  (lambda (x)
+    (syntax-case x ()
+		 ((_ test then)
+		  (make <gnu.expr.IfExp>
+		    (syntax->expression (syntax test))
+		    (syntax->expression (syntax then))
+		    (static-field <gnu.commonlisp.lang.Lisp2> 'nilExpr)))
+		 ((_ test then else ...)
+		  (make <gnu.expr.IfExp>
+		    (syntax->expression (syntax test))
+		    (syntax->expression (syntax then))
+		    (syntax-body->expression (syntax (begin else ...)))))
+		 ((_ . rest)
+		  (syntax-error (syntax rest)
+				"too few expressions for 'if'")))))
+
 (define-syntax catch
   (syntax-rules ()
 		((catch tag body ...)
@@ -42,17 +59,7 @@
 
 (define (quit-char) #\bel)
 
-(define (make-local-variable symbol)
-  (invoke-static <gnu.jemacs.buffer.Buffer> 'makeBufferLocal symbol #f)
-  symbol)
-(define (make-variable-buffer-local symbol)
-  (invoke-static <gnu.jemacs.buffer.Buffer> 'makeBufferLocal symbol #t)
-  symbol)
-
 (define (emacs:read #!optional (port (current-input-port)))
-  (let ((lexer
-	 ((primitive-constructor <gnu.jemacs.lang.ELispReader> (<input-port>))
-	  port)))
-    ((primitive-virtual-method <gnu.jemacs.lang.ELispReader> "readObject"
-			       <object> ())
-     lexer)))
+  ((<gnu.kawa.lispexpr.LispReader> port):readObject))
+
+
