@@ -14,13 +14,13 @@ import gnu.kawa.lispexpr.LangPrimType;
  * @author Per Bothner
  */
 
-public class AddOp extends ProcedureN implements CanInline, Inlineable
+public class AddOp extends ArithOp
 {
   int plusOrMinus = 1;
 
   public AddOp(String name, int plusOrMinus)
   {
-    setName(name);
+    super(name);
     this.plusOrMinus = plusOrMinus;
   }
 
@@ -91,7 +91,31 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
 
   public static Object $Mn(Object arg1)
   {
-    return ((Numeric) arg1).neg();
+    int code = Arithmetic.classifyValue(arg1);
+    switch (code)
+      {
+      case Arithmetic.INT_CODE:
+	return new Integer(- Arithmetic.asInt(arg1));
+      case Arithmetic.LONG_CODE:
+	return new Long(- Arithmetic.asLong(arg1));
+      case Arithmetic.BIGINTEGER_CODE:
+	return Arithmetic.asBigInteger(arg1).negate();
+      case Arithmetic.INTNUM_CODE:
+	return IntNum.neg(Arithmetic.asIntNum(arg1));
+      case Arithmetic.BIGDECIMAL_CODE:
+	return Arithmetic.asBigDecimal(arg1).negate();
+      case Arithmetic.RATNUM_CODE:
+	return RatNum.neg(Arithmetic.asRatNum(arg1));
+      case Arithmetic.FLOAT_CODE:
+	return new Float(- Arithmetic.asFloat(arg1));
+      case Arithmetic.DOUBLE_CODE:
+	return new Double(- Arithmetic.asDouble(arg1));
+      case Arithmetic.FLONUM_CODE:
+	return new DFloNum(- Arithmetic.asDouble(arg1));
+      default:
+        return Arithmetic.asNumeric(arg1).neg();
+      }
+
   }
 
   public static Object $Pl$V (Object arg1, Object arg2,
@@ -137,7 +161,7 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
 
   public static Expression pairwise(Procedure proc,
                                     Expression rproc, Expression[] args,
-				    ExpWalker walker)
+				    InlineCalls walker)
   {
     int len = args.length;
     Expression prev = args[0];
@@ -148,15 +172,17 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
         args2[1] = args[i];
         ApplyExp next = new ApplyExp(rproc, args2);
         if (proc instanceof CanInline)
-          prev = ((CanInline) proc).inline(next, walker);
+          prev = ((CanInline) proc).inline(next, walker, true);
         else
           prev = next;
       }
     return prev;
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     // Inlining may yield PrimProcedure instructions of bytecode instructions
     // which we don't know how to interpret (yet).
     if (! walker.getCompilation().mustCompile)
@@ -209,12 +235,16 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
       }
     if (args.length == 2)
       {
-	return primInline(plusOrMinus > 0 ? 96 /* iadd */ : 100 /* isub */,
-			  exp);
+	return primInline(primitiveOpcode(), exp);
       }
     return exp;
   }
 
+  public int primitiveOpcode ()
+  {
+    return plusOrMinus > 0 ? 96 /* iadd */ : 100 /* isub */;
+  }
+
   public static Expression primInline (int opcode, ApplyExp exp)
   {
     Expression[] args = exp.getArgs();
@@ -262,163 +292,4 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
     return exp;
   }
 
-  /*
-  static ClassType typeInteger = ClassType.make("java.lang.Integer");
-  static ClassType typeLong = ClassType.make("java.lang.Long");
-  */
-  static ClassType typeIntNum = ClassType.make("gnu.math.IntNum");
-  static ClassType typeDFloNum = ClassType.make("gnu.math.DFloNum");
-  static ClassType typeRealNum = ClassType.make("gnu.math.RealNum");
-  static ClassType typeNumeric = ClassType.make("gnu.math.Numeric");
-
-  public void compile (ApplyExp exp, Compilation comp, Target target)
-  {
-    Expression[] args = exp.getArgs();
-    int len = args.length;
-    if (len == 0)
-      {
-	comp.compileConstant(IntNum.zero(), target);
-	return;
-      }
-    Type type = getReturnType(args);
-    Type ttype = target.getType();
-    if (len == 1 || target instanceof IgnoreTarget)
-      {
-	// FIXME implement optimization for unary
-	ApplyExp.compile(exp, comp, target);
-	return;
-      }
-    PrimType ptype = null;
-    if (ttype instanceof PrimType)
-      {
-	char sig = type.getSignature().charAt(0);
-	if (sig == 'V' || sig == 'Z' || sig == 'C')
-	 ptype = null; // error
-	else if (sig == 'D' || sig == 'F')
-	  {
-	    if (type.isSubtype(typeRealNum))
-	      ptype = LangPrimType.doubleType;
-	  }
-	else
-	  {
-	    if (type.isSubtype(typeIntNum))
-	      ptype = sig == 'J' ? LangPrimType.longType : LangPrimType.intType;
-	  }
-      }
-    if (ptype != null)
-      {
-	CodeAttr code = comp.getCode();
-	// FIXME would be nice to use iinc when appropriate!
-	// We would need to use a special LocalVariableTarget,
-	// created by SetExp when dest is a local variable.
-	// Then if len==2 && ptype==LangPrimType.intType
-	// && target instanceof LocalVariableTarget
-	// && one arg is QuoteExp && other arg is same local as target
-	// => then emit iinc.
-	args[0].compile(comp, ttype);
-	for (int i = 1;  i < len;  i++)
-	  {
-	    args[i].compile(comp, ptype);
-	    if (plusOrMinus > 0)
-	      code.emitAdd(ptype);
-	    else
-	      code.emitSub(ptype);
-	  }
-	target.compileFromStack(comp, ttype);
-      }
-    else if (type.isSubtype(typeDFloNum))
-      {
-	PrimType dtype = LangPrimType.doubleType;
-	Target dtarget = new StackTarget(dtype);
-	CodeAttr code = comp.getCode();
-	args[0].compile(comp, dtarget);
-	for (int i = 1;  i < len;  i++)
-	  {
-	    args[i].compile(comp, dtarget);
-	    if (plusOrMinus > 0)
-	      code.emitAdd(dtype);
-	    else
-	      code.emitSub(dtype);
-	  }
-	target.compileFromStack(comp, dtype);
-      }
-    else
-      ApplyExp.compile(exp, comp, target);
-  }
-
-  /** Classify an expression according to its numeric type.
-   * kind==0:  not a number.
-   * kind==1:  a non-real number
-   * kind==2:  real number
-   * kind==3:  floating-point
-   * kind==4:  exact integer
-   */
-  public static int classify (Type type)
-  {
-    int kind = 0;
-    if (type instanceof PrimType)
-      {
-	char sig = type.getSignature().charAt(0);
-	if (sig == 'V' || sig == 'Z' || sig == 'C')
-	  return 0;
-	else if (sig == 'D' || sig == 'F')
-	  return 3;
-	else
-	  return 4;
-      }
-    else if (type.isSubtype(typeIntNum))
-      return 4;
-    else if (type.isSubtype(typeDFloNum))
-      return 3;
-    else if (type.isSubtype(typeRealNum))
-      return 2;
-    else if (type.isSubtype(typeNumeric))
-      return 1;
-    else
-      return 0;
-  }
-
-  public Type getReturnType (Expression[] args)
-  {
-    int len = args.length;
-    if (len == 0)
-      return typeIntNum;
-    Type type = Type.pointer_type;
-    int kind0 = 0;
-    for (int i = 0;  i < len;  i++)
-      {
-	Expression arg = args[i];
-	int kind = classify(arg.getType());
-
-	if (kind == 0)
-	  return Type.pointer_type;
-
-	if (i == 0)
-	  kind0 = kind;
-
-	if (kind0 == 4 && kind == 4)
-	  type = typeIntNum;
-	else if (kind0 >= 2 && kind >= 2)
-	  {
-	    if (kind0 == 3 || kind == 3)
-	      {
-		type = typeDFloNum;
-		kind0 = 3;
-	      }
-	    else
-	      {
-		type = typeRealNum;
-		kind0 = 2;
-	      }
-	  }
-	else if (kind0 >= 1 && kind >= 1)
-	  {
-	    type = typeNumeric;
-	    kind0 = 1;
-	  }
-	else
-	  return Type.pointer_type;
-      }
-    return type;
-  }
 }
