@@ -130,29 +130,46 @@ public class ApplyToArgs extends ProcedureN
 
   Language language;
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
     Expression[] args = exp.getArgs();
     int nargs = args.length - 1;
     if (nargs >= 0)
       {
         Expression proc = args[0];
-        args[0] = proc;
+        if (! argsInlined)
+          {
+            if (proc instanceof LambdaExp)
+              {
+                Expression[] rargs = new Expression[nargs];
+                System.arraycopy(args, 1, rargs, 0, nargs);
+                return walker.walk(new ApplyExp(proc, rargs));
+              }
+            proc = walker.walk(proc);
+            args[0] = proc;
+          }
         Type ptype = proc.getType();
         ApplyExp result;
         Compilation comp = walker.getCompilation();
         Language language = comp.getLanguage();
-        // This might be more cleanly handled at the type specifier. FIXME
-        if (Invoke.checkKnownClass(ptype, comp) < 0)
-          return exp;
-        ClassType ctype;
         if (ptype.isSubtype(Compilation.typeProcedure))
           {
             Expression[] rargs = new Expression[nargs];
             System.arraycopy(args, 1, rargs, 0, nargs);
-            result = new ApplyExp(proc, rargs);
+            return proc.inline(new ApplyExp(proc, rargs), walker, null,
+                               argsInlined);
           }
-        else if (ptype.isSubtype(Compilation.typeType)
+        if (! argsInlined)
+          {
+            for (int i = 1; i <= nargs;  i++)
+              args[i] = walker.walk(args[i]);
+          }
+        // This might be more cleanly handled at the type specifier. FIXME
+        if (Invoke.checkKnownClass(ptype, comp) < 0)
+          return exp;
+        ClassType ctype;
+        if (ptype.isSubtype(Compilation.typeType)
                  || language.getTypeFor(proc,false) != null)
           {
             result = new ApplyExp(Invoke.make, args);
@@ -169,7 +186,7 @@ public class ApplyToArgs extends ProcedureN
             // We search for a "get(int)" method, rather than just using
             // typeList.getDeclaredMethod("get", 1) to see if we make a
             // a virtual call rather than an interface call.
-            Method get = ctype.getMethod("get", new Type[] { Type.int_type });
+            Method get = ctype.getMethod("get", new Type[] { Type.intType  });
             result = new ApplyExp(get, args);
           }
         else
@@ -190,13 +207,14 @@ public class ApplyToArgs extends ProcedureN
   public Object applyN (Object[] args) throws Throwable
   {
     Object proc = args[0];
-    Object[] rargs = new Object[args.length-1];
-    System.arraycopy(args, 1, rargs, 0, rargs.length);
     if (proc instanceof Procedure)
       {
+        Object[] rargs = new Object[args.length-1];
+        System.arraycopy(args, 1, rargs, 0, rargs.length);
         return ((Procedure) proc).applyN(rargs);
       }
-    if (proc instanceof gnu.bytecode.Type)
+    if (proc instanceof gnu.bytecode.Type
+        || proc instanceof Class)
       {
         return gnu.kawa.reflect.Invoke.make.applyN(args);
       }
@@ -210,7 +228,7 @@ public class ApplyToArgs extends ProcedureN
       {
         if (args.length != 2)
           throw new WrongArguments(this, args.length); // FIXME
-        int index = ((Number) rargs[0]).intValue();
+        int index = ((Number) args[1]).intValue();
         /* #ifdef JAVA2 */
         return ((java.util.List) proc).get(index);
         /* #else */
@@ -222,7 +240,7 @@ public class ApplyToArgs extends ProcedureN
       {
         if (args.length != 2)
           throw new WrongArguments(this, args.length); // FIXME
-        return java.lang.reflect.Array.get(proc, ((Number) rargs[0]).intValue());
+        return java.lang.reflect.Array.get(proc, ((Number) args[1]).intValue());
       }
     throw new WrongType(this, 0, proc, "procedure");
   }
