@@ -8,6 +8,7 @@ import gnu.math.RatNum;
 import gnu.math.IntNum;
 import java.io.PrintWriter;
 import gnu.text.Char;
+import gnu.expr.Keyword;
 import gnu.kawa.xml.XmlNamespace;
 import gnu.text.Printable;
 /* #ifdef use:java.util.regex */
@@ -94,8 +95,8 @@ public class DisplayFormat extends AbstractFormat
 	if (list != value)
 	  out.writeSpaceFill();
 	Pair pair = (Pair) list;
-	writeObject(pair.car, (Consumer) out);
-	list = pair.cdr;
+	writeObject(pair.getCar(), (Consumer) out);
+	list = pair.getCdr();
       }
     if (list != LList.Empty)
       {
@@ -113,8 +114,12 @@ public class DisplayFormat extends AbstractFormat
         && ! (obj instanceof gnu.kawa.xml.UntypedAtomic)
         && ! (obj instanceof Values)
         && (getReadableOutput()
-            || ! (obj instanceof FString
-                  || obj instanceof Char
+            || ! (obj instanceof Char
+                  /* #ifdef use:java.lang.CharSequence */
+                  || obj instanceof CharSequence
+                  /* #else */
+                  // || obj instanceof String || obj instanceof CharSeq
+                  /* #endif */
                   || obj instanceof Character)))
       {
         ((OutPort) out).writeWordStart();
@@ -142,7 +147,7 @@ public class DisplayFormat extends AbstractFormat
             write(sym.getLocalPart(), out);
           }
         else
-          writeObject(obj.toString(), out);
+          writeSymbol(sym, out, readable);
       }
     /* #ifdef use:java.net.URI */
     /* #ifdef use:java.lang.CharSequence */
@@ -155,18 +160,35 @@ public class DisplayFormat extends AbstractFormat
       }
     /* #endif */
     /* #endif */
-    else if (obj instanceof CharSeq)
+    else if
+      /* #ifdef use:java.lang.CharSequence */
+      (obj instanceof CharSequence) 
+      /* #else */
+      // (obj instanceof CharSeq || obj instanceof String) 
+      /* #endif */
       {
-	CharSeq str = (CharSeq) obj;
+        /* #ifdef use:java.lang.CharSequence */
+	CharSequence str = (CharSequence) obj;
+        /* #else */
+	// String str = obj.toString();
+        /* #endif */
 	if (getReadableOutput () && out instanceof PrintWriter)
 	  Strings.printQuoted(str, (PrintWriter) out, 1);
-	else if (obj instanceof FString) // FIXME Do we need this case?
-	  {
-	    FString fstr = (FString) obj;
-	    out.write(fstr.data, 0, fstr.size());
-	  }
-	else
-	  str.consume(0, str.size(), out);
+        else if (obj instanceof String)
+          {
+            out.write((String) obj);
+          }
+	else if (obj instanceof CharSeq)
+          {
+            CharSeq seq = (CharSeq) obj;
+            seq.consume(0, seq.size(), out);
+          }
+        else
+          {
+            int len = str.length();
+            for (int i = 0; i < len;  i++)
+              out.write(str.charAt(i));
+          }
       }
     else if (obj instanceof LList && out instanceof OutPort)
       writeList((LList) obj, (OutPort) out);
@@ -275,8 +297,6 @@ public class DisplayFormat extends AbstractFormat
           }
 	if (asString == null)
 	  write("#!null", out);
-	else if (readable && obj instanceof String)
-          writeReadableSymbol(asString, out);
         else
           write(asString, out);
       }
@@ -337,12 +357,41 @@ public class DisplayFormat extends AbstractFormat
                     + "|([-+]|[.][.][.])");
   /* #endif */
 
-  void writeReadableSymbol (String sym, Consumer out)
+  void writeSymbol (Symbol sym, Consumer out, boolean readable)
+  {
+    String prefix = sym.getPrefix();
+    Namespace namespace = sym.getNamespace();
+    String uri = namespace == null ? null : namespace.getName();
+    boolean suffixColon = false;
+    if (namespace == Keyword.keywordNamespace)
+      {
+        if (language == 'C' || language == 'E')
+          out.write(':');
+        else
+          suffixColon = true;
+      }
+    else if (prefix != null && prefix.length() > 0)
+      {
+        writeSymbol(prefix, out, readable);
+        out.write(':');
+      }
+    else if (uri != null && uri.length() > 0)
+      {
+        out.write('{');
+        out.write(uri);
+        out.write('}');
+      }
+    writeSymbol(sym.getName(), out, readable);
+    if (suffixColon)
+      out.write(':');
+  }
+
+  void writeSymbol (String sym, Consumer out, boolean readable)
   {
     /* #ifdef use:java.util.regex */
     /* Use |...| if symbol doesn't follow R5RS conventions
        for identifiers or has a colon in the interior. */
-    if (! r5rsIdentifierMinusInteriorColons.matcher(sym).matches())
+    if (readable && ! r5rsIdentifierMinusInteriorColons.matcher(sym).matches())
       {
         int len = sym.length();
         if (len == 0)
