@@ -12,7 +12,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
 {
   public static final GetNamedPart getNamedPart = new GetNamedPart();
 
-  /** PREFIX:<> is equivalent to the ClassType bound to PREFIX. */
+  /** {@code PREFIX:<>} is equivalent to the {@code ClassType} bound to {@code PREFIX}. */
   public static final String CLASSTYPE_FOR = "<>";
 
   /** Pseudo-method-name for the cast operation. */
@@ -25,8 +25,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
   {
     String name1;
     Object name2;
-    if (part2 instanceof QuoteExp
-        && (name2 = ((QuoteExp) part2).getValue()) instanceof String
+    if ((name2 = part2.valueIfConstant()) instanceof SimpleSymbol
         && ((part1 instanceof ReferenceExp
              && (name1 = ((ReferenceExp) part1).getSimpleName()) != null)
             || (part1 instanceof GetNamedExp
@@ -43,11 +42,11 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
     if (combinedName != null)
       {
         Translator tr = (Translator) Compilation.getCurrent();
-        Declaration decl = tr.lexical.lookup(combinedName, false/*FIXME*/);
+        Symbol symbol = Namespace.EmptyNamespace.getSymbol(combinedName);
+        Declaration decl = tr.lexical.lookup(symbol, false/*FIXME*/);
         if (! Declaration.isUnknown(decl))
           return new ReferenceExp(decl);
 
-        Symbol symbol = env.defaultNamespace().lookup(combinedName);
         Object property = null; // FIXME?
         if (symbol != null && env.isBound(symbol, property))
           return new ReferenceExp(combinedName);
@@ -87,8 +86,10 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
     return makeExp(new QuoteExp(type), new QuoteExp(member));
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     if (args.length != 2 || ! (args[1] instanceof QuoteExp)
         || ! (exp instanceof GetNamedExp))
@@ -114,7 +115,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
       : comp.mainClass;
     GetNamedExp nexp = (GetNamedExp) exp;
 
-    if (typeval instanceof Type)
+    if (typeval != null)
       {
         if (mname.equals(CLASSTYPE_FOR))
           return new QuoteExp(typeval);
@@ -129,7 +130,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
               return nexp.setProcedureKind('C');
           }
       }
-    if (typeval instanceof ClassType)
+    if (typeval instanceof ObjectType)
       {
         if (mname.length() > 1 && mname.charAt(0) == '.')
           {
@@ -141,7 +142,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
         if (Invoke.checkKnownClass(typeval, comp) < 0)
           return exp;
         PrimProcedure[] methods
-          = ClassMethods.getMethods((ClassType) typeval,
+          = ClassMethods.getMethods((ObjectType) typeval,
                                     Compilation.mangleName(mname),
                                     '\0', caller, language);
         if (methods != null && methods.length > 0)
@@ -151,7 +152,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
           }
         ApplyExp aexp = new ApplyExp(SlotGet.staticField, args);
         aexp.setLine(exp);
-        return ((InlineCalls) walker).walkApplyOnly(aexp);
+        return walker.walkApplyOnly(aexp);
                             
       }
     if (typeval != null)
@@ -167,7 +168,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
     */
 
     if (type.isSubtype(Compilation.typeClassType)
-        || type.isSubtype(Type.java_lang_Class_type))
+        || type.isSubtype(Type.javalangClassType))
       // The container evaluates to a class (so we should look for a static
       // field or method), but we don't know which class at compile-time.
       // However, we should still optimize it somewhat, above.  FIXME.
@@ -176,8 +177,6 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
     if (type instanceof ObjectType)
       {
         ObjectType otype = (ObjectType) type;
-        ClassType ctype
-          = type instanceof ClassType ? (ClassType) type : Type.pointer_type;
         PrimProcedure[] methods
           = ClassMethods.getMethods(otype, Compilation.mangleName(mname),
                                     'V', caller, language);
@@ -186,16 +185,6 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
             nexp.methods = methods;
             return nexp.setProcedureKind('M');
           }
-        Member part = SlotGet.lookupMember(ctype, mname, caller);
-        if (part != null
-            || (mname.equals("length") && type instanceof ArrayType))
-          {
-            // FIXME: future kludge to avoid re-doing SlotGet.getField.
-            // args = new Expression[] { context, new QuoteExp(part) });
-            ApplyExp aexp = new ApplyExp(SlotGet.field, args);
-            aexp.setLine(exp);
-            return ((InlineCalls) walker).walkApplyOnly(aexp);
-          }
 
         if (type.isSubtype(typeHasNamedParts))
           {
@@ -210,9 +199,21 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
                     return QuoteExp.getInstance(val);
                   }
               }
+            args = new Expression[] { args[0], QuoteExp.getInstance(mname) };
             return new ApplyExp(typeHasNamedParts.getDeclaredMethod("get", 1),
                                 args).setLine(exp);
           }
+
+        Member part = SlotGet.lookupMember(otype, mname, caller);
+        if (part != null
+            || (mname.equals("length") && type instanceof ArrayType))
+          {
+            // FIXME: future kludge to avoid re-doing SlotGet.getField.
+            // args = new Expression[] { context, new QuoteExp(part) });
+            ApplyExp aexp = new ApplyExp(SlotGet.field, args);
+            aexp.setLine(exp);
+            return walker.walkApplyOnly(aexp);
+          }
       }
 
     if (comp.getBooleanOption("warn-invoke-unknown-method", ! comp.immediate))
@@ -287,6 +288,17 @@ public class GetNamedPart extends Procedure2 implements HasSetter, CanInline
       return ((HasNamedParts) container).get(name);
     if (container instanceof Class)
       container = (ClassType) Type.make((Class) container);
+    if (container instanceof Package)
+      {
+        try
+          {
+            String pname = ((Package) container).getName();
+            return ClassType.getContextClass(pname + '.' + name);
+          }
+        catch (Throwable ex)
+          {
+          }
+      }
     if (container instanceof Type)
       return getTypePart((Type) container, name);
     return getMemberPart(container, part.toString());
@@ -369,8 +381,11 @@ class GetNamedExp extends ApplyExp
     return this;
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
   {
+    if (! argsInlined)
+      exp.walkArgs(walker);
     Expression[] pargs = getArgs();
     Expression context = pargs[0];
     Expression[] args = exp.getArgs();
@@ -485,8 +500,10 @@ class NamedPart extends ProcedureN
     return 0xfffff000;
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     switch (kind)
       {
@@ -507,7 +524,7 @@ class NamedPart extends ProcedureN
           }
         ApplyExp aexp = new ApplyExp(proc, xargs);
         aexp.setLine(exp);
-        return ((InlineCalls) walker).walkApplyOnly(aexp);
+        return walker.walkApplyOnly(aexp);
       }
     return exp;
   }
@@ -634,8 +651,10 @@ class NamedPartSetter extends gnu.mapping.Setter
     return 0xfffff000;
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     NamedPart get = (NamedPart) this.getter;
     if (get.kind == 'D')
       {
@@ -658,7 +677,7 @@ class NamedPartSetter extends gnu.mapping.Setter
           return exp;
         ApplyExp aexp = new ApplyExp(proc, xargs);
         aexp.setLine(exp);
-        return ((InlineCalls) walker).walkApplyOnly(aexp);
+        return walker.walkApplyOnly(aexp);
       }
     return exp;
   }
