@@ -12,7 +12,7 @@ import gnu.mapping.*;
  * @author Per Bothner
  */
 
-public class MultiplyOp extends ProcedureN implements CanInline
+public class MultiplyOp extends ArithOp
 {
   public static final MultiplyOp $St = new MultiplyOp("*");
 
@@ -21,6 +21,16 @@ public class MultiplyOp extends ProcedureN implements CanInline
     super(name);
   }
 
+  public Object defaultResult ()
+  {
+    return IntNum.one();
+  }
+
+  public int primitiveOpcode ()
+  {
+    return 104;
+  }
+
   public static Object apply (Object arg1, Object arg2)
   {
     return ((Numeric) arg1).mul(arg2);
@@ -92,8 +102,10 @@ public class MultiplyOp extends ProcedureN implements CanInline
     return result;
    }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     if (! walker.getCompilation().mustCompile)
       return exp;
     Expression folded = exp.inlineIfConstant(this, walker);
