@@ -233,8 +233,10 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
     return getLanguage().booleanObject(applyN(flags, args));
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression folded = exp.inlineIfConstant(this, walker);
     if (folded != exp)
       return folded;
@@ -268,10 +270,10 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
 		&& (kind0 < long_KIND || kind1 < long_KIND))
 	      {
 		Type[] ctypes = new Type[2];
-		ctypes[0] = AddOp.typeIntNum;
+		ctypes[0] = Arithmetic.typeIntNum;
 		if (kind1 >= long_KIND)
 		  {
-		    ctypes[1] = Type.long_type;
+		    ctypes[1] = Type.longType;
 		  }
 		else if (kind0 >= long_KIND
 			 // Simple check to avoid re-ordering side-effects.
@@ -280,7 +282,7 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
 			     || arg0 instanceof ReferenceExp
 			     || arg1 instanceof ReferenceExp))
 		  {
-		    ctypes[1] = Type.long_type;
+		    ctypes[1] = Type.longType;
 		    args = new Expression[2];
 		    args[0] = arg1;
 		    args[1] = arg0;
@@ -288,9 +290,9 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
 		      mask ^= TRUE_IF_GRT|TRUE_IF_LSS;
 		  }
 		else
-		  ctypes[1] = AddOp.typeIntNum;
+		  ctypes[1] = Arithmetic.typeIntNum;
 		Method cmeth
-		  = AddOp.typeIntNum.getDeclaredMethod("compare", ctypes);
+		  = Arithmetic.typeIntNum.getDeclaredMethod("compare", ctypes);
 		PrimProcedure compare = new PrimProcedure(cmeth);
 		arg0 = new ApplyExp(compare, args);
 		arg1 = new QuoteExp(IntNum.zero());
@@ -298,11 +300,11 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
 	      }
 	    Type commonType;
 	    if (kind0 >= int_KIND && kind1 >= int_KIND)
-	      commonType = Type.int_type;
+	      commonType = Type.intType;
 	    else if (kind0 >= long_KIND && kind1 >= long_KIND)
-	      commonType = Type.long_type;
+	      commonType = Type.longType;
 	    else
-	      commonType = Type.double_type;
+	      commonType = Type.doubleType;
 	    StackTarget subTarget = new StackTarget(commonType);
 	    ConditionalTarget ctarget = (ConditionalTarget) target;
 	    
@@ -388,13 +390,13 @@ public class NumberCompare extends ProcedureN implements CanInline, Inlineable
 	  return long_KIND;
 	return int_KIND;
       }
-     if (type.isSubtype(AddOp.typeIntNum))
+     if (type.isSubtype(Arithmetic.typeIntNum))
        return IntNum_KIND;
-     if (type.isSubtype(AddOp.typeDFloNum))
+     if (type.isSubtype(Arithmetic.typeDFloNum))
        return double_KIND;
-     if (type.isSubtype(AddOp.typeRealNum))
+     if (type.isSubtype(Arithmetic.typeRealNum))
        return RealNum_KIND;
-     if (type.isSubtype(AddOp.typeNumeric))
+     if (type.isSubtype(Arithmetic.typeNumeric))
        return Numeric_KIND;
     return Unknown_KIND;
   }
