@@ -36,8 +36,10 @@ public class Setter extends Procedure1 implements CanInline, HasSetter
     return ((Procedure)arg).getSetter();
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     if (args.length == 1)
       {
@@ -153,8 +155,11 @@ class SetArrayExp extends ApplyExp
     elementType = arrayType.getComponentType();
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
   {
+    if (! argsInlined)
+      exp.walkArgs(walker);
     Expression[] args = exp.getArgs();
     if (args.length == 2)
       {
@@ -177,19 +182,22 @@ class SetListExp extends ApplyExp
     super(func, args);
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker, Declaration decl)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            Declaration decl, boolean argsInlined)
   {
+    if (! argsInlined)
+      exp.walkArgs(walker);
     Expression[] args = exp.getArgs();
     if (args.length == 2)
       {
         Expression[] xargs = new Expression[4];
         xargs[0] = this.getArgs()[0];
         xargs[1] = QuoteExp.getInstance("set");
-        xargs[2] = Convert.makeCoercion(args[0], Type.int_type);
+        xargs[2] = Convert.makeCoercion(args[0], Type.intType);
         xargs[3] = args[1];
         Expression set
           = walker.walkApplyOnly(new ApplyExp(Invoke.invoke, xargs));
-        return Convert.makeCoercion(set, Type.void_type);
+        return Convert.makeCoercion(set, Type.voidType);
       }
     return exp;
   }
