@@ -29,15 +29,15 @@ public class DefineNamespace extends Syntax
                                      ScopeExp defs, Translator tr)
   {
     Pair p1, p2;
-    if (! (st.cdr instanceof Pair)
-        || ! ((p1 = (Pair) st.cdr).car instanceof String)
-	|| ! (p1.cdr instanceof Pair)
-	|| (p2 = (Pair) p1.cdr).cdr != LList.Empty)
+    if (! (st.getCdr() instanceof Pair)
+        || ! ((p1 = (Pair) st.getCdr()).getCar() instanceof Symbol)
+	|| ! (p1.getCdr() instanceof Pair)
+	|| (p2 = (Pair) p1.getCdr()).getCdr() != LList.Empty)
       {
 	tr.error('e', "invalid syntax for define-namespace");
 	return false;
       }
-    String name = (String) p1.car;
+    Symbol name = (Symbol) p1.getCar();
     Declaration decl = defs.getDefine(name, 'w', tr);
     tr.push(decl);
     decl.setFlag(Declaration.IS_CONSTANT|Declaration.IS_NAMESPACE_PREFIX);
@@ -51,9 +51,14 @@ public class DefineNamespace extends Syntax
     Translator.setLine(decl, p1);
     Expression value;
     String literal = null;
-    if (p2.car instanceof FString)
+    if
+      /* #ifdef use:java.lang.CharSequence */
+      (p2.getCar() instanceof CharSequence) 
+      /* #else */
+      // (p2.getCar() instanceof CharSeq || p2.getCar() instanceof String) 
+      /* #endif */
       {
-        literal = p2.car.toString();
+        literal = p2.getCar().toString();
         Namespace namespace;
         if (literal.startsWith("class:"))
           {
@@ -64,7 +69,7 @@ public class DefineNamespace extends Syntax
           }
         else if (makeXML)
           {
-            namespace = XmlNamespace.getInstance(name, literal);
+            namespace = XmlNamespace.getInstance(name.getName(), literal);
             decl.setType(ClassType.make("gnu.kawa.xml.XmlNamespace"));
           }
         else
