@@ -22,7 +22,7 @@ public abstract class LispLanguage extends Language
   static public final String unquotesplicing_sym = "unquote-splicing";
   static public final String quasiquote_sym = "quasiquote";
   /** Used for Kawa infix ':' operator. */
-  static public final String lookup_sym = "$lookup$";
+  static public final Symbol lookup_sym = Namespace.EmptyNamespace.getSymbol("$lookup$");
 
   /** The default <code>ReadTable</code> for this language. */
   public ReadTable defaultReadTable = createReadTable();
@@ -35,9 +35,9 @@ public abstract class LispLanguage extends Language
     return new LispReader(inp, messages);
   }
 
-  public Compilation getCompilation (Lexer lexer, SourceMessages messages)
+  public Compilation getCompilation (Lexer lexer, SourceMessages messages, NameLookup lexical)
   {
-    return new Translator (this, messages);
+    return new Translator(this, messages, lexical);
   }
 
   public boolean parse (Compilation comp, int options)
@@ -159,4 +159,13 @@ public abstract class LispLanguage extends Language
       return getSymbol((String) sym);
     return (Symbol) sym;
   }
+
+  /** If a symbol is lexically unbound, look for a default binding.
+   * The default implementation does nothing.
+   * @return null if no binidng, or an Expression.
+   */
+  public Expression checkDefaultBinding (Symbol name, Translator tr)
+  {
+    return null;
+  }
 }
