@@ -24,21 +24,6 @@ public class LispReader extends Lexer
     super(port, messages);
   }
 
-  /** Resolve a unit name, if possible.
-   * Returns null if the unit name is unknown. */
-  public static Object lookupUnit (String name)
-  {
-    name = (name + "$unit").intern();
-    try
-      {
-	return Environment.getCurrent().getChecked(name);
-      }
-    catch (UnboundLocationException ex)
-      {
-	return name;
-      }
-  }
-
   /** Read a #|...|#-style comment (which may contain other nested comments).
     * Assumes the initial "#|" has already been read.
     */
@@ -160,12 +145,6 @@ public class LispReader extends Lexer
    */
   protected boolean seenEscapes;
 
-  /** True if ":IDENTIFIER" should be treated as a keyword. */
-  protected boolean initialColonIsKeyword = true;
-
-  /** True if "IDENTIFIER:" should be treated as a keyword. */
-  protected boolean finalColonIsKeyword = true;
-
   void readToken(int ch, char readCase, ReadTable rtable)
       throws java.io.IOException, SyntaxException
   {
@@ -287,6 +266,8 @@ public class LispReader extends Lexer
     if (ch < 0 || ch == ':' || (entry = rtable.lookup(ch)) == null
         || ch == rtable.postfixLookupOperator)
       return false;
+    if (ch == ',')
+      return true;
     int kind = entry.getKind();
     return kind == ReadTable.CONSTITUENT
       || kind == ReadTable.NON_TERMINATING_MACRO
@@ -314,7 +295,7 @@ public class LispReader extends Lexer
         ch = port.read();
         Object rightOperand = readValues(ch, rtable.lookup(ch), rtable);
         value = LList.list2(value,
-                            LList.list2(LispLanguage.quote_sym, rightOperand));
+                            LList.list2(rtable.makeSymbol(LispLanguage.quasiquote_sym), rightOperand));
         value = PairWithPosition.make(LispLanguage.lookup_sym, value,
                                       port.getName(), line+1, column+1);
       }
@@ -354,6 +335,22 @@ public class LispReader extends Lexer
   static final int SCM_COMPLEX = 1;
   public static final int SCM_NUMBERS = SCM_COMPLEX;
 
+  public static Object parseNumber
+  /* #ifdef use:java.lang.CharSequence */
+  (CharSequence str, int radix)
+  /* #else */
+  // (CharSeq str, int radix)
+  /* #endif */
+  {
+    char[] buf;
+    if (str instanceof FString)
+      buf = ((FString) str).data;
+    else
+      buf = str.toString().toCharArray();
+    int len = str.length();
+    return parseNumber(buf, 0, len, '\0', radix, LispReader.SCM_NUMBERS);
+  }
+
   /** Parse a number.
    * @param buffer contains the characters of the number
    * @param start startinging index of the number in the buffer
@@ -483,7 +480,7 @@ public class LispReader extends Lexer
 
     int realStart = pos - 1;
     boolean hash_seen = false;
-    char exp_seen = '\000';
+    int exp_seen = -1;
     int digits_start = -1;
     int decimal_point = -1;
     boolean copy_needed = false;
@@ -535,6 +532,7 @@ public class LispReader extends Lexer
 		    break loop;
 		  }
 		char next = buffer[pos];
+                int exp_pos = pos-1;
 		if (next == '+' || next == '-')
 		  {
 		    if (++ pos >= end
@@ -546,13 +544,13 @@ public class LispReader extends Lexer
 		    pos--;
 		    break loop;
 		  }
-		if (exp_seen != '\000')
+		if (exp_seen >= 0)
 		  return "duplicate exponent";
 		if (radix != 10)
 		  return "exponent in non-decimal number";
 		if (digits_start < 0)
 		  return "mantissa with no digits";
-		exp_seen = ch;
+		exp_seen = exp_pos;
 		for (;;)
 		  {
 		    pos++;
@@ -564,7 +562,7 @@ public class LispReader extends Lexer
 		  return "multiple fraction symbol '/'";
 		if (digits_start < 0)
 		  return "no digits before fraction symbol '/'";
-		if (exp_seen != '\000' || decimal_point >= 0)
+		if (exp_seen >= 0 || decimal_point >= 0)
 		  return "fraction symbol '/' following exponent or '.'";
 		numerator = valueOf(buffer, digits_start, pos - digits_start,
 				    radix, negative, lvalue);
@@ -595,13 +593,22 @@ public class LispReader extends Lexer
     boolean inexact = (exactness == 'i' || exactness == 'I'
 		       || (exactness == ' ' && hash_seen));
     RealNum number = null;
-    if (exp_seen != '\000' || decimal_point >= 0)
+    if (exp_seen >= 0 || decimal_point >= 0)
       {
 	if (digits_start > decimal_point && decimal_point >= 0)
 	  digits_start = decimal_point;
 	if (numerator != null)
 	  return "floating-point number after fraction symbol '/'";
 	String str = new String(buffer, digits_start, pos - digits_start);
+        if (exp_seen >= 0)
+          {
+            char exp = buffer[exp_seen];
+            if (exp != 'e' && exp != 'E')
+              {
+                int prefix = exp_seen - digits_start;
+                str = str.substring(0, prefix)+'e'+str.substring(prefix+1);
+              }
+          }
 	double d = Convert.parseDouble(str);
 	number = new DFloNum(negative ? - d : d);
       }
@@ -702,92 +709,7 @@ public class LispReader extends Lexer
 		return Complex.make(IntNum.zero (), number);
 	      }
 	  }
-	if (lcount > 0)
-	  {
-	    Object unit = null;
-	    for (;;)
-	      {
-		String word = new String(buffer, pos - lcount, lcount);
-		Object u = lookupUnit(word);
-
-		int power = 1;
-		if (pos < end)
-		  {
-		    ch = buffer[pos];
-		    if (ch == '^' && ++pos < end)
-		      ch = buffer[pos];
-		    boolean neg = ch == '-';
-		    if ((ch == '-' || ch == '+') && ++pos < end)
-		      ch = buffer[pos];
-		    power = -1;
-		    for (;;)
-		      {
-			int d = Character.digit(ch, 10);
-			if (d < 0)
-			  {
-			    if (power < 0)
-			      return "junk after unit name";
-			    break;
-			  }
-			power = power < 0 ? d  : 10 * power + d;
-			if (++pos == end)
-			  break;
-			if (power > 1000000)
-			  return "unit power too large";
-			ch = buffer[pos];
-		      }
-		    if (neg) power = -power;
-		  }
-
-		// "expt" and "*" are too open to name clashes. FIXME.
-		if (power != 1)
-		  {
-		    if (u instanceof Unit)
-		      u = Unit.pow((Unit) u, power);
-		    else
-		      u = LList.list3("expt", u, IntNum.make(power));
-		  }
-		if (unit == null)
-		  unit = u;
-		else if (u instanceof Unit && unit instanceof Unit)
-		  unit = Unit.times((Unit) unit, (Unit) u);
-		else
-		  unit = LList.list3("*", unit, u);
-		if (pos >= end)
-		  break;
-		ch = buffer[pos++];
-		if (ch == '*')
-		  {
-		    if (pos == end)
-		      return "end of token after '*'";
-		    ch = buffer[pos++];
-		  }
-		lcount = 0;
-		for (;;)
-		  {
-		    if (! Character.isLetter(ch))
-		      {
-			pos--;
-			break;
-		      }
-		    lcount++;
-		    if (pos == end)
-		      break;
-		    ch = buffer[pos++];
-		  }
-		if (lcount == 0)
-		  return "excess junk after unit";
-	      }
-
-	    if (unit == null)
-	      return "expected unit";
-	    else if (unit instanceof Unit)
-	      return Quantity.make(number, (Unit) unit);
-	    else
-	      return LList.list3("*", number, unit);
-	  }
-	else
-	  return "excess junk after number";
+        return "excess junk after number";
 	
       }
     return number;
@@ -856,13 +778,13 @@ public class LispReader extends Lexer
 
     int len = endPos - startPos;
 
-    if (initialColonIsKeyword && packageMarker == startPos && len > 1)
+    if (rtable.initialColonIsKeyword && packageMarker == startPos && len > 1)
       {
 	startPos++;
 	String str = new String(tokenBuffer, startPos, endPos-startPos);
 	return Keyword.make(str.intern());
     }
-    if (finalColonIsKeyword && packageMarker == endPos - 1 && len > 1)
+    if (rtable.finalColonIsKeyword && packageMarker == endPos - 1 && len > 1)
       {
 	String str = new String(tokenBuffer, startPos, len - 1);
 	return Keyword.make(str.intern());
@@ -877,12 +799,14 @@ public class LispReader extends Lexer
 			       '\0', 0, SCM_NUMBERS);
     if (value != null && ! (value instanceof String))
       return value;
+    /* Common Lisp only?  FIXME
     if (isPotentialNumber(tokenBuffer, startPos, endPos))
       {
 	error(value == null ? "not a valid number"
 	      : "not a valid number: " + value);
 	return IntNum.zero();
       }
+    */
     return returnSymbol(startPos, endPos, rtable);
   }
 
@@ -916,35 +840,50 @@ public class LispReader extends Lexer
       case 'e':  c = 27;  break;  // escape
       case '\"': c = 34;  break;  // quote
       case '\\': c = 92;  break;  // backslash
-      case ' ': // Skip to end of line, inclusive.
+      case ' ':  // Skip to end of line, inclusive.
+      case '\n': // Skip initial whitespace on following line.
+      case '\r':
+      case '\t':
 	for (;;)
 	  {
-	    c = read();
 	    if (c < 0)
 	      {
-		eofError("unexpected EOF in character literal");
+		eofError("unexpected EOF in literal");
 		return -1;
 	      }
 	    if (c == '\n')
-	      return -2;
+	      break;
 	    if (c == '\r')
 	      {
 		if (peek() == '\n')
 		  skip();
-		return -2;
+                c = '\n';
+                break;
 	      }
 	    if (c != ' ' && c != '\t')
 	      {
 		unread(c);
 		break;
 	      }
+	    c = read();
 	  }
-      case '\r':
-	if (peek() == '\n')
-	  skip();
-	return -2;
-      case '\n':
-	return -2;
+        if (c != '\n')
+          break; // ERROR
+        // FIXME: if legacy-compatible non-R6RS-mode: return -2;
+        for (;;)
+          {
+            c = read();
+            if (c < 0)
+              {
+                eofError("unexpected EOF in literal");
+                return -1;
+              }
+            if (c != ' ' && c != '\t')
+              {
+                unread(c);
+                return -2;
+              }
+          }
       case 'M':
 	c = read();
 	if (c != '-')
@@ -1019,8 +958,12 @@ public class LispReader extends Lexer
 	      c = (c << 4) + v;
 	    else
 	      {
-		if (d >= 0)
-		  unread(d);
+                if (d != ';')
+                  {
+                    // FIXME: if strict-R6RS: ERROR
+                    if (d >= 0)
+                      unread(d);
+                  }
 		break;
 	      }
 	  }
@@ -1060,16 +1003,9 @@ public class LispReader extends Lexer
       return Pair.make(car, LList.Empty);
   }
 
-  public Object makePair (Object car, Object cdr)
-  {
-    Object pair = makePair(car, -1, -1);
-    setCdr(pair, cdr);
-    return pair;
-  }
-
   protected void setCdr (Object pair, Object cdr)
   {
-    ((Pair) pair).cdr = cdr;
+    ((PairWithPosition) pair).setCdrBackdoor(cdr);
   }
 
   /** Read a number from a LispReader
@@ -1113,14 +1049,31 @@ public class LispReader extends Lexer
     int startPos = reader.tokenBufferLength;
     reader.tokenBufferAppend(ch);
     reader.readToken(reader.read(), 'D', ReadTable.getCurrent());
+    char[] tokenBuffer = reader.tokenBuffer;
     int length = reader.tokenBufferLength - startPos;
     if (length == 1)
-      return Char.make(reader.tokenBuffer[startPos]);
-    String name = new String(reader.tokenBuffer, startPos, length);
+      return Char.make(tokenBuffer[startPos]);
+    String name = new String(tokenBuffer, startPos, length);
     ch = Char.nameToChar(name);
     if (ch >= 0)
       return Char.make(ch);
-    ch = Character.digit(reader.tokenBuffer[startPos], 8);
+    ch = tokenBuffer[startPos];
+    if (ch == 'x' || ch == 'X')
+      {
+        int value = 0;
+        for (int i = 1; ;  i++)
+          {
+             if (i == length)
+	      return Char.make(value);
+             int v = Character.digit (tokenBuffer[startPos + i], 16);
+             if (v < 0)
+               break;
+             value = 16 * value + v;
+             if (value > 0x10FFFF)
+               break;
+          }
+      }
+    ch = Character.digit(ch, 8);
     if (ch >= 0)
       {
 	int value = ch;
@@ -1128,7 +1081,7 @@ public class LispReader extends Lexer
 	  {
 	    if (i == length)
 	      return Char.make(value);
-	    ch = Character.digit(reader.tokenBuffer[startPos + i], 8);
+	    ch = Character.digit(tokenBuffer[startPos + i], 8);
 	    if (ch < 0)
 	      break;
 	    value = 8 * value + ch;
@@ -1174,6 +1127,8 @@ public class LispReader extends Lexer
       return Special.dfault;
     if (name.equals("undefined"))
       return Special.undefined;
+    if (name.equals("abstract"))
+      return Special.abstractSpecial;
     if (name.equals("null"))
       return null;
     reader.error("unknown named constant #!"+name);
