@@ -24,6 +24,24 @@ public class ReadTable extends RangeTable
   /** A character such that PreOpWord -> ($lookup$ Pre 'Word), if > 0. */
   public char postfixLookupOperator = (char) (-1);
 
+  /** True if ":IDENTIFIER" should be treated as a keyword. */
+  protected boolean initialColonIsKeyword;
+
+  /** True if "IDENTIFIER:" should be treated as a keyword. */
+  protected boolean finalColonIsKeyword;
+
+  /** Set whether ":IDENTIFIER" should be treated as a keyword. */
+  public void setInitialColonIsKeyword (boolean whenInitial)
+  {
+    initialColonIsKeyword = whenInitial;
+  }
+
+  /** Set whether "IDENTIFIER:" should be treated as a keyword. */
+  public void setFinalColonIsKeyword (boolean whenFinal)
+  {
+    finalColonIsKeyword = whenFinal;
+  }
+
   static final ThreadLocation current = new ThreadLocation("read-table");
 
   public ReadTable()
@@ -68,14 +86,14 @@ public class ReadTable extends RangeTable
     set('\177',entry);
     set('\b', entry);
     set('\"', new ReaderString());
-    set('#',  ReaderDispatch.create());
+    set('#',  ReaderDispatch.create(this));
     set(';',  ReaderIgnoreRestOfLine.getInstance());
     set('(',  ReaderParens.getInstance('(', ')'));
 
-    set('\'', new ReaderQuote(LispLanguage.quote_sym));
-    set('`',  new ReaderQuote(LispLanguage.quasiquote_sym));
-    set(',',  new ReaderQuote(LispLanguage.unquote_sym,
-				 '@', LispLanguage.unquotesplicing_sym));
+    set('\'', new ReaderQuote(makeSymbol(LispLanguage.quote_sym)));
+    set('`',  new ReaderQuote(makeSymbol(LispLanguage.quasiquote_sym)));
+    set(',',  new ReaderQuote(makeSymbol(LispLanguage.unquote_sym),
+                              '@', makeSymbol(LispLanguage.unquotesplicing_sym)));
 
     setBracketMode();  // Sets the entries for '[', ']', and '<'.
 
@@ -198,6 +216,6 @@ public class ReadTable extends RangeTable
 
   protected Object makeSymbol (String name)
   {
-    return name.intern();
+    return Namespace.EmptyNamespace.getSymbol(name.intern());
   }
 }
