@@ -8,6 +8,9 @@ import gnu.mapping.Values;
 import gnu.mapping.Procedure;
 import gnu.bytecode.Type;
 import gnu.lists.*;
+/* #ifdef use:java.util.regex */
+import java.util.regex.*;
+/* #endif */
 
 public class ReaderDispatchMisc extends ReadTableEntry
 {
@@ -86,6 +89,10 @@ public class ReaderDispatchMisc extends ReadTableEntry
 	reader.tokenBufferAppend('#');
 	reader.tokenBufferAppend(ch);
 	return LispReader.readNumberWithRadix(2, reader, 0);
+        /* #ifdef use:java.util.regex */
+      case '/':
+        return readRegex(in, ch, count);
+        /* #endif */
       case '|':
 	port = reader.getPort();
 	if (port instanceof InPort)
@@ -103,6 +110,23 @@ public class ReaderDispatchMisc extends ReadTableEntry
 	      ((InPort) port).readState = saveReadState;
 	  }
 	return Values.empty;
+      case ';':
+	port = reader.getPort();
+	if (port instanceof InPort)
+	  {
+	    saveReadState = ((InPort) port).readState;
+	    ((InPort) port).readState = ';';
+	  }
+	try
+	  {
+            reader.readObject();
+	  }
+	finally
+	  {
+	    if (port instanceof InPort)
+	      ((InPort) port).readState = saveReadState;
+	  }
+	return Values.empty;
       case ',':
 	port = reader.getPort();
         Object list;
@@ -110,9 +134,9 @@ public class ReaderDispatchMisc extends ReadTableEntry
             && ((length
                  = LList.listLength(list = reader.readObject(), false))
                 > 0)
-            && ((Pair) list).car instanceof String)
+            && ((Pair) list).getCar() instanceof String)
           {
-            name = (String) ((Pair) list).car;
+            name = (String) ((Pair) list).getCar();
             Object proc = ReadTable.getCurrent().getReaderCtor(name);
             if (proc == null)
               in.error("unknown reader constructor "+name);
@@ -123,12 +147,12 @@ public class ReaderDispatchMisc extends ReadTableEntry
                 length--;  // Subtract 1 for the constructor name.
                 int parg = proc instanceof Type ? 1 : 0;
                 Object[] args = new Object[parg+length];
-                Object argList = ((Pair) list).cdr;
+                Object argList = ((Pair) list).getCdr();
                 for (int i = 0;  i < length;  i++)
                   {
                     Pair pair = (Pair) argList;
-                    args[parg+i] = pair.car;
-                    argList = pair.cdr;
+                    args[parg+i] = pair.getCar();
+                    argList = pair.getCdr();
                   }
                 try
                   {
@@ -153,4 +177,81 @@ public class ReaderDispatchMisc extends ReadTableEntry
 	return Values.empty;
       }
   }
+
+  public static Pattern readRegex (Lexer in, int ch, int count)
+    throws java.io.IOException, SyntaxException
+  {
+    int startPos = in.tokenBufferLength;
+    LineBufferedReader port = in.getPort();
+    char saveReadState = '\0';
+    int flags = 0;
+    if (port instanceof InPort)
+      {
+	saveReadState = ((InPort) port).readState;
+	((InPort) port).readState = '/';
+      }
+    try
+      {
+	for (;;)
+	  {
+	    int next;
+
+	    int c = port.read();
+            if (c < 0)
+              in.eofError("unexpected EOF in regex literal");
+	    if (c == ch)
+              break;
+            if (c == '\\')
+              {
+                c = port.read();
+                if ((c == ' ' ||  c == '\t' || c == '\r' || c == '\n')
+                    && in instanceof LispReader)
+                  {
+                    c = ((LispReader) in).readEscape(c);
+                    if (c == -2)
+                      continue;
+                  }
+                if (c < 0)
+                  in.eofError("unexpected EOF in regex literal");
+                if (c != ch)
+                  in.tokenBufferAppend('\\');
+              }
+            in.tokenBufferAppend(c);
+          }
+        String pattern = new String(in.tokenBuffer, startPos,
+                                    in.tokenBufferLength - startPos);
+        for (;;)
+          {
+            int c = in.peek();
+            if (c == 'i' || c == 'I')
+              flags |= Pattern.CASE_INSENSITIVE|Pattern.UNICODE_CASE;
+            else if (c == 's' || c == 'S')
+              flags |= Pattern.DOTALL;
+            else if (c == 'm' || c == 'M')
+              flags |= Pattern.MULTILINE;
+            // Think this through more before adidng this feature:
+            // Perhaps we should use the 'x' handling from
+            // gnu.xquery.util.StringUtils.makePattern (which is
+            // smart enogh to handle space in character classes).
+            // Perhaps we should handle Scheme comments?
+            /*else if (c == 'x' || c == 'X')
+              flags |= Pattern.COMMENTS;
+            */
+            else if (Character.isLetter(c))
+              {
+                in.error("unrecognized regex option '"+((char) c)+'\'');
+              }
+            else
+              break;
+            in.skip();
+          }
+        return Pattern.compile(pattern, flags);
+      }
+    finally
+      {
+	in.tokenBufferLength = startPos;
+	if (port instanceof InPort)
+	  ((InPort) port).readState = saveReadState;
+      }
+  }
 }
