@@ -15,15 +15,16 @@ public class ArrayNew extends Procedure1 implements Inlineable, Externalizable
 
   public Object apply1 (Object count)
   {
-    return java.lang.reflect.Array.newInstance(element_type.getReflectClass(),
+    Class clas = element_type.getImplementationType().getReflectClass();
+    return java.lang.reflect.Array.newInstance(clas,
 					       ((Number) count).intValue());
   }
 
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
-    exp.getArgs()[0].compile(comp, Type.int_type);
+    exp.getArgs()[0].compile(comp, Type.intType);
     CodeAttr code = comp.getCode();
-    code.emitNewArray(element_type);
+    code.emitNewArray(element_type.getImplementationType());
     target.compileFromStack(comp, ArrayType.make(element_type));
   }
 
