@@ -2,8 +2,9 @@ package gnu.kawa.reflect;
 import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import java.io.*;
 
-public class ArraySet extends Procedure3 implements Inlineable
+public class ArraySet extends Procedure3 implements Inlineable, Externalizable
 {
   Type element_type;
 
@@ -22,7 +23,7 @@ public class ArraySet extends Procedure3 implements Inlineable
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     Expression[] args = exp.getArgs();
-    args[0].compile(comp, new ArrayType(element_type));
+    args[0].compile(comp, ArrayType.make(element_type));
     args[1].compile(comp, Type.int_type);
     args[2].compile(comp, element_type);
     comp.getCode().emitArrayStore(element_type);
@@ -33,4 +34,15 @@ public class ArraySet extends Procedure3 implements Inlineable
   {
     return Type.void_type;
   }
+
+  public void writeExternal(ObjectOutput out) throws IOException
+  {
+    out.writeObject(element_type);
+  }
+
+  public void readExternal(ObjectInput in)
+    throws IOException, ClassNotFoundException
+  {
+    element_type = (Type) in.readObject();
+  }
 }
