@@ -9,7 +9,7 @@ import gnu.expr.*;
  * (For now, only fields are supported.)
  */
 
-public class ClassMemberLocation extends Location
+public abstract class ClassMemberLocation extends Location
 {
   Object instance;
   ClassType type;
@@ -34,6 +34,13 @@ public class ClassMemberLocation extends Location
     this.mname = mname;
   }
 
+  public ClassMemberLocation(Object instance, java.lang.reflect.Field field)
+  {
+    this.instance = instance;
+    this.rfield = field;
+    this.mname = field.getName();
+  }
+
   public String getMemberName()
   {
     return mname;
@@ -44,13 +51,6 @@ public class ClassMemberLocation extends Location
     return type;
   }
 
-  public ClassMemberLocation(Object instance, java.lang.reflect.Field field)
-  {
-    this.instance = instance;
-    this.rfield = field;
-    this.mname = field.getName();
-  }
-
   void setup ()
   {
     if (rfield == null)
@@ -109,6 +109,23 @@ public class ClassMemberLocation extends Location
     return rfld;
   }
 
+  /** Return the {@code Class} this member is in. */
+
+  public Class getRClass ()
+  {
+    java.lang.reflect.Field rfld = this.rfield;
+    if (rfld != null)
+      return rfld.getDeclaringClass();
+    try
+      {
+        return type.getReflectClass();
+      }
+    catch (Exception ex)
+      {
+        return null;
+      }   
+  }
+
   public Object get (Object defaultValue)
   {
     java.lang.reflect.Field rfld = getRField();
@@ -173,7 +190,8 @@ public class ClassMemberLocation extends Location
 	sym = Symbol.make(uri == null ? "" : uri,
 			  fdname.toString().intern());
       }
-    boolean isFinal = (rfield.getModifiers() & Access.FINAL) != 0;
+    int rModifiers = rfield.getModifiers();
+    boolean isFinal = (rModifiers & Access.FINAL) != 0;
     Location loc;
     Object property = null;
     if (isAlias && isFinal)
@@ -184,17 +202,21 @@ public class ClassMemberLocation extends Location
       {
 	if (isFinal)
 	  property = language.getEnvPropertyFor(rfield, fvalue);
-	loc = new ClassMemberLocation(instance, rfield);
+        boolean isStatic = (rModifiers & Access.STATIC) != 0;
+        if (isStatic)
+          loc = new StaticFieldLocation(rfield);
+        else
+          loc = new FieldLocation(instance, rfield);
+
       }
     env.addLocation(sym, property, loc);
   }
 
   /** Import all the public fields of an object. */
-  public static void defineAll(Object instance, Environment env)
+  public static void defineAll (Object instance, Language language, Environment env)
     throws IllegalAccessException
   {
     Class clas = instance.getClass();
-    Language language = Language.getDefaultLanguage();
     java.lang.reflect.Field[] fields = clas.getFields();
     for (int i = fields.length;  --i >= 0; )
       {
