@@ -96,16 +96,18 @@ public class ClassMethods extends Procedure2
                                            ClassType caller,
                                            Language language)
   {
-    MethodFilter filter = new MethodFilter(mname, 0, 0, caller);
     // FIXME kludge until we handle "language types".
     if (dtype == Type.tostring_type)
       dtype = Type.string_type;
+    MethodFilter filter = new MethodFilter(mname, 0, 0, caller,
+                                           mode == 'P' ? null : dtype);
     boolean named_class_only = mode == 'P' || "<init>".equals(mname);
     Vector methods = new Vector();
-    dtype.getMethods(filter, named_class_only ? 0 : 2,
-		     methods,
-                     caller == null ? "-" : caller.getPackageName());
-
+    dtype.getMethods(filter, named_class_only ? 0 : 2, methods);
+    if (! named_class_only &&
+        // If not redundant (i.e. not a normal ClassType), also search Object.
+        ! (dtype instanceof ClassType && ! ((ClassType) dtype).isInterface()))
+      Type.pointer_type.getMethods(filter, 0, methods);
     int mlength = (named_class_only ? methods.size()
 		   : removeRedundantMethods(methods));
 
@@ -114,6 +116,13 @@ public class ClassMethods extends Procedure2
     for (int i = mlength;  --i >= 0; )
     {
       Method method = (Method) methods.elementAt(i);
+      if (! named_class_only && method.getDeclaringClass() != dtype)
+        {
+          // Override declared type of method so it matches receiver,
+          // like javac does, for improved binary compatibility.
+          method = new Method(method,
+                              (ClassType) dtype.getImplementationType());
+        }
       PrimProcedure pproc = new PrimProcedure(method, mode, language);
       result[count++] = pproc;
     }
@@ -245,15 +254,17 @@ class MethodFilter implements gnu.bytecode.Filter
   int modifiers;
   int modmask;
   ClassType caller;
+  ObjectType receiver;
 
   public MethodFilter(String name, int modifiers, int modmask,
-		      ClassType caller)
+		      ClassType caller, ObjectType receiver)
   {
     this.name = name;
     this.nlen = name.length();
     this.modifiers = modifiers;
     this.modmask = modmask;
     this.caller = caller;
+    this.receiver = receiver;
   }
 
   public boolean select(Object value)
@@ -262,6 +273,7 @@ class MethodFilter implements gnu.bytecode.Filter
     String mname = method.getName();
     int mmods = method.getModifiers();
     if ((mmods & modmask) != modifiers
+        || (mmods & Access.SYNTHETIC) != 0
 	|| ! mname.startsWith(name))
       return false;
     int mlen = mname.length();
@@ -273,7 +285,9 @@ class MethodFilter implements gnu.bytecode.Filter
 	&& (mlen != nlen + 4
 	    || ! mname.endsWith("$V$X")))
       return false;
+    ClassType declaring = receiver instanceof ClassType ? (ClassType) receiver
+      : method.getDeclaringClass();
     return caller == null
-      || caller.isAccessible(method.getDeclaringClass(), mmods);
+      || caller.isAccessible(declaring, receiver, method.getModifiers());
   }
 }
