@@ -63,7 +63,8 @@ public class FieldLocation extends ClassMemberLocation
             flags |= CONSTANT;
             if (ftype.isSubtype(Compilation.typeProcedure))
               flags |= PROCEDURE;
-            if (ftype.isSubtype(ClassType.make("kawa.lang.Syntax")))
+            if (ftype instanceof ClassType
+                && ((ClassType) ftype).isSubclass("kawa.lang.Syntax"))
               flags |= SYNTAX;
           }
         else
@@ -122,6 +123,11 @@ public class FieldLocation extends ClassMemberLocation
     super(instance, type, mname);
   }
 
+  public FieldLocation (Object instance, java.lang.reflect.Field field)
+  {
+    super(instance, field);
+    type = (ClassType) Type.make(field.getDeclaringClass());
+  }
 
   public void setDeclaration (Declaration decl)
   {
@@ -151,7 +157,7 @@ public class FieldLocation extends ClassMemberLocation
 	gnu.bytecode.Field procField = t.getDeclaredField(fname);
 	if (procField == null)
 	  return null;
-        ModuleInfo info = ModuleInfo.find(t.getName());
+        ModuleInfo info = ModuleInfo.find(t);
         ModuleExp mexp = info.getModuleExp();
         for (d = mexp.firstDecl();  d != null; d = d.nextDecl())
           {
@@ -264,6 +270,35 @@ public class FieldLocation extends ClassMemberLocation
       }
   }
 
+  public Object setWithSave (Object newValue, CallContext ctx)
+  {
+    if ((flags & KIND_FLAGS_SET) == 0)
+      setKindFlags();
+    if ((flags & INDIRECT_LOCATION) == 0)
+      return super.setWithSave(newValue, ctx);
+    else
+      {
+	Object v;
+	if ((flags & VALUE_SET) != 0)
+	  v = value;
+	else
+	  {
+	    flags |= VALUE_SET;
+	    v = getFieldValue();
+	    value = v;
+	  }
+	return ((Location) v).setWithSave(newValue, ctx);
+      }
+  }
+
+  public void setRestore (Object oldValue, CallContext ctx)
+  {
+    if ((flags & INDIRECT_LOCATION) == 0)
+      super.setRestore(oldValue, ctx);
+    else
+      ((Location) value).setRestore(oldValue, ctx);
+  }
+
   public boolean isConstant ()
   {
     if ((flags & KIND_FLAGS_SET) == 0)
