@@ -63,13 +63,19 @@ public class SlotGet extends Procedure2
           isName = mname;
         fname = null;
       }
-    else if (! (arg2 instanceof String) && ! (arg2 instanceof FString))
-      throw new WrongType(this, 2, arg2, "string");
-    else
+    else if (arg2 instanceof SimpleSymbol
+             /* #ifdef use:java.lang.CharSequence */
+             || arg2 instanceof CharSequence
+             /* #else */
+             // || arg2 instanceof gnu.lists.CharSeq || arg2 instanceof String
+             /* #endif */
+             )
       {
         name = arg2.toString();
         fname = gnu.expr.Compilation.mangleNameIfNeeded(name);
       }
+    else
+      throw new WrongType(this, 2, arg2, "string");
     // "intern" fname if it is "class" or "length":
     if ("class".equals(fname))
       fname = "class";
@@ -198,7 +204,7 @@ public class SlotGet extends Procedure2
    * @param name the source (unmangled) name of the property.
    */
   public static Member
-  lookupMember (ClassType clas, String name, ClassType caller)
+  lookupMember (ObjectType clas, String name, ClassType caller)
   {
     gnu.bytecode.Field field
       = clas.getField(Compilation.mangleNameIfNeeded(name), -1);
@@ -206,8 +212,7 @@ public class SlotGet extends Procedure2
       {
         if (caller == null)
           caller = Type.pointer_type;
-        if (caller.isAccessible(field.getDeclaringClass(),
-                                field.getModifiers()))
+        if (caller.isAccessible(field, clas))
           return field;
       }
 
@@ -220,8 +225,10 @@ public class SlotGet extends Procedure2
       return method;
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Compilation comp = walker.getCompilation();
     Language language = comp.getLanguage();
     Type type;
@@ -271,27 +278,27 @@ public class SlotGet extends Procedure2
         if (part instanceof gnu.bytecode.Field)
           {
             gnu.bytecode.Field field = (gnu.bytecode.Field) part;
-            ctype = field.getDeclaringClass();
             int modifiers = field.getModifiers();
             boolean isStaticField = (modifiers & Access.STATIC) != 0;
             if (isStatic && ! isStaticField)
               return new ErrorExp("cannot access non-static field `" + name
                                   + "' using `" + getName() + '\'', comp);
-	    if (caller != null && ! caller.isAccessible(ctype, modifiers))
-	      return new ErrorExp("field "+ctype.getName()+'.'+name
-                                  +" is not accessible here", comp);
+	    if (caller != null
+                && ! caller.isAccessible(field, ctype))
+	      return new ErrorExp("field "+field.getDeclaringClass().getName()
+                                  +'.'+name+" is not accessible here", comp);
           }
 
         else if (part instanceof gnu.bytecode.Method)
           {
             gnu.bytecode.Method method = (gnu.bytecode.Method) part;
-            ctype = method.getDeclaringClass();
+            ClassType dtype = method.getDeclaringClass();
 	    int modifiers = method.getModifiers();
             boolean isStaticMethod = method.getStaticFlag();
             if (isStatic && ! isStaticMethod)
               return new ErrorExp("cannot call non-static getter method `"
                                   + name + "' using `" + getName() + '\'', comp);
-	    if (caller != null && ! caller.isAccessible(ctype, modifiers))
+	    if (caller != null && ! caller.isAccessible(dtype, ctype, modifiers))
 	      return new ErrorExp( "method "+method +" is not accessible here", 
                                    comp);
           }
@@ -327,7 +334,7 @@ public class SlotGet extends Procedure2
                            QuoteExp.getInstance(isName),
                            QuoteExp.getInstance(language)});
         nexp.setLine(exp);
-        return ((InlineCalls) walker).walkApplyOnly(nexp);
+        return walker.walkApplyOnly(nexp);
       }
     return exp;
   }
