@@ -102,12 +102,13 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
 
   public Object apply3 (Object obj, Object fname, Object value)
   {
-    apply(isStatic, obj, (String) fname, value);
+    // Should actually check type rather than just calling toString.  FIXME.
+    apply(isStatic, obj, fname.toString(), value);
     return returnSelf ? obj : Values.empty;
   }
 
   public static Member
-  lookupMember (ClassType clas, String name, ClassType caller)
+  lookupMember (ObjectType clas, String name, ClassType caller)
   {
     gnu.bytecode.Field field
       = clas.getField(Compilation.mangleNameIfNeeded(name), -1);
@@ -115,8 +116,7 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
       {
         if (caller == null)
           caller = Type.pointer_type;
-        if (caller.isAccessible(field.getDeclaringClass(),
-                                field.getModifiers()))
+        if (caller.isAccessible(field, clas))
           return field;
       }
 
@@ -170,14 +170,16 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
       }
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     // Unlike, for SlotGet, we do the field-lookup at compile time
     // rather than inline time.  The main reason is that optimizing
     // (set! CLASS-OR-OBJECT:FIELD-NAME VALUE) is tricky, since (currently)
     // afte we've inlined setter, this method doesn't get called.
     if (isStatic && walker.getCompilation().mustCompile)
-      return Invoke.inlineClassName (exp, 0, (InlineCalls) walker);
+      return Invoke.inlineClassName (exp, 0, walker);
     else
       return exp;
   }
@@ -228,10 +230,10 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
 	if (part != null)
 	  {
 	    int modifiers = part.getModifiers();
-            ClassType ptype = part.getDeclaringClass();
 	    boolean isStaticField = (modifiers & Access.STATIC) != 0;
-	    if (caller != null && ! caller.isAccessible(ptype, modifiers))
-	      comp.error('e', "slot '"+name +"' in "+ptype.getName()
+	    if (caller != null && ! caller.isAccessible(part, ctype))
+	      comp.error('e', "slot '"+name
+                         +"' in "+part.getDeclaringClass().getName()
 			 +" not accessible here");
 	    args[0].compile(comp,
 			    isStaticField ? Target.Ignore
