@@ -42,8 +42,10 @@ public class TypeSwitch extends MethodProc implements CanInline, Inlineable
     defaultProc.check1(selector, ctx);
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     for (int i = 1;  i < args.length;  i++)
       {
@@ -52,6 +54,7 @@ public class TypeSwitch extends MethodProc implements CanInline, Inlineable
 	    LambdaExp lexp = (LambdaExp) args[i];
 	    lexp.setInlineOnly(true);
 	    lexp.returnContinuation = exp;
+            lexp.inlineHome = walker.getCurrentLambda();
 	  }
       }
     return exp;
