@@ -106,57 +106,58 @@ public class KawaPageServlet extends KawaServlet
 	return null;
       }
 
-    URLConnection connection = url.openConnection();
     String urlString = url.toExternalForm();
-    long lastModified = connection.getLastModified();
     if (minfo == null || ! urlString.equals(minfo.getSourceAbsPathname()))
       minfo = mmanager.findWithURL(url);
-    if (minfo.lastModifiedTime == lastModified)
-      {
-        minfo.lastCheckedTime = now;
-        return mcontext.findInstance(minfo);
-      }
+    if (minfo.checkCurrent(mmanager, now))
+      return mcontext.findInstance(minfo);
 
-    minfo.lastModifiedTime = lastModified;
-    minfo.lastCheckedTime = now;
     mmap.put(path, minfo);
 
-    InputStream resourceStream = connection.getInputStream();
+    Path absPath = minfo.getSourceAbsPath();
+    InputStream resourceStream = absPath.openInputStream();
             
     Language language
       = Language.getInstanceFromFilenameExtension(path);
-    if (language == null)
-      language = Language.detect(resourceStream);
-    if (language == null)
+    if (language != null)
+      ctx.getServletContext().log("Compile "+path+" - a "+language.getName()+" source file (based on extension)");
+    else
       {
-        if (path != upath)
+        language = Language.detect(resourceStream);
+        if (language != null)
+          ctx.getServletContext().log("Compile "+path+" - a "+language.getName()+" source file (detected from content)");
+        else
           {
-            ctx.response.reset();
-            ctx.response.sendError(HttpServletResponse.SC_NOT_FOUND, path);
+            if (path != upath)
+              {
+                ctx.response.reset();
+                ctx.response.sendError(HttpServletResponse.SC_NOT_FOUND, path);
+                return null;
+              }
+            String contentType = context.getMimeType(path);
+            response.setContentType(contentType);
+            ServletOutputStream out = response.getOutputStream();
+            byte[] buffer = new byte[4*1024];
+            for (;;)
+              {
+                int n = resourceStream.read(buffer);
+                if (n < 0)
+                  break;
+                out.write(buffer, 0, n);
+              }
+            resourceStream.close();
             return null;
           }
-        String contentType = context.getMimeType(path);
-        response.setContentType(contentType);
-        ServletOutputStream out = response.getOutputStream();
-        byte[] buffer = new byte[4*1024];
-        for (;;)
-          {
-            int n = resourceStream.read(buffer);
-            if (n < 0)
-              break;
-            out.write(buffer, 0, n);
-          }
-        resourceStream.close();
-        return null;
       }
 
-    InPort port = new InPort(resourceStream, minfo.getSourceAbsPath());
+    InPort port = new InPort(resourceStream, absPath);
     Language.setDefaultLanguage(language);
     SourceMessages messages = new SourceMessages();
     Compilation comp;
     try
       {
-        comp = language.parse(port, messages, Language.PARSE_IMMEDIATE);
+        comp = language.parse(port, messages, minfo);
+        comp.immediate = true;
         /*
         int dot = path.indexOf('.');
         if (dot < 0)
@@ -164,7 +165,6 @@ public class KawaPageServlet extends KawaServlet
         String name = path.substring(path.lastIndexOf('/')+1, dot);
         comp.getModule().setName(name);
         */
-        language.resolve(comp);
       }
     catch (SyntaxException ex)
       {
@@ -178,10 +178,9 @@ public class KawaPageServlet extends KawaServlet
     if (! messages.seenErrors())
       {
         ModuleExp mexp = comp.getModule();
-        comp.addMainClass(mexp);
-        comp.walkModule(mexp);
-        comp.setState(Compilation.WALKED);
-        cl = ModuleExp.evalToClass(comp, url);
+        // FIXME - the env should probably be session-specific.
+        Environment env = Environment.getCurrent();
+        cl = (Class) ModuleExp.evalModule1(env, comp, url, null);
       }
 
     // FIXME: we could output a nice pretty HTML table of the errors
@@ -197,8 +196,7 @@ public class KawaPageServlet extends KawaServlet
         return null;
       }
 
-    minfo.moduleClass = cl;
-    minfo.className = cl.getName();
+    minfo.setModuleClass(cl);
 
     if (saveClass)
       comp.outputClass(context.getRealPath("WEB-INF/classes")+'/');
