@@ -7,10 +7,13 @@ import javax.servlet.http.*;
 import gnu.mapping.*;
 import gnu.xml.*;
 import java.io.IOException;
+import gnu.text.Path;
 
 public abstract class KawaServlet
-extends HttpServlet implements CpsMethodContainer
+extends HttpServlet
 {
+  public abstract void run(CallContext ctx) throws Throwable;
+
    public void doPost (HttpServletRequest request,
 		       HttpServletResponse response)
      throws ServletException, IOException
@@ -35,12 +38,13 @@ extends HttpServlet implements CpsMethodContainer
     ctx.consumer = new ServletPrinter(response);  // FIXME - should re-use
     ctx.request = request;
     ctx.response = response;
+    Path.setCurrentPath(Path.valueOf(request.getRequestURL().toString()));
     ctx.servlet = this;
     ctx.values = Values.noArgs;
 
-    /* FIXME should use fluid binding!
-    gnu.expr.Interpreter interp = gnu.expr.Interpreter.getInterpreter();
-    String lang = interp.getName();
+    /* FIXME should use fluid binding - or ThreadLocation!
+    gnu.expr.Language language = gnu.expr.Language.getDefaultLanguage();
+    String lang = language.getName();
     Environment env = Environment.getCurrent();
     if (lang == "XQuery")
       {
@@ -58,11 +62,10 @@ extends HttpServlet implements CpsMethodContainer
       }
     */
 
-    ctx.consumer.beginDocument();
+    ctx.consumer.startDocument();
     try
       {
-	apply(ctx);
-	ctx.run();
+	run(ctx);
       }
     catch (Throwable throwable)
       {
@@ -78,11 +81,5 @@ extends HttpServlet implements CpsMethodContainer
       }
     ctx.consumer.endDocument();
   }
-
-  public void apply(CpsMethodProc proc, CallContext context)
-  {
-  }
-
-  public abstract void apply(CallContext context)  throws Throwable;
 }
 
