@@ -1,4 +1,4 @@
-;; Copyright (c) 2005, 2006 Per Bothner
+;; Copyright (c) 2005, 2006, 2007 Per Bothner
 ;;
 ;; Permission is hereby granted, free of charge, to any person
 ;; obtaining a copy of this software and associated documentation
@@ -108,7 +108,7 @@
 		(> (vector-length obj) 1)
 		(eq (vector-ref obj 0) %test-runner-cookie)))
 	 (define (alloc)
-	   (let ((runner (make-vector 22)))
+	   (let ((runner (make-vector 23)))
 	     (vector-set! runner 0 %test-runner-cookie)
 	     runner))
 	 (begin
@@ -156,6 +156,7 @@
 )
 
 (define (test-runner-reset runner)
+    (test-result-alist! runner '())
     (test-runner-pass-count! runner 0)
     (test-runner-fail-count! runner 0)
     (test-runner-xpass-count! runner 0)
@@ -781,23 +782,34 @@
  (kawa
   (define-syntax %test-error
     (syntax-rules ()
+      ((%test-error r #t expr)
+       (cond ((%test-on-test-begin r)
+	      (test-result-set! r 'expected-error #t)
+	      (%test-on-test-end r
+				 (try-catch
+				  (let ()
+				    (test-result-set! r 'actual-value expr)
+				    #f)
+				  (ex <java.lang.Throwable>
+				      (test-result-set! r 'actual-error ex)
+				      #t)))
+	      (%test-report-result))))
       ((%test-error r etype expr)
-       (let ()
-	 (if (%test-on-test-begin r)
-	     (let ((et etype))
-	       (test-result-set! r 'expected-error et)
-	       (%test-on-test-end r
-				  (try-catch
-				   (let ()
-				     (test-result-set! r 'actual-value expr)
-				     #f)
-				   (ex <java.lang.Throwable>
-				       (test-result-set! r 'actual-error ex)
-				       (cond ((and (instance? et <gnu.bytecode.ClassType>)
-						   (gnu.bytecode.ClassType:isSubclass et <java.lang.Throwable>))
-					      (instance? ex et))
-					     (else #t)))))
-	       (%test-report-result))))))))
+       (if (%test-on-test-begin r)
+	   (let ((et etype))
+	     (test-result-set! r 'expected-error et)
+	     (%test-on-test-end r
+				(try-catch
+				 (let ()
+				   (test-result-set! r 'actual-value expr)
+				   #f)
+				 (ex <java.lang.Throwable>
+				     (test-result-set! r 'actual-error ex)
+				     (cond ((and (instance? et <gnu.bytecode.ClassType>)
+						 (gnu.bytecode.ClassType:isSubclass et <java.lang.Throwable>))
+					    (instance? ex et))
+					   (else #t)))))
+	     (%test-report-result)))))))
  ((and srfi-34 srfi-35)
   (define-syntax %test-error
     (syntax-rules ()
