@@ -1,4 +1,4 @@
-// Copyright (c) 2005  Per M.A. Bothner
+// Copyright (c) 2005, 2009  Per M.A. Bothner
 // This is free software;  for terms and warranty disclaimer see COPYING.
 
 package gnu.kawa.util;
@@ -17,18 +17,19 @@ public class PreProcess
   int lineno;
 
   static final String JAVA4_FEATURES = "+JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio";
-  static final String NO_JAVA4_FEATURES = "-use:java.util.IdentityHashMap -use:java.lang.CharSequence -use:java.lang.Throwable.getCause -use:java.net.URI -use:java.util.regex -use:org.w3c.dom.Node -JAXP-1.3 -JAVA5 -JAVA6 -JAXP-QName -use:java.text.Normalizer -SAX2 -use:java.nio";
-  static final String JAVA5_FEATURES = "+JAVA5 "+JAVA4_FEATURES+" +use:org.w3c.dom.Node +JAXP-1.3 -JAXP-QName";
+  static final String NO_JAVA4_FEATURES = "-use:java.util.IdentityHashMap -use:java.lang.CharSequence -use:java.lang.Throwable.getCause -use:java.net.URI -use:java.util.regex -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAVA5 -JAVA6 -JAXP-QName -use:java.text.Normalizer -SAX2 -use:java.nio -Android";
+  static final String JAVA5_FEATURES = "+JAVA5 "+JAVA4_FEATURES+" +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName";
 
   static String[] version_features = {
     "java1", "-JAVA2 "+NO_JAVA4_FEATURES,
     "java2", "+JAVA2 "+NO_JAVA4_FEATURES,
     // We don't use Node for java4 because there are inconsistencies between
     // the version of DOM used in Java4 and the one in Java 5 (and GCJ).
-    "java4", "-JAVA5 "+JAVA4_FEATURES+" -use:org.w3c.dom.Node -JAXP-1.3 -JAXP-QName -JAVA6 -use:java.text.Normalizer",
-    "java4x", "-JAVA5 "+JAVA4_FEATURES+" +use:org.w3c.dom.Node +JAXP-1.3 -JAXP-QName -JAVA6 -use:java.text.Normalizer",
-    "java5", JAVA5_FEATURES+" -JAVA6 -use:java.text.Normalizer",
-    "java6", JAVA5_FEATURES+" +JAVA6 +use:java.text.Normalizer",
+    "java4", "-JAVA5 "+JAVA4_FEATURES+" -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAXP-QName -JAVA6 -use:java.text.Normalize -Android",
+    "java4x", "-JAVA5 "+JAVA4_FEATURES+" +use:org.w3c.dom.Node +JAXP-1.3 +use:javax.xml.transform -JAXP-QName -JAVA6 -use:java.text.Normalizer -Android",
+    "java5", JAVA5_FEATURES+" -JAVA6 -use:java.text.Normalizer -Android",
+    "java6", JAVA5_FEATURES+" +JAVA6 +use:java.text.Normalizer -Android",
+    "android", "+JAVA5 "+JAVA4_FEATURES+" +use:org.w3c.dom.Node +JAXP-1.3 -JAXP-QName -use:javax.xml.transform -JAVA6 -use:java.text.Normalizer +Android",
   };
 
   void error(String msg)
