@@ -28,6 +28,25 @@ public class XmlNamespace extends Namespace
       }
   }
 
+  /* #ifdef JAVA5 */
+  /** Emitted by compiler to handle literals. */
+  public static XmlNamespace make (String name, String prefix)
+  {
+    return getInstance(prefix, name);
+  }
+  /* #else */
+  // /** Only for use by compiler to handle literals. */
+  // public XmlNamespace (String uri, String prefix)
+  // {
+  //   setName(uri.intern());
+  //   prefix = prefix.intern();
+  // }
+  /* #endif */
+  /** Only for use when serializing. */
+  public XmlNamespace ()
+  {
+  }
+
   public Object get (String name)
   {
     ElementType type = ElementType.make(getSymbol(name));
