@@ -162,7 +162,7 @@ public class Symbol
    * {@code "prefix:name=value"} - such as on the Kawa command-line -
    * where we don't yet know the namespace-uri.  Code that later looks
    * for a value should look both under the true namespace-uri and
-   less LOG.JA   * using this method (or {@link Namespace#makeUnknownNamespace}).
+   * the prefix combined with {@link Namespace#makeUnknownNamespace(prefix)}.
    */
   public static Symbol makeWithUnknownNamespace (String local, String prefix)
   {
