@@ -33,7 +33,11 @@ public class Values extends TreeList implements Printable, Externalizable
     return isEmpty() ? noArgs : toArray();
   }
 
-  public static Object values$V(Object[] vals)
+  /* #ifdef JAVA5 */
+  public static Object values(Object... vals)
+  /* #else */
+  // public static Object values$V(Object[] vals)
+  /* #endif */
   {
     return make(vals);
   }
@@ -163,7 +167,7 @@ public class Values extends TreeList implements Printable, Externalizable
     return isEmpty() ? empty : this;
   }
 
-  /** Helper method called by code using a SeriesTarget.
+  /** Helper method called by compiled code.
    * The compiled code iterates through zero or more values.
    * Return the index of the next value, or -1 if currently at eof.
    * A non-Values object is treated as a singleton value,
@@ -177,7 +181,7 @@ public class Values extends TreeList implements Printable, Externalizable
       return curIndex == 0 ? 1 : -1;
   }
 
-  /** Helper method called by code using a SeriesTarget.
+  /** Helper method called by compiled code.
    * The compiled code iterates through zero or more values.
    * Extract the object referenced by the curIndex.
    * A non-Values object is treated as a singleton value.
