@@ -87,6 +87,19 @@ public class IntNum extends RatNum implements Externalizable
     return result;
   }
 
+  public static IntNum asIntNumOrNull (Object value)
+  {
+    if (value instanceof IntNum)
+      return (IntNum) value;
+    if (value instanceof BigInteger)
+      return IntNum.valueOf(value.toString(), 10);
+    if (value instanceof Number
+        && (value instanceof Integer || value instanceof Long
+            || value instanceof Short || value instanceof Byte))
+      return IntNum.make(((Number) value).longValue());
+    return null;
+  }
+
   /** Make an IntNum from an unsigned 64-bit value. */
   public static IntNum makeU (long value)
   {
@@ -955,13 +968,17 @@ public class IntNum extends RatNum implements Externalizable
 	  return IntNum.abs(x);
 	yval = 1;
       }
-    int len = (xval > yval ? xval : yval) + 1;
+    int len = xval > yval ? xval : yval;
     int[] xwords = new int[len];
     int[] ywords = new int[len];
     x.getAbsolute (xwords);
     y.getAbsolute (ywords);
     len = MPN.gcd (xwords, ywords, len);
     IntNum result = new IntNum (0);
+    // If the high-order bit in the result is set, the number needs one more
+    // word to make the 2^er complement positive.
+    if (xwords[len-1] < 0)
+      xwords[len++] = 0;
     result.ival = len;
     result.words = xwords;
     return result.canonicalize ();
@@ -1081,12 +1098,49 @@ public class IntNum extends RatNum implements Externalizable
     return result.canonicalize ();
   }
 
+  /* #ifdef JAVA5 */
   public void format (int radix, StringBuffer buffer)
+  {
+    if (radix == 10)
+      {
+        if (words == null)
+          {
+            buffer.append(ival);
+            return;
+          }
+        else if (ival <= 2)
+          {
+            buffer.append(longValue());
+            return;
+          }
+      }
+    buffer.append(toString(radix));
+  }
+  /* #endif */
+
+  public void format (int radix,
+                      /* #ifdef JAVA5 */
+                      StringBuilder buffer
+                      /* #else */
+                      // StringBuffer buffer
+                      /* #endif */
+                      )
   {
     if (words == null)
-      buffer.append(Integer.toString (ival, radix));
+      {
+        if (radix == 10)
+          buffer.append(ival);
+        else
+          buffer.append(Integer.toString (ival, radix));
+      }
     else if (ival <= 2)
-      buffer.append(Long.toString (longValue (), radix));
+      {
+        long lval = longValue();
+        if (radix == 10)
+          buffer.append(lval);
+        else
+          buffer.append(Long.toString (lval, radix));
+      }
     else
       {
 	boolean neg = isNegative ();
@@ -1119,12 +1173,32 @@ public class IntNum extends RatNum implements Externalizable
 	  }
 	else
 	  {
+            int chars_per_word = MPN.chars_per_word(radix);
+            int wradix = radix;
+            for (int j = chars_per_word; --j > 0; )
+              wradix = wradix * radix;
 	    int i = buffer.length();
 	    for (;;)
 	      {
-		int digit = MPN.divmod_1 (work, work, len, radix);
-		buffer.append (Character.forDigit (digit, radix));
+		int wdigit = MPN.divmod_1(work, work, len, wradix);
 		while (len > 0 && work[len-1] == 0) len--;
+                for (int j = chars_per_word; --j >= 0; ) {
+                  int digit;
+                  if (len == 0 && wdigit == 0)
+                    break;
+                  if (wdigit < 0) // Overflow
+                    {
+                      long ldigit = (long) wdigit & 0xFFFFFFFF;
+                      digit = (int)(ldigit % radix);
+                      wdigit = (int) (wdigit / radix);
+                    }
+                  else
+                    {
+                      digit = wdigit % radix;
+                      wdigit = wdigit / radix;
+                    }
+                  buffer.append (Character.forDigit(digit, radix));
+                }
 		if (len == 0)
 		  break;
 	      }
@@ -1150,7 +1224,11 @@ public class IntNum extends RatNum implements Externalizable
     else if (ival <= 2)
       return Long.toString (longValue (), radix);
     int buf_size = ival * (MPN.chars_per_word (radix) + 1);
-    StringBuffer buffer = new StringBuffer (buf_size);
+    /* #ifdef JAVA5 */
+    StringBuilder buffer = new StringBuilder (buf_size);
+    /* #else */
+    // StringBuffer buffer = new StringBuffer (buf_size);
+    /* #endif */
     format(radix, buffer);
     return buffer.toString ();
   }
@@ -1554,7 +1632,7 @@ public class IntNum extends RatNum implements Externalizable
     throws IOException, ClassNotFoundException
   {
     int i = in.readInt();
-    if (ival <= (int) 0xC0000000)
+    if (i <= (int) 0xC0000000)
       {
 	i &= 0x7fffffff;
 	if (i == 1)
