@@ -1,7 +1,7 @@
 package gnu.text;
 import java.io.*;
-import java.util.Hashtable;
 import gnu.lists.Consumer;
+import gnu.kawa.util.*;
 
 /**
  * A wrapper for characters.
@@ -35,7 +35,7 @@ public class Char
   {
   }
 
-  private Char (int ch)
+  Char (int ch)
   {
     value = ch;
   }
@@ -73,8 +73,7 @@ public class Char
 
   static Char[] ascii;
 
-  static Char temp = new Char (0);
-  static Hashtable hashTable;
+  static CharMap hashTable = new CharMap();
 
   static
   {
@@ -87,18 +86,9 @@ public class Char
   {
     if (ch < 128)
       return ascii[ch];
-    else
+    synchronized (hashTable)
       {
-	// Re-writing this will allow equals to just use ==.  FIXME.
-	temp.value = ch;
-	if (hashTable == null)
-	  hashTable = new Hashtable ();
-	Object entry = hashTable.get (temp);
-	if (entry != null)
-	  return (Char) entry;
-	Char newChar = new Char (ch);
-	hashTable.put (newChar, newChar);
-	return newChar;
+        return hashTable.get(ch);
       }
   }
 
@@ -110,9 +100,10 @@ public class Char
       && ((Char)obj).intValue() == value;
   }
 
-  static char[] charNameValues = { ' ', '\t', '\n', '\n'
-				   ,'\r', '\f', '\b',
-				   '\033', '\177', '\177', '\007', '\0' };
+  static char[] charNameValues = { ' ', '\t', '\n', '\n',
+				   '\r', '\f', '\b', '\033',
+                                   '\177', '\177', '\177',
+                                   '\007', '\007', '\013', '\0' };
   static String[] charNames = { "space",
 				"tab",
 				"newline",
@@ -121,9 +112,12 @@ public class Char
 				"page",
 				"backspace",
 				"esc",
+				"delete",
 				"del",
 				"rubout",
+				"alarm",
 				"bel",
+				"vtab",
 				"nul"};
 
   public static int nameToChar(String name)
@@ -216,14 +210,10 @@ public class Char
 	    return sbuf.toString();
 	  }
       }
-    if (ch < 8)
+    if (ch < ' ' || ch > 0x7F)
       {
-	sbuf.append('0');  // make sure there at least two octal digits
-	sbuf.append(ch);
-      }
-    else if (ch < ' ' || ch > 0x7F)
-      {
-	sbuf.append(Integer.toString(ch, 8));
+        sbuf.append('x');
+        sbuf.append(Integer.toString(ch, 16));
       }
     else
       sbuf.append((char) ch);
@@ -232,8 +222,9 @@ public class Char
 
   /**
    * @serialData Writes the char value as a char.
-   *   If the value is > 0xFFFF, write a pair of surrogate values.
-   *   If the value is is a high surrogate only, write it followed by '\0'.
+   *   If the value is {@code > 0xFFFF}, write a pair of surrogate values.
+   *   If the value is is a high surrogate only,
+   *   write it followed by {@code '\0'}.
    */
   public void writeExternal(ObjectOutput out) throws IOException
   {
@@ -275,3 +266,35 @@ public class Char
     return value - ((Char) o).value;
   }
 }
+
+/** Helper class for mapping Unicode scalar value to Char object. */
+
+class CharMap extends AbstractWeakHashTable<Char,Char>
+{
+  public Char get (int key)
+  {
+    cleanup();
+    int hash = key;
+    int index = hashToIndex(hash);
+    for (WeakHashNode<Char,Char> node = table[index];
+	 node != null;  node = node.next)
+      {
+        Char val = getEntryValue(node);
+        if (val.intValue() == key)
+          return val;
+      }
+    Char val = new Char(key);
+    super.put(val, val);
+    return val;
+  }
+
+  protected Char getKeyFromValue (Char ch)
+  {
+    return ch;
+  }
+
+  protected boolean matches (Char oldValue, Char newValue)
+  {
+    return oldValue.intValue() == newValue.intValue();
+  }
+}
