@@ -1171,7 +1171,8 @@ public class PrettyWriter extends java.io.Writer
       {
 	throw new RuntimeException(ex.toString());
       }
-    bufferStartColumn += count;
+    bufferFillPointer = count; // For the sake of the following:
+    bufferStartColumn = getColumnNumber();
     System.arraycopy(buffer, count, buffer, 0, newFillPtr);
     bufferFillPointer = newFillPtr;
     bufferOffset += count;
@@ -1215,6 +1216,17 @@ public class PrettyWriter extends java.io.Writer
     buffer = null;
   }
 
+  /** Flush and close this local Writer, but not underlying Writers. */
+  public void closeThis()  throws IOException
+  {
+    if (out != null)
+      { 
+	forcePrettyOutput();
+        out = null;
+      }
+    buffer = null;
+  }
+
   /** Not meaningful if {@code prettyPrintingMode > 0}. */
   public int getColumnNumber ()
   {
