@@ -7,6 +7,8 @@ package gnu.text;
  * Has a "current line number" which clients can use as the default line
  * number, or clients can explicitly provide a line number.
  * Does not handle localization of messages.
+ *
+ * Similar functionality as JAVA6's javax.tools.DiagnosticCollector.
  */
 
 public class SourceMessages implements SourceLocator
@@ -165,6 +167,14 @@ public class SourceMessages implements SourceLocator
 			  current_line, current_column, message));
   }
 
+  public void error(char severity, String message, Throwable exception)
+  {
+    SourceError err = new SourceError(severity, current_filename,
+                                      current_line, current_column, message);
+    err.fakeException = exception;
+    error(err);
+  }
+
   public void error(char severity, String message, String code)
   {
     SourceError err = new SourceError(severity, current_filename,
@@ -250,6 +260,13 @@ public class SourceMessages implements SourceLocator
     this.locator = locator == this ? null : locator;
   }
 
+  public final SourceLocator swapSourceLocator (SourceLocator locator)
+  {
+    SourceLocator save = this.locator;
+    this.locator = locator;
+    return save;
+  }
+
   /** Copies the current position of locator. */
   public final void setLocation (SourceLocator locator)
   {
