@@ -411,7 +411,7 @@ public class XMLPrinter extends OutPort
                 // If we're not canonicalizing, we just want to suppress
                 // duplicates, rather than putting them in order.
                 // Note we put the bindings in reverse order, since that's
-                // the following print loop expects.
+                // what the following print loop expects.
                 if (! sortNamespaces)
                   continue;
                 if (prefix == null)
@@ -643,9 +643,9 @@ public class XMLPrinter extends OutPort
   public static String formatDecimal (BigDecimal dec)
   {
     /* #ifdef JAVA5 */
-    // return formatDecimal(dec.toPlainString());
+    return formatDecimal(dec.toPlainString());
     /* #else */
-    return formatDecimal(dec.toString());
+    // return formatDecimal(dec.toString());
     /* #endif */
   }
 
