@@ -7,17 +7,17 @@ import gnu.kawa.xml.*;
 import gnu.xml.*;
 import gnu.mapping.*;
 import gnu.bytecode.*;
+import gnu.kawa.util.HashNode;
 import gnu.kawa.reflect.StaticFieldLocation;
+import gnu.kawa.reflect.SingletonType;
 import gnu.kawa.functions.GetNamedPart;
 import gnu.xquery.util.NamedCollator;
 import gnu.xquery.util.QNameUtils;
-import java.util.Vector;
-import gnu.math.DateTime;
-import gnu.math.IntNum;
+import kawa.standard.Scheme;
 
 public class XQResolveNames extends ResolveNames
 {
-  XQParser parser;
+  public XQParser parser;
 
   /** Code number for the special <code>last</code> function. */
   public static final int LAST_BUILTIN = -1;
@@ -25,8 +25,8 @@ public class XQResolveNames extends ResolveNames
   /** Code number for the special <code>position</code> function. */
   public static final int POSITION_BUILTIN = -2;
 
-  /** Value of {@code xs:QName()} constructor. */
-  public static final int XS_QNAME_BUILTIN = -3;
+  /** Code number for internal function to handle extensions. */
+  public static final int HANDLE_EXTENSION_BUILTIN = -3;
 
   /** Code number for the special <code>compare</code> function. */
   public static final int COMPARE_BUILTIN = -4;
@@ -106,8 +106,12 @@ public class XQResolveNames extends ResolveNames
   public static final int CAST_AS_BUILTIN = -33;
   public static final int CASTABLE_AS_BUILTIN = -34;
 
-  /** Code number for internal function to handle extensions. */
-  public static final int HANDLE_EXTENSION_BUILTIN = -35;
+  /** Value of {@code xs:QName()} constructor. */
+  public static final int XS_QNAME_BUILTIN = -35;
+
+  /** Like {@code XS_QNAME_BUILTIN}, but ignore the default
+   * element namespace.  The is appropriate fro resolving atributes. */
+  public static final int XS_QNAME_IGNORE_DEFAULT_BUILTIN = -36;
 
   public static final Declaration handleExtensionDecl
     = makeBuiltin("(extension)", HANDLE_EXTENSION_BUILTIN);
@@ -125,6 +129,9 @@ public class XQResolveNames extends ResolveNames
   public static final Declaration xsQNameDecl
     = makeBuiltin(Symbol.make(XQuery.SCHEMA_NAMESPACE, "QName"), XS_QNAME_BUILTIN);
 
+  public static final Declaration xsQNameIgnoreDefaultDecl
+    = makeBuiltin(Symbol.make(XQuery.SCHEMA_NAMESPACE, "(QName-ignore-default)"), XS_QNAME_IGNORE_DEFAULT_BUILTIN);
+
   public static final Declaration staticBaseUriDecl
     = makeBuiltin("static-base-uri", STATIC_BASE_URI_BUILTIN);
 
@@ -270,7 +277,8 @@ public class XQResolveNames extends ResolveNames
     if (exp.getBinding() == null)
       {
 	Object symbol = exp.getSymbol();
-        boolean function = exp.isProcedureName();
+        boolean needFunction = exp.isProcedureName();
+        boolean needType = exp.getFlag(ReferenceExp.TYPE_NAME);
         int namespace = call == null ? XQuery.VALUE_NAMESPACE
           : XQuery.namespaceForFunctions(call.getArgCount());
 	Declaration decl = lookup.lookup(symbol, namespace);
@@ -305,16 +313,14 @@ public class XQResolveNames extends ResolveNames
             if (name.indexOf(':') < 0)
               {
                 name = name.intern();
-                if (function)
+                if (needFunction)
                   {
                     for (int i = 0;  i < functionNamespacePath.length;  i++)
                       {
                         sym = functionNamespacePath[i].getSymbol(name);
-                        decl = lookup.lookup(sym, function);
+                        decl = lookup.lookup(sym, namespace);
                         if (decl != null)
                           break;
-                        if (! function)
-                          continue;
                         decl = flookup(sym);
                         if (decl != null)
                           break;
@@ -323,20 +329,26 @@ public class XQResolveNames extends ResolveNames
               }
             if (decl == null)
               {
-                sym = parser.namespaceResolve(name, function);
+                sym = parser.namespaceResolve(name, needFunction);
                 if (sym != null)
                   {
                     decl = lookup.lookup(sym, namespace);
-                    if (decl == null && function)
+                    if (decl == null
+                        && (needFunction || needType))
                       {
                         String uri = sym.getNamespaceURI();
+                        Type type = null;
                         if (XQuery.SCHEMA_NAMESPACE.equals(uri))
                           {
-                            Type type =
-                              XQuery.getStandardType(sym.getName());
-                            if (type != null)
-                              return QuoteExp.getInstance(type);
+                            type = XQuery.getStandardType(sym.getName());
                           }
+                        else if (needType && uri == ""
+                            && ! getCompilation().isPedantic())
+                          {
+                            type = Scheme.string2Type(sym.getName());
+                          }
+                        if (type != null)
+                          return new QuoteExp(type).setLine(exp);
                         if (uri != null && uri.length() > 6 &&
                             uri.startsWith("class:"))
                           {
@@ -350,10 +362,12 @@ public class XQResolveNames extends ResolveNames
           }
         if (decl != null)
           exp.setBinding(decl);
-        else if (function)
+        else if (needFunction)
           error('e', "unknown function "+symbol);
+        else if (needType)
+          messages.error('e', exp, "unknown type "+symbol, "XPST0051");
         else
-          messages.error('e',"unknown variable $"+symbol, "XPST0008");
+          messages.error('e', exp, "unknown variable $"+symbol, "XPST0008");
       }
     return exp;
   }
@@ -421,6 +435,14 @@ public class XQResolveNames extends ResolveNames
       }
     moduleDecl = exp.firstDecl();
     exp.body = walkStatements(exp.body);
+
+    // Remove old hidden declarations, for GC and speed.
+    for (Declaration decl = exp.firstDecl();
+         decl != null;  decl = decl.nextDecl())
+      {
+        if (decl.getSymbol() != null)
+          lookup.removeSubsumed(decl);
+      }
   }
 
   /**
@@ -520,7 +542,6 @@ public class XQResolveNames extends ResolveNames
 	Declaration decl = ((ReferenceExp) func).getBinding();
 	int code;
         Expression err;
-        ModuleExp mexp;
 	if (decl != null && (code = decl.getCode()) < 0)
 	  {
 	    switch (code)
@@ -537,48 +558,80 @@ public class XQResolveNames extends ResolveNames
                   decl.setCanRead(true);
 		return new ReferenceExp(sym, decl);
               case CAST_AS_BUILTIN:
-                {
-		  Expression[] args = exp.getArgs();
-                  if (args[0].valueIfConstant() == Compilation.typeSymbol)
-                    return walkApplyExp(XQParser.castQName(args[1]));
-                  func
-                    = XQParser.makeFunctionExp("gnu.xquery.util.CastAs", "castAs");
-                  return new ApplyExp(func, args);
-                }
               case CASTABLE_AS_BUILTIN:
                 {
 		  Expression[] args = exp.getArgs();
-                  if (args[1].valueIfConstant() == Compilation.typeSymbol
-                      && args[0] instanceof QuoteExp)
+                  Expression texp = args[code == CAST_AS_BUILTIN ? 0 : 1];
+                  Expression qexp = texp;
+                  if (texp instanceof ApplyExp)
                     {
-                      Object value = ((QuoteExp) args[0]).getValue();  
-                      try
-                        {
-                          QNameUtils.resolveQName(value,
-                                                  parser.constructorNamespaces,
-                                                  parser.prologNamespaces);
-                          return XQuery.trueExp;
-                        }
-                      catch (RuntimeException ex)
-                        {	
-                          return XQuery.falseExp;
-                        }
+                      ApplyExp taexp = (ApplyExp) texp;
+                      if (taexp.getFunction().valueIfConstant()
+                          == XQParser.proc_OccurrenceType_getInstance)
+                        qexp = taexp.getArg(0);
                     }
-                  func = XQParser.makeFunctionExp("gnu.xquery.lang.XQParser",                                       "castableAs");
-                  return new ApplyExp(func, args);
+                  Object value = qexp.valueIfConstant();
+                  String msg = null;
+                  if (value == SingletonType.getInstance())
+                    msg = "type to 'cast as' or 'castable as' must be atomic";
+                  else if (value == XDataType.anyAtomicType)
+                    msg = "type to 'cast as' or 'castable as' cannot be anyAtomicType";
+                  else if (value == XDataType.anySimpleType)
+                    msg = "type to 'cast as' or 'castable as' cannot be anySimpleType";
+                  else if (value == XDataType.untypedType)
+                    msg = "type to 'cast as' or 'castable as' cannot be untyped";
+                  else if (value == XDataType.NotationType)
+                    msg = "type to 'cast as' or 'castable as' cannot be NOTATION";
+                  if (msg != null)
+                    messages.error('e', texp, msg, "XPST0080");
+                  boolean toQName = (value == Compilation.typeSymbol
+                                     && ! (texp instanceof ApplyExp));
+                  if (code == CAST_AS_BUILTIN)
+                    {
+                      if (toQName)
+                        return walkApplyExp(XQParser.castQName(args[1], true));
+                      func
+                        = XQParser.makeFunctionExp("gnu.xquery.util.CastAs", "castAs");
+                    }
+                  else
+                    {
+                      if (toQName && args[0] instanceof QuoteExp)
+                        {
+                          value = ((QuoteExp) args[0]).getValue();  
+                          try
+                            {
+                              QNameUtils.resolveQName(value,
+                                                      parser.constructorNamespaces,
+                                                      parser.prologNamespaces);
+                              return XQuery.trueExp;
+                            }
+                          catch (RuntimeException ex)
+                            {	
+                              return XQuery.falseExp;
+                            }
+                        }
+                      func = XQParser.makeFunctionExp("gnu.xquery.lang.XQParser",                                       "castableAs");
+                    }
+                  return new ApplyExp(func, args).setLine(exp);
                 }
 	      case XS_QNAME_BUILTIN:
+	      case XS_QNAME_IGNORE_DEFAULT_BUILTIN:
 		{
 		  Expression[] args = exp.getArgs();
                   if ((err = checkArgCount(args, decl, 1, 1)) != null)
                     return err;
+                  NamespaceBinding constructorNamespaces
+                    = parser.constructorNamespaces;
+                  if (code == XS_QNAME_IGNORE_DEFAULT_BUILTIN)
+                    constructorNamespaces
+                      = new NamespaceBinding(null, "", constructorNamespaces);
 		  if (args[0] instanceof QuoteExp)
 		    {
 		      try
 			{
 			  Object val = ((QuoteExp) args[0]).getValue();
 			  val = QNameUtils.resolveQName(val,
-                                                        parser.constructorNamespaces,
+                                                        constructorNamespaces,
                                                         parser.prologNamespaces);
 			  return new QuoteExp(val);
 			}
@@ -589,7 +642,7 @@ public class XQResolveNames extends ResolveNames
 		    }
 		  Expression[] xargs = {
 		    args[0],
-		    new QuoteExp(parser.constructorNamespaces),
+		    new QuoteExp(constructorNamespaces),
 		    new QuoteExp(parser.prologNamespaces) };
 		  Method meth
 		    = (ClassType.make("gnu.xquery.util.QNameUtils")
@@ -737,10 +790,7 @@ public class XQResolveNames extends ResolveNames
                     : QuoteExp.voidExp;
                   ApplyExp aexp
                     = new ApplyExp(meth, new Expression[]{ uri, base });
-                  if (code == DOC_BUILTIN)
-                    aexp.setType(NodeType.documentNodeTest);
-                  else
-                    aexp.setType(XDataType.booleanType);
+                  aexp.setType(NodeType.documentNodeTest);
                   return aexp;
                 }
               case DOC_BUILTIN:
@@ -752,7 +802,7 @@ public class XQResolveNames extends ResolveNames
                   if (code == DOC_BUILTIN)
                     {
                       mname = "docCached";
-                      if (parser.warnOldVersion
+                      if (XQParser.warnOldVersion
                           && "document".equals(decl.getName()))
                         getCompilation()
                           .error('w', "replace 'document' by 'doc'");
@@ -870,8 +920,10 @@ public class XQResolveNames extends ResolveNames
 
 	// Add namespaces nodes that might be needed.
 	NamespaceBinding nsBindings = make.getNamespaceNodes();
-	nsBindings = maybeAddNamespace(MakeElement.getTagName(exp),
-				       nsBindings);
+        Symbol tag = make.tag;
+        if (tag == null)
+          tag = MakeElement.getTagName(exp);
+	nsBindings = maybeAddNamespace(tag, false, nsBindings);
 	Expression[] args = exp.getArgs();
         Symbol[] attrSyms = new Symbol[args.length];
         int nattrSyms = 0;
@@ -902,8 +954,8 @@ public class XQResolveNames extends ResolveNames
                                 messages.error('e', XMLFilter.duplicateAttributeMessage(sym, elementName), "XQST0040");
                               }
                           }
+                        nsBindings = maybeAddNamespace(sym, true, nsBindings);
                       }
-                    nsBindings = maybeAddNamespace(sym, nsBindings);
                   }
 	      }
 	  }
@@ -929,16 +981,20 @@ public class XQResolveNames extends ResolveNames
       return gnu.kawa.functions.GetModuleClass.getModuleClassURI(comp);
   }
 
-  static NamespaceBinding maybeAddNamespace(Symbol qname,
+  static NamespaceBinding maybeAddNamespace(Symbol qname, boolean isAttribute,
 					    NamespaceBinding bindings)
   {
     if (qname == null) // Happens if prevously-reported unknown prefix.
       return bindings;
     String prefix = qname.getPrefix();
     String uri = qname.getNamespaceURI();
-    return NamespaceBinding.maybeAdd(prefix == "" ? null : prefix,
-                                     uri == "" ? null : uri,
-                                     bindings);
+    if (prefix == "")
+      prefix = null;
+    if (uri == "")
+      uri = null;
+    if (isAttribute && prefix == null && uri == null)
+      return bindings;
+    return NamespaceBinding.maybeAdd(prefix, uri, bindings);
   }
 
   /** Wrap a (known) procedure value as a Declaration. */
