@@ -94,9 +94,9 @@ public class XQuery extends Language
     return new XQParser(inp, messages, this);
   }
 
-  public Compilation getCompilation (Lexer lexer, SourceMessages messages)
+  public Compilation getCompilation (Lexer lexer, SourceMessages messages, NameLookup lexical)
   {
-    return new Compilation(this, messages, ((XQParser) lexer).lexical);
+    return new Compilation(this, messages, lexical);
   }
 
   /** Special parser flag used by <code>evalToFocusProc</code>. */
@@ -122,8 +122,8 @@ public class XQuery extends Language
 	Declaration dotDecl = lexp.addDeclaration(XQParser.DOT_VARNAME);
 	dotDecl.setFlag(Declaration.IS_SINGLE_VALUE);
 	dotDecl.noteValue (null);  // Does not have a known value.
-	lexp.addDeclaration(XQParser.POSITION_VARNAME, Type.int_type);
-	lexp.addDeclaration(XQParser.LAST_VARNAME, Type.int_type);
+	lexp.addDeclaration(XQParser.POSITION_VARNAME, Type.intType);
+	lexp.addDeclaration(XQParser.LAST_VARNAME, Type.intType);
 	tr.push(lexp);
 	lexp.body = parser.parse(tr);
 	tr.pop(lexp);
@@ -819,7 +819,10 @@ public class XQuery extends Language
       "ID", XStringType.IDType,
       "IDREF", XStringType.IDREFType,
       "ENTITY", XStringType.ENTITYType,
-      "anyAtomicType", XDataType.anyAtomicType
+      "anyAtomicType", XDataType.anyAtomicType,
+      "anySimpleType", XDataType.anySimpleType,
+      "untyped", XDataType.untypedType,
+      "anyType", Type.objectType
     };
 
   public static Type getStandardType (String name)
