@@ -19,19 +19,19 @@ public class RunXQTS extends FilterConsumer
   static XQuery xqueryLanguage = XQuery.getInstance();
 
   /* #ifdef JAVA5 */
-  // HashMap<String,String> expectedFailures = new HashMap<String,String>();
-  // HashMap<String,String> modules = new HashMap<String,String>();
-  // HashMap<String,Object> sources = new HashMap<String,Object>();
-  // Stack<Symbol> externalVariablesSet = new Stack<Symbol>();
-  // Stack<String> outputFileAlts = new Stack<String>();
-  // Stack<String> outputCompareAlts = new Stack<String>();
+  HashMap<String,String> expectedFailures = new HashMap<String,String>();
+  HashMap<String,String> modules = new HashMap<String,String>();
+  HashMap<String,Object> sources = new HashMap<String,Object>();
+  Stack<Symbol> externalVariablesSet = new Stack<Symbol>();
+  Stack<String> outputFileAlts = new Stack<String>();
+  Stack<String> outputCompareAlts = new Stack<String>();
   /* #else */
-  Hashtable expectedFailures = new Hashtable();
-  Hashtable modules = new Hashtable();
-  Hashtable sources = new Hashtable();
-  Stack externalVariablesSet = new Stack();
-  Stack outputFileAlts = new Stack();
-  Stack outputCompareAlts = new Stack();
+  // Hashtable expectedFailures = new Hashtable();
+  // Hashtable modules = new Hashtable();
+  // Hashtable sources = new Hashtable();
+  // Stack externalVariablesSet = new Stack();
+  // Stack outputFileAlts = new Stack();
+  // Stack outputCompareAlts = new Stack();
   /* #endif */
   ModuleManager manager = ModuleManager.getInstance();
   Object failExpected;
@@ -190,9 +190,9 @@ public class RunXQTS extends FilterConsumer
   Object currentElementType;
   Symbol currentElementSymbol;
   /* #ifdef JAVA5 */
-  // Stack<Object> elementTypeStack = new Stack<Object>();
+  Stack<Object> elementTypeStack = new Stack<Object>();
   /* #else */
-  Stack elementTypeStack = new Stack();
+  // Stack elementTypeStack = new Stack();
   /* #endif */
   boolean inStartTag;
   int attrValueStart;
@@ -210,20 +210,16 @@ public class RunXQTS extends FilterConsumer
     super(out);
     this.cout = out;
 
-    expectFailures("K-ReplaceFunc-8", "allow bad regex replacement string");
     expectFailures("K2-MatchesFunc-1", "allow bad regex pattern");
-    expectFailures("static-context-1", "unchecked unknownType in element(*,TypeName)");
-    expectFailures("fn-abs-more-args-023|fn-abs-more-args-024",
-                   "testsuite error (4023): -0 is not  valid unsignedLong/unsignedShort literal");
+    expectFailures("K2-MatchesFunc-8|K2-MatchesFunc-9|K2-MatchesFunc-10|K2-ReplaceFunc-3", "backwards references in regep not implemented");
     expectFailures("ST-WhereExpr001",
                    "testsuite error (4024) - should not require error");
-    expectFailures("K2-SeqExprCast-207",
-                   "was testsuite error(4252) - now mismatch between '>' and '&gt;'");
     /* #ifndef JAVA5 */
-    expectFailures("surrogates12|surrogates13|surrogates14|surrogates15",
-                   "surrogates not handled by java.util.regex");
+    // expectFailures("surrogates12|surrogates13|surrogates14|surrogates15",
+    //                "surrogates not handled by java.util.regex");
     /* #endif */
-    expectFailures("K-SeqExprInstanceOf-53", "too lenient about non-stanadrd types: void");
+    expectFailures("K2-Literals-34", "MainModule without QueryBody"); // should be fixed
+    expectFailures("K2-NameTest-68", "depends on typing");
     expectFailures("ST-Axes001|ST-Axes002|ST-Axes003|ST-Axes004|ST-Axes005|"
                    +"ST-Axes006|ST-Axes007|ST-Axes008|ST-Axes009|ST-Axes010|"
                    +"ST-Axes011|ST-Axes012|ST-Axes013|ST-Axes014|ST-Axes015",
@@ -271,7 +267,7 @@ public class RunXQTS extends FilterConsumer
                    "some string functions don't support collation argument");
     expectFailures("caselessmatch04",
                    "regex/unicode special case");
-    expectFailures("string-queries-results-q4|K2-FunctionProlog-7|K2-FunctionProlog-17|K2-FunctionProlog-18|K2-FunctionProlog-19|K2-FunctionProlog-22",
+    expectFailures("string-queries-results-q4|K2-FunctionProlog-7|K2-FunctionProlog-17|K2-FunctionProlog-18|K2-FunctionProlog-19|K2-FunctionProlog-22|XMark-Q18|XMark-All",
                    "function conversion incorrect for user-defined functions");
     expectFailures("caselessmatch10|caselessmatch11",
                    // Need to translate [xxx-[yyy]] to [xxx&&[^yyy]].
@@ -396,6 +392,7 @@ public class RunXQTS extends FilterConsumer
         outputCompareAlts.clear();
         expectedErrorsBuf.setLength(1);
         manager.clear();
+        Document.clearLocalCache();
       }
     else if (tagMatches("query"))
       {
@@ -507,6 +504,8 @@ public class RunXQTS extends FilterConsumer
           }
         throw ex;
       }
+    // This is mainly to avoid memory leaking.
+    NameLookup.setInstance(xqueryLanguage.getEnvironment(), null);
     Compilation comp;
     Procedure withContextProc = null;
     try
@@ -521,6 +520,12 @@ public class RunXQTS extends FilterConsumer
         if (messages.seenErrors())
           throw new SyntaxException(messages);
       }
+    catch (OutOfMemoryError ex)
+      {
+	System.err.println("caught "+ex);
+	System.exit(-1);
+        return;
+      }
     catch (SyntaxException ex)
       {
         in.close();
@@ -592,6 +597,11 @@ public class RunXQTS extends FilterConsumer
         else
           ModuleExp.evalModule(env, ctx, comp, null, null);
       }
+    catch (OutOfMemoryError ex)
+      {
+	System.err.println("caught "+ex);
+	System.exit(-1);
+      }
     catch (Throwable ex)
       {
         if (ex instanceof NumberFormatException
@@ -834,11 +844,11 @@ public class RunXQTS extends FilterConsumer
             i1 = start_attr1;
             i2 = start_attr2;
             /* #ifdef JAVA5 */
-            // Stack<String> attrs1 = new Stack<String>();
-            // Stack<String> attrs2 = new Stack<String>();
+            Stack<String> attrs1 = new Stack<String>();
+            Stack<String> attrs2 = new Stack<String>();
             /* #else */
-            Stack attrs1 = new Stack();
-            Stack attrs2 = new Stack();
+            // Stack attrs1 = new Stack();
+            // Stack attrs2 = new Stack();
             /* #endif */
             for (;;)
               {
@@ -963,13 +973,20 @@ public class RunXQTS extends FilterConsumer
       || c == '.' || c == '-' || c == '+' || c == 'E';
   }
 
-  String selectedTest;
-
   public String getElementValue ()
   {
     return cout.toSubString(elementStartIndex[nesting]);
   }
 
+  String selectedTest;
+
+  boolean isSelected (String testName)
+  {
+    if ("Constr-cont-document-3".equals(testName))
+      return false;
+    return selectedTest == null || selectedTest.equals(testName);
+  }
+
   public void endElement()
   {
     if (inStartTag)
@@ -977,8 +994,7 @@ public class RunXQTS extends FilterConsumer
     if (tagMatches("test-case"))
       {
         if (--maxTests == 0)  System.exit(0); // FIXME
-        if (selectedTest == null
-            || selectedTest.equals(testName))
+        if (isSelected(testName))
           {
             xqlog.startElement(testCaseElementType);
             writeAttribute("name", testName);
@@ -1009,28 +1025,31 @@ public class RunXQTS extends FilterConsumer
       }
     else if (tagMatches("input-query"))
       {
-        String variable = attributes.getValue("variable");
-        Symbol symbol = Symbol.parse(variable);
-        String name = attributes.getValue("name");
-        String filename
-          = directory + '/' + XQueryQueryOffsetPath + testFilePath
-          + name + XQueryFileExtension;
-        InPort in;
-        try
+        if (isSelected(testName))
           {
-            in = InPort.openFile(filename);
-            Object value = XQuery.getInstance().eval(in);
-            in.close();
-            Environment current = Environment.getCurrent();
-            current.put(symbol, null, value);
-            externalVariablesSet.push(symbol);
-          }
-        catch (Throwable ex)
-          {
-            System.err.println("input-query: cannot open "+filename);
-            System.err.println("caught "+ex);
-            ex.printStackTrace();
-            System.exit(-1);
+            String variable = attributes.getValue("variable");
+            Symbol symbol = Symbol.parse(variable);
+            String name = attributes.getValue("name");
+            String filename
+              = directory + '/' + XQueryQueryOffsetPath + testFilePath
+              + name + XQueryFileExtension;
+            InPort in;
+            try
+              {
+                in = InPort.openFile(filename);
+                Object value = XQuery.getInstance().eval(in);
+                in.close();
+                Environment current = Environment.getCurrent();
+                current.define(symbol, null, value);
+                externalVariablesSet.push(symbol);
+              }
+            catch (Throwable ex)
+              {
+                System.err.println("input-query for "+testName+": cannot open "+filename);
+                System.err.println("caught "+ex);
+                ex.printStackTrace();
+                System.exit(-1);
+              }
           }
       }
     else if (tagMatches("input-file") || tagMatches("contextItem"))
@@ -1058,14 +1077,14 @@ public class RunXQTS extends FilterConsumer
           {
             Object value = gnu.kawa.xml.Document.parseCached(path);
             if (symbol != null)
-              Environment.getCurrent().put(symbol, null, value);
+              Environment.getCurrent().define(symbol, null, value);
             else
               contextItem = value;
           }
         catch (Throwable ex)
           {
             System.err.println("caught "+ex);
-            System.err.println("reading data file "+path);
+            System.err.println("reading data file "+path+" for "+testName);
             System.err.println("inputFile:"+inputFile+" variable:"+variable+" path:"+path);
             ex.printStackTrace();
             System.exit(-1);
@@ -1133,10 +1152,8 @@ public class RunXQTS extends FilterConsumer
         String mpath = directory + '/' + mfile + XQueryFileExtension;
         String mclass = Compilation.mangleURI(uri)
           + '.' + XQuery.makeClassName(mpath);
-
-        ModuleInfo minfo = manager.findWithClassName(mclass);
-        minfo.sourcePath = mfile + XQueryFileExtension;
-        minfo.setSourceAbsPath(Path.valueOf(mpath));
+        ModuleInfo minfo = manager.findWithSourcePath(mpath);
+        minfo.className = mclass;
         minfo.setNamespaceUri(uri);
       }
     /*
