@@ -51,8 +51,11 @@ public class RelativeStep extends MethodProc implements CanInline, Inlineable
     filter.finish();
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    // FIXME make use of type of E1 to set dot in E2.
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     Expression exp1 = args[0];
     Expression exp2 = args[1];
@@ -68,6 +71,7 @@ public class RelativeStep extends MethodProc implements CanInline, Inlineable
 
     lexp2.setInlineOnly(true);
     lexp2.returnContinuation = exp;
+    lexp2.inlineHome = walker.getCurrentLambda();
 
     exp2 = lexp2.body;
 
@@ -157,7 +161,7 @@ public class RelativeStep extends MethodProc implements CanInline, Inlineable
           }
       }
     // Now we can rewrite 'descendant-or-self::node()/B' (which is the
-    // expansion of the abbreviated syntax '//B') to /descdendant::B'.
+    // expansion of the abbreviated syntax '//B') to /descendant::B'.
     if (exp1 instanceof ApplyExp && exp2 instanceof ApplyExp)
       {
         ApplyExp aexp1 = (ApplyExp) exp1;
@@ -170,9 +174,8 @@ public class RelativeStep extends MethodProc implements CanInline, Inlineable
             && (exp12 = aexp1.getArg(1)) instanceof LambdaExp)
           {
             LambdaExp lexp12 = (LambdaExp) exp12;
-            ApplyExp aexp12;
             if (lexp12.body instanceof ApplyExp
-                && (aexp12 = (ApplyExp) lexp12.body).getFunction().valueIfConstant() == DescendantOrSelfAxis.anyNode)
+                && ((ApplyExp) lexp12.body).getFunction().valueIfConstant() == DescendantOrSelfAxis.anyNode)
               {
                 exp.setArg(0, aexp1.getArg(0));
                 aexp2.setFunction(new QuoteExp(DescendantAxis.make(((ChildAxis) p2).getNodePredicate())));
@@ -232,9 +235,7 @@ public class RelativeStep extends MethodProc implements CanInline, Inlineable
           }
       }
 
-    if (! (target instanceof ConsumerTarget
-           || (target instanceof SeriesTarget
-               && (expectedKind == 'A' || expectedKind == 'S'))))
+    if (! (target instanceof ConsumerTarget))
       {
 	ConsumerTarget.compileUsingConsumer(exp, comp, target);
 	return;
