@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2003  Per M.A. Bothner and Brainfood Inc.
+// Copyright (c) 2001, 2003, 2008  Per M.A. Bothner and Brainfood Inc.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.xquery.util;
@@ -293,9 +293,9 @@ public class StringUtils
     arg2 = SequenceUtils.coerceToZeroOrOne(arg2, "concat", 2);
     String str2 = TextUtils.stringValue(arg2);
     /* #ifdef JAVA5 */
-    // StringBuilder result = new StringBuilder(str1);
+    StringBuilder result = new StringBuilder(str1);
     /* #else */
-    StringBuffer result = new StringBuffer(str1);
+    // StringBuffer result = new StringBuffer(str1);
     /* #endif */
     result.append(str2);
     int count = args.length;
@@ -368,29 +368,20 @@ public class StringUtils
 
   public static String encodeForUri (Object arg)
   {
-    return encodeForUri(arg, 'U');
+    String str = coerceToString(arg, "encode-for-uri", 1, "");
+    return URIPath.encodeForUri(str, 'U');
   }
 
   public static String iriToUri (Object arg)
   {
-    return encodeForUri(arg, 'I');
+    String str = coerceToString(arg, "iri-to-uru", 1, "");
+    return URIPath.encodeForUri(str, 'I');
   }
 
   public static String escapeHtmlUri (Object arg)
   {
-    return encodeForUri(arg, 'H');
-  }
-
-  static String encodeForUri (Object arg, char mode)
-  {
-    String str;
-    if (arg instanceof String || arg instanceof UntypedAtomic)
-      str = arg.toString();
-    else if (arg == null || arg == Values.empty)
-      str = "";
-    else
-      throw new ClassCastException();
-    return URIPath.encodeForUri(str, mode);
+    String str = coerceToString(arg, "escape-html-uri", 1, "");
+    return URIPath.encodeForUri(str, 'H');
   }
 
   public static String normalizeSpace (Object arg)
@@ -443,11 +434,22 @@ public class StringUtils
           case 'x':
             StringBuffer sbuf = new StringBuffer();
             int plen = pattern.length();
-            for (int j = 0; j < plen;  j++)
+            int inBracket = 0;
+            for (int j = 0; j < plen;  )
               {
-                char pch = pattern.charAt(j);
-                if (! Character.isWhitespace(pch))
-                  sbuf.append(pch);
+                char pch = pattern.charAt(j++);
+                if (pch == '\\' && j < plen)
+                  { 
+                    sbuf.append(pch);
+                    pch = pattern.charAt(j++);
+                  }
+                else if (pch == '[')
+                  inBracket++;
+                else if (pch == ']')
+                  inBracket--;
+                else if (inBracket == 0 && Character.isWhitespace(pch))
+                  continue;
+                sbuf.append(pch);
               }
             pattern = sbuf.toString();
             break;
@@ -458,6 +460,35 @@ public class StringUtils
             throw new IllegalArgumentException("unknown 'replace' flag");
           }
       }
+    
+    if (pattern.indexOf("{Is") >= 0)
+      { // Change "\p{IsXxxx}" to "\P{InXxxx}".
+        StringBuffer sbuf = new StringBuffer();
+        int plen = pattern.length();
+        for (int j = 0; j < plen;  )
+          {
+            char pch = pattern.charAt(j++);
+            if (pch == '\\' && j + 4 < plen)
+              { 
+                sbuf.append(pch);
+                pch = pattern.charAt(j++);  
+                sbuf.append(pch);
+                if ((pch == 'p' || pch == 'P')
+                    && pattern.charAt(j) == '{'
+                    && pattern.charAt(j+1) == 'I'
+                    && pattern.charAt(j+2) == 's')
+                  {
+                    sbuf.append('{');
+                    sbuf.append('I');
+                    sbuf.append('n');
+                    j += 3;
+                  }
+              }
+            else
+              sbuf.append(pch);
+          }
+        pattern = sbuf.toString();
+      }
     return Pattern.compile(pattern, fl);
   }
   /* #endif */
@@ -470,13 +501,7 @@ public class StringUtils
   public static boolean matches (Object arg, String pattern, String flags)
   {
     /* #ifdef use:java.util.regex */
-    String str;
-    if (arg instanceof String || arg instanceof UntypedAtomic)
-      str = arg.toString();
-    else if (arg == null || arg == Values.empty)
-      str = "";
-    else
-      throw new ClassCastException();
+    String str = coerceToString(arg, "matches", 1, "");
     return makePattern(pattern, flags).matcher(str).find();
     /* #else */
     // throw new Error("fn:matches requires java.util.regex (JDK 1.4 or equivalent)");
@@ -493,13 +518,18 @@ public class StringUtils
                                  String replacement, String flags)
   {
     /* #ifdef use:java.util.regex */
-    String str;
-    if (arg instanceof String || arg instanceof UntypedAtomic)
-      str = arg.toString();
-    else if (arg == null || arg == Values.empty)
-      str = "";
-    else
-      throw new ClassCastException();
+    String str = coerceToString(arg, "replace", 1, "");
+    int rlen = replacement.length();
+    for (int i = 0; i < rlen; )
+      {
+        char rch = replacement.charAt(i++);
+        if (rch == '\\')
+          {
+            if (i >= rch
+                || ! ((rch = replacement.charAt(i++)) == '\\' || rch == '$'))
+              throw new IllegalArgumentException("invalid replacement string [FORX0004]");
+          }
+      }
     return makePattern(pattern, flags).matcher(str).replaceAll(replacement);
     /* #else */
     // throw new Error("fn:replace requires java.util.regex (JDK 1.4 or equivalent)");
@@ -515,13 +545,7 @@ public class StringUtils
                                  String flags, CallContext ctx)
   {
     /* #ifdef use:java.util.regex */
-    String str;
-    if (arg instanceof String || arg instanceof UntypedAtomic)
-      str = arg.toString();
-    else if (arg == null || arg == Values.empty)
-      str = "";
-    else
-      throw new ClassCastException();
+    String str = coerceToString(arg, "tokenize", 1, "");
     Consumer out = ctx.consumer;
     Matcher matcher = makePattern(pattern, flags).matcher(str);
     int len = str.length();
