@@ -1,10 +1,9 @@
 package kawa;
 
 import java.awt.*;
+import javax.swing.*;
 import java.awt.event.*;
 import gnu.mapping.*;
-import gnu.text.Path;
-
 import gnu.expr.Language;
 import kawa.standard.Scheme;
 
@@ -13,7 +12,8 @@ import kawa.standard.Scheme;
   * @author Per Bothner (extensive changes).
   */
 
-public class GuiConsole extends Frame implements ActionListener {
+public class GuiConsole extends JFrame
+  implements ActionListener, ReplDocument.DocumentCloseListener {
   private static String CLOSE = "Close";
   private static String EXIT = "Exit";
   private static String NEW = "New";
@@ -22,17 +22,15 @@ public class GuiConsole extends Frame implements ActionListener {
 
   static int window_number = 0;
 
-  Language language;
-  Environment environment;
-  Future thread;
-
-  gnu.text.QueueReader in_r;
-  OutPort out_p, err_p;
-
-  MessageArea message = null;
+  ReplPane pane;
+  ReplDocument document;
 
   public static void main(String[] args) {
-    Language language = Scheme.getInstance();
+    repl.noConsole = false;
+    int iArg = repl.processArgs(args, 0, args.length);
+    repl.getLanguage();
+    repl.setArgs(args, iArg);
+    repl.checkInitFile();
     new GuiConsole();
   }
 
@@ -41,55 +39,46 @@ public class GuiConsole extends Frame implements ActionListener {
     this(Language.getDefaultLanguage(), Environment.getCurrent(), false);
   }
 
-  public GuiConsole(Language language, Environment penvironment, boolean shared)
+  public GuiConsole (ReplDocument doc)
   {
     super("Kawa");
-    this.language = language;
+    init(doc);
+  }
 
-    in_r = new gnu.text.QueueReader ();
-    message = new MessageArea(in_r);
+  void init(ReplDocument doc)
+  {
+    document = doc;
+    document.addDocumentCloseListener(this);
+    pane = new ReplPane(document);
     window_number++;
-    kawa.repl.exitIncrement();
-
-    out_p = new OutPort(message.getStdout(), true,
-                        Path.valueOf("<msg_stdout>"));
-    err_p = new OutPort(message.getStderr(), true,
-                        Path.valueOf("<msg_stderr>"));
-    InPort in_p = new GuiInPort(in_r, Path.valueOf("<msg_stdin>"),
-                                out_p, message);
-
     this.setLayout(new BorderLayout(0,0));
-
-    this.add("Center",message);
-
+    this.add("Center", new JScrollPane(pane));
     setupMenus();
     //pack();
     setLocation(100 * window_number, 50 * window_number);
     setSize(700,500);
     setVisible(true);
+  }
 
-    thread = new Future (new kawa.repl(language),
-			 penvironment, in_p, out_p, err_p);
-    Environment env = thread.getEnvironment();
-    if (shared)
-      env.setIndirectDefines();
-    this.environment = env;
-    thread.start();
+  public GuiConsole(Language language, Environment penvironment, boolean shared)
+  {
+    super("Kawa");
+    repl.getLanguage(); // In case a new GuiConsole is created from Java.
+    init(new ReplDocument(language, penvironment, shared));
+
+    // Uncomment to test same ReplDocument in two JFrames.
+    // new GuiConsole(document);
+  }
+
+  public void closed (ReplDocument doc)
+  {
+    close();
   }
 
   void close () {
-    in_r.appendEOF();
+    document.removeDocumentCloseListener(this);
     dispose();
-    // Give thread chance to finish and clean up
-    try {
-      Thread.sleep(100);
-    } catch (InterruptedException ex) {
-    }
-    // Thread.stop is deprecated in JDK 1.2, but I see no good
-    // alternative.  (Thread.destroy is not implemented!)
-    thread.stop(); 
-    kawa.repl.exitDecrement();
- }
+  }
 
   private void setupMenus() {
     MenuBar menubar;
@@ -136,19 +125,20 @@ public class GuiConsole extends Frame implements ActionListener {
     this.setMenuBar(menubar);
   }
 
-  public void actionPerformed(ActionEvent e) {
+  public void actionPerformed(ActionEvent e)
+  {
     String cmd = e.getActionCommand();
 
     if (cmd.equals(NEW))
-      new GuiConsole(language, Environment.getGlobal(), false);
+      new GuiConsole(document.language, Environment.getGlobal(), false);
     else if (cmd.equals(NEW_SHARED))
-      new GuiConsole(language, environment, true);
+      new GuiConsole(document.language, document.environment, true);
     else if (cmd.equals(EXIT))
       System.exit(0);
     else if (cmd.equals(CLOSE))
       close();
     else if (cmd.equals(PURGE_MESSAGE)) {
-      message.deleteOldText();
+      pane.document.deleteOldText();
     }
     else
       OutPort.outDefault().println("Unknown menu action: "+cmd);
