@@ -14,7 +14,7 @@ public class Eval extends Procedure1or2
   public static final Eval eval = new Eval();
   static { eval.setName("eval"); }
 
-  final static String evalFunctionName = "atEvalLevel";
+  final static String evalFunctionName = "atEvalLevel$";
 
   public static void eval (Object sexpr, Environment env, CallContext ctx)
     throws Throwable
@@ -74,9 +74,10 @@ public class Eval extends Procedure1or2
     Environment saveGlobalEnv = Environment.getCurrent();
     try
       {
-	if (env != saveGlobalEnv)
+ 	if (env != saveGlobalEnv)
 	  Environment.setCurrent(env);
-	Translator tr = new Translator(language, messages);
+	Translator tr = new Translator(language, messages,
+                                       NameLookup.getInstance(env, language));
         tr.immediate = true;
 	ModuleExp mod = tr.pushNewModule((String) null);
         Compilation save_comp = Compilation.getCurrent();
@@ -95,7 +96,7 @@ public class Eval extends Procedure1or2
 
 	if (body instanceof PairWithPosition)
 	  mod.setFile(((PairWithPosition) body).getFileName());
-	mod.setName (evalFunctionName);
+	mod.setName(evalFunctionName + (++ModuleExp.interactiveCounter));
 	ModuleExp.evalModule(env, ctx, tr, null, null);
 	if (messages.seenErrors())
 	  throw new RuntimeException("invalid syntax in eval form:\n"
