@@ -4,6 +4,7 @@ import gnu.expr.*;
 import gnu.lists.*;
 import gnu.bytecode.Type;
 import gnu.kawa.functions.Convert;
+import gnu.kawa.lispexpr.LangObjType;
 
 /**
  * The Syntax transformer that re-writes the lambda builtin.
@@ -29,7 +30,7 @@ public class Lambda extends Syntax
 
   public Expression rewriteForm (Pair form, Translator tr)
   {
-    Expression exp = rewrite(form.cdr, tr);
+    Expression exp = rewrite(form.getCdr(), tr);
     Translator.setLine(exp, form);
     return exp;
   }
@@ -42,7 +43,7 @@ public class Lambda extends Syntax
     LambdaExp lexp = new LambdaExp();
     Pair pair = (Pair) obj;
     Translator.setLine(lexp, pair);
-    rewrite(lexp, pair.car, pair.cdr, tr, null);
+    rewrite(lexp, pair.getCar(), pair.getCdr(), tr, null);
     if (tr.getMessages().getErrorCount() > old_errors)
       return new ErrorExp("bad lambda expression");
     return lexp;
@@ -80,7 +81,7 @@ public class Lambda extends Syntax
     int rest_args = -1;
     int key_args = -1;
     Pair pair;
-    for (; ;  bindings = pair.cdr)
+    for (; ;  bindings = pair.getCdr())
       {
 	if (bindings instanceof SyntaxForm)
 	  {
@@ -92,7 +93,7 @@ public class Lambda extends Syntax
 	  break;
 	pair = (Pair) bindings;
         // An initial pass to count the parameters.
-	Object pair_car = pair.car;
+	Object pair_car = pair.getCar();
 	if (pair_car instanceof SyntaxForm)
 	  pair_car = ((SyntaxForm) pair_car).form;
 	if (pair_car == optionalKeyword)
@@ -135,8 +136,8 @@ public class Lambda extends Syntax
 	      }
 	    key_args = 0;
 	  }
-        else if (tr.matches(pair.car, "::") && pair.cdr instanceof Pair)
-          pair = (Pair) pair.cdr;
+        else if (tr.matches(pair.getCar(), "::") && pair.getCdr() instanceof Pair)
+          pair = (Pair) pair.getCdr();
 	else if (key_args >= 0)
 	  key_args++;
 	else if (rest_args >= 0)
@@ -145,9 +146,9 @@ public class Lambda extends Syntax
 	  opt_args++;
 	else
 	  lexp.min_args++;
-	bindings = pair.cdr;
+	bindings = pair.getCdr();
       }
-    if (bindings instanceof String || bindings instanceof Symbol)
+    if (bindings instanceof Symbol)
       {
 	if (opt_args >= 0 || key_args >= 0 || rest_args >= 0)
 	  {
@@ -186,7 +187,7 @@ public class Lambda extends Syntax
     opt_args = 0;
     key_args = 0;
     Object mode = null;
-    for (; ;  bindings = pair.cdr)
+    for (; ;  bindings = pair.getCdr())
       {
 	if (bindings instanceof SyntaxForm)
 	  {
@@ -200,7 +201,7 @@ public class Lambda extends Syntax
 	if (! (bindings instanceof Pair))
 	  break;
 	pair = (Pair) bindings;
-	Object pair_car = pair.car;
+	Object pair_car = pair.getCar();
 	if (pair_car instanceof SyntaxForm)
 	  {
 	    SyntaxForm sf = (SyntaxForm) pair_car;
@@ -223,19 +224,20 @@ public class Lambda extends Syntax
 	    tr.syntaxError("'::' must follow parameter name");
 	    return;
 	  }
-	if (pair_car instanceof String || pair_car instanceof Symbol)
+        pair_car = tr.namespaceResolve(pair_car);
+	if (pair_car instanceof Symbol)
           {
             name = pair_car;
-            if (pair.cdr instanceof Pair
-                && tr.matches((p = (Pair) pair.cdr).car, "::"))
+            if (pair.getCdr() instanceof Pair
+                && tr.matches((p = (Pair) pair.getCdr()).getCar(), "::"))
               {
-                if (! (pair.cdr instanceof Pair))
+                if (! (pair.getCdr() instanceof Pair))
                   {
                     tr.syntaxError("'::' not followed by a type specifier"
                                    + " (for parameter '" + name + "')");
                     return;
                   }
-                p = (Pair) p.cdr;
+                p = (Pair) p.getCdr();
                 typeSpecPair = p;
                 pair = p;
               }
@@ -243,32 +245,32 @@ public class Lambda extends Syntax
 	else if (pair_car instanceof Pair)
 	  {
 	    p = (Pair) pair_car;
-	    pair_car = p.car;
+	    pair_car = p.getCar();
 	    if (pair_car instanceof SyntaxForm)
 	      {
 		SyntaxForm sf = (SyntaxForm) pair_car;
 		pair_car = sf.form;
 		templateScope = sf.scope;
 	      }
-	    if ((pair_car instanceof String
-		 || pair_car instanceof Symbol)
-		&& p.cdr instanceof Pair)
+            pair_car = tr.namespaceResolve(pair_car);
+	    if (pair_car instanceof Symbol
+		&& p.getCdr() instanceof Pair)
 	      {
 		name = pair_car;
-		p = (Pair) p.cdr;
-		if (tr.matches(p.car, "::"))
+		p = (Pair) p.getCdr();
+		if (tr.matches(p.getCar(), "::"))
 		  {
-		    if (! (p.cdr instanceof Pair))
+		    if (! (p.getCdr() instanceof Pair))
 		      {
 			tr.syntaxError("'::' not followed by a type specifier"
 				       + " (for parameter '" + name + "')");
 			return;
 		      }
-		    p = (Pair) p.cdr;
+		    p = (Pair) p.getCdr();
 		    typeSpecPair = p;
-		    if (p.cdr instanceof Pair)
-		      p = (Pair) p.cdr;
-		    else if (p.cdr == LList.Empty)
+		    if (p.getCdr() instanceof Pair)
+		      p = (Pair) p.getCdr();
+		    else if (p.getCdr() == LList.Empty)
 		      p = null;
 		    else
 		      {
@@ -279,10 +281,10 @@ public class Lambda extends Syntax
 		  }
 		if (p != null && mode != null)
 		  {
-		    defaultValue = p.car;
-		    if (p.cdr instanceof Pair)
-		      p = (Pair) p.cdr;
-		    else if (p.cdr == LList.Empty)
+		    defaultValue = p.getCar();
+		    if (p.getCdr() instanceof Pair)
+		      p = (Pair) p.getCdr();
+		    else if (p.getCdr() == LList.Empty)
 		      p = null;
 		    else
 		      {
@@ -300,10 +302,10 @@ public class Lambda extends Syntax
 			return;
 		      }
 		    typeSpecPair = p;
-		    if (p.cdr != LList.Empty)
+		    if (p.getCdr() != LList.Empty)
 		      {
 			tr.syntaxError("junk at end of specifier for parameter '"
-				       + name + '\''+" after type "+p.car);
+				       + name + '\''+" after type "+p.getCar());
 			return;
 		      }
 		  }
@@ -328,7 +330,7 @@ public class Lambda extends Syntax
 	    decl.setFlag(Declaration.TYPE_SPECIFIED);
 	  }
 	else if (mode == restKeyword)
-	  decl.setType(Compilation.scmListType);
+	  decl.setType(LangObjType.listType);
 	decl.noteValue(null);  // Does not have a known value.
 	addParam(decl, templateScope, lexp, tr);
 	tr.popPositionOf(savePos);
@@ -339,10 +341,10 @@ public class Lambda extends Syntax
 	bindings = sf.form;
 	templateScopeRest = sf.scope;
       }
-    if (bindings instanceof String || bindings instanceof Symbol)
+    if (bindings instanceof Symbol)
       {
 	Declaration decl = new Declaration(bindings);
-	decl.setType(Compilation.scmListType);
+        decl.setType(LangObjType.listType);
 	decl.noteValue (null);  // Does not have a known value.
 	addParam(decl, templateScopeRest, lexp, tr);
       }
@@ -375,8 +377,8 @@ public class Lambda extends Syntax
 	if (! (body instanceof Pair))
 	  break;
 	Pair pair1 = (Pair) body;
-	Object attrName = Translator.stripSyntax(pair1.car);
-	Object pair1_cdr = pair1.cdr;
+	Object attrName = Translator.stripSyntax(pair1.getCar());
+	Object pair1_cdr = pair1.getCdr();
 	while (pair1_cdr instanceof SyntaxForm)
 	  {
 	    syntax = (SyntaxForm) pair1_cdr;
@@ -407,8 +409,14 @@ public class Lambda extends Syntax
 	  {
 	    Expression attrExpr = tr.rewrite_car(pair2, syntax);
 	    if (! (attrExpr instanceof QuoteExp)
-		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof String
-		      || attrValue instanceof FString))
+		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof SimpleSymbol
+                      /* #ifdef use:java.lang.CharSequence */
+		      || attrValue instanceof CharSequence
+                      /* #else */
+		      // || attrValue instanceof String
+                      // || attrValue instanceof CharSeq
+                      /* #endif */
+                      ))
 	      tr.error('e', "access: value not a constant symbol or string");
 	    else if (lexp.nameDecl == null)
 	      tr.error('e', "access: not allowed for anonymous function");
@@ -438,8 +446,14 @@ public class Lambda extends Syntax
 	  {
 	    Expression attrExpr = tr.rewrite_car(pair2, syntax);
 	    if (! (attrExpr instanceof QuoteExp)
-		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof String
-		      || attrValue instanceof FString))
+		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof SimpleSymbol
+                      /* #ifdef use:java.lang.CharSequence */
+		      || attrValue instanceof CharSequence
+                      /* #else */
+		      // || attrValue instanceof String
+                      // || attrValue instanceof CharSeq
+                      /* #endif */
+		      ))
 	      tr.error('e', "allocation: value not a constant symbol or string");
 	    else if (lexp.nameDecl == null)
 	      tr.error('e', "allocation: not allowed for anonymous function");
@@ -463,7 +477,7 @@ public class Lambda extends Syntax
 	  }
 	else if (attrName == kawa.standard.object.throwsKeyword)
 	  {
-	    attrValue = pair2.car;
+	    attrValue = pair2.getCar();
 	    int count = Translator.listLength(attrValue);
 	    if (count < 0)
 	      tr.error('e', "throws: not followed by a list");
@@ -490,7 +504,7 @@ public class Lambda extends Syntax
 			tr.error('e', "throws not followed by a classname");
 			tr.popPositionOf(savePos);
 		      }
-		    attrValue = pair3.cdr; 
+		    attrValue = pair3.getCdr(); 
 		  }
 		lexp.setExceptions(exps);
 	      }
@@ -505,7 +519,7 @@ public class Lambda extends Syntax
 	  {
 	    tr.error('w', "unknown procedure property "+attrName);
 	  }
-	body = pair2.cdr;
+	body = pair2.getCdr();
       }
     accessFlag |= allocationFlag;
     if (accessFlag != 0)
@@ -520,14 +534,14 @@ public class Lambda extends Syntax
     while (body instanceof Pair)
       {
 	Pair pair = (Pair) body;
-	if (! (pair.cdr instanceof Pair))
+	if (! (pair.getCdr() instanceof Pair))
 	  break;
-	Object attrName = pair.car;
+	Object attrName = pair.getCar();
 	if (tr.matches(attrName, "::"))
 	  attrName = null;
 	else if (! (attrName instanceof Keyword))
 	  break;
-	body = ((Pair) pair.cdr).cdr;
+	body = ((Pair) pair.getCdr()).getCdr();
       }
     return body;
   }
@@ -587,9 +601,12 @@ public class Lambda extends Syntax
     tr.curLambda = saveLambda;
     Expression[] exps;
     int len;
+    Object val;
     if (lexp.body instanceof BeginExp
         && (len = (exps = ((BeginExp) lexp.body).getExpressions()).length) > 1
-        && exps[0] instanceof ReferenceExp)
+        && (exps[0] instanceof ReferenceExp
+            || ((val = exps[0].valueIfConstant()) instanceof Type
+                || val instanceof Class)))
       {
 	// Handle '<TYPENAME> BODY':
         Expression rexp = exps[0];
@@ -600,18 +617,12 @@ public class Lambda extends Syntax
           {
             Expression[] new_body = new Expression[len];
             System.arraycopy(exps, 1, new_body, 0, len);
-            lexp.body = new BeginExp(new_body);
+            lexp.body = BeginExp.canonicalize(new_body);
           }
         Convert.setCoercedReturnValue(lexp, rexp, tr.getLanguage());
       }
-    else if (lexp.returnType != null
-             && lexp.returnType != Type.pointer_type
-             && lexp.returnType != Type.void_type)
-      {
-	Expression value = lexp.body;
-	lexp.body = Convert.makeCoercion(value, lexp.returnType);
-	lexp.body.setLine(value);
-      }
+    else
+      lexp.setCoercedReturnType(lexp.returnType);
     tr.pop(lexp);
     lexp.countDecls();
     tr.popRenamedAlias(numRenamedAlias);
