@@ -265,7 +265,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		program.append((char) MATCH_PAIR);
 		Pair pair = (Pair) pattern;
 		SyntaxForm car_syntax = syntax;
-		Object next = pair.cdr;
+		Object next = pair.getCdr();
 		while (next instanceof SyntaxForm)
 		  {
 		    syntax = (SyntaxForm) next;
@@ -273,10 +273,10 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		  }
 		boolean repeat = false;
 		if (next instanceof Pair
-		    && tr.matches(((Pair) next).car, SyntaxRule.dots3))
+		    && tr.matches(((Pair) next).getCar(), SyntaxRule.dots3))
 		  {
 		    repeat = true;
-		    next = ((Pair) next).cdr;
+		    next = ((Pair) next).getCdr();
 		    while (next instanceof SyntaxForm)
 		      {
 			syntax = (SyntaxForm) next;
@@ -287,7 +287,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		int subvar0 = patternNames.size();
 		if (context == 'P')
 		  context = '\0';
-		translate(pair.car, program, literal_identifiers,
+		translate(pair.getCar(), program, literal_identifiers,
 			  repeat ? nesting + 1 : nesting,
 			  literals, car_syntax,
 			  context == 'V' ? '\0' : 'P', tr);
@@ -332,14 +332,27 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		tr.popPositionOf(savePos);
 	      }
 	  }
-	else if (pattern instanceof String || pattern instanceof Symbol)
+	else if (pattern instanceof Symbol)
 	  {
 	    for (int j = literal_identifiers.length;  --j >= 0; )
 	      {
-		ScopeExp scope = syntax == null ? tr.currentScope()
-		  : syntax.scope;
-		if (literalIdentifierEq(pattern, scope,
-					literal_identifiers[j]))
+                ScopeExp current = tr.currentScope();
+                ScopeExp scope1 = syntax == null ? current : syntax.scope;
+                ScopeExp scope2;
+                Object literal = literal_identifiers[j];
+                if (literal instanceof SyntaxForm)
+                  {
+                    SyntaxForm syntax2 = (SyntaxForm) literal;
+                    
+                    literal = syntax2.form;
+                    scope2 = syntax2.scope;
+                  }
+                else if (tr.currentMacroDefinition != null)
+                  scope2 = tr.currentMacroDefinition.getCapturedScope();
+                else
+                  scope2 = current;
+		if (literalIdentifierEq(pattern, scope1,
+					literal, scope2))
 		  {
 		    int i = SyntaxTemplate.indexOf(literals, pattern);
 		    if (i < 0)
@@ -419,12 +432,12 @@ public class SyntaxPattern extends Pattern implements Externalizable
       value = (value << 13) | ((ch = program.charAt(pc++)) >> 3);
     if ((ch & 7) == MATCH_ANY_CAR)
       {
-	if (syntax != null && ! (p.car instanceof SyntaxForm))
-	  p = Translator.makePair(p, syntax.fromDatum(p.car), p.cdr);
+	if (syntax != null && ! (p.getCar() instanceof SyntaxForm))
+	  p = Translator.makePair(p, syntax.fromDatum(p.getCar()), p.getCdr());
 	vars[start_vars + value] = p;
 	return true;
       }
-    return match (p.car, vars, start_vars, pc_start, syntax);
+    return match (p.getCar(), vars, start_vars, pc_start, syntax);
   }
 
   public boolean match (Object obj, Object[] vars, int start_vars,
@@ -476,7 +489,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		    break;
 		  }
 		else if (o instanceof Pair)
-		  o = ((Pair) o).cdr;
+		  o = ((Pair) o).getCdr();
 		else 
 		  return false;
 	      }
@@ -490,7 +503,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 	      return false;
 	    pc += value;
 	    value = 0;
-	    obj = p.cdr;
+	    obj = p.getCdr();
 	    continue;	
 	  case MATCH_LREPEAT:
 	    int repeat_pc = pc;
@@ -546,7 +559,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 		p = (Pair) obj;
 		if (! match_car (p, vars, start_vars, repeat_pc, syntax))
 		  return false;
-		obj = p.cdr;
+		obj = p.getCdr();
 		for (int j = 0;  j < subvarN;  j++)
 		  arrays[j][i] = vars[subvar0+j];
 	      }
@@ -558,11 +571,33 @@ public class SyntaxPattern extends Pattern implements Externalizable
 	    continue;
 	  case MATCH_EQUALS:
 	    Object lit = literals[value];
-	    // We should be using Translator's matches routine, but the current
-	    // Translator isn't available, so here is a special-purpose kludge.
-	    if (lit instanceof String && obj instanceof Symbol)
-	      obj = ((Symbol) obj).getName();
-	    return lit.equals(obj);
+            Object id1, id2;
+            ScopeExp sc1, sc2;
+            Translator tr = (Translator) Compilation.getCurrent();
+            if (lit instanceof SyntaxForm)
+              {
+                SyntaxForm sf = (SyntaxForm) lit;
+                id1 = sf.form;
+                sc1 = sf.scope;
+              }
+            else
+              {
+                id1 = lit;
+                Syntax curSyntax = tr.getCurrentSyntax();
+                sc1 = curSyntax instanceof Macro ? ((Macro) curSyntax).getCapturedScope() : null;
+              }
+            if (obj instanceof SyntaxForm)
+              {
+                SyntaxForm sf = (SyntaxForm) obj;
+                id2 = sf.form;
+                sc2 = sf.scope;
+              }
+            else
+              {
+                id2 = obj;
+                sc2 = syntax == null ? tr.currentScope() : syntax.scope;
+              }
+            return literalIdentifierEq(id1, sc1, id2, sc2);
 	  case MATCH_ANY:
 	    if (syntax != null)
 	      obj = syntax.fromDatum(obj);
@@ -600,23 +635,17 @@ public class SyntaxPattern extends Pattern implements Externalizable
     return vars;
   }
 
-  public static boolean literalIdentifierEq (Object id1, ScopeExp sc1,
-					     Object literal2)
-  {
-    if (literal2 instanceof SyntaxForm)
-      {
-	SyntaxForm syntax = (SyntaxForm) literal2;
-	return literalIdentifierEq(id1, sc1, syntax.form, syntax.scope);
-      }
-    return literalIdentifierEq(id1, sc1, literal2, null);
-  }
-
   public static boolean literalIdentifierEq (Object id1, ScopeExp sc1,
 					     Object id2, ScopeExp sc2)
   {
-    if (id1 != id2)
+    if (id1 != id2 && (id1 == null || id2 == null || ! id1.equals(id2)))
       return false;
+    if (sc1 == sc2)
+      return true;
     Declaration d1 = null, d2 = null;
+    // Ending the look before we get to ModuleExp isn't really right,
+    // but it's a hassle dealing the global Environment.
+    // FIXME when we re-do the library/globals imlementation.
     while (sc1 != null && ! (sc1 instanceof ModuleExp))
       {
 	d1 = sc1.lookup(id1);
@@ -626,7 +655,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
       }
     while (sc2 != null && ! (sc2 instanceof ModuleExp))
       {
-	d2 = sc1.lookup(id1);
+	d2 = sc2.lookup(id2);
 	if (d2 != null)
 	  break;
 	sc2 = sc2.outer;
@@ -655,7 +684,7 @@ public class SyntaxPattern extends Pattern implements Externalizable
 	  }
 	Pair pair = (Pair) list;
 	tr.pushPositionOf(pair);
-	Object literal = pair.car;
+	Object literal = pair.getCar();
 	Object wrapped;
 	if (literal instanceof SyntaxForm)
 	  {
@@ -664,11 +693,10 @@ public class SyntaxPattern extends Pattern implements Externalizable
 	  }
 	else
 	  wrapped = literal; // FIXME
-	if (! (literal instanceof String 
-               || literal instanceof gnu.mapping.Symbol))
+	if (! (literal instanceof Symbol))
           tr.error('e', "non-symbol '"+literal+"' in literals list");
 	literals[i] = wrapped;
-	list = pair.cdr;
+	list = pair.getCdr();
       }
     tr.popPositionOf(savePos);
     return literals;
