@@ -4,6 +4,8 @@ import gnu.expr.*;
 import gnu.kawa.reflect.*;
 import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
+import gnu.bytecode.ArrayClassLoader;
+import gnu.bytecode.ZipLoader;
 import gnu.text.SourceMessages;
 import gnu.lists.*;
 import gnu.kawa.lispexpr.*;
@@ -47,6 +49,20 @@ public class Translator extends Compilation
 
   public LambdaExp curMethodLambda;
 
+  public static final Declaration getNamedPartDecl;
+  public static final StaticFieldLocation getNamedPartLocation;
+  static {
+    // Declare the special symbol $lookup$ (from the reader)
+    // and bind it to getNamedPartDecl.
+    String cname = "gnu.kawa.functions.GetNamedPart";
+    String fname = "getNamedPart";
+    getNamedPartDecl = Declaration.getDeclarationFromStatic(cname, fname);
+    StaticFieldLocation loc = new StaticFieldLocation(cname, fname);
+    loc.setProcedure();
+    loc.setDeclaration(getNamedPartDecl);
+    getNamedPartLocation = loc;
+  }
+
   /** Return true if decl is lexical and not fluid. */
   public boolean isLexical (Declaration decl)
   {
@@ -70,9 +86,9 @@ public class Translator extends Compilation
 
   private static Expression errorExp = new ErrorExp ("unknown syntax error");
 
-  public Translator (Language language, SourceMessages messages)
+  public Translator (Language language, SourceMessages messages, NameLookup lexical)
   {
-    super(language, messages);
+    super(language, messages, lexical);
     this.env = Environment.getCurrent();
   }
 
@@ -86,7 +102,7 @@ public class Translator extends Compilation
   public final Expression rewrite_car (Pair pair, SyntaxForm syntax)
   {
     if (syntax == null || syntax.scope == current_scope
-	|| pair.car instanceof SyntaxForm)
+	|| pair.getCar() instanceof SyntaxForm)
       return rewrite_car(pair, false);
     ScopeExp save_scope = current_scope;
     try
@@ -102,7 +118,7 @@ public class Translator extends Compilation
 
   public final Expression rewrite_car (Pair pair, boolean function)
   {
-    Object car = pair.car;
+    Object car = pair.getCar();
     if (pair instanceof PairWithPosition)
       return rewrite_with_position (car, function, (PairWithPosition) pair);
     else
@@ -154,7 +170,7 @@ public class Translator extends Compilation
     return null;
   }
 
-  final boolean selfEvaluatingSymbol (Object obj)
+  public final boolean selfEvaluatingSymbol (Object obj)
   {
     return ((LispLanguage) getLanguage()).selfEvaluatingSymbol(obj);
   }
@@ -174,9 +190,30 @@ public class Translator extends Compilation
     if (form instanceof SyntaxForm)
       {
 	// FIXME
-	return literal == ((SyntaxForm) form).form;
+	form = ((SyntaxForm) form).form;
       }
-    if (form instanceof Symbol && ! selfEvaluatingSymbol(form))
+    if (form instanceof SimpleSymbol && ! selfEvaluatingSymbol(form))
+      {
+	ReferenceExp rexp = getOriginalRef(lexical.lookup(form, -1));
+	if (rexp != null)
+	  form = rexp.getSymbol();
+      }
+    return form instanceof SimpleSymbol
+      && ((Symbol) form).getLocalPart() == literal;
+  }
+
+  public boolean matches(Object form, SyntaxForm syntax, Symbol literal)
+  {
+    if (syntax != null)
+      {
+        // FIXME
+      }
+    if (form instanceof SyntaxForm)
+      {
+	// FIXME
+	form = ((SyntaxForm) form).form;
+      }
+    if (form instanceof SimpleSymbol && ! selfEvaluatingSymbol(form))
       {
 	ReferenceExp rexp = getOriginalRef(lexical.lookup(form, -1));
 	if (rexp != null)
@@ -220,7 +257,7 @@ public class Translator extends Compilation
   Syntax check_if_Syntax (Declaration decl)
   {
     Declaration d = Declaration.followAliases(decl);
-
+    Object obj = null;
     Expression dval = d.getValue();
     if (dval != null && d.getFlag(Declaration.IS_SYNTAX))
       {
@@ -237,8 +274,7 @@ public class Translator extends Compilation
               }
             else if (current_scope instanceof TemplateScope)
               macroContext = ((TemplateScope) current_scope).macroContext;
-            Object obj = dval.eval(env);
-            return obj instanceof Syntax ? (Syntax) obj : null;
+            obj = dval.eval(env);
           }
         catch (Throwable ex)
           {
@@ -246,14 +282,19 @@ public class Translator extends Compilation
             error('e', "unable to evaluate macro for "+decl.getSymbol());
           }
       }
-    return null;
+    else if (decl.getFlag(Declaration.IS_SYNTAX) && ! decl.needsContext())
+      {
+	StaticFieldLocation loc = StaticFieldLocation.make(decl);
+	obj = loc.get(null);
+      }
+    return obj instanceof Syntax ? (Syntax) obj : null;
   }
 
   public Expression rewrite_pair (Pair p, boolean function)
   {
-    if (p.car instanceof Syntax)
-      return apply_rewrite((Syntax) p.car, p);
-    Object cdr = p.cdr;
+    if (p.getCar() instanceof Syntax)
+      return apply_rewrite((Syntax) p.getCar(), p);
+    Object cdr = p.getCdr();
 
     Expression func = rewrite_car (p, true);
     Object proc = null;
@@ -316,9 +357,9 @@ public class Translator extends Compilation
     int cdr_length = listLength(cdr);
 
     if (cdr_length == -1)
-      return syntaxError("circular list is not allowed after "+p.car);
+      return syntaxError("circular list is not allowed after "+p.getCar());
     if (cdr_length < 0)
-      return syntaxError("dotted list ["+cdr+"] is not allowed after "+p.car);
+      return syntaxError("dotted list ["+cdr+"] is not allowed after "+p.getCar());
 
     boolean mapKeywordsToAttributes = false;
     Stack vec = new Stack();
@@ -358,7 +399,7 @@ public class Translator extends Compilation
           }
 
         vec.addElement(arg);
-	cdr = cdr_pair.cdr;
+	cdr = cdr_pair.getCdr();
       }
     Expression[] args = new Expression[vec.size()];
     vec.copyInto(args);
@@ -366,12 +407,23 @@ public class Translator extends Compilation
     if (save_scope != current_scope)
       setCurrentScope(save_scope);
 
+    if (func instanceof ReferenceExp
+        && (((ReferenceExp) func).getBinding()==getNamedPartDecl))
+      {
+        Expression part1 = args[0];
+        Expression part2 = args[1];
+        Symbol sym = namespaceResolve(part1, part2);
+        if (sym != null)
+          return rewrite(sym, function);
+        // FIXME don't copy the args array in makeExp ...
+        return GetNamedPart.makeExp(part1, part2);
+      }
     return ((LispLanguage) getLanguage()).makeApply(func, args);
   }
 
-  public Symbol namespaceResolve (Expression context, Expression member)
+  public Namespace namespaceResolvePrefix (Expression context)
   {
-    if (context instanceof ReferenceExp && member instanceof QuoteExp)
+    if (context instanceof ReferenceExp)
       {
         ReferenceExp rexp = (ReferenceExp) context;
         Declaration decl = rexp.getBinding();
@@ -394,14 +446,28 @@ public class Translator extends Compilation
             Namespace ns = (Namespace) val;
             String uri = ns.getName();
             if (uri != null && uri.startsWith("class:"))
-              return null; 
-            String mem = ((QuoteExp) member).getValue().toString().intern();
-            return ns.getSymbol(mem);
+              return null;
+            return ns;
           }
       }
     return null;
   }
 
+  public Symbol namespaceResolve (Namespace ns, Expression member)
+  {
+    if (ns != null && member instanceof QuoteExp)
+      {
+        String mem = ((QuoteExp) member).getValue().toString().intern();
+        return ns.getSymbol(mem);
+      }
+    return null;
+  }
+
+  public Symbol namespaceResolve (Expression context, Expression member)
+  {
+    return namespaceResolve(namespaceResolvePrefix(context), member);
+  }
+
   public static Object stripSyntax (Object obj)
   {
     while (obj instanceof SyntaxForm)
@@ -415,7 +481,7 @@ public class Translator extends Compilation
       obj = ((SyntaxForm) obj).form;
     if (! (obj instanceof Pair))
       return null;
-    return stripSyntax(((Pair) obj).car);
+    return stripSyntax(((Pair) obj).getCar());
   }
 
   public static Object safeCdr (Object obj)
@@ -424,7 +490,7 @@ public class Translator extends Compilation
       obj = ((SyntaxForm) obj).form;
     if (! (obj instanceof Pair))
       return null;
-    return stripSyntax(((Pair) obj).cdr);
+    return stripSyntax(((Pair) obj).getCdr());
   }
 
   /** Returns the length of a syntax list.
@@ -451,15 +517,15 @@ public class Translator extends Compilation
 	if (! (fast instanceof Pair))
 	  return -1-n;
 	n++;
-	Object next = ((Pair) fast).cdr;
+	Object next = ((Pair) fast).getCdr();
 	while (next instanceof SyntaxForm)
 	  next = ((SyntaxForm) next).form;
 	if (next == LList.Empty)
 	  return n;
 	if (! (next instanceof Pair))
 	  return -1-n;
-	slow = ((Pair)slow).cdr;
-	fast = ((Pair)next).cdr;
+	slow = ((Pair)slow).getCdr();
+	fast = ((Pair)next).getCdr();
 	n++;
 	if (fast == slow)
 	  return Integer.MIN_VALUE;
@@ -502,36 +568,27 @@ public class Translator extends Compilation
 
   public Object namespaceResolve (Object name)
   {
-    if (! (name instanceof String))
+    if (! (name instanceof SimpleSymbol))
       {
         Pair p;
         if (name instanceof Pair
             && safeCar(p = (Pair) name) == LispLanguage.lookup_sym
-            && p.cdr instanceof Pair
-            && (p = (Pair) p.cdr).cdr instanceof Pair)
+            && p.getCdr() instanceof Pair
+            && (p = (Pair) p.getCdr()).getCdr() instanceof Pair)
           {
-            Expression part1 = rewrite(p.car);
-            Expression part2 = rewrite(((Pair) p.cdr).car);
-
+            Expression part1 = rewrite(p.getCar());
+            Expression part2 = rewrite(((Pair) p.getCdr()).getCar());
             Symbol sym = namespaceResolve(part1, part2);
             if (sym != null)
               return sym;
             String combinedName = GetNamedPart.combineName(part1, part2);
             if (combinedName != null)
-              return combinedName;
+              return Namespace.EmptyNamespace.getSymbol(combinedName);
           }
       }
     return name;
   }
 
-  public void setCurrentScope (ScopeExp scope)
-  {
-    super.setCurrentScope(scope);
-    while (scope != null && ! (scope instanceof PatternScope))
-      scope = scope.outer;
-    patternScope = (PatternScope) scope;
-  }
-
   /**
    * Re-write a Scheme expression in S-expression format into internal form.
    */
@@ -556,8 +613,7 @@ public class Translator extends Compilation
       return rewrite_with_position (exp, function, (PairWithPosition) exp);
     else if (exp instanceof Pair)
       return rewrite_pair((Pair) exp, function);
-    else if (exp instanceof String
-	     || (exp instanceof Symbol && ! selfEvaluatingSymbol(exp)))
+    else if (exp instanceof Symbol && ! selfEvaluatingSymbol(exp))
       {
 	Declaration decl = lexical.lookup(exp, function);
         Declaration cdecl = null;
@@ -568,8 +624,7 @@ public class Translator extends Compilation
         ScopeExp scope = current_scope;
         int decl_nesting = decl == null ? -1 : ScopeExp.nesting(decl.context);
         String dname;
-        if (exp instanceof String
-            || (exp instanceof Symbol && ((Symbol) exp).hasEmptyNamespace()))
+        if (exp instanceof Symbol && ((Symbol) exp).hasEmptyNamespace())
           dname = exp.toString();
         else
           {
@@ -613,7 +668,6 @@ public class Translator extends Compilation
           }
 
 	Object nameToLookup;
-	Symbol symbol = null;
 	if (decl != null)
 	  {
 	    nameToLookup = decl.getSymbol();
@@ -633,8 +687,7 @@ public class Translator extends Compilation
 	  {
 	    nameToLookup = exp;
 	  }
-	symbol = exp instanceof String ? env.getSymbol((String) exp)
-	  : (Symbol) exp;
+	Symbol symbol = (Symbol) exp;
 	boolean separate = getLanguage().hasSeparateFunctionNamespace();
         if (decl != null)
           {
@@ -675,13 +728,31 @@ public class Translator extends Compilation
                     if (! inlineOk(null)
                         // A kludge - we get a bunch of testsuite failures
                         // if we don't inline $lookup$.  FIXME.
-                        && decl != kawa.standard.Scheme.getNamedPartDecl)
+                        && decl != getNamedPartDecl)
                       decl = null;
-                    if (decl != null && ! decl.isStatic())
+                    else if (immediate)
                       {
-                        cdecl = new Declaration("(module-instance)");
-                        cdecl.setValue(new QuoteExp(floc.getInstance()));
+                        if (! decl.isStatic())
+                          {
+                            cdecl = new Declaration("(module-instance)");
+                            cdecl.setValue(new QuoteExp(floc.getInstance()));
+                          }
                       }
+                    else if (decl.isStatic())
+                      {
+                        // If the class has been loaded through ZipLoader
+                        // or ArrayClassLoader then it might not be visible
+                        // if loaded through some other ClassLoader.
+                        Class fclass = floc.getRClass();
+                        ClassLoader floader;
+                        if (fclass == null
+                            || ((floader = fclass.getClassLoader())
+                                instanceof ZipLoader)
+                            || floader instanceof ArrayClassLoader)
+                          decl = null;
+                      }
+                    else
+                      decl = null;
                   }
                 catch (Throwable ex)
                   {
@@ -691,6 +762,13 @@ public class Translator extends Compilation
                     decl = null;
                   }
               }
+            else if (loc == null || ! loc.isBound())
+              {
+                Expression e
+                  = ((LispLanguage) getLanguage()).checkDefaultBinding(symbol, this);
+                if (e != null)
+                  return e;
+              }
 	    /*
             else if (Compilation.inlineOk && function)
               {
@@ -701,8 +779,13 @@ public class Translator extends Compilation
               }
             */
           }
-	if (decl != null && decl.getContext() instanceof PatternScope)
-	  return syntaxError("reference to pattern variable "+decl.getName()+" outside syntax template");
+	if (decl != null)
+          {
+            // In Scheme/Lisp a variable cannot donate multiple values.
+            decl.setFlag(Declaration.IS_SINGLE_VALUE);
+            if (decl.getContext() instanceof PatternScope)
+              return syntaxError("reference to pattern variable "+decl.getName()+" outside syntax template");
+          }
 
 	ReferenceExp rexp = new ReferenceExp (nameToLookup, decl);
         rexp.setContextDecl(cdecl);
@@ -715,6 +798,8 @@ public class Translator extends Compilation
       return rewrite(((LangExp) exp).getLangValue(), function);
     else if (exp instanceof Expression)
       return (Expression) exp;
+    else if (exp == Special.abstractSpecial)
+      return QuoteExp.abstractExp;
     else
       return QuoteExp.getInstance(Quote.quote(exp, this));
   }
@@ -767,8 +852,8 @@ public class Translator extends Compilation
       return;
     setLine(saved);
     positionPair = (PairWithPosition) saved;
-    if (positionPair.car == Special.eof)
-      positionPair = (PairWithPosition) positionPair.cdr;
+    if (positionPair.getCar() == Special.eof)
+      positionPair = (PairWithPosition) positionPair.getCdr();
   }
 
   /** Set the line position of the argument to the current position. */
@@ -790,7 +875,6 @@ public class Translator extends Compilation
         texp = new InlineCalls(this).walk(texp);
 	if (texp instanceof ErrorExp)
 	  return null;
-        texp = new InlineCalls(this).walk(texp);
 	Type type = getLanguage().getTypeFor(texp);
 	 if (type == null)
 	   {
@@ -893,28 +977,45 @@ public class Translator extends Compilation
         Pair st_pair = (Pair) st;
         Declaration saveContext = macroContext;
         Syntax syntax = null;
-        ScopeExp save_scope = current_scope;
+        ScopeExp savedScope = current_scope;
+        Object savedPosition = pushPositionOf(st);
         try
           {
-            Object obj = st_pair.car;
+            Object obj = st_pair.getCar();
             if (obj instanceof SyntaxForm)
               {
-                SyntaxForm sf = (SyntaxForm) st_pair.car;
+                SyntaxForm sf = (SyntaxForm) st_pair.getCar();
                 setCurrentScope(sf.scope);
                 obj = sf.form;
               }
             Pair p;
             if (obj instanceof Pair
-                && (p = (Pair) obj).car == LispLanguage.lookup_sym
-                && p.cdr instanceof Pair
-                && (p = (Pair) p.cdr).cdr instanceof Pair)
+                && (p = (Pair) obj).getCar() == LispLanguage.lookup_sym
+                && p.getCdr() instanceof Pair
+                && (p = (Pair) p.getCdr()).getCdr() instanceof Pair)
               {
-                Expression part1 = rewrite(p.car);
-                Expression part2 = rewrite(((Pair) p.cdr).car);
-                obj = namespaceResolve(part1, part2);
+                Expression part1 = rewrite(p.getCar());
+                Expression part2 = rewrite(((Pair) p.getCdr()).getCar());
+                Object value1 = part1.valueIfConstant();
+                Object value2 = part2.valueIfConstant();
+                if (value1 instanceof Class && value2 instanceof Symbol)
+                  {
+                    try
+                      {
+                        obj = GetNamedPart.getNamedPart(value1, (Symbol)value2);
+                        if (obj instanceof Syntax)
+                          syntax = (Syntax) obj;
+                      }
+                    catch (Throwable ex)
+                      {
+                        obj = null;
+                      }
+                  }
+                else
+                  obj = namespaceResolve(part1, part2);
               }
-            if (obj instanceof String
-                || (obj instanceof Symbol && ! selfEvaluatingSymbol(obj)))
+            
+            if (obj instanceof Symbol && ! selfEvaluatingSymbol(obj))
               {
                 Expression func = rewrite(obj, true);
                 if (func instanceof ReferenceExp)
@@ -938,8 +1039,9 @@ public class Translator extends Compilation
           }
         finally
           {
-            if (save_scope != current_scope)
-              setCurrentScope(save_scope);
+            if (savedScope != current_scope)
+              setCurrentScope(savedScope);
+            popPositionOf(savedPosition);
           }
 	if (syntax != null)
 	  {
@@ -991,7 +1093,7 @@ public class Translator extends Compilation
                     f = wrapSyntax(f, sf);
                     if (lastPair == null)
 		      return f;
-                    lastPair.cdr = f;
+                    lastPair.setCdrBackdoor(f);
                     return list;
                   }
 		formStack.add(wrapSyntax(popForms(first), sf));
@@ -1006,14 +1108,14 @@ public class Translator extends Compilation
 	  {
 	    Pair pair = (Pair) body;
 	    int first = formStack.size();
-	    scanForm(pair.car, defs);
+	    scanForm(pair.getCar(), defs);
             if (getState() == Compilation.PROLOG_PARSED)
               {
                 // We've seen a require form during the initial pass when
                 // we're looking module names.  Defer the require and any
                 // following forms in this body.
-                if (pair.car != pendingForm)
-                  pair = makePair(pair, pendingForm, pair.cdr);
+                if (pair.getCar() != pendingForm)
+                  pair = makePair(pair, pendingForm, pair.getCdr());
                 pendingForm = new Pair(kawa.standard.begin.begin, pair);
                 return LList.Empty;
               }
@@ -1027,12 +1129,12 @@ public class Translator extends Compilation
 		    if (lastPair == null)
 		      list = npair;
 		    else
-		      lastPair.cdr = npair;
+		      lastPair.setCdrBackdoor(npair);
 		    lastPair = npair;
 		  }
 		formStack.setSize(first);
 	      }
-	    body = pair.cdr;
+	    body = pair.getCdr();
 	  }
 	else
 	  {
@@ -1068,7 +1170,7 @@ public class Translator extends Compilation
 	scanBody(exp, defs, false);
 	if (formStack.size() == first)
 	  formStack.add(syntaxError ("body with no expressions"));
-	int ndecls = defs.countDecls();
+	int ndecls = defs.countNonDynamicDecls();
 	if (ndecls != 0)
 	  {
 	    Expression[] inits = new Expression[ndecls];
@@ -1221,8 +1323,19 @@ public class Translator extends Compilation
 		 || mexp.getFlag(ModuleExp.SUPERTYPE_SPECIFIED))
 	  decl.setFlag(Declaration.NONSTATIC_SPECIFIED);
       }
-    if (! moduleStatic)
-      mexp.declareThis(null);
+  }
+
+  static void vectorReverse (Vector vec, int start, int count)
+  {
+    // See http://www.azillionmonkeys.com/qed/case8.html
+    int j = count / 2;
+    int last = start + count - 1;
+    for (int i = 0; i < j; i++)
+      {
+        Object tmp = vec.elementAt(start + i);
+        vec.setElementAt(vec.elementAt(last - i), start+i);
+        vec.setElementAt(tmp, last - i);
+      }
   }
 
   public void resolveModule(ModuleExp mexp)
@@ -1233,20 +1346,31 @@ public class Translator extends Compilation
         ModuleInfo info = (ModuleInfo) pendingImports.elementAt(i++);
         ScopeExp defs = (ScopeExp) pendingImports.elementAt(i++);
         Expression posExp = (Expression) pendingImports.elementAt(i++);
+        Integer savedSize = (Integer) pendingImports.elementAt(i++);
         if (mexp == defs)
           {
             // process(BODY_PARSED);
             Expression savePos = new ReferenceExp((Object) null);
             savePos.setLine(this);
             setLine(posExp);
+            int beforeSize = formStack.size();
             kawa.standard.require.importDefinitions(null, info, null,
                                                     formStack, defs, this);
+            int desiredPosition = savedSize.intValue();
+            if (savedSize != beforeSize)
+              {
+                // Move the forms resulting from the import to desiredPosition:
+                int curSize = formStack.size();
+                int count = curSize - desiredPosition;
+                // See http://www.azillionmonkeys.com/qed/case8.html
+                vectorReverse(formStack, desiredPosition, beforeSize-desiredPosition);
+                vectorReverse(formStack, beforeSize, curSize - beforeSize);
+                vectorReverse(formStack, desiredPosition, count);
+              }
             setLine(savePos);
-            pendingImports.setElementAt(null, i-3);
-            pendingImports.setElementAt(null, i-2);
-            pendingImports.setElementAt(null, i-1);
           }
       }
+    pendingImports = null;
 
     processAccesses();
 
@@ -1256,7 +1380,9 @@ public class Translator extends Compilation
       {
 	Compilation.setCurrent(this);
 	mexp.body = makeBody(firstForm, mexp);
-	lexical.pop(mexp);
+        // In immediate mode need to preseve Declaration for current "seesion".
+        if (! immediate)
+	  lexical.pop(mexp);
       }
     finally
       {
