@@ -14,38 +14,38 @@
   (path:isAbsolute))
 (define (path-scheme (p :: path))
   (let ((s (p:getScheme)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-authority (p :: path))
   (let ((s (p:getAuthority)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-user-info (p :: path))
   (let ((s (p:getUserInfo)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-host (p :: path))
   (p:getHost))
 (define (path-file (p :: path))
   (let ((s (p:getPath)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-directory (p :: path))
   (let ((s (p:getDirectory)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f (s:toString))))
 (define (path-parent (p :: path))
   (let ((s (p:getParent)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f (s:toString))))
 (define (path-last (p :: path))
   (let ((s (p:getLast)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-extension (p :: path))
   (let ((s (p:getExtension)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-port (p :: path)) :: <int>
   (p:getPort))
 (define (path-query (p :: path))
   (let ((s (p:getQuery)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 (define (path-fragment (p :: path))
   (let ((s (p:getFragment)))
-    (if (eq? s #!null) #f (<string> s))))
+    (if (eq? s #!null) #f s)))
 
 #|
 (resolve-path path) ;; resolves symlinks
@@ -58,34 +58,30 @@
 (simplify-path)
 |#
 
-(define (file-exists? (filename :: path)) :: <boolean>
-  (filename:exists))
+(define (file-exists? (file :: path)) :: <boolean>
+  (file:exists))
 
-(define (file-directory? (filename :: path)) :: <boolean>
-  (filename:isDirectory))
+(define (file-directory? (file :: path)) :: <boolean>
+  (file:isDirectory))
 
-(define (file-readable? filename)
-  ((primitive-virtual-method "java.io.File" "canRead" "boolean" ())
-   (->pathname filename)))
+(define (file-readable? (file :: filepath)) :: <boolean>
+  ((file:toFile):canRead))
 
-(define (file-writable? filename)
-  ((primitive-virtual-method "java.io.File" "canWrite" "boolean" ())
-   (->pathname filename)))
+(define (file-writable? (file :: filepath)) :: <boolean>
+  ((file:toFile):canWrite))
 
 ;(define (file-modification-time (filename :: path)) :: <long>
 ;  (filename:getLastModified))
 
-(define (delete-file filename)
-  ((primitive-virtual-method "java.io.File" "delete" "boolean" ())
-   (->pathname filename)))
+(define (delete-file (file :: filepath)) :: <void>
+  (if (not (file:delete))
+      (primitive-throw (<java.io.IOException>:new
+			((format #f "cannot delete ~a" file):toString)))))
 
-(define (rename-file oldname newname)
-  ((primitive-virtual-method "java.io.File" "renameTo" "boolean"
-			     ("java.io.File"))
-   (->pathname oldname) (->pathname newname)))
+(define (rename-file (oldname :: filepath) (newname :: filepath))
+  ((oldname:toFile):renameTo (newname:toFile)))
 
-
-(define (copy-file from to)
+(define (copy-file (from :: path) (to :: path)) :: <void>
   (let ((in (open-input-file from))
 	(out (open-output-file to)))
     (do ((ch (read-char in) (read-char in)))
@@ -95,23 +91,29 @@
 	 #!void)
       (write-char ch out))))
 
-(define (create-directory dirname)
-  ((primitive-virtual-method <java.io.File> "mkdir" <boolean> ())
-   (->pathname dirname)))
+(define (create-directory (dirname :: filepath))
+  ((dirname:toFile):mkdir))
+
+;; In Scsh and Gambit.
+(define (directory-files (dir :: filepath))
+  (let ((files ((java.io.File (dir:toFile)):list)))
+     (if (eq? files #!null) #f
+         (gnu.lists.LList:makeList files 0)))) 
+
+;; (define (directory-for-each proc directory) ...)
 
 ; Taken from MIT Scheme
 (define (->pathname filename) :: path
   (path filename))
   
 (define (%file-separator)
-  (make <string>
-   (invoke-static <java.lang.System> 'getProperty 'file.separator)))
+  (invoke-static <java.lang.System> 'getProperty 'file.separator))
 
 (define (system-tmpdir)
   (let ((name :: <java.lang.String> ; Java2 only
 	 (invoke-static <java.lang.System> 'getProperty 'java.io.tmpdir)))
     (if (not (eq? name #!null))
-	(make <string> name)
+	name
 	(let ((sep (%file-separator)))
 	  (if (equal? sep "\\") "C:\\temp" "/tmp")))))
 
@@ -128,14 +130,16 @@
        (gnu.kawa.functions.GetModuleClass:getModuleClassURI
 	(gnu.expr.Compilation:getCurrent))))))
 
-(define-syntax resource-uri
+(define-syntax resource-url
   (syntax-rules ()
-    ((resource-uri uri)
-     ((module-uri):resolve uri))))
+    ((resource-url uri)
+     (gnu.text.URLPath:valueOf
+      (((((module-uri):resolve uri):toURL):openConnection):getURL)))))
 
 ; From MzLib.  Scsh has (create-temp-file [prefix]).
-(define (make-temporary-file #!optional (fmt "kawa~d.tmp")) :: <string>
-  (make <string> (gnu.kawa.functions.FileUtils:createTempFile fmt)))
+(define (make-temporary-file #!optional (fmt :: <string> "kawa~d.tmp"))
+  :: filepath
+  (filepath (gnu.kawa.functions.FileUtils:createTempFile (fmt:toString))))
 
 ;;; The definition of include is based on that in the portable implementation
 ;;; of syntax-case psyntax.ss, which is again based on Chez Scheme.
@@ -171,12 +175,12 @@
 		(let* ((path-pair :: <gnu.lists.PairWithPosition>
 				  (syntax-object->datum (syntax (filename))))
 		       (base :: path (path-pair:getFileName))
-		       (fname ((field path-pair 'car):toString)))
+		       (fname ((path-pair:getCar):toString)))
 		  (list
 		   (datum->syntax-object (syntax filename) 'include)
 		   (datum->syntax-object
 		    (syntax filename)
-		    (make <string> ((base:resolve fname):toString))))))))
+		    ((base:resolve fname):toString)))))))
 
 #|
 (define-syntax source-file
