@@ -13,7 +13,12 @@
 	   (= (java.lang.Math:IEEEremainder
 	       (gnu.math.DFloNum:doubleValue x)
 	       1.0)
-	      0.0))))
+	      0.0))
+      (and (instance? x <java.lang.Number>)
+	   (or (instance? x <java.lang.Long>)
+	       (instance? x <java.lang.Integer>)
+	       (instance? x <java.lang.Short>)
+	       (instance? x <java.math.BigInteger>)))))
 
 (define (exact? x) :: <boolean> 
   (and (instance? x <number>) (invoke (as <number> x) 'isExact)))
@@ -56,26 +61,32 @@
 (define (abs (x :: <number>)) :: <number>
   (invoke x 'abs))
 
-(define (quotient (x :: <real>) (y :: <real>)) :: <real>
-  (if (and (instance? x <integer>) (instance? y <integer>))
-      (invoke-static <integer> 'quotient x y)
-      (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))))
+(define-procedure quotient
+  ;; (x :: <real>) (y :: <real>)) :: <real>
+  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
+	  (invoke-static <integer> 'quotient x y))
+  (lambda ((x :: <real>) (y :: <real>)) :: <real>
+	  (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))))
 
-(define (remainder (x :: <real>) (y :: <real>)) :: <real> 
-  (if (and (instance? x <integer>) (instance? y <integer>))
-      (invoke-static <integer> 'remainder x y)
-      (if (zero? y)
-	  (if (exact? y) x (exact->inexact x))
-	  (- x (* (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))
-		  y)))))
+(define-procedure remainder
+  ;; (x :: <real>) (y :: <real>)) :: <real> 
+  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
+	  (invoke-static <integer> 'remainder x y))
+  (lambda ((x :: <real>) (y :: <real>)) :: <real>
+	  (if (zero? y)
+	      (if (exact? y) x (exact->inexact x))
+	      (- x (* (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))
+		      y)))))
 
-(define (modulo (x :: <real>) (y :: <real>)) :: <real>
-  (if (and (instance? x <integer>) (instance? y <integer>))
-      (invoke-static <integer> 'modulo x y)
-      (if (zero? y)
-	  (if (exact? y) x (exact->inexact x))
-	  (- x (* (invoke (/ x y) 'toInt (static-field <number> 'FLOOR))
-		  y)))))
+(define-procedure modulo
+  ;; (x :: <real>) (y :: <real>)) :: <real>
+  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
+	  (invoke-static <integer> 'modulo x y))
+  (lambda ((x :: <real>) (y :: <real>)) :: <real>
+	  (if (zero? y)
+	      (if (exact? y) x (exact->inexact x))
+	      (- x (* (invoke (/ x y) 'toInt (static-field <number> 'FLOOR))
+		      y)))))
 
 (define (gcd #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
@@ -84,16 +95,16 @@
 	(let ((result :: <integer> (args 0)))
 	  (do ((i :: <int> 1 (+ i 1)))
 	      ((>= i n) result)
-	    (set! result (<integer>:gcd result (<integer>:@ (args i)))))))))
+	    (set! result (gnu.math.IntNum:gcd result (<integer>:@ (args i)))))))))
 
 (define (lcm #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
     (if (zero? n)
 	1
-	(let ((result :: <integer> (<integer>:abs (<integer>:@ (args 0)))))
+	(let ((result :: <integer> (gnu.math.IntNum:abs (<integer>:@ (args 0)))))
 	  (do ((i :: <int> 1 (+ i 1)))
 	      ((>= i n) result)
-	    (set! result (<integer>:lcm result (<integer>:@ (args i)))))))))
+	    (set! result (gnu.math.IntNum:lcm result (<integer>:@ (args i)))))))))
 
 (define (numerator (x :: <rational>)) :: <integer>
   (invoke x 'numerator))
@@ -146,10 +157,13 @@
   (lambda ((x :: <double>)) :: <double>
 	  (java.lang.Math:atan x)))
 
-(define (sqrt (num :: <quantity>)) :: <quantity>
-  (gnu.math.Quantity:make
-   (invoke (invoke num 'number) 'sqrt)
-   (invoke (invoke num 'unit) 'sqrt)))
+(define-procedure sqrt
+  (lambda ((num :: <quantity>)) :: <quantity>
+	  (gnu.math.Quantity:make
+	   (invoke (invoke num 'number) 'sqrt)
+	   (invoke (invoke num 'unit) 'sqrt)))
+  (lambda ((x :: <double>)) :: <double>
+	  (java.lang.Math:sqrt x)))
 
 (define (make-rectangular (x :: <real>) (y :: <real>)) :: <complex>
   (invoke-static <complex> 'make x y))
@@ -179,23 +193,43 @@
       (invoke (as <real> num) 'toExact)
       num))
 
-(define (arithmetic-shift (value :: <integer>) (amount :: <int>)) :: <integer>
-  (invoke-static <integer> 'shift value amount))
+(define (bitwise-arithmetic-shift (value :: <integer>) (amount :: <int>)) :: <integer>
+  (gnu.math.IntNum:shift value amount))
 
-(define (lognot (i :: <integer>)) :: <integer>
-  (invoke-static <gnu.math.BitOps> 'not i))
+(define (bitwise-arithmetic-shift-left (value :: <integer>) (amount :: <int>)) :: <integer>
+  (if (< amount 0) (throw 'misc-error "shift amount must be non-negative"))
+  (gnu.math.IntNum:shift value amount))
+
+(define (bitwise-arithmetic-shift-right (value :: <integer>) (amount :: <int>)) :: <integer>
+  (if (< amount 0) (throw 'misc-error "shift amount must be non-negative"))
+  (gnu.math.IntNum:shift value (- amount)))
+
+(define (bitwise-not (i :: <integer>)) :: <integer>
+  (gnu.math.BitOps:not i))
 
 (define (logop (op :: <int>) (i :: <integer>) (j :: <integer>)) :: <integer>
   (invoke-static <gnu.math.BitOps> 'bitOp op i j))
 
-(define (logbit? (i :: <integer>) (bitno :: <int>)) :: <boolean>
-  (invoke-static <gnu.math.BitOps> 'bitValue i bitno))
+(define (bitwise-bit-set? (i :: <integer>) (bitno :: <int>)) :: <boolean>
+  (gnu.math.BitOps:bitValue i bitno))
 
-(define (bit-extract (i :: <integer>) (start :: <int>) (end :: <int>))
+(define (bitwise-copy-bit (i :: integer) (bitno :: int) (new-value :: int))
+  :: integer
+  (gnu.math.BitOps:setBitValue i bitno new-value))
+
+(define (bitwise-copy-bit-field (to :: integer) (start :: int) (end :: int) (from :: integer)) ::  integer
+  (let* ((mask1 (bitwise-arithmetic-shift-left -1 start))
+	 (mask2 (bitwise-not (bitwise-arithmetic-shift-left -1 end)))
+	 (mask (bitwise-and mask1 mask2)))
+    (bitwise-if mask
+		(bitwise-arithmetic-shift-left from start)
+		to)))
+
+(define (bitwise-bit-field (i :: <integer>) (start :: <int>) (end :: <int>))
   :: <integer>
   (invoke-static <gnu.math.BitOps> 'extract i start end))
 
-(define (logand #!rest (args :: <Object[]>)) :: <integer>
+(define (bitwise-and #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
     (if (zero? n)
 	-1
@@ -205,7 +239,7 @@
 	    (let ((arg-i :: <integer> (args i)))
 	      (set! result (gnu.math.BitOps:and result arg-i))))))))
 
-(define (logior #!rest (args :: <Object[]>)) :: <integer>
+(define (bitwise-ior #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
     (if (zero? n)
 	0
@@ -214,7 +248,7 @@
 	      ((>= i n) result)
 	    (set! result (gnu.math.BitOps:ior result (args i))))))))
 
-(define (logxor #!rest (args :: <Object[]>)) :: <integer>
+(define (bitwise-xor #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
     (if (zero? n)
 	0
@@ -223,24 +257,51 @@
 	      ((>= i n) result)
 	    (set! result (gnu.math.BitOps:xor result (args i))))))))
 
+(define (bitwise-if (e1 :: integer) (e2 :: integer) (e3  integer)) :: integer
+  (bitwise-ior (bitwise-and e1 e2)
+	       (bitwise-and (bitwise-not e1) e3)))
+
 (define (logtest (i :: <integer>) (j :: <integer>))
   (invoke-static <gnu.math.BitOps> 'test i j))
 
 (define (logcount (i :: <integer>)) :: <int>
-  (invoke-static <gnu.math.BitOps> 'bitCount i))
+  (gnu.math.BitOps:bitCount
+   (if (>= i 0) i (gnu.math.BitOps:not i))))
 
-(define (integer-length (i :: <integer>)) :: <int>
+(define (bitwise-bit-count (i :: <integer>)) :: <int>
+  (if (>= i 0)
+      (gnu.math.BitOps:bitCount i)
+      (- -1 (gnu.math.BitOps:bitCount (gnu.math.BitOps:not i)))))  
+
+(define (bitwise-length (i :: <integer>)) :: <int>
   (invoke i 'intLength))
 
+(define (bitwise-first-bit-set (i :: <integer>)) :: <int>
+  (gnu.math.BitOps:lowestBitSet i))
+
+(define (bitwise-rotate-bit-field (n :: integer) (start :: int) (end :: int) (count :: int)) :: integer
+  (let ((width (- end start)))
+    (if (positive? width)
+	(let* ((count (modulo count width))
+	       (field0 (bitwise-bit-field n start end))
+	       (field1 (bitwise-arithmetic-shift-left field0 count))
+	       (field2 (bitwise-arithmetic-shift-right
+			field0
+			(- width count)))
+	       (field (bitwise-ior field1 field2)))
+	  (bitwise-copy-bit-field n start end field))
+	n)))
+
+(define (bitwise-reverse-bit-field (n :: integer) (start :: int) (end :: int)) :: integer
+  (gnu.math.BitOps:reverseBits n start end))
+
 (define (number->string (arg :: <java.lang.Number>)
 			#!optional (radix :: <int> 10))
-  (make <string>
+  (make <gnu.lists.FString>
     (gnu.kawa.functions.Arithmetic:toString arg radix)))
 
 (define (string->number (str :: <string>) #!optional (radix :: <int> 10))
-  (let ((result (gnu.kawa.lispexpr.LispReader:parseNumber
-		 str:data 0 (str:length) #\000 radix
-		 gnu.kawa.lispexpr.LispReader:SCM_NUMBERS)))
+  (let ((result (gnu.kawa.lispexpr.LispReader:parseNumber str radix)))
     (if (instance? result <gnu.math.Numeric>) result #f)))
 
 (define (quantity->number (q :: <quantity>)) :: <complex>
