@@ -38,9 +38,8 @@
     object)))
 
 (define (record-type-name (rtd :: <class-type>))
-  (make <string>
-   (invoke-static <gnu.expr.Compilation> 'demangleName
-		  (invoke rtd 'getName) #t)))
+  (invoke-static <gnu.expr.Compilation> 'demangleName
+		  (invoke rtd 'getName) #t))
 
 (define (record-type-field-names rtd)
   ((primitive-static-method <record> "typeFieldNames"
