@@ -46,8 +46,8 @@
     (let loop ((p lst) (i :: <int> 0))
       (if (null? p) arr
 	  (let ((pp :: <pair> p))
-	    ((primitive-array-set <String>) arr i (field pp 'car))
-	    (loop (field pp 'cdr) (+ i 1)))))))
+	    ((primitive-array-set <String>) arr i pp:car)
+	    (loop pp:cdr (+ i 1)))))))
 
 (define (tokenize-string-to-string-array (string :: <String>))
   (let* ((toks (make <java.util.StringTokenizer> string))
@@ -67,8 +67,8 @@
     (let loop ((p rlist) (i :: <int> (- count 1)))
       (if (null? p) arr
 	  (let ((pp :: <pair> p))
-	    ((primitive-array-set <String>) arr i (field pp 'car))
-	    (loop (field pp 'cdr) (- i 1)))))))
+	    ((primitive-array-set <String>) arr i  pp:car)
+	    (loop pp:cdr (- i 1)))))))
 
 (define (tokenize-string-using-shell string)
   (let ((arr :: <java.lang.String[]>
@@ -79,7 +79,7 @@
     arr))
 
 (define command-parse :: <function>
-  (if (equal? (java.lang.System:getProperty '|file.separator|) '|/|)
+  (if (equal? (java.lang.System:getProperty "file.separator") "/")
       tokenize-string-using-shell
       tokenize-string-to-string-array))
 
@@ -89,7 +89,7 @@
   (let* ((messages :: <gnu.text.SourceMessages>
 		   (make <gnu.text.SourceMessages>))
 	 (comp :: <gnu.expr.Compilation>
-	       (invoke-static <kawa.lang.CompileFile> 'read source messages)))
+	       (invoke-static <kawa.lang.CompileFile> 'read (source:toString) messages)))
     (if (invoke messages 'seenErrors)
 	(primitive-throw (make <gnu.text.SyntaxException> messages)))
     (invoke comp 'compileToArchive
