@@ -169,7 +169,8 @@ public class repl extends Procedure0or1
 	  scmHomeDirectory = Boolean.FALSE;
 	Environment.getCurrent().put("home-directory", scmHomeDirectory);
 	if (initFile != null && initFile.exists())
-	  Shell.runFile(initFile.getPath(), 0);
+	  if (! Shell.runFile(initFile.getPath(), 0))
+            System.exit(-1);
       }
   }
 
@@ -236,7 +237,16 @@ public class repl extends Procedure0or1
 	    if (arg.equals ("-c"))
 	      checkInitFile();
 	    Language language = Language.getDefaultLanguage();
-	    Shell.runString(args[iArg], language, Environment.getCurrent());
+            SourceMessages messages = new SourceMessages();
+            Throwable ex = Shell.run(language, Environment.getCurrent(),
+                                     new CharArrayInPort(args[iArg]),
+                                     OutPort.outDefault(),
+                                     null, messages);
+            if (ex != null)
+              {
+                Shell.printError(ex, messages, OutPort.errDefault());
+                System.exit(-1);
+              }
 	    something_done = true;
 	  }
 	else if (arg.equals ("-f"))
@@ -248,7 +258,8 @@ public class repl extends Procedure0or1
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    Shell.runFile (filename, 0);
+	    if (! Shell.runFile (filename, 0))
+              System.exit(-1);
 	    something_done = true;
 	  }
 	else if (arg.startsWith("--script"))
@@ -273,7 +284,8 @@ public class repl extends Procedure0or1
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    Shell.runFile(filename, skipLines);
+	    if (! Shell.runFile(filename, skipLines))
+              System.exit(-1);
             return -1;
 	  }
 	else if (arg.equals("\\"))
@@ -283,6 +295,7 @@ public class repl extends Procedure0or1
 	      bad_option (arg);
 	    String filename = args[iArg];
 	    InPort freader;
+            SourceMessages messages = new SourceMessages();
 	    try
 	      {
 		InputStream fstream = new BufferedInputStream(new FileInputStream(filename));
@@ -349,21 +362,26 @@ public class repl extends Procedure0or1
 		// FIXME adjust line number
 		setArgs(args, iArg+1);
 		checkInitFile();
-		kawa.standard.load.loadSource(freader, Environment.user(), null);
-		return -1;
-	      }
-	    catch (SyntaxException ex)
-	      {
-		ex.printAll(OutPort.errDefault(), 20);
-	      }
-	    catch (java.io.FileNotFoundException ex)
-	      {
-		System.err.println("Cannot open file "+filename);
-		System.exit(1);
+                OutPort err = OutPort.errDefault();
+                Throwable ex = Shell.run(Language.getDefaultLanguage(),
+                                         Environment.getCurrent(),
+                                         freader, OutPort.outDefault(), null,
+                                         messages);
+                messages.printAll(err, 20);
+                if (ex != null)
+                  {
+                    if (ex instanceof SyntaxException)
+                      {
+                        SyntaxException se = (SyntaxException) ex;
+                        if (se.getMessages() == messages)
+                          System.exit(1);
+                      }
+                    throw ex;
+                  }
 	      }
 	    catch (Throwable ex)
 	      {
-		ex.printStackTrace(System.err);
+                Shell.printError(ex, messages,  OutPort.errDefault());
 		System.exit(1);
 	      }
 	    return -1;
@@ -383,17 +401,7 @@ public class repl extends Procedure0or1
 	    getLanguage();
 	    setArgs (args, iArg);
 	    checkInitFile();
-	    // Do this instead of just new GuiConsole in case we have
-	    // configured --without-awt.
-	    try
-	      {
-		Class.forName("kawa.GuiConsole").newInstance();
-	      }
-	    catch (Exception ex)
-	      {
-		System.err.println("failed to create Kawa window: "+ex);
-		System.exit (-1);
-	      }
+            startGuiConsole();
 	    something_done = true;
 	  }
 	else if (arg.equals ("-d"))
@@ -404,6 +412,29 @@ public class repl extends Procedure0or1
             ModuleManager manager = ModuleManager.getInstance();
 	    manager.setCompilationDirectory(args[iArg]);
 	  }
+        else if (arg.equals("--target") || arg.equals("target"))
+          {
+	    iArg++;
+	    if (iArg == maxArg)
+	      bad_option (arg);
+            arg = args[iArg];
+            if (arg.equals("7"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_7_VERSION;
+            if (arg.equals("6") || arg.equals("1.6"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_6_VERSION;
+            else if (arg.equals("5") || arg.equals("1.5"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_5_VERSION;
+            else if (arg.equals("1.4"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_4_VERSION;
+            else if (arg.equals("1.3"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_3_VERSION;
+            else if (arg.equals("1.2"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_2_VERSION;
+            else if (arg.equals("1.1"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_1_VERSION;
+            else
+              bad_option(arg);
+          }
 	else if (arg.equals ("-P"))
 	  {
 	    iArg++;
@@ -561,6 +592,8 @@ public class repl extends Procedure0or1
 	  {
 	    gnu.expr.Compilation.inlineOk = false;
 	  }
+        else if (arg.equals("--no-console"))
+          noConsole = true;
 	else if (arg.equals("--inline"))
 	  {
 	    gnu.expr.Compilation.inlineOk = true;
@@ -600,7 +633,7 @@ public class repl extends Procedure0or1
 	    System.out.print("Kawa ");
 	    System.out.print(Version.getVersion());
 	    System.out.println();
-	    System.out.println("Copyright (C) 2007 Per Bothner");
+	    System.out.println("Copyright (C) 2009 Per Bothner");
 	    something_done = true;
 	  }
 	else if (arg.length () > 0 && arg.charAt(0) == '-')
@@ -663,7 +696,7 @@ public class repl extends Procedure0or1
           {
             int ci = arg.indexOf('=');
             if (ci <= 0)
-              return iArg;
+              break;
             String key = arg.substring(0, ci);
             String value = arg.substring(ci+1);
             for (int i = 0; ; i++)
@@ -712,6 +745,7 @@ public class repl extends Procedure0or1
         String arg = args[i];
         getLanguageFromFilenameExtension(arg);
         Language language = Language.getDefaultLanguage();
+        Compilation comp = null;
         try
           {
             InPort fstream;
@@ -726,7 +760,7 @@ public class repl extends Procedure0or1
                 break; // Kludge to shut up compiler.
               }
             
-            Compilation comp
+            comp
               = language.parse(fstream, messages, Language.PARSE_PROLOG);
 
             if (compilationTopname != null)
@@ -748,11 +782,7 @@ public class repl extends Procedure0or1
           {
             if (! (ex instanceof SyntaxException)
                 || ((SyntaxException) ex).getMessages() != messages)
-              {
-                System.err.println("Internal error while compiling "+arg);
-                ex.printStackTrace(System.err);
-                System.exit(-1);
-              }
+              internalError(ex, comp, arg);
           }
         if (messages.seenErrorsOrWarnings())
           {
@@ -783,25 +813,33 @@ public class repl extends Procedure0or1
           }
         catch (Throwable ex)
           {
-            StringBuffer sbuf = new StringBuffer();
-            String file = comp.getFileName();
-            int line = comp.getLineNumber();
-            if (file != null && line > 0)
-              {
-                sbuf.append(file);
-                sbuf.append(':');
-                sbuf.append(line);
-                sbuf.append(": ");
-              }
-            sbuf.append("internal error while compiling ");
-            sbuf.append(arg);
-            System.err.println(sbuf.toString());
-            ex.printStackTrace(System.err);
-            System.exit(-1);
+            internalError(ex, comp, arg);
           }
       }
   }
 
+  static void internalError (Throwable ex, Compilation comp, Object arg)
+  {
+    StringBuffer sbuf = new StringBuffer();
+    if (comp != null)
+      {
+        String file = comp.getFileName();
+        int line = comp.getLineNumber();
+        if (file != null && line > 0)
+          {
+            sbuf.append(file);
+            sbuf.append(':');
+            sbuf.append(line);
+            sbuf.append(": ");
+          }
+      }
+    sbuf.append("internal error while compiling ");
+    sbuf.append(arg);
+    System.err.println(sbuf.toString());
+    ex.printStackTrace(System.err);
+    System.exit(-1);
+  }
+
   /** A list of standard command-line fluid names to map to static fields.
    * For each entry:
    * element 0 is a property name (before the '=' in the comamnd-line);
@@ -828,21 +866,30 @@ public class repl extends Procedure0or1
 	int iArg = processArgs(args, 0, args.length);
 	if (iArg < 0)
 	  return;
+        boolean ok;
 	if (iArg < args.length)
 	  {
 	    String filename = args[iArg];
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    Shell.runFile(filename, 0);
+	    ok = Shell.runFile(filename, 0);
 	  }
 	else
 	  {
 	    getLanguage();
 	    setArgs (args, iArg);
 	    checkInitFile();
-	    Shell.run(Language.getDefaultLanguage(), Environment.getCurrent());
-	  }
+            if (shouldUseGuiConsole())
+              startGuiConsole();
+            else
+              {
+                ok = Shell.run(Language.getDefaultLanguage(),
+                               Environment.getCurrent());
+                if (! ok)
+                  System.exit(-1);
+              }
+          }
       }
     finally
       {
@@ -853,5 +900,39 @@ public class repl extends Procedure0or1
 	  }
 	exitDecrement();
       }
-   }
+  }
+
+  public static boolean noConsole;
+
+  public static boolean shouldUseGuiConsole ()
+  {
+    if (noConsole)
+      return true;
+    try
+      {
+        if ((Class.forName("java.lang.System")
+             .getMethod("console", new Class[0])
+             .invoke(new Object[0])) == null)
+          return true;
+      }
+    catch (Throwable ex)
+      {
+      }
+    return false;
+  }
+
+  private static void startGuiConsole ()
+  {
+    // Do this instead of just new GuiConsole in case we have
+    // configured --without-awt.
+    try
+      {
+        Class.forName("kawa.GuiConsole").newInstance();
+      }
+    catch (Exception ex)
+      {
+        System.err.println("failed to create Kawa window: "+ex);
+        System.exit (-1);
+      }
+  }
 }
