@@ -4,17 +4,24 @@ import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
 import gnu.bytecode.ArrayType;
 import gnu.mapping.*;
+import gnu.math.IntNum;
 import gnu.expr.*;
 import java.util.Hashtable;
+import java.util.Vector;
+import gnu.text.Lexer;
 import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
 import gnu.lists.AbstractFormat;
+import gnu.kawa.functions.Apply;
 import gnu.kawa.functions.ApplyToArgs;
 import gnu.kawa.functions.DisplayFormat;
 import gnu.kawa.functions.NumberCompare;
 import gnu.kawa.functions.GetNamedPart;
+import gnu.kawa.functions.MultiplyOp;
 import gnu.kawa.reflect.ClassMethods;
 import gnu.kawa.reflect.StaticFieldLocation;
+import gnu.math.DFloNum;
+import gnu.kawa.xml.XmlNamespace;
 import gnu.math.Unit;
 
 public class Scheme extends LispLanguage
@@ -42,9 +49,9 @@ public class Scheme extends LispLanguage
   public static final NumberCompare numLss;
   public static final NumberCompare numLEq;
 
+  public static final Apply apply;
   public static final ApplyToArgs applyToArgs;
   static final Declaration applyFieldDecl;
-  public static final Declaration getNamedPartDecl;
 
   static {
     // (null-environment)
@@ -56,12 +63,13 @@ public class Scheme extends LispLanguage
     instance = new Scheme(kawaEnvironment);
     instanceOf = new gnu.kawa.reflect.InstanceOf(instance, "instance?");
     not = new not(instance, "not");
-    map = new map(true);
-    forEach = new map(false);
     applyToArgs = new ApplyToArgs("apply-to-args", instance);
     applyFieldDecl
       = Declaration.getDeclarationFromStatic("kawa.standard.Scheme",
                                              "applyToArgs");
+    map = new map(true, applyToArgs, applyFieldDecl);
+    forEach = new map(false, applyToArgs, applyFieldDecl);
+    apply = new Apply("apply", applyToArgs);
     isEq = new gnu.kawa.functions.IsEq(instance, "eq?");
     isEqv = new gnu.kawa.functions.IsEqv(instance, "eqv?", isEq);
     isEqual = new gnu.kawa.functions.IsEqual(instance, "equal?");
@@ -78,15 +86,7 @@ public class Scheme extends LispLanguage
 
     // Declare the special symbol $lookup$ (from the reader)
     // and bind it to getNamedPartDecl.
-    String cname = "gnu.kawa.functions.GetNamedPart";
-    String fname = "getNamedPart";
-    getNamedPartDecl = Declaration.getDeclarationFromStatic(cname, fname);
-    Symbol lookup_sym = instance.getSymbol("$lookup$");
-    StaticFieldLocation loc
-      = StaticFieldLocation.define(instance.environ, lookup_sym, null,
-                                   cname, fname);
-    loc.setProcedure();
-    loc.setDeclaration(getNamedPartDecl);
+    instance.environ.addLocation(LispLanguage.lookup_sym, null, Translator.getNamedPartLocation);
 
     repl = new kawa.repl(instance);
     instance.initScheme();
@@ -127,7 +127,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("%let", "kawa.standard.let", "let");
       defSntxStFld("let", "kawa.lib.std_syntax");
       defSntxStFld("let*", "kawa.lib.std_syntax");
-      defSntxStFld("letrec", "kawa.lib.std_syntax");
+      defSntxStFld("letrec", "kawa.lib.prim_syntax");
 
       defSntxStFld("begin", "kawa.standard.begin", "begin");
       defSntxStFld("do", "kawa.lib.std_syntax");
@@ -195,7 +195,6 @@ public class Scheme extends LispLanguage
       defProcStFld("cddddr", "kawa.lib.lists");
       defProcStFld("null?", "kawa.lib.lists");
       defProcStFld("list?", "kawa.lib.lists");
-      defProcStFld("list", "gnu.kawa.functions.MakeList");
       defProcStFld("length", "kawa.lib.lists");
       defProcStFld("append", "kawa.standard.append", "append");
       defProcStFld("reverse", "kawa.lib.lists");
@@ -282,40 +281,47 @@ public class Scheme extends LispLanguage
       defProcStFld("char>?", "kawa.lib.characters");
       defProcStFld("char<=?", "kawa.lib.characters");
       defProcStFld("char>=?", "kawa.lib.characters");
-      defProcStFld("char-ci=?", "kawa.lib.characters");
-      defProcStFld("char-ci<?", "kawa.lib.characters");
-      defProcStFld("char-ci>?", "kawa.lib.characters");
-      defProcStFld("char-ci<=?", "kawa.lib.characters");
-      defProcStFld("char-ci>=?", "kawa.lib.characters");
-      defProcStFld("char-alphabetic?", "kawa.lib.characters");
-      defProcStFld("char-numeric?", "kawa.lib.characters");
-      defProcStFld("char-whitespace?", "kawa.lib.characters");
-      defProcStFld("char-upper-case?", "kawa.lib.characters");
-      defProcStFld("char-lower-case?", "kawa.lib.characters");
+      defProcStFld("char-ci=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-ci<?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-ci>?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-ci<=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-ci>=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-alphabetic?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-numeric?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-whitespace?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-upper-case?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-lower-case?", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-title-case?", "kawa.lib.rnrs.unicode");
       defProcStFld("char->integer", "kawa.lib.characters");
       defProcStFld("integer->char", "kawa.lib.characters");
-      defProcStFld("char-upcase", "kawa.lib.characters");
-      defProcStFld("char-downcase", "kawa.lib.characters");
+      defProcStFld("char-upcase", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-downcase", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-titlecase", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-foldcase", "kawa.lib.rnrs.unicode");
+      defProcStFld("char-general-category", "kawa.lib.rnrs.unicode");
       
       //-- Section 6.7  -- complete
       defProcStFld("string?", "kawa.lib.strings");
       defProcStFld("make-string", "kawa.lib.strings");
-      defProcStFld("string", "kawa.lib.strings");
       defProcStFld("string-length", "kawa.lib.strings");
       defProcStFld("string-ref", "kawa.lib.strings");
       defProcStFld("string-set!", "kawa.lib.strings");
 
       defProcStFld("string=?", "kawa.lib.strings");
-      defProcStFld("string-ci=?", "kawa.lib.strings");
       defProcStFld("string<?", "kawa.lib.strings");
       defProcStFld("string>?", "kawa.lib.strings");
       defProcStFld("string<=?", "kawa.lib.strings");
       defProcStFld("string>=?", "kawa.lib.strings");
 
-      defProcStFld("string-ci<?", "kawa.lib.strings");
-      defProcStFld("string-ci>?", "kawa.lib.strings");
-      defProcStFld("string-ci<=?", "kawa.lib.strings");
-      defProcStFld("string-ci>=?", "kawa.lib.strings");
+      defProcStFld("string-ci=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-ci<?", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-ci>?", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-ci<=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-ci>=?", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-normalize-nfd", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-normalize-nfkd", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-normalize-nfc", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-normalize-nfkc", "kawa.lib.rnrs.unicode");
 
       defProcStFld("substring", "kawa.lib.strings");
       defProcStFld("string-append", "kawa.lib.strings");
@@ -328,7 +334,6 @@ public class Scheme extends LispLanguage
       //-- Section 6.8  -- complete
       defProcStFld("vector?", "kawa.lib.vectors");
       defProcStFld("make-vector", "kawa.lib.vectors");
-      defProcStFld("vector", "kawa.lib.vectors");
       defProcStFld("vector-length", "kawa.lib.vectors");
       defProcStFld("vector-ref", "kawa.lib.vectors");
       defProcStFld("vector-set!", "kawa.lib.vectors");
@@ -342,7 +347,7 @@ public class Scheme extends LispLanguage
 
       //-- Section 6.9  -- complete [except restricted call/cc]
       defProcStFld("procedure?", "kawa.lib.misc");
-      defProcStFld("apply", "gnu.kawa.functions.Apply", "apply");
+      defProcStFld("apply", "kawa.standard.Scheme", "apply");
       defProcStFld("map", "kawa.standard.Scheme", "map");
       defProcStFld("for-each", "kawa.standard.Scheme", "forEach");
       defProcStFld("call-with-current-continuation",
@@ -433,25 +438,52 @@ public class Scheme extends LispLanguage
 
       defProcStFld("exit", "kawa.lib.thread");
 
-      defProcStFld("arithmetic-shift", "kawa.lib.numbers");
-      defProcStFld("ash", "kawa.lib.numbers", "arithmetic$Mnshift");
-      defProcStFld("logand", "kawa.lib.numbers");
-      defProcStFld("logior", "kawa.lib.numbers");
-      defProcStFld("logxor", "kawa.lib.numbers");
-      defProcStFld("lognot", "kawa.lib.numbers");
+      defProcStFld("bitwise-arithmetic-shift", "kawa.lib.numbers");
+      defProcStFld("arithmetic-shift", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-arithmetic-shift"));
+      defProcStFld("ash", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-arithmetic-shift"));
+      defProcStFld("bitwise-arithmetic-shift-left", "kawa.lib.numbers");
+      defProcStFld("bitwise-arithmetic-shift-right", "kawa.lib.numbers");
+      defProcStFld("bitwise-and", "kawa.lib.numbers");
+      defProcStFld("logand", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-and"));
+      defProcStFld("bitwise-ior", "kawa.lib.numbers");
+      defProcStFld("logior", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-ior"));
+      defProcStFld("bitwise-xor", "kawa.lib.numbers");
+      defProcStFld("logxor", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-xor"));
+      defProcStFld("bitwise-if", "kawa.lib.numbers");
+      defProcStFld("bitwise-not", "kawa.lib.numbers");
+      defProcStFld("lognot", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-not"));
       defProcStFld("logop", "kawa.lib.numbers");
-      defProcStFld("logbit?", "kawa.lib.numbers");
+      defProcStFld("bitwise-bit-set?", "kawa.lib.numbers");
+      defProcStFld("logbit?", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-bit-set?"));
       defProcStFld("logtest", "kawa.lib.numbers");
+      defProcStFld("bitwise-bit-count", "kawa.lib.numbers");
       defProcStFld("logcount", "kawa.lib.numbers");
-      defProcStFld("bit-extract", "kawa.lib.numbers");
-      defProcStFld("integer-length", "kawa.lib.numbers");
+      defProcStFld("bitwise-copy-bit", "kawa.lib.numbers");
+      defProcStFld("bitwise-copy-bit-field", "kawa.lib.numbers");
+      defProcStFld("bitwise-bit-field", "kawa.lib.numbers");
+      defProcStFld("bit-extract", "kawa.lib.numbers",
+                   Compilation.mangleNameIfNeeded("bitwise-bit-field"));
+      defProcStFld("bitwise-length", "kawa.lib.numbers");
+      defProcStFld("integer-length", "kawa.lib.numbers", "bitwise$Mnlength");
+      defProcStFld("bitwise-first-bit-set", "kawa.lib.numbers");
+      defProcStFld("bitwise-rotate-bit-field", "kawa.lib.numbers");
+      defProcStFld("bitwise-reverse-bit-field", "kawa.lib.numbers");
 
       // These are from SLIB.
       defProcStFld("string-upcase!", "kawa.lib.strings");
       defProcStFld("string-downcase!", "kawa.lib.strings");
       defProcStFld("string-capitalize!", "kawa.lib.strings");
-      defProcStFld("string-upcase", "kawa.lib.strings");
-      defProcStFld("string-downcase", "kawa.lib.strings");
+      defProcStFld("string-upcase", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-downcase", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-titlecase", "kawa.lib.rnrs.unicode");
+      defProcStFld("string-foldcase", "kawa.lib.rnrs.unicode");
       defProcStFld("string-capitalize", "kawa.lib.strings");
       defSntxStFld("primitive-virtual-method",
                    "kawa.standard.prim_method", "virtual_method");
@@ -502,6 +534,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("define-macro", "kawa.lib.syntax");
       defSntxStFld("%define-macro", "kawa.standard.define_syntax",
                    "define_macro");
+      defSntxStFld("define-syntax-case", "kawa.lib.syntax");
       defSntxStFld("syntax-case", "kawa.standard.syntax_case", "syntax_case");
       defSntxStFld("%define-syntax", "kawa.standard.define_syntax",
                    "define_syntax");
@@ -513,9 +546,11 @@ public class Scheme extends LispLanguage
       defProcStFld("syntax-body->expression", "kawa.lib.prim_syntax");
       defProcStFld("generate-temporaries", "kawa.lib.std_syntax");
       defSntxStFld("with-syntax", "kawa.lib.std_syntax");
-      defProcStFld("syntax-source", "kawa.lib.syntax");
-      defProcStFld("syntax-line", "kawa.lib.syntax");
-      defProcStFld("syntax-column", "kawa.lib.syntax");
+      defProcStFld("identifier?", "kawa.lib.std_syntax");
+      defProcStFld("free-identifier=?", "kawa.lib.std_syntax");
+      defProcStFld("syntax-source", "kawa.lib.std_syntax");
+      defProcStFld("syntax-line", "kawa.lib.std_syntax");
+      defProcStFld("syntax-column", "kawa.lib.std_syntax");
       defSntxStFld("define-for-syntax", "kawa.lib.prim_syntax");
       defSntxStFld("include", "kawa.lib.files");
       defSntxStFld("include-relative", "kawa.lib.files");
@@ -563,8 +598,8 @@ public class Scheme extends LispLanguage
       defProcStFld("environment-bound?", "kawa.lib.misc");
       defProcStFld("scheme-implementation-version", "kawa.lib.misc");
       defProcStFld("scheme-window", "kawa.lib.windows");
-      defSntxStFld("define-procedure", "kawa.lib.syntax");
-      defProcStFld("add-procedure-properties", "kawa.lib.syntax");
+      defSntxStFld("define-procedure", "kawa.lib.std_syntax");
+      defProcStFld("add-procedure-properties", "kawa.lib.std_syntax");
       defProcStFld("make-procedure",
                    "gnu.kawa.functions.MakeProcedure", "makeProcedure");
       defProcStFld("procedure-property", "kawa.lib.misc");
@@ -590,7 +625,7 @@ public class Scheme extends LispLanguage
       defProcStFld("gentemp", "kawa.lib.syntax");
       defSntxStFld("defmacro", "kawa.lib.syntax");
       defProcStFld("setter", "gnu.kawa.functions.Setter", "setter");
-      defSntxStFld("resource-uri", "kawa.lib.files");
+      defSntxStFld("resource-url", "kawa.lib.files");
 
       defProcStFld("resolve-uri", "kawa.lib.files");
       defSntxStFld("module-uri", "kawa.lib.files");
@@ -622,6 +657,7 @@ public class Scheme extends LispLanguage
                    "define_variable");
       defSntxStFld("define-member-alias", "kawa.standard.define_member_alias",
                    "define_member_alias");
+      defSntxStFld("import", "kawa.lib.syntax");
       defSntxStFld("require", "kawa.standard.require", "require");
       defSntxStFld("module-name", "kawa.standard.module_name",
                    "module_name");
@@ -632,6 +668,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("module-static", "kawa.standard.module_static",
                    "module_static");
       defSntxStFld("module-export", "kawa.standard.export", "module_export");
+      defSntxStFld("export", "kawa.standard.export", "export");
       defSntxStFld("module-compile-options",
                    "kawa.standard.module_compile_options",
                    "module_compile_options");
@@ -750,10 +787,15 @@ public class Scheme extends LispLanguage
       defAliasStFld("*print-miser-width*",
                     "gnu.text.PrettyWriter", "miserWidthLoc");
       defAliasStFld("html", "gnu.kawa.xml.XmlNamespace", "HTML");
+      defAliasStFld("unit", "kawa.standard.Scheme", "unitNamespace");
 
       defAliasStFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
       defAliasStFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
       defAliasStFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
+      defAliasStFld("vector", "gnu.kawa.lispexpr.LangObjType", "vectorType");
+      defAliasStFld("string", "gnu.kawa.lispexpr.LangObjType", "stringType");
+      defAliasStFld("list", "gnu.kawa.lispexpr.LangObjType", "listType");
+      defAliasStFld("regex", "gnu.kawa.lispexpr.LangObjType", "regexType");
       defProcStFld("path?", "kawa.lib.files");
       defProcStFld("filepath?", "kawa.lib.files");
       defProcStFld("URI?", "kawa.lib.files");
@@ -790,37 +832,6 @@ public class Scheme extends LispLanguage
     return "Scheme";
   }
 
-  public NamedLocation lookupBuiltin (Symbol name, Object property, int hash)
-  {
-    NamedLocation loc = super.lookupBuiltin(name, property, hash);
-    if (loc == null && property == null)
-      {
-        // Special handling for names of the form "<TYPE>".  I.e. if an
-        // identifier of the form is unbound, then get a matching Type.
-        // Also, handles U$unit by doing Unit.lookup("U").  (The Scheme reader
-        // translates a quantity like 2in to (* 2 in$unit).  The advantage is
-        // is that we can have clean scoping rules for unit names;  the downside
-        // is that 2in is no longer a literal.)
-        String nam = name.getName();
-	Object val = null;
-	String uri = name.getNamespaceURI();
-        int len = nam.length();
-	if (nam.endsWith("$unit"))
-	  val = Unit.lookup(nam.substring(0, nam.length()-5));
-	else if (len > 2 && nam.charAt(0) == '<' && nam.charAt(len-1) == '>')
-          {
-            String tname = nam.substring(1, len-1);
-            val = Scheme.string2Type(tname);
-          }
-	if (val != null)
-          {
-            loc = new PlainLocation(name, null);
-            loc.set(val);
-          }
-      }
-    return loc;
-  }
-
   /** Evalutate Scheme expressions from string.
    * @param string the string constaining Scheme expressions
    * @param env the Environment to evaluate the string in
@@ -921,7 +932,7 @@ public class Scheme extends LispLanguage
     if (types == null)
       {
 	booleanType
-	  = new LangPrimType(Type.boolean_type, Scheme.getInstance());
+	  = new LangPrimType(Type.booleanType, Scheme.getInstance());
 	types = new Hashtable ();
 	types.put ("void", LangPrimType.voidType);
 	types.put ("int", LangPrimType.intType);
@@ -934,27 +945,33 @@ public class Scheme extends LispLanguage
 	types.put ("double", LangPrimType.doubleType);
 	types.put ("never-returns", Type.neverReturnsType);
 
-	types.put ("Object", Type.pointer_type);
-	types.put ("java.lang.Object", Type.pointer_type);
-	types.put ("String", Type.tostring_type);
+	types.put ("Object", Type.objectType);
+	types.put ("String", Type.toStringType);
 
-	types.put ("object", Type.pointer_type);
+	types.put ("object", Type.objectType);
 	types.put ("number", ClassType.make("gnu.math.Numeric"));
 	types.put ("quantity", ClassType.make("gnu.math.Quantity"));
 	types.put ("complex", ClassType.make("gnu.math.Complex"));
 	types.put ("real", ClassType.make("gnu.math.RealNum"));
 	types.put ("rational", ClassType.make("gnu.math.RatNum"));
-	types.put ("integer", ClassType.make("gnu.math.IntNum"));
-	types.put ("symbol", ClassType.make("java.lang.String"));
+	types.put ("integer", LangObjType.integerType);
+	types.put ("symbol", ClassType.make("gnu.mapping.Symbol"));
 	types.put ("keyword", ClassType.make("gnu.expr.Keyword"));
-	types.put ("list", ClassType.make("gnu.lists.LList"));
 	types.put ("pair", ClassType.make("gnu.lists.Pair"));
 	types.put ("pair-with-position",
 		   ClassType.make("gnu.lists.PairWithPosition"));
-	types.put ("string", ClassType.make("gnu.lists.FString"));
+        /* #ifdef use:java.lang.CharSequence */
+	types.put ("string", ClassType.make("java.lang.CharSequence"));
+        /* #else */
+        // /* better would be a union of CharSeq and java.lang.String. */
+	// types.put ("string", ClassType.make("gnu.lists.FString"));
+        /* #endif */
+	types.put ("constant-string", ClassType.make("java.lang.String"));
 	types.put ("abstract-string", ClassType.make("gnu.lists.CharSeq"));
 	types.put ("character", ClassType.make("gnu.text.Char"));
-	types.put ("vector", ClassType.make("gnu.lists.FVector"));
+	types.put ("vector", LangObjType.vectorType);
+	types.put ("string", LangObjType.stringType);
+	types.put ("list", LangObjType.listType);
 	types.put ("function", ClassType.make("gnu.mapping.Procedure"));
 	types.put ("procedure", ClassType.make("gnu.mapping.Procedure"));
 	types.put ("input-port", ClassType.make("gnu.mapping.InPort"));
@@ -962,8 +979,9 @@ public class Scheme extends LispLanguage
 	types.put ("string-output-port",
                    ClassType.make("gnu.mapping.CharArrayOutPort"));
 	types.put ("record", ClassType.make("kawa.lang.Record"));
-	types.put ("type", ClassType.make("gnu.bytecode.Type"));
-	types.put ("class-type", ClassType.make("gnu.bytecode.ClassType"));
+	types.put ("type", LangObjType.typeType);
+	types.put ("class-type", LangObjType.typeClassType);
+	types.put ("class", LangObjType.typeClass);
 
         types.put ("s8vector", ClassType.make("gnu.lists.S8Vector"));
         types.put ("u8vector", ClassType.make("gnu.lists.U8Vector"));
@@ -1003,13 +1021,25 @@ public class Scheme extends LispLanguage
     if (clas.isPrimitive())
       return getNamedType(name);
     if ("java.lang.String".equals(name))
-      return Type.tostring_type;
+      return Type.toStringType;
+    if ("gnu.math.IntNum".equals(name))
+      return LangObjType.integerType;
+    if ("gnu.lists.FVector".equals(name))
+      return LangObjType.vectorType;
+    if ("gnu.lists.LList".equals(name))
+      return LangObjType.listType;
     if ("gnu.text.Path".equals(name))
       return LangObjType.pathType;
     if ("gnu.text.URIPath".equals(name))
       return LangObjType.URIType;
     if ("gnu.text.FilePath".equals(name))
       return LangObjType.filepathType;
+    if ("java.lang.Class".equals(name))
+      return LangObjType.typeClass;
+    if ("gnu.bytecode.Type".equals(name))
+      return LangObjType.typeType;
+    if ("gnu.bytecode.ClassType".equals(name))
+      return LangObjType.typeClassType;
     return Type.make(clas);
   }
 
@@ -1045,33 +1075,298 @@ public class Scheme extends LispLanguage
     return getInstance().getTypeFor(exp);
   }
 
+  public static final Namespace unitNamespace =
+    Namespace.make("http://kawa.gnu.org/unit", "unit");
+
+  /** If a symbol is lexically unbound, look for a default binding.
+   * If the symbol is the name of an existing Java type/class,
+   * return that Class.
+   * Handles both with and without (semi-deprecated) angle-brackets:
+   * {@code <java.lang.Integer>} and {@code java.lang.Integer}.
+   * Also handles arrays, such as {@code java.lang.String[]}.
+   */
+  public Expression checkDefaultBinding (Symbol symbol, Translator tr)
+  {
+    Namespace namespace = symbol.getNamespace();
+    String local = symbol.getLocalPart();
+    if (namespace instanceof XmlNamespace)
+      return QuoteExp.getInstance(((XmlNamespace) namespace).get(local));
+    if (namespace.getName() == unitNamespace.getName())
+      {
+        Object val = Unit.lookup(local);
+        if (val != null)
+          return QuoteExp.getInstance(val);
+      }
+    String name = symbol.toString();
+    int len = name.length();
+    if (len == 0)
+      return null;
+    if (len > 1 && name.charAt(len-1) == '?')
+      {
+        int llen = local.length();
+        if (llen > 1)
+          {
+            String tlocal = local.substring(0, llen-1).intern();
+            Symbol tsymbol = namespace.getSymbol(tlocal);
+            Expression texp = tr.rewrite(tsymbol, false);
+            if (texp instanceof ReferenceExp)
+              {
+                Declaration decl = ((ReferenceExp) texp).getBinding();
+                if (decl == null || decl.getFlag(Declaration.IS_UNKNOWN))
+                  texp = null;
+              }
+            else if (! (texp instanceof QuoteExp))
+              texp = null;
+            if (texp != null)
+              {
+                LambdaExp lexp = new LambdaExp(1);
+                lexp.setSymbol(symbol);
+                Declaration param = lexp.addDeclaration((Object) null);
+                lexp.body = new ApplyExp(instanceOf,
+                                         new Expression[] {
+                                           new ReferenceExp(param), texp, });
+                return lexp;
+              }
+          }
+      }
+    char ch0 = name.charAt(0);
+    if (ch0 == '-' || ch0 == '+' || Character.digit(ch0, 10) >= 0)
+      {
+        // 1: initial + or -1 seen.
+        // 2: digits seen
+        // 3: '.' seen
+        // 4: fraction seen
+        // 5: [eE][=+]?[0-9]+ seen
+        int state = 0;
+        int i = 0;
+        for (;  i < len;  i++)
+          {
+            char ch = name.charAt(i);
+            if (Character.digit(ch, 10) >= 0)
+              state = state < 3 ? 2 : state < 5 ? 4 : 5;
+            else if ((ch == '+' || ch == '-') && state == 0)
+              state = 1;
+            else if (ch == '.' && state < 3)
+              state = 3;
+            else if ((ch == 'e' || ch == 'E') && (state == 2 || state == 4)
+                     && i + 1 < len)
+              {
+                int j = i+1;
+                char next = name.charAt(j);
+                if ((next == '-' || next == '+') && ++j < len)
+                  next = name.charAt(j);
+                if (Character.digit(next, 10) < 0)
+                  break;
+                state = 5;
+                i = j+1;
+              }
+            else
+              break;
+          }
+      tryQuantity:
+        if (i < len && state > 1)
+          {
+            DFloNum num = new DFloNum(name.substring(0,i));
+            boolean div = false;
+            Vector vec = new Vector();
+            for (; i < len ;)
+              {
+                char ch = name.charAt(i++);
+                if (ch == '*')
+                  {
+                    if (i == len) break tryQuantity;
+                    ch = name.charAt(i++);
+                  }
+                else if (ch == '/')
+                  {
+                    if (i == len || div) break tryQuantity; 
+                    div = true;
+                    ch = name.charAt(i++);
+                  }
+                int unitStart = i-1;
+                int unitEnd;
+                for (;;)
+                  {
+                    if (! Character.isLetter(ch))
+                      {
+                        unitEnd = i - 1;
+                        if (unitEnd == unitStart)
+                          break tryQuantity;
+                        break;
+                      }
+                    if (i == len)
+                      {
+                        unitEnd = i;
+                        ch = '1';
+                        break;
+                      }
+                    ch = name.charAt(i++);
+                  }
+                vec.addElement(name.substring(unitStart, unitEnd));
+                boolean expRequired = false;
+                if (ch == '^')
+                  {
+                    expRequired = true;
+                    if (i == len) break tryQuantity; 
+                    ch = name.charAt(i++);
+                  }
+                boolean neg = div;
+                if (ch == '+')
+                  {
+                    expRequired = true;
+                    if (i == len) break tryQuantity; 
+                    ch = name.charAt(i++);
+                  }
+                else if (ch == '-')
+                  {
+                    expRequired = true;
+                    if (i == len) break tryQuantity; 
+                    ch = name.charAt(i++);
+                    neg = ! neg;
+                  }
+                int nexp = 0;
+                int exp = 0;
+                for (;;)
+                  {
+                    int dig = Character.digit(ch, 10);
+                    if (dig <= 0)
+                      {
+                        i--;
+                        break;
+                      }
+                    exp = 10 * exp + dig;
+                    nexp++;
+                    if (i == len)
+                      break;
+                    ch = name.charAt(i++);
+                  }
+                if (nexp == 0)
+                  {
+                    exp = 1;
+                    if (expRequired)
+                      break tryQuantity;
+                  }
+                if (neg)
+                  exp = - exp;
+                vec.addElement(IntNum.make(exp));
+              }
+            if (i == len)
+              {
+                int nunits = vec.size() >> 1;
+                Expression[] units = new Expression[nunits];
+                for (i = 0;  i < nunits;  i++)
+                  {
+                    String uname = (String) vec.elementAt(2*i);
+                    Symbol usym = unitNamespace.getSymbol(uname.intern());
+                    Expression uref = tr.rewrite(usym);
+                    IntNum uexp = (IntNum) vec.elementAt(2*i+1);
+                    if (uexp.longValue() != 1)
+                      uref = new ApplyExp(expt.expt,
+                                          new Expression[] { uref , QuoteExp.getInstance(uexp) });
+                    units[i] = uref;
+                  }
+                Expression unit;
+                if (nunits == 1)
+                  unit = units[0];
+                else
+                  unit = new ApplyExp(MultiplyOp.$St, units);
+                return new ApplyExp(MultiplyOp.$St,
+                                    new Expression[] { QuoteExp.getInstance(num),
+                                                       unit });
+              }
+          }
+      }
+    if (len > 2 && ch0 == '<' && name.charAt(len-1) == '>')
+      {
+        name = name.substring(1, len-1);
+        len -= 2;
+      }
+    int rank = 0;
+    while (len > 2 && name.charAt(len-2) == '[' && name.charAt(len-1) == ']')
+      {
+        len -= 2;
+        rank++;
+      }
+
+    String cname = name;
+    if (rank != 0)
+      cname = name.substring(0, len);
+    try
+      { 
+        Class clas;
+        Type type = getNamedType(cname);
+        if (type != null)
+          {
+            // Somewhat inconsistent: Types named by getNamedType are Type,
+            // while standard type/classes are Class.  FIXME.
+            while (--rank >= 0)
+              type = gnu.bytecode.ArrayType.make(type);
+            return QuoteExp.getInstance(type);
+          }
+        else
+          {
+            type = Type.lookupType(cname);
+            if (type instanceof gnu.bytecode.PrimType)
+              clas = type.getReflectClass();
+            else
+              {
+                if (cname.indexOf('.') < 0)
+                  cname = (tr.classPrefix
+                           + Compilation.mangleNameIfNeeded(cname));
+                clas = ClassType.getContextClass(cname);
+              }
+          }
+        if (clas != null)
+          {
+            if (rank > 0)
+              {
+                type = Type.make(clas);
+                while (--rank >= 0)
+                  type = gnu.bytecode.ArrayType.make(type);
+                clas = type.getReflectClass();
+              }
+            return QuoteExp.getInstance(clas);
+          }
+      }
+    catch (ClassNotFoundException ex)
+      {
+        Package pack = gnu.bytecode.ArrayClassLoader.getContextPackage(name);
+        if (pack != null)
+          return QuoteExp.getInstance(pack);
+      }
+    catch (Throwable ex)
+      {
+      }
+    return null;
+  }
+
   public Expression makeApply (Expression func, Expression[] args)
   {
-    if (func instanceof ReferenceExp
-        && (((ReferenceExp) func).getBinding()==getNamedPartDecl))
-      {
-        // FIXME don't copy the args array in makeExp ...
-        return GetNamedPart.makeExp(args[0], args[1]);
-      }
     Expression[] exps = new Expression[args.length+1];
     exps[0] = func;
     System.arraycopy(args, 0, exps, 1, args.length);
     return new ApplyExp(new ReferenceExp(applyFieldDecl), exps);
   }
 
+  public Symbol asSymbol (String ident)
+  {
+    return Namespace.EmptyNamespace.getSymbol(ident);
+  }
+
   public ReadTable createReadTable ()
   {
     ReadTable tab = ReadTable.createInitial();
     tab.postfixLookupOperator = ':';
     ReaderDispatch dispatchTable = (ReaderDispatch) tab.lookup('#');
-    dispatchTable.set('\'', new ReaderQuote("syntax"));
-    dispatchTable.set('`', new ReaderQuote("quasisyntax"));
+    dispatchTable.set('\'', new ReaderQuote(asSymbol("syntax")));
+    dispatchTable.set('`', new ReaderQuote(asSymbol("quasisyntax")));
     dispatchTable.set(',', ReaderDispatchMisc.getInstance());
     tab.putReaderCtorFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
     tab.putReaderCtorFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
     tab.putReaderCtorFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
     tab.putReaderCtorFld("namespace", "kawa.lib.misc", "namespace");
     tab.putReaderCtorFld("duration", "kawa.lib.numbers", "duration");
+    tab.setFinalColonIsKeyword(true);
     return tab;
   }
 
