@@ -13,7 +13,7 @@ public class define_alias extends Syntax
   public boolean scanForDefinitions (Pair st, java.util.Vector forms,
                                      ScopeExp defs, Translator tr)
   {
-    Object formCdr = st.cdr;
+    Object formCdr = st.getCdr();
     SyntaxForm formSyntax = null;
     while (formCdr instanceof SyntaxForm)
       {
@@ -24,13 +24,13 @@ public class define_alias extends Syntax
       {
         Pair p = (Pair) formCdr;
         SyntaxForm nameSyntax = formSyntax;
-        Object name = p.car;
+        Object name = p.getCar();
         while (name instanceof SyntaxForm)
           {
             nameSyntax = (SyntaxForm) name;
             name = nameSyntax.form;
           }
-        formCdr = p.cdr;
+        formCdr = p.getCdr();
         while (formCdr instanceof SyntaxForm)
           {
             formSyntax = (SyntaxForm) formCdr;
@@ -38,24 +38,43 @@ public class define_alias extends Syntax
           }
         if ((name instanceof String || name instanceof Symbol)
             && formCdr instanceof Pair
-            && (p = (Pair) formCdr).cdr == LList.Empty)
+            && (p = (Pair) formCdr).getCdr() == LList.Empty)
           {
             Declaration decl = tr.define(name, nameSyntax, defs);
             decl.setIndirectBinding(true);
             decl.setAlias(true);
             Expression arg = tr.rewrite_car(p, formSyntax);
             if (arg instanceof ReferenceExp)
-              ((ReferenceExp) arg).setDontDereference(true);
+              {
+                ReferenceExp rarg = (ReferenceExp) arg;
+                Declaration d = Declaration.followAliases(rarg.getBinding());
+                Expression dval;
+                if (d != null
+                    && ((dval = d.getValue()) instanceof ClassExp
+                        || dval instanceof ModuleExp))
+                  {
+                    decl.setIndirectBinding(false);
+                    decl.setFlag(Declaration.IS_CONSTANT);
+                  }
+                else
+                  rarg.setDontDereference(true);
+              }
+            else if (arg instanceof QuoteExp)
+              {
+                decl.setIndirectBinding(false);
+                decl.setFlag(Declaration.IS_CONSTANT);
+              }
             else
-              arg = location.rewrite(arg, tr);
+              {
+                arg = location.rewrite(arg, tr);
+                decl.setType(ClassType.make("gnu.mapping.Location"));
+              }
             tr.mustCompileHere(); // For simplicity.
             tr.push(decl);
             SetExp sexp = new SetExp(decl, arg);
             tr.setLineOf(sexp);
             decl.noteValue(arg);
             sexp.setDefining (true);
-            if (! (arg instanceof ReferenceExp))
-              decl.setType(ClassType.make("gnu.mapping.Location"));
             forms.addElement(sexp);
             return true;
           }
