@@ -8,10 +8,13 @@ public class export extends Syntax
   public static final export module_export = new export();
   static { module_export.setName("module-export"); }
 
+  public static final export export = new export();
+  static { module_export.setName("export"); }
+
   public boolean scanForDefinitions (Pair st, java.util.Vector forms,
                                      ScopeExp defs, Translator tr)
   {
-    Object list = st.cdr;
+    Object list = st.getCdr();
     Object savePos = tr.pushPositionOf(st);
     try
       {
@@ -35,7 +38,7 @@ public class export extends Syntax
             if (list instanceof Pair)
               {
                 st = (Pair) list;
-                Object symbol = st.car;
+                Object symbol = st.getCar();
                 while (symbol instanceof SyntaxForm)
                   {
                     nameSyntax = (SyntaxForm) symbol;
@@ -62,7 +65,7 @@ public class export extends Syntax
                     if (decl.getFlag(Declaration.NOT_DEFINING))
                       Translator.setLine(decl, st);
                     decl.setFlag(Declaration.EXPORT_SPECIFIED);
-                    list = st.cdr;
+                    list = st.getCdr();
                     continue;
                   }
               }
