@@ -3,6 +3,7 @@ import gnu.lists.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.kawa.functions.IsEq;
+import gnu.kawa.functions.ApplyToArgs;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.reflect.SlotGet;
 
@@ -15,10 +16,16 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
   /** True if we should collect the result into a list. */
   boolean collect;
 
-  public map (boolean collect)
+  final ApplyToArgs applyToArgs;
+  final Declaration applyFieldDecl;
+
+  public map (boolean collect,
+              ApplyToArgs applyToArgs, Declaration applyFieldDecl)
   {
     super (collect ? "map" : "for-each");
     this.collect = collect;
+    this.applyToArgs = applyToArgs;
+    this.applyFieldDecl = applyFieldDecl;
   }
 
   /** An optimized single-list version of map. */
@@ -29,13 +36,13 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
     while (list != LList.Empty)
       {
 	Pair pair = (Pair) list;
-	Pair new_pair = new Pair (proc.apply1 (pair.car), LList.Empty);
+	Pair new_pair = new Pair (proc.apply1(pair.getCar()), LList.Empty);
 	if (last == null)
 	  result = new_pair;
 	else
-	  last.cdr = new_pair;
+	  last.setCdr(new_pair);
 	last = new_pair;
-	list = pair.cdr;
+	list = pair.getCdr();
       }
     return result;
   }
@@ -46,26 +53,30 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
     while (list != LList.Empty)
       {
 	Pair pair = (Pair) list;
-	proc.apply1 (pair.car);
-	list = pair.cdr;
+	proc.apply1(pair.getCar());
+	list = pair.getCdr();
       }
   }
 
   public Object apply2 (Object arg1, Object arg2) throws Throwable
   {
-    Procedure proc = (Procedure) arg1;
-    if (collect)
-      return map1 (proc, arg2);
-    forEach1 (proc, arg2);
-    return Values.empty;
+    if (arg1 instanceof Procedure)
+      {
+        Procedure proc = (Procedure) arg1;
+        if (collect)
+          return map1 (proc, arg2);
+        forEach1 (proc, arg2);
+        return Values.empty;
+      }
+    return applyN(new Object[] { arg1, arg2 });
   }
 
   public Object applyN (Object[] args) throws Throwable
   {
-    Procedure proc = (Procedure) (args[0]);
     int arity = args.length - 1;
-    if (arity == 1)
+    if (arity == 1 && args[0] instanceof Procedure)
       {
+        Procedure proc = (Procedure) (args[0]);
 	if (collect)
 	  return map1 (proc, args[1]);
 	forEach1 (proc, args[1]);
@@ -79,7 +90,22 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
       result = Values.empty;;
     Object[] rest = new Object [arity];
     System.arraycopy (args, 1, rest, 0, arity);
-    Object[] each_args = new Object [arity];
+    Procedure proc;
+    int need_apply;
+    Object[] each_args;
+    if (args[0] instanceof Procedure)
+      {
+        need_apply = 0;
+        each_args = new Object[arity];
+        proc = (Procedure) args[0];
+      }
+    else
+      {
+        need_apply = 1;
+        each_args = new Object[arity+1];
+        each_args[0] = args[0];
+        proc = applyToArgs;
+      }
     for (;;)
       {
 	for (int i = 0;  i < arity;  i++)
@@ -88,8 +114,8 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
 	    if (list == LList.Empty)
 	      return result;
 	    Pair pair = (Pair) list;
-	    each_args[i] = pair.car;
-	    rest[i] = pair.cdr;
+	    each_args[need_apply+i] = pair.getCar();
+	    rest[i] = pair.getCdr();
 	  }
 	Object value = proc.applyN (each_args);
 	if (collect)
@@ -98,21 +124,23 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
 	    if (last == null)
 	      result = new_pair;
 	    else
-	      last.cdr = new_pair;
+	      last.setCdr(new_pair);
 	    last = new_pair;
 	  }
       }
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    // FIXME: We should inline the list arguments first before inlining the
+    // procedure argument, for better type inference etc.
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     int nargs = args.length;
     if (nargs < 2)
       return exp;  // ERROR
 
-    InlineCalls inliner = (InlineCalls) walker;
-
     nargs--;
 
     Expression proc = args[0];
@@ -154,18 +182,18 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
 	pargs[i].noteValue(inits3[i]);
       }
     Declaration resultDecl = collect ? lexp.addDeclaration("result") : null;
-    
-    Expression[] doArgs = new Expression[nargs];
+    Expression[] doArgs = new Expression[1+nargs];
     Expression[] recArgs = new Expression[collect ? nargs + 1 : nargs];
     for (int i = 0;  i < nargs;  i++)
       {
-	doArgs[i] = inliner.walkApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "car"));
-	recArgs[i] = inliner.walkApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "cdr"));
+	doArgs[i+1] = walker.walkApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "car"));
+	recArgs[i] = walker.walkApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "cdr"));
       }
     if (! procSafeForMultipleEvaluation)
       proc = new ReferenceExp(procDecl);
-    Expression doit = inliner.walkApplyOnly(new ApplyExp(proc, doArgs));
-    Expression rec = inliner.walkApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), recArgs));
+    doArgs[0] = proc;
+    Expression doit = walker.walkApplyOnly(new ApplyExp(new ReferenceExp(applyFieldDecl), doArgs));
+    Expression rec = walker.walkApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), recArgs));
     if (collect)
       {
 	Expression[] consArgs = new Expression[2];
@@ -191,14 +219,14 @@ public class map  extends gnu.mapping.ProcedureN implements CanInline
 	Expression result
 	  = collect ? (Expression) new ReferenceExp(resultDecl)
 	  : (Expression) QuoteExp.voidExp;
-	lexp.body = new IfExp(inliner.walkApplyOnly(new ApplyExp(isEq, compArgs)),
+	lexp.body = new IfExp(walker.walkApplyOnly(new ApplyExp(isEq, compArgs)),
 			      result, lexp.body);
 	initArgs[i] = args[i+1];
       }
     if (collect)
       initArgs[nargs] = empty;
 
-    Expression body = inliner.walkApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), initArgs));
+    Expression body = walker.walkApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), initArgs));
     if (collect)
       {
 	Expression[] reverseArgs = new Expression[1];
