@@ -12,7 +12,7 @@ public class module_compile_options extends Syntax
   public boolean scanForDefinitions (Pair st, java.util.Vector forms,
                                      ScopeExp defs, Translator tr)
   {
-    Object rest = with_compile_options.getOptions(st.cdr, null, this, tr);
+    Object rest = with_compile_options.getOptions(st.getCdr(), null, this, tr);
     if (rest != LList.Empty)
       tr.error('e', getName() + " key must be a keyword");
     return true;
