@@ -11,14 +11,14 @@ public class module_name extends Syntax
 
   public void scanForm (Pair form, ScopeExp defs, Translator tr)
   {
-    Object form_cdr = form.cdr;
+    Object form_cdr = form.getCdr();
     SyntaxForm nameSyntax = null;
     while (form_cdr instanceof SyntaxForm)
       {
 	nameSyntax = (SyntaxForm) form_cdr;
 	form_cdr = nameSyntax.form;
       }
-    Object arg = form_cdr instanceof Pair ? ((Pair) form_cdr).car : null;
+    Object arg = form_cdr instanceof Pair ? ((Pair) form_cdr).getCar() : null;
     while (arg instanceof SyntaxForm)
       {
 	nameSyntax = (SyntaxForm) arg;
@@ -28,19 +28,19 @@ public class module_name extends Syntax
     Pair p;
     String err = null;
     Declaration decl = null;
-    if (arg instanceof Pair && (p = (Pair) arg).car == "quote")
+    if (arg instanceof Pair && (p = (Pair) arg).getCar() == "quote")
       {
-	arg = p.cdr;
+	arg = p.getCdr();
 	if (! (arg instanceof Pair)
-	    || (p = (Pair) arg).cdr != LList.Empty
-	    || ! (p.car instanceof String))
+	    || (p = (Pair) arg).getCdr() != LList.Empty
+	    || ! (p.getCar() instanceof String))
 	  err = "invalid quoted symbol for 'module-name'";
         else
-          name = (String) p.car;
+          name = (String) p.getCar();
       }
-    else if (arg instanceof FString)
+    else if (arg instanceof FString || arg instanceof String)
       name = arg.toString();
-    else if (arg instanceof String || arg instanceof Symbol)
+    else if (arg instanceof Symbol)
       {
 	name = arg.toString();
 	int len = name.length();
@@ -49,10 +49,8 @@ public class module_name extends Syntax
 	    && name.charAt(len-1) == '>')
 	  {
 	    name = name.substring(1, len-1);
-            decl = tr.define(arg, nameSyntax, defs);
 	  }
-	else
-	  err = "not implemented: plain name in module-name";
+        decl = tr.define(arg, nameSyntax, defs);
       }
     else
       err = "un-implemented expression in module-name";
@@ -85,10 +83,12 @@ public class module_name extends Syntax
 
         if (decl != null)
           {
-            decl.noteValue(module);
+            decl.noteValue(new QuoteExp(tr.mainClass, Compilation.typeClass));
             decl.setFlag(Declaration.IS_CONSTANT|Declaration.PRIVATE_SPECIFIED);
+            if (module.outer == null)
+              decl.setFlag(Declaration.STATIC_SPECIFIED);
             decl.setPrivate(true);
-            decl.setType(Compilation.typeClassType);
+            decl.setType(Compilation.typeClass);
           }
         tr.mustCompileHere();
       }
