@@ -4,6 +4,7 @@ import kawa.lang.*;
 import gnu.lists.*;
 import java.util.Vector;
 import gnu.mapping.Symbol;
+import gnu.mapping.Namespace;
 import gnu.bytecode.Type;
 import gnu.kawa.functions.Convert;
 
@@ -15,6 +16,8 @@ public class object extends Syntax
 
   Lambda lambda;
   public static final Keyword accessKeyword = Keyword.make("access");
+  public static final Keyword classNameKeyword = Keyword.make("class-name");
+  public static final Keyword interfaceKeyword = Keyword.make("interface");
   public static final Keyword throwsKeyword = Keyword.make("throws");
   static final Keyword typeKeyword = Keyword.make("type");
   public static final Keyword allocationKeyword = Keyword.make("allocation");
@@ -24,6 +27,8 @@ public class object extends Syntax
   static final Keyword init_valueKeyword = Keyword.make("init-value");
   static final Keyword init_keywordKeyword = Keyword.make("init-keyword");
 
+  static final Symbol coloncolon = Namespace.EmptyNamespace.getSymbol("::");
+
   public object(Lambda lambda)
   {
     this.lambda = lambda;
@@ -31,16 +36,16 @@ public class object extends Syntax
 
   public Expression rewriteForm (Pair form, Translator tr)
   {
-    if (! (form.cdr instanceof Pair))
+    if (! (form.getCdr() instanceof Pair))
       return tr.syntaxError("missing superclass specification in object");
-    Pair pair = (Pair) form.cdr;
+    Pair pair = (Pair) form.getCdr();
     ObjectExp oexp = new ObjectExp();
-    if (pair.car instanceof FString)
+    if (pair.getCar() instanceof FString)
       {
-        // oexp.setName(pair.car.toString());
-	if (! (pair.cdr instanceof Pair))
+        // oexp.setName(pair.getCar().toString());
+	if (! (pair.getCdr() instanceof Pair))
 	  return tr.syntaxError("missing superclass specification after object class name");
-	pair = (Pair) pair.cdr;
+	pair = (Pair) pair.getCdr();
       }
     Object[] saved = scanClassDef(pair, oexp, tr);
     if (saved != null)
@@ -54,10 +59,12 @@ public class object extends Syntax
   public Object[] scanClassDef (Pair pair, ClassExp oexp, Translator tr)
   {
     tr.mustCompileHere();
-    Object superlist = pair.car;
-    Object components = pair.cdr;
+    Object superlist = pair.getCar();
+    Object components = pair.getCdr();
+    Object classNamePair = null;
     LambdaExp method_list = null;
     LambdaExp last_method = null;
+    int classAccessFlag = 0;
     // First pass (get Declarations).
     Vector inits = new Vector(20);
     for (Object obj = components;  obj != LList.Empty;  )
@@ -72,18 +79,62 @@ public class object extends Syntax
 	    return null;
 	  }
 	pair = (Pair) obj;
-	Object pair_car = pair.car;
+	Object pair_car = pair.getCar();
 	while (pair_car instanceof SyntaxForm)
 	  pair_car = ((SyntaxForm) pair_car).form;
+	obj = pair.getCdr(); // Next member.
+	Object savedPos1 = tr.pushPositionOf(pair);
+        if (pair_car instanceof Keyword)
+          {
+            while (obj instanceof SyntaxForm)
+              obj = ((SyntaxForm) obj).form;
+            if (obj instanceof Pair)
+              {
+                if (pair_car == interfaceKeyword)
+                  {
+                    Object val = ((Pair) obj).getCar();
+                    if (val == Boolean.FALSE)
+                      oexp.setFlag(ClassExp.CLASS_SPECIFIED);
+                    else
+                      oexp.setFlag(ClassExp.INTERFACE_SPECIFIED);
+                    obj = ((Pair) obj).getCdr();
+                    tr.popPositionOf(savedPos1);
+                    continue;
+                  }
+                if (pair_car == classNameKeyword)
+                  {
+                    if (classNamePair != null)
+                      tr.error('e', "duplicate class-name specifiers");
+                    classNamePair = obj;
+                    obj = ((Pair) obj).getCdr();
+                    tr.popPositionOf(savedPos1);
+                    continue;
+                  }
+                if (pair_car == accessKeyword)
+                  {
+                    if (oexp.nameDecl == null)
+                      tr.error('e', "access specifier for anonymous class");
+                    else if (classAccessFlag != 0)
+                      tr.error('e', "duplicate access specifiers");
+                    else
+                      {
+                        classAccessFlag = matchAccess(((Pair) obj).getCar(), tr);
+                        if ((classAccessFlag & (Declaration.PRIVATE_ACCESS|Declaration.PROTECTED_ACCESS)) != 0)
+                          tr.error('e', "invalid class access specifier");
+                      }
+                    obj = ((Pair) obj).getCdr();
+                    tr.popPositionOf(savedPos1);
+                    continue;
+                  }
+              }
+          }
 	if (! (pair_car instanceof Pair))
 	  {
 	    tr.error('e', "object member not a list");
 	    return null;
 	  }
-	obj = pair.cdr; // Next member.
-	Object savedPos1 = tr.pushPositionOf(pair);
 	pair = (Pair) pair_car;
-	pair_car = pair.car;
+	pair_car = pair.getCar();
 	while (pair_car instanceof SyntaxForm)
 	  pair_car = ((SyntaxForm) pair_car).form;
 	if (pair_car instanceof String || pair_car instanceof Symbol
@@ -94,7 +145,6 @@ public class object extends Syntax
 	    Object args;
 	    Declaration decl;
 	    int allocationFlag = 0;
-	    String accessFlagName = null;
 	    int accessFlag = 0;
 	    if (sname instanceof Keyword)
 	      {
@@ -107,7 +157,7 @@ public class object extends Syntax
 		decl.setSimple(false);
 		decl.setFlag(Declaration.FIELD_OR_METHOD);
 		Translator.setLine(decl, pair);
-		args = pair.cdr;
+		args = pair.getCdr();
 	      }
 	    int nKeywords = 0;
 	    boolean seenInit = false;
@@ -118,19 +168,19 @@ public class object extends Syntax
                   args = ((SyntaxForm) args).form;
 		pair = (Pair) args;
 		Pair keyPair = pair;
-		Object key = pair.car;
+		Object key = pair.getCar();
                 while (key instanceof SyntaxForm)
                   key = ((SyntaxForm) key).form;
 		Object savedPos2 = tr.pushPositionOf(pair);
-		args = pair.cdr;
-		if ((key == "::" || key instanceof Keyword)
+		args = pair.getCdr();
+		if ((key == coloncolon || key instanceof Keyword)
 		    && args instanceof Pair)
 		  {
 		    nKeywords++;
 		    pair = (Pair) args;
-		    Object value = pair.car;
-		    args = pair.cdr;
-		    if (key == "::" || key == typeKeyword)
+		    Object value = pair.getCar();
+		    args = pair.getCdr();
+		    if (key == coloncolon || key == typeKeyword)
 		      typePair = pair;
 		    else if (key == allocationKeyword)
 		      {
@@ -168,38 +218,14 @@ public class object extends Syntax
 		      }
 		    else if (key == accessKeyword)
 		      {
-			String newAccessFlag = null;
-			if (matches(value, "private", tr))
-			  {
-			    newAccessFlag = "private";
-			    accessFlag = Declaration.PRIVATE_ACCESS;
-			  }
-			else if (matches(value, "protected", tr))
-			  {
-			    newAccessFlag = "protected";
-			    accessFlag = Declaration.PROTECTED_ACCESS;
-			  }
-			else if (matches(value, "public", tr))
-			  {
-			    newAccessFlag = "public";
-			    accessFlag = Declaration.PUBLIC_ACCESS;
-			  }
-			else if (matches(value, "package", tr))
-			  {
-			    newAccessFlag = "package";
-			    accessFlag = Declaration.PACKAGE_ACCESS;
-			  }
-			else
-			  {
-			    tr.error('e', "unknown access specifier");
-			  }
-			if (accessFlagName != null && newAccessFlag != null)
-			  {
-			    tr.error('e', "duplicate access specifiers - "
-				     + accessFlagName + " and "
-				     + newAccessFlag);
-			  }
-			accessFlagName = newAccessFlag;
+                        int newAccessFlag = matchAccess(value, tr);
+                        if (newAccessFlag == 0)
+                          tr.error('e', "unknown access specifier");
+                        else if (accessFlag != 0)
+                          tr.error('e', "duplicate access specifiers - "
+                                   + accessString(accessFlag) + " and "
+                                   + accessString(newAccessFlag));
+			accessFlag = newAccessFlag;
 		      }
 		    else
 		      {
@@ -214,12 +240,12 @@ public class object extends Syntax
 		  }
 		else if (args instanceof Pair
 			 && nKeywords == 0 && ! seenInit && typePair == null
-			 && (pair = (Pair) args).cdr == LList.Empty)
+			 && (pair = (Pair) args).getCdr() == LList.Empty)
 		  {
 		    // Backward compatibility.
 		    typePair = keyPair;
 		    initPair = pair;
-		    args = pair.cdr;
+		    args = pair.getCdr();
 		    seenInit = true;
 		  }
 		else
@@ -266,7 +292,7 @@ public class object extends Syntax
 	else if (pair_car instanceof Pair)
 	  { // Method declaration.
 	    Pair mpair = (Pair) pair_car;
-	    Object mname = mpair.car;
+	    Object mname = mpair.getCar();
 	    if (! (mname instanceof String)
 		&& ! (mname instanceof Symbol))
 	      {
@@ -293,12 +319,16 @@ public class object extends Syntax
 	  tr.error ('e', "invalid field/method definition");
 	tr.popPositionOf(savedPos1);
       }
+  if (classAccessFlag != 0)
+    oexp.nameDecl.setFlag(classAccessFlag);
+
     Object[] result = {
       oexp,
       components,
       inits,
       method_list,
-      superlist
+      superlist,
+      classNamePair
     };
     return result;
   }
@@ -310,6 +340,7 @@ public class object extends Syntax
     Vector inits = (Vector) saved[2];
     LambdaExp method_list = (LambdaExp) saved[3];
     Object superlist = saved[4];
+    Object classNamePair = saved[5];
     oexp.firstChild = method_list;
 
     int num_supers = Translator.listLength(superlist);
@@ -328,11 +359,22 @@ public class object extends Syntax
 	  }
 	Pair superpair = (Pair) superlist;
 	supers[i] = tr.rewrite_car(superpair, false);
-	superlist = superpair.cdr;
+	superlist = superpair.getCdr();
+      }
+
+    if (classNamePair != null)
+      {
+        Expression classNameExp = tr.rewrite_car((Pair) classNamePair, false);
+        Object classNameVal = classNameExp.valueIfConstant();
+        String classNameSpecifier;
+        if (classNameVal instanceof String
+            && (classNameSpecifier = (String) classNameVal).length() > 0)
+          oexp.classNameSpecifier = classNameSpecifier;
+        else
+          tr.error('e', "class-name specifier must be a non-empty string literal");
       }
     oexp.supers = supers;
 
-    oexp.setClassName(tr);
     oexp.setTypes(tr);
 
     // First a pass over init-form: specifiers, since these are evaluated
@@ -360,7 +402,7 @@ public class object extends Syntax
 	  }
 	Pair pair = (Pair) obj;
 	Object savedPos1 = tr.pushPositionOf(pair);
-	Object pair_car = pair.car;
+	Object pair_car = pair.getCar();
 	SyntaxForm memberSyntax = componentsSyntax;
 	while (pair_car instanceof SyntaxForm)
 	  {
@@ -369,9 +411,16 @@ public class object extends Syntax
 	  }
 	try
 	  {
-	    obj = pair.cdr; // Next member.
+	    obj = pair.getCdr(); // Next member.
+            if (pair_car instanceof Keyword
+                && obj instanceof Pair)
+              {
+                // Handled at scan time.
+                obj = ((Pair) obj).getCdr();
+                continue;
+              }
 	    pair = (Pair) pair_car;
-	    pair_car = pair.car;
+	    pair_car = pair.getCar();
 	    SyntaxForm memberCarSyntax = memberSyntax;
 	    while (pair_car instanceof SyntaxForm)
 	      {
@@ -383,7 +432,7 @@ public class object extends Syntax
 	      { // Field declaration.
 		Object type = null;
 		int nKeywords = 0;
-		Object args = pair_car instanceof Keyword ? pair : pair.cdr;
+		Object args = pair_car instanceof Keyword ? pair : pair.getCdr();
 		Pair initPair = null;
                 SyntaxForm initSyntax = null;
 		while (args != LList.Empty)
@@ -394,19 +443,19 @@ public class object extends Syntax
                         args = memberSyntax.form;
                       }
 		    pair = (Pair) args;
-		    Object key = pair.car;
+		    Object key = pair.getCar();
                     while (key instanceof SyntaxForm)
                       key = ((SyntaxForm) key).form;
 		    Object savedPos2 = tr.pushPositionOf(pair);
-		    args = pair.cdr;
-		    if ((key == "::" || key instanceof Keyword)
+		    args = pair.getCdr();
+		    if ((key == coloncolon || key instanceof Keyword)
 			&& args instanceof Pair)
 		      {
 			nKeywords++;
 			pair = (Pair) args;
-			Object value = pair.car;
-			args = pair.cdr;
-			if (key == "::" || key == typeKeyword)
+			Object value = pair.getCar();
+			args = pair.getCdr();
+			if (key == coloncolon || key == typeKeyword)
 			  type = value;
 			else if (key == initKeyword
 				 || key == initformKeyword
@@ -429,13 +478,13 @@ public class object extends Syntax
 		      }
 		    else if (args instanceof Pair && nKeywords == 0
 			     && initPair == null && type == null
-			     && (pair = (Pair) args).cdr == LList.Empty)
+			     && (pair = (Pair) args).getCdr() == LList.Empty)
 		      {
 			// Backward compatibility.
 			type = key;
 			initPair = pair;
                         initSyntax = memberSyntax;
-			args = pair.cdr;
+			args = pair.getCdr();
 		      }
 		    else
 		      {
@@ -465,14 +514,14 @@ public class object extends Syntax
 		  tr.setCurrentScope(memberSyntax.scope);
                 if ("*init*".equals(meth.getName()))
                   {
-                    meth.setReturnType(Type.void_type);
+                    meth.setReturnType(Type.voidType);
                     if (! oexp.isSimple())
                       tr.error('e', "'*init*' methods only supported for simple classes");
                   }
                 Translator.setLine(meth, pair);
                 LambdaExp saveLambda = tr.curMethodLambda;
                 tr.curMethodLambda = meth;
-		lambda.rewrite(meth, ((Pair) pair_car).cdr, pair.cdr, tr,
+		lambda.rewrite(meth, ((Pair) pair_car).getCdr(), pair.getCdr(), tr,
 			       memberCarSyntax != null
 			       && (memberSyntax == null
 				   || memberCarSyntax.scope != memberSyntax.scope)
@@ -542,7 +591,7 @@ public class object extends Syntax
         initValue = sexp;
       }
     else
-      initValue = Convert.makeCoercion(initValue, new QuoteExp(Type.void_type));
+      initValue = Convert.makeCoercion(initValue, new QuoteExp(Type.voidType));
     ((BeginExp) initMethod.body).add(initValue);
     tr.curMethodLambda = saveLambda;
     tr.pop(initMethod);
@@ -560,13 +609,40 @@ public class object extends Syntax
     else if (exp instanceof FString)
       value = ((FString) exp).toString();
     else if (exp instanceof Pair
-	     && tr.matches((pair = (Pair) exp).car, Scheme.quote_sym)
-	     && pair.cdr instanceof Pair
-	     && (pair = (Pair) pair.cdr).cdr == LList.Empty
-	     && pair.car instanceof String)
-      value = (String) pair.car;
+	     && tr.matches((pair = (Pair) exp).getCar(), Scheme.quote_sym)
+	     && pair.getCdr() instanceof Pair
+	     && (pair = (Pair) pair.getCdr()).getCdr() == LList.Empty
+	     && pair.getCar() instanceof gnu.mapping.SimpleSymbol)
+      value = pair.getCar().toString();
     else
       return false;
     return tag == null || tag.equals(value);
   }
+
+  static int matchAccess (Object value, Translator tr)
+  {
+    if (matches(value, "private", tr))
+      return Declaration.PRIVATE_ACCESS;
+    else if (matches(value, "protected", tr))
+      return Declaration.PROTECTED_ACCESS;
+    else if (matches(value, "public", tr))
+      return Declaration.PUBLIC_ACCESS;
+    else if (matches(value, "package", tr))
+      return Declaration.PACKAGE_ACCESS;
+    else
+      return 0;
+  }
+
+  static String accessString (int accessFlag)
+  {
+    if (accessFlag == Declaration.PRIVATE_ACCESS)
+      return "private";
+    if (accessFlag == Declaration.PROTECTED_ACCESS)
+      return "protected";
+    if (accessFlag == Declaration.PUBLIC_ACCESS)
+      return "public";
+    if (accessFlag == Declaration.PACKAGE_ACCESS)
+      return "package";
+    return "<internal error>";
+  }
 }
