@@ -87,8 +87,14 @@ public class require extends Syntax
     map("testing", SLIB_PREFIX + "testing");
     map("srfi-69", SLIB_PREFIX + "srfi69");
     map("hash-table", SLIB_PREFIX + "srfi69");
+    map("basic-hash-tables", SLIB_PREFIX + "srfi69");
+    map("srfi-95", "kawa.lib.srfi95");
+    map("sorting-and-merging", "kawa.lib.srfi95");
+    map("regex", "kawa.lib.kawa.regex");
+    map("pregexp", SLIB_PREFIX + "pregexp");
     map("gui", SLIB_PREFIX + "gui");
     map("swing-gui", SLIB_PREFIX + "swing");
+    map("android-defs", "gnu.kawa.android.defs");
   }
 
   public static String mapFeature(String featureName)
@@ -98,7 +104,8 @@ public class require extends Syntax
 
   public static Object find(String typeName)
   {
-    return ModuleInfo.find(typeName).getInstance();
+    return ModuleManager.getInstance()
+      .findWithClassName(typeName).getInstance();
   }
 
   public boolean scanForDefinitions (Pair st, Vector forms,
@@ -112,30 +119,35 @@ public class require extends Syntax
         // it be emitted at the end of the 'body'.
         return true;
       }
-    Pair args = (Pair) st.cdr;
-    Object name = args.car;
+    Pair args = (Pair) st.getCdr();
+    Object name = args.getCar();
     Type type = null;
     Pair p;
     if (name instanceof Pair
-        && tr.matches((p = (Pair) name).car, Scheme.quote_sym))
+        && tr.matches((p = (Pair) name).getCar(), Scheme.quote_sym))
       {
-	name = p.cdr;
+	name = p.getCdr();
 	if (! (name instanceof Pair)
-	    || (p = (Pair) name).cdr != LList.Empty
-	    || ! (p.car instanceof String))
+	    || (p = (Pair) name).getCdr() != LList.Empty
+	    || ! (p.getCar() instanceof Symbol))
 	  {
 	    tr.error('e', "invalid quoted symbol for 'require'");
 	    return false;
 	  }
-	name = mapFeature((String) p.car);
+	name = mapFeature(p.getCar().toString());
 	if (name == null)
 	  {
-	    tr.error('e', "unknown feature name '"+p.car+"' for 'require'");
+	    tr.error('e', "unknown feature name '"+p.getCar()+"' for 'require'");
 	    return false;
 	  }
 	type = ClassType.make((String) name);
       }
-    else if (name instanceof FString)
+    else if
+      /* #ifdef use:java.lang.CharSequence */
+      (name instanceof CharSequence)
+      /* #else */
+      // (name instanceof String || name instanceof CharSeq)
+      /* #endif */
       {
         String sourceName = name.toString();
         ModuleInfo info = lookupModuleFromSourcePath(sourceName, defs);
@@ -146,43 +158,48 @@ public class require extends Syntax
           }
         return importDefinitions(null, info, null, forms, defs, tr);
       }
-    else
+    else if (name instanceof Symbol && ! tr.selfEvaluatingSymbol(name))
       {
-	if (name instanceof String)
-	  {
-	    String str = (String) name;
-	    int len = str.length();
-	    if (len > 2
-		&& str.charAt(0) == '<'
-		&& str.charAt(len-1) == '>')
-	      {
-		str = str.substring(1, len-1);
-		if (str.indexOf('.') < 0)
-		  str = tr.classPrefix + str;
-                if (args.cdr instanceof Pair
-                    && ((Pair) args.cdr).car instanceof FString)
+        type = tr.getLanguage().getTypeFor(tr.rewrite(name, false));
+        if (type instanceof ClassType && args.getCdr() instanceof Pair)
+          {
+            name = ((Pair) args.getCdr()).getCar();
+            if
+              /* #ifdef use:java.lang.CharSequence */
+              (name instanceof CharSequence)
+              /* #else */
+              // (name instanceof String || name instanceof CharSeq)
+              /* #endif */
+              {
+                String sourceName = name.toString();
+                ModuleInfo info = lookupModuleFromSourcePath(sourceName, defs);
+                if (info == null)
                   {
-                    String sourceName = ((Pair) args.cdr).car.toString();
-                    ModuleInfo info = lookupModuleFromSourcePath(sourceName, defs);
-                    if (info == null)
-                      {
-                        tr.error('e', "malformed URL: "+sourceName);
-                        return false;
-                      }
-                    return importDefinitions(str,
-                                             info,
-                                             null, forms, defs, tr);
+                    tr.error('e', "malformed URL: "+sourceName);
+                    return false;
                   }
-		type = Scheme.string2Type(str);
-	      }
+                return importDefinitions(type.getName(), info, null,
+                                         forms, defs, tr);
+              }
 	  }
       }
-    if (type == null)
+    if (! (type instanceof ClassType))
       {
 	tr.error('e', "invalid specifier for 'require'");
 	return false;
       }
-    importDefinitions(null, ModuleInfo.find(type), null, forms, defs, tr);
+    ModuleInfo minfo;
+    try
+      {
+        minfo = ModuleInfo.find((ClassType) type);
+      }
+    catch (Exception ex)
+      {
+	tr.error('e', "unknown class "+type.getName());
+	return false;
+      }
+    importDefinitions(null, minfo, null,
+                      forms, defs, tr);
     return true;
   }
 
@@ -200,15 +217,12 @@ public class require extends Syntax
    *   or null if unknown.
    */
   public static boolean
-  importDefinitions (String className, ModuleInfo info, String uri,
-                     Vector forms,
-                     ScopeExp defs, Compilation tr)
+  importDefinitions (String className, ModuleInfo info, Procedure renamer,
+                     Vector forms, ScopeExp defs, Compilation tr)
   {
     ModuleManager manager = ModuleManager.getInstance();
-    String sourceName = info.sourcePath;
     long now;
     if ((info.getState() & 1) == 0
-        && sourceName != null
         && ! info.checkCurrent(manager, (now = System.currentTimeMillis())))
       {
         SourceMessages messages = tr.getMessages();
@@ -219,8 +233,10 @@ public class require extends Syntax
             InPort fstream = InPort.openFile(info.getSourceAbsPath());
             info.clearClass();
             info.className = className;
-            comp = language.parse(fstream, messages, info);
-            comp.immediate = tr.immediate;
+            int options = Language.PARSE_PROLOG;
+            if (tr.immediate)
+              options |= Language.PARSE_IMMEDIATE;
+            comp = language.parse(fstream, messages, options, info);
           }
         catch (java.io.FileNotFoundException ex)
           {
@@ -253,6 +269,7 @@ public class require extends Syntax
           {
             // Oops.  We found a cycle.
             tr.pushPendingImport(info, defs);
+            tr.pendingImports.push(Integer.valueOf(forms.size()));
             return true;
           }
       }
@@ -260,8 +277,9 @@ public class require extends Syntax
     ClassType type = info.getClassType();
     String tname = info.className;
     boolean immediate = tr.immediate && defs instanceof ModuleExp;
-    boolean isRunnable = info.getState() < Compilation.RESOLVED
-      || type.isSubtype(Compilation.typeRunnable);
+    boolean isRunnable = (info.getState() < Compilation.RESOLVED
+                          ? info.getCompilation().makeRunnable()
+                          : type.isSubtype(Compilation.typeRunnable));
     Declaration decl = null;
     ClassType thisType = ClassType.make("kawa.standard.require");
     Expression[] args = { new QuoteExp(tname) };
@@ -277,12 +295,35 @@ public class require extends Syntax
     for (Declaration fdecl = mod.firstDecl();
          fdecl != null;  fdecl = fdecl.nextDecl())
       {
-        Object fdname = fdecl.getSymbol();
+        if (fdecl.isPrivate())
+          continue;
+
+        Symbol aname = (Symbol) fdecl.getSymbol();
+        if (renamer != null)
+          {
+            Object mapped;
+            try
+              {
+                mapped = renamer.apply1(aname);
+              }
+            catch (Throwable ex)
+              {
+                mapped = ex;
+              }
+            if (mapped == null)
+              continue;
+            if (! (mapped instanceof Symbol))
+              {
+                tr.error('e', "internal error - import name mapper returned non-symbol: "+mapped.getClass().getName());
+                continue;
+              }
+            aname = (Symbol) mapped;
+          }
         boolean isStatic = fdecl.getFlag(Declaration.STATIC_SPECIFIED);
         if (! isStatic && decl == null)
           {
             String iname = tname.replace('.', '$') + "$instance";
-            decl = new Declaration(iname.intern(), type);
+            decl = new Declaration(SimpleSymbol.valueOf(iname), type);
             if (! immediate)
               decl.setPrivate(true);
             decl.setFlag(Declaration.IS_CONSTANT
@@ -304,9 +345,6 @@ public class require extends Syntax
             decl.setFlag(Declaration.TYPE_SPECIFIED);
           }
 
-        if (fdecl.isPrivate())
-          continue;
-
         if (fdecl.field != null)
           {
             String fname = fdecl.field.getName();
@@ -323,18 +361,7 @@ public class require extends Syntax
         // But if the binding is re-exported (or EXTERNAL_ACCESS
         // gets set), then we need a separate declaration.
         // (If EXTERNAL_ACCESS, the field gets PRIVATE_PREFIX.)
-        Object aname;
 
-        if (fdname instanceof Symbol)
-          aname = fdname;
-        else
-          {
-            String sname = fdname.toString();
-            if (uri == null)
-              aname = sname.intern();
-            else
-              aname = Symbol.make(uri, sname);
-          }
         boolean isImportedInstance
           = fdecl.field != null && fdecl.field.getName().endsWith("$instance");
 
@@ -382,8 +409,8 @@ public class require extends Syntax
             if (! immediate)
               adecl.setPrivate(true);
           } 
-        if (fdecl.getFlag(Declaration.IS_CONSTANT))
-         adecl.setFlag(Declaration.IS_CONSTANT);
+        // Imported variables should be read-only.
+        adecl.setFlag(Declaration.IS_CONSTANT);
         if (fdecl.getFlag(Declaration.IS_SYNTAX))
           adecl.setFlag(Declaration.IS_SYNTAX);
         if (fdecl.isProcedureDecl())
@@ -432,7 +459,7 @@ public class require extends Syntax
                 String iname
                   = (xdecl.field.getDeclaringClass().getName().replace('.', '$')
                      + "$instance");
-                Declaration cdecl = defs.lookup(iname.intern());
+                Declaration cdecl = defs.lookup(SimpleSymbol.valueOf(iname));
                 cdecl.setFlag(Declaration.EXPORT_SPECIFIED);
                 aref.setContextDecl(cdecl);
               }
