@@ -1,9 +1,8 @@
-(test-init "macros" 96)
+(test-init "macros" 101)
 
 (test 'ok 'letxx (let ((xx #f)) (cond (#t xx 'ok))))
 
-;;; FIXME this does not work - hygiene problem!
-;;;(test 'ok 'let=> (let ((=> #f)) (cond (#t => 'ok))))
+(test 'ok 'let=> (let ((=> #f)) (cond (#t => 'ok))))
 
 (load (string-append src-prefix "mac1.scm"))
 (test '(1 2) 'something (something 1 2))
@@ -381,7 +380,7 @@
     ((_ slotname)
      (field (this) (quote slotname)))))
 (define-simple-class <xclass> ()
-  (x :init 0)
+  (x init: 0)
   ((incx) <void>
    (set! (slot x) (+ 1 (slot x))))
   ((incx2) <void>
@@ -479,3 +478,96 @@
 		      (syntax
 		       (lambda args (begin bodye1))))))))
   (test 3 'savannah-bug-18105 ((crashing-syntax (arg1 arg2) 3) 1 2)))
+
+;; Luis Casillas <luis@casillas.org> posted to Kawa list 2007-02-02:
+(define (alter-syntax-datum proc stx)
+  ;; must use define-syntax-datum in PLT
+  (datum->syntax-object stx (proc (syntax-object->datum stx))))
+(define-syntax define-symbol-altering-macro
+  (syntax-rules ()
+    ((_ (macro-name arg) expr . exprs)
+     (define-symbol-altering-macro macro-name (lambda (arg) expr . exprs)))
+    ((_ macro-name proc)
+     (define-syntax macro-name
+       (lambda (stx)
+         (syntax-case stx ()
+           ((_ sym . args)
+            (let ((new-sym (alter-syntax-datum proc (syntax sym))))
+              ;; must use #, in PLT
+              #`(,new-sym . args)))))))))
+(define-symbol-altering-macro (call-reversename sym)
+  (string->symbol
+   (list->string
+    (reverse
+     (string->list
+      (symbol->string sym))))))
+(test 7 'symbol-altering-macro (call-reversename xam 3 2 7 6))
+
+;; Based on a bug reported by Dan Stanger <DStanger@EatonVance.Com>.
+(define (test-literal-capture-1)
+  (letrec-syntax
+      ((define-input
+	 (syntax-rules () ((define-input var) (define var 1))))
+       (test-out
+	(syntax-rules (test-content-type! define-input)
+	  ((test-out (test-content-type! expr ...)) (test-content-type! expr ...))
+	  ((test-out (define-input form)) (error (define-input form)))
+	  ((test-out expr) (list expr)))))
+
+    (test-out (symbol->string (quote b)))))
+(test '("b") test-literal-capture-1)
+
+;; A test submitted by Felix Klock <felix_klock_ii@mac.com>
+(define (test-literal-capture-2)
+  (let-syntax ((testm1 (syntax-rules (local-macro)
+                         ((testm1 (local-macro e))  `((lit1 ,e)))
+                         ((testm1 e)                `((els1 ,e))))))
+    (append
+     (testm1 'in-a)
+     (testm1 (local-macro  'in-b))
+     (let-syntax ((local-macro (syntax-rules ()
+                                 ((local-macro e) `((2 ,e))))))
+       (append
+        (testm1 (local-macro  'in-c))
+        (let-syntax ((testm2 (syntax-rules (local-macro)
+                               ((testm2 (local-macro e))
+                                `((lit2 ,e)))
+                               ((testm2 e)
+                                `((els2 ,e))))))
+          (testm2 (local-macro  'in-d)))
+        (let-syntax ((testm3 (syntax-rules (local-macro)
+                               ((testm3 (local-macro e))
+                                `((lit3 ,e)))
+                               ((testm3 e)
+                                `((els3 ,e)))))
+                     (local-macro (syntax-rules ()
+                                    ((local-macro e)
+                                     `((4 ,e))))))
+          (testm3 (local-macro 'in-e)))
+        (let-syntax ((testm5 (syntax-rules (local-macro)
+                               ((testm5 (local-macro e))
+                                `((lit5 ,e)))
+                               ((testm4 e)
+                                `((els5 ,e))))))
+          (let-syntax ((local-macro (syntax-rules ()
+                                      ((local-macro e)
+                                       `((6 ,e))))))
+            (testm5 (local-macro 'in-f))))
+        (let-syntax ((local-macro (syntax-rules ()
+                                    ((local-macro e)
+                                     `((7 ,e))))))
+          (let-syntax ((testm8 (syntax-rules (local-macro)
+                                 ((testm8 (local-macro e))
+                                  `((lit8 ,e)))
+                                 ((testm8 e)
+                                  `((els8 ,e))))))
+            (testm8 (local-macro 'in-g))))
+        )))))
+(test '((els1 in-a) (lit1 in-b) (els1 ((2 in-c))) (lit2 in-d) (els3 ((4 in-e))) (els5 ((6 in-f))) (lit8 in-g))
+      test-literal-capture-2)
+
+;; Savannah bug #26993 "String literals in syntax-rules don't match".
+(define-syntax foo-26993
+   (syntax-rules ()
+     ((foo-26993 "foo") 'ok)))
+(test 'ok 'test-savannah-26993 (foo-26993 "foo"))
