@@ -1,4 +1,4 @@
-(test-init "Miscellaneous" 151)
+(test-init "Miscellaneous" 176)
 
 ;;; DSSSL spec example 11
 (test '(3 4 5 6) (lambda x x) 3 4 5 6)
@@ -115,7 +115,7 @@
 (section "new-line handling")
 ;;; Test that #\return and #\newline are read robustly.
 
-(define cr-test-string "a \"bRLc\" dRklLXY")
+(define cr-test-string (string-copy "a \"bRLc\" dRklLXY"))
 (do ((i 0 (+ i 1)))
     ((= i (string-length cr-test-string)) #t)
   (if (char=? #\R (string-ref cr-test-string i))
@@ -497,16 +497,16 @@
 (define (fl-f y) (+ 10 y))
 (fluid-let ((fl-x 2)) (fl-f 1))
 
-(test '|10a| 'to-hex-1 (java.lang.Integer:toHexString 266))
+(test "10a" 'to-hex-1 (java.lang.Integer:toHexString 266))
 (define (to-hex (x :: <int>)) (java.lang.Integer:toHexString x))
-(test '|10b| to-hex 267)
+(test "10b" to-hex 267)
 (define-namespace Long "class:java.lang.Long")
-(test '|10d| 'to-hex-1 (Long:toHexString 269))
+(test "10d" 'to-hex-1 (Long:toHexString 269))
 (define (long-to-hex (x :: <long>)) (Long:toHexString x))
-(test '|10e| long-to-hex 270)
-(test '|123| Long:toString (Long:new '00123))
+(test "10e" long-to-hex 270)
+(test "123" Long:toString (Long:new '00123))
 (define (to-int-string x :: <long>) (java.lang.Object:toString (Long:new x)))
-(test '|124| to-int-string '00124)
+(test "124" to-int-string '00124)
 
 ;;; Based on bug report 2002-12-3 from Petter &Ouml;sterlund <petos@fyrplus.se>
 (define (fie-6)
@@ -557,7 +557,6 @@
 (test "[ 23]" test-printf "[%3d]" 23)
 (test "[3.50 ]" test-printf "[%-5.2f]" 3.5)
 
-
 (define fluid-stack '())
 (define fluid-let-test-level 'main)
 (define (push-fluid-let-test-level!)
@@ -581,7 +580,8 @@
   (test "" string-append/shared)
   (test "" string-append/shared "")
   (test "abc" string-append/shared str)
-  (test "abc123xy" string-append/shared str "123" "xy")
+  (set! str (string-append/shared str "123" "xy"))
+  (test "abc123xy" 'string-append/shared str)
   (test #t equal? "abc123xy" str))
 
 (test "Test." 'from-psyntax
@@ -712,3 +712,180 @@
       (as-xml (h:code "Foo")))
 (test "<b xmlns=\"http://www.w3.org/1999/xhtml\"><code>Foo</code></b>" 'html-contructor-4
       (as-xml (html:b (html:code "Foo"))))
+
+(test "<list><b xmlns=\"http://www.w3.org/1999/xhtml\">bold 1</b> <b xmlns=\"http://www.w3.org/1999/xhtml\">bold2</b></list>" 'html-contructor-4 (as-xml (map html:b '("bold 1" "bold2"))))
+
+;; Based on Savannah bug#18736, "intenal compile error -- svn rev 5816".
+;; From Thomas Kirk <tk@research.att.com>
+(test #t 'test-savannah-18736
+      (let* ((elapsed 0)
+	     (oldtime (java.lang.System:currentTimeMillis))
+	     (val ((lambda () (sleep 0.015))))
+	     (ignored
+	      (begin
+		(set! elapsed (- (java.lang.System:currentTimeMillis) oldtime))
+		val)))
+	;; While time resolution is non-portable, assume a sleep of 20ms
+	;; will be detectable as taking at least 10ms.
+	(>= elapsed 10)))
+
+;; Test for Savannah bug #18909 "Recursive call to function in closure causes
+;; NullPointerException".  Chris Wegrzyn <chris.wegrzyn@gmail.com>
+(define (savannah-18909-outerproc foo)
+  (define (innerproc)
+    (if foo
+	(lambda () (innerproc))
+	'()))
+  (innerproc))
+(define savannah-18909-destroy ((savannah-18909-outerproc #t))) 
+(test savannah-18909-destroy 'savannah-18909 (savannah-18909-destroy))
+
+;; Bug reported by Yaroslav Kavenchuk <kavenchuk@jenty.by> 2008-02-26:
+(define primes (<integer[]> 2 3 5 7 11 13))
+(test 11 'primes-integer-indexing (primes 4))
+
+(define sum 0)
+(define (test-exit-with-finally-1 x)
+  (call-with-current-continuation
+   (lambda (exit)
+     (try-finally
+      (if (< x 0)
+	  (exit (list x))
+	  (* 2 x))
+      (set! sum (+ sum 1))))))
+(test '(8 10 (-9) (-1) 24) 'test-exit-with-finally-1
+      (map test-exit-with-finally-1 '(4 5 -9 -1 12)))
+
+(set! sum 0)
+(define list-inner '())
+(define (test-exit-with-finally-2 x)
+  (call-with-current-continuation
+   (lambda (exit1)
+     (try-finally
+      (call-with-current-continuation
+       (lambda (exit2)
+	 (try-finally
+	  (begin
+	    (if (< x 0)
+		(exit2 (list 2 x)))
+	    (if (odd? x)
+		(exit1 (list 1 x)))
+	    (set! list-inner (cons x list-inner))
+	    (list 0 x))
+	  (set! sum (+ sum 1)) #| Inner finally |#)))
+      (set! sum (+ sum 10)) #| Outer finally |#))))
+(test '((0 4) (1 5) (2 -9) (0 14) (2 -1) (0 12) (1 7))
+	'test-exit-with-finally-2
+      (map test-exit-with-finally-2 '(4 5 -9 14 -1 12 7)))
+(test "Sum: 77 Inner: (12 14 4)"
+      'test-exit-with-finally-2-results
+      (format #f "Sum: ~s Inner: ~s" sum list-inner))
+
+;; R6RS and SRFI-62 S-expression comments
+(test 5 'srfi-62-test-1 (+ 1 #;(* 2 3) 4))
+(test '(x z) 'srfi-62-test-2 (list 'x #;'y 'z))
+(test 12 'srfi-62-test-3 (* 3 4 #;(+ 1 2)))
+(test 16 'srfi-62-test-4 (#;sqrt abs -16))
+(test '(a d) 'srfi-62-test-5 (list 'a #; #;'b 'c 'd))
+(test '(a e) 'srfi-62-test-6 (list 'a #;(list 'b #;c 'd) 'e))
+(test '(a . c) 'srfi-62-test-7 '(a . #;b c))
+(test '(a . b) 'srfi-62-test-8 '(a . b #;c))
+
+;; Savannah bug #26940 "Compiler stuck in endless loop"
+;; Reported by Helmut Eller
+(define (mutual-tailcals x)
+  ;; Note that the order of the functions is reversed relative to the
+  ;; Savannah bug report, because I recently fixed the implementation of
+  ;; the letrec macro, which used to create declarations in reverse order.
+  (letrec ((f0 (lambda () (if (= x 0) 1 (f1))))
+           (f1 (lambda () (if (= x 0) (f0) (f2))))
+	   (f2 (lambda () (if (= x 0) (f1) 0))))
+    (f2)))
+(test 0 mutual-tailcals 4)
+
+;; Savannah bug #24249 "Local define miscompiled"
+(let ()
+  ;; Added 'list' to suppress tail-call-optimization.
+  (define (baz) (list (bar)))
+  (define (bar)
+    (let ((k (lambda () #f)))
+      (cond ((not (procedure? k))
+	     (error 'bad-k k)))
+      k))
+  (define (foo) (bar))
+  (test "#<procedure k>" 'test-savannah-24249 ((foo):toString)))
+
+(require <InliningTest>)
+(test 16 inline-two-calls 5)
+(test 7 inline-two-calls -5)
+(test #f check-even 200001)
+(test #t check-even 18)
+
+;; Savannah bug #27011: ArrayIndexOutOfBoundsException after 20 local variables
+(define (big-let) ; no argument!
+  (define x0  "a")
+  (define x1  "a")
+  (define x2  "a")
+  (define x3  "a")
+  (define x4  "a")
+  (define x5  "a")
+  (define x6  "a")
+  (define x7  "a")
+  (define x8  "a")
+  (define x9  "a")
+  (define x10 "a")
+  (define x11 "a")
+  (define x12 "a")
+  (define x13 "a")
+  (define x14 "a")
+  (define x15 "a")
+  (define x16 "a")
+  (define x17 "a")
+  (define x18 "a")
+  (define x19 "a")
+  (define x20 "a")
+  (set! x0  x1 )
+  (set! x1  x2 )
+  (set! x2  x3 ) 
+  (set! x3  x4 )
+  (set! x4  x5 )
+  (set! x5  x6 )
+  (set! x6  x7 )
+  (set! x7  x8 )
+  (set! x8  x9 )
+  (set! x9  x10)
+  (set! x10 x11)
+  (set! x11 x12)
+  (set! x12 x13)
+  (set! x13 x14)
+  (set! x14 x15)
+  (set! x15 x16)
+  (set! x16 x17)
+  (set! x17 x18)
+  (set! x18 x19)
+  (set! x19 x20)
+  (set! x20 x0 ) 
+  x0)
+(test "a" big-let)
+
+;; Savannah bug #27014 "AND vs. VOID"
+(begin
+  (define (foo) (and (bar) (bar)))
+  (define baz #f)
+  (define (bar) (set! baz #f)))
+
+;; Savannah bug #27019 "setLength method of StringBuilder not found"
+(define sb (java.lang.StringBuilder "abcdef"))
+(define (set-length (builder :: java.lang.StringBuilder) (len :: int))
+  (with-compile-options warn-invoke-unknown-method: #t
+			warn-as-error: #t
+			(builder:setLength len)))
+(set-length sb 4)
+(test "abcd" 'test-savannah-27019 (sb:toString))
+
+;; Savannah bug #27188 "Sequence printing"
+(define sublist-27188 (invoke #(10 11 12 13 14 15) 'subList 1 3))
+(test "(#sequence 11 12)" 'test-savannah-27188 (format #f "~s" sublist-27188))
+
+(require "test-cycle12.scm")
+(test '(8 12) c1x-c2x)
