@@ -1,4 +1,4 @@
-(test-init "Objects" 129)
+(test-init "Objects" 131)
 
 ;; Force procedure to be applied without being inlined:
 (define-syntax force-eval
@@ -40,7 +40,7 @@
 (set! lt1:a! 9)
 (test (make-funny-record2 14 9) 'funny-record lt1)
 
-(test '(10 \10 20 \20) 'object-with-field-1
+(test '(10 "10" 20 "20") 'object-with-field-1
        (let*
 	   ((obj (object (<object>)
 			 (fld 10)
@@ -54,7 +54,7 @@
 		(str2 (as <String> obj)))
 	     (list val1 str1 val2 str2)))))
 
-(test '(100 \100 20 \20) 'object-with-field-2
+(test '(100 "100" 20 "20") 'object-with-field-2
        (let*
 	   ((val0 100)
 	    (obj (object (<object>)
@@ -122,7 +122,7 @@
              (format #f "{arg1: ~s arg2: ~s}" arg1 arg2)))))
 
 (test "{arg1: 23 arg2: 12}" 'object-with-closure-3
-      (symbol->string (as <String> ((document-filter 23) 12))))
+      (as <String> ((document-filter 23) 12)))
 
 (define i100 (force-eval make <integer> ival: 100))
 (define i200 (force-compile make <integer> ival: 200))
@@ -163,7 +163,7 @@
 (test 2 list-length-3 '(a b))
 (test 0 length (classify))
 
-(test 3 length-diff1 'abcdef 'abc)
+(test 3 length-diff1 "abcdef" "abc")
 (test 3 length-diff2 'abcdef 'abc)
 (test 3 length-diff3 'abcdef 'abc)
 
@@ -292,6 +292,10 @@
 (require <MyModule>)
 (test '(#t 5 6) my-func-t 5 6)
 
+(define TestCapturedFieldRef-instance (<TestCapturedFieldRef> 7))
+(incr-field-function 2)
+(test 109 'test-captured-field-ref TestCapturedFieldRef-instance:var)
+
 (define-record-type pare
   (kons x y-part)
   pare?
@@ -337,8 +341,8 @@
 (define colon-test-list-1 (list 11 12 13 14))
 (test 13 'colon-test-3 colon-test-list-1:cdr:cdr:car)
 (test '(12 13 14) 'colon-test-4 colon-test-list-1:cdr)
-(test '|(11 12 13 14)| 'colon-test-5 (colon-test-list-1:toString))
-(test '|(12 13 14)| 'colon-test-6 (colon-test-list-1:cdr:toString))
+(test "(11 12 13 14)" 'colon-test-5 (colon-test-list-1:toString))
+(test "(12 13 14)" 'colon-test-6 (colon-test-list-1:cdr:toString))
 
 ;; Test for Savannah bug #4289
 (define pa-data (pa-new 10))
@@ -363,6 +367,12 @@
 (require <cycle1>)
 (test #t is-even? 8)
 (test #f is-even? 3)
+(define c1-n1 (cycle1-name1))
+(define c1-n2 (cycle1-name2))
+(define c1-n3 (cycle1-name3))
+(test "cycle1" 'check-reference-module-name
+      (and (eq? c1-n1 c1-n2) (eq? c1-n1 c1-n3)
+	   (java.lang.Class? c1-n1) (invoke c1-n1 'getName)))
 (require <cycle2>)
 (test #t is-even? 8)
 (test #f is-even? 3)
